Package: optedr
Title: Calculating Optimal and D-Augmented Designs for Single- and
        Multi-Factor Models
Version: 3.0.1
Authors@R: 
    c(person(given = "Carlos",
           family = "de la Calle-Arroyo",
           role = c("aut", "cre"),
           email = "carlos.calle.arroyo@gmail.com",
           comment = c(ORCID = "0000-0002-5099-888X")),
      person(given = "Jesús",
           family = "López-Fidalgo",
           role = c("aut"),
           comment = c(ORCID = "0000-0001-7502-8188")), 
      person(given = "Licesio J.",
           family = "Rodríguez-Aragón",
           role = c("aut"),
           comment = c(ORCID = "0000-0003-4970-3877")) 
           )
Description: Calculates D-, Ds-, A-, I- and L-optimal designs, weighted combinations of these via a Compound criterion, and KL-optimal designs for model discrimination, for non-linear single- and multi-factor models, via an implementation of the cocktail algorithm (Yu, 2011, <doi:10.1007/s11222-010-9183-2>). Multi-factor models use design variables x1, x2, … with a named-list design space; single-factor models remain backward compatible. Compares designs via their efficiency, augments any design with a controlled efficiency loss, and provides efficient rounding functions to convert approximate designs to exact ones.
License: GPL-3
Encoding: UTF-8
URL: https://github.com/kezrael/optedr,
        https://github.com/Kezrael/optedr
BugReports: https://github.com/Kezrael/optedr/issues
RoxygenNote: 7.3.1
VignetteBuilder: knitr
Suggests: testthat (>= 3.0.0), mockery, knitr, rmarkdown, markdown, DT,
        shinydashboard, shinyalert, plotly, hrbrthemes, shinyjs,
        orthopolynom, magrittr, tidyverse, nleqslv
Imports: ggplot2, purrr, rlang, crayon, cli, shiny, utils
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-03 09:58:42 UTC; Carlos
Author: Carlos de la Calle-Arroyo [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5099-888X>),
  Jesús López-Fidalgo [aut] (ORCID:
    <https://orcid.org/0000-0001-7502-8188>),
  Licesio J. Rodríguez-Aragón [aut] (ORCID:
    <https://orcid.org/0000-0003-4970-3877>)
Maintainer: Carlos de la Calle-Arroyo <carlos.calle.arroyo@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-03 16:00:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(plot,optdes)
S3method(print,augment_region)
S3method(print,optdes)
S3method(summary,optdes)
export(augment_design)
export(combinatorial_round)
export(design_efficiency)
export(efficient_round)
export(get_augment_region)
export(make_glm_family)
export(make_kl_fun)
export(opt_des)
export(shiny_augment)
export(shiny_optimal)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# optedr

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/optedr)](https://CRAN.R-project.org/package=optedr)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
<!-- badges: end -->

## Overview

**optedr** is an optimal experimental design suite for non-linear and
multi-factor models. Its main capabilities are:

- Calculating **D-, Ds-, A-, I- and L-optimal** designs for non-linear
  models with any number of factors.
- Calculating **Compound** designs combining multiple criteria with
  user-specified weights.
- Calculating **KL-Optimality** designs for model discrimination,
  supporting Normal, Poisson, Binomial and Gamma families.
- **Augmenting** existing designs with controlled D-efficiency loss.
- Evaluating the **efficiency** of a design against the optimum.
- **Rounding** approximate designs to exact designs.

## Installation

You can install the released version of optedr from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("optedr")
```

You can install the latest version from [GitHub](https://github.com/)
with:

``` r
devtools::install_github("kezrael/optedr")
```

## Functions

- `opt_des()` — calculates optimal designs.
- `design_efficiency()` — evaluates the efficiency of a design against
  the optimum.
- `get_augment_region()` — computes the candidate region for
  augmentation with controlled efficiency loss.
- `augment_design()` — adds points to a design and rescales weights.
- `efficient_round()` — rounds approximate designs to exact designs
  (multiplier method).
- `combinatorial_round()` — rounds approximate designs to exact designs
  (exhaustive floor/ceiling search).
- `make_kl_fun()` — constructs a KL divergence function from two
  model-family pairs.
- `shiny_optimal()` / `shiny_augment()` — interactive Shiny demos.

The `optdes` object returned by `opt_des()` has its own `print()`,
`summary()` and `plot()` methods.

## Usage

``` r
library(optedr)
```

    #> ℹ Loading optedr

### Optimal design

`opt_des()` requires a `criterion`, a `model`, `parameters` and their
nominal values, and a `design_space`. For multi-factor models pass
`design_space` as a named list — variable names must match those in
`model`.

``` r
library(optedr)

result_D <- opt_des(
  criterion    = "D-Optimality",
  model        = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters   = c("Vmax", "K1", "K2"),
  par_values   = c(1, 1, 1),
  design_space = list(x1 = c(0.1, 10), x2 = c(0.1, 10))
)
#> 
#> ℹ Stop condition reached: difference between sensitivity and criterion < 1e-05
#> ⠙ Calculating optimal design 22 done (11/s) | 2sℹ The lower bound for efficiency is 99.9990099490678%
#>                                                  
result_D
#> Optimal design for D-Optimality (2 factors):
#>           x1         x2    Weight
#> 1  0.8332604 10.0000000 0.3333301
#> 2 10.0000000  0.8332352 0.3333300
#> 3 10.0000000 10.0000000 0.3333399
```

`plot()` returns a heatmap of the sensitivity function, with support
points overlaid in red. By the Equivalence Theorem the design is optimal
if and only if the sensitivity equals the number of parameters at every
support point and is below it everywhere else.

``` r
plot(result_D)
```

<img src="man/figures/README-opt-plot-1.png" width="80%" />

`summary()` gives additional detail: criterion value, Atwood convergence
criterion, and the design space.

### Other criteria

The same interface supports five optimality criteria:

| Criterion         | Key argument | Objective                               |
|-------------------|--------------|-----------------------------------------|
| `"D-Optimality"`  | —            | Minimise $-\log\det M$                  |
| `"Ds-Optimality"` | `par_int`    | Focus on a subset of parameters         |
| `"A-Optimality"`  | —            | Minimise $\operatorname{tr} M^{-1}$     |
| `"I-Optimality"`  | `reg_int`    | Minimise integrated prediction variance |
| `"L-Optimality"`  | `matB`       | Minimise $\operatorname{tr}(B\,M^{-1})$ |

For example, A-Optimality minimises the average variance of all
estimators:

``` r
result_A <- opt_des(
  criterion    = "A-Optimality",
  model        = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters   = c("Vmax", "K1", "K2"),
  par_values   = c(1, 1, 1),
  design_space = list(x1 = c(0.1, 10), x2 = c(0.1, 10))
)
#> 
#> ℹ Stop condition not reached, max iterations performed
#> ⠙ Calculating optimal design 22 done (14/s) | 1.6sℹ The lower bound for efficiency is 99.9999976902864%
#>                                                    
result_A
#> Optimal design for A-Optimality (2 factors):
#>           x1         x2    Weight
#> 1  0.6569581 10.0000000 0.3725354
#> 2 10.0000000  0.6569844 0.3725333
#> 3 10.0000000 10.0000000 0.2549313
```

### Compound criterion

Combine any set of criteria as a weighted sum. Weights are normalised
automatically.

``` r
result_DI <- opt_des(
  criterion    = "Compound",
  model        = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters   = c("Vmax", "K1", "K2"),
  par_values   = c(1, 1, 1),
  design_space = list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
  compound     = list(
    list(criterion = "D-Optimality", weight = 0.6),
    list(criterion = "I-Optimality", weight = 0.4,
         reg_int = list(x1 = c(1, 5), x2 = c(1, 5)))
  )
)
#> ⠙ Calculating optimal design 7 done (3.3/s) | 2.1s⠹ Calculating optimal design 8 done (3.2/s) | 2.5s⠸ Calculating optimal design 9 done (3.1/s) | 2.9s⠼ Calculating optimal design 10 done (3/s) | 3.3s ⠴ Calculating optimal design 11 done (3.1/s) | 3.5s⠦ Calculating optimal design 13 done (3.4/s) | 3.8s⠧ Calculating optimal design 14 done (3.4/s) | 4.1s⠇ Calculating optimal design 16 done (3.6/s) | 4.4s⠏ Calculating optimal design 17 done (3.7/s) | 4.6s⠋ Calculating optimal design 18 done (3.8/s) | 4.8s⠙ Calculating optimal design 20 done (3.9/s) | 5.1s⠹ Calculating optimal design 21 done (4/s) | 5.3s  
#> ℹ Stop condition not reached, max iterations performed
#> ⠸ Calculating optimal design 22 done (4/s) | 5.5sℹ The lower bound for efficiency is 98.0491819762597%
#>                                                   
result_DI
#> Optimal design for Compound (0.60*D + 0.40*I):
#>           x1        x2    Weight
#> 1  0.9122428  8.703202 0.3548138
#> 2  1.6738998  1.439094 0.1073170
#> 3 10.0000000  1.012563 0.3389087
#> 4 10.0000000 10.000000 0.1989605
```

### Design efficiency

`design_efficiency()` measures the fraction of the optimal information
that a given design achieves. An efficiency of 0.80 means the design
needs $1/0.80 = 1.25\times$ more observations to match the optimal.

``` r
corners <- data.frame(
  x1     = c(0.1, 10,  0.1, 10),
  x2     = c(0.1, 0.1, 10,  10),
  Weight = rep(0.25, 4)
)
eff <- design_efficiency(corners, result_D)
#> ℹ The efficiency of the design is 19.3350596170462%
```

### Augmenting designs

`get_augment_region()` finds which points can be added to an existing
design while keeping its D-efficiency above a threshold `delta_val`.
`augment_design()` then adds the chosen point and rescales weights. Both
functions work in 1D and multi-dimensional spaces.

``` r
init_des <- data.frame(
  x1     = c(0.8, 10,  5),
  x2     = c(10,  0.8, 5),
  Weight = c(1/3, 1/3, 1/3)
)

region <- get_augment_region(
  criterion           = "D-Optimality",
  init_design         = init_des,
  alpha               = 0.25,
  model               = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters          = c("Vmax", "K1", "K2"),
  par_values          = c(1, 1, 1),
  design_space        = list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
  calc_optimal_design = FALSE,
  delta_val           = 0.85
)
```

<img src="man/figures/README-augment-1.png" width="80%" />

`region$region` contains the candidate points ranked by efficiency gain.
Pick the best and augment:

``` r
best <- region$region[which.max(region$region$efficiency), ]

augmented <- augment_design(
  criterion           = "D-Optimality",
  init_design         = init_des,
  alpha               = 0.25,
  model               = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters          = c("Vmax", "K1", "K2"),
  par_values          = c(1, 1, 1),
  design_space        = list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
  calc_optimal_design = FALSE,
  delta_val           = 0.85,
  new_points          = data.frame(x1 = best$x1, x2 = best$x2, Weight = 1)
)
```

<img src="man/figures/README-augment-add-1.png" width="80%" />

    #> ℹ 1903 candidate points with efficiency >= 0.85 (from LHS sample of 2000)
    #> Sample of candidate points:
    #>           x1        x2 efficiency
    #> 1  6.7385595 0.2924680  0.8909334
    #> 2  1.9821626 1.1154958  0.9394470
    #> 3  4.0971554 8.4766962  1.0007669
    #> 4  6.0239305 8.7571051  1.1028042
    #> 5  4.6024281 0.4528793  0.9276096
    #> 6  9.4770793 4.5600493  1.0465406
    #> 7  0.9444112 2.8258679  0.9368173
    #> 8  8.0232267 1.8904368  0.8922271
    #> 9  3.1618572 4.4661287  0.8625847
    #> 10 5.0458372 9.1524421  1.0662419
    #> 11 7.0391033 7.7941107  1.1160897
    #> 12 0.4494250 7.4074376  0.9311998
    #> 13 1.5010313 3.8698458  0.8872015
    #> 14 8.1851605 4.7172844  1.0284918
    #> 15 3.2994805 9.4786392  0.9698640
    augmented
    #>          x1        x2 Weight
    #> 1  0.800000 10.000000   0.25
    #> 2 10.000000  0.800000   0.25
    #> 3  5.000000  5.000000   0.25
    #> 4  9.912547  9.954952   0.25

### Rounding to exact designs

Approximate designs assign continuous weights. Two functions convert
them to integer replication counts for a given total $n$:

``` r
# Multiplier method
efficient_round(result_D$optdes, n = 12)
#>           x1         x2 Weight
#> 1  0.8332604 10.0000000      4
#> 2 10.0000000  0.8332352      4
#> 3 10.0000000 10.0000000      4

# Exhaustive floor/ceiling search (optimal for small number of support points)
combinatorial_round(result_D, n = 12)
#>           x1         x2 Weight
#> 1  0.8332604 10.0000000      4
#> 2 10.0000000  0.8332352      4
#> 3 10.0000000 10.0000000      4
```

### KL-Optimality

KL-Optimality finds the design that maximises the Kullback-Leibler
divergence between a reference model and the most adversarial rival, for
Normal, Poisson, Binomial and Gamma families.

``` r
result_kl <- opt_des(
  "KL-Optimality",
  model        = y ~ Vmax * x / (Km + x),
  parameters   = c("Vmax", "Km"),
  par_values   = c(2, 1),
  design_space = c(0.1, 5),
  rival_model  = y ~ a * x,
  rival_params = c("a"),
  rival_pars   = c(1),
  rival_lower  = c(0.2),
  rival_upper  = c(2.5),
  family       = "Normal",
  phi          = 1
)
#> 
#> ℹ Stop condition reached: difference between sensitivity and criterion < 1e-05
#> ⠙ Calculating optimal design 7 done (8.1/s) | 863msℹ The lower bound for efficiency is 99.9957589718079%
#>                                                     
result_kl
#> Optimal design for KL-Optimality:
#>      Point    Weight
#> 1 1.127578 0.8159831
#> 2 5.000000 0.1840169
plot(result_kl)
```

<img src="man/figures/README-kl-1.png" width="80%" />

`make_kl_fun()` automates the KL function construction for custom
model-family pairs, including different error structures or families.

## Learn more

- `vignette("optedr-intro")` — optimality criteria, multi-dimensional
  design spaces, compound designs, efficiency and rounding.
- `vignette("optedr-augment")` — augmenting designs in 1D, 2D and 3D
  with controlled efficiency loss.
- `vignette("optedr-kl")` — KL-Optimality for model discrimination,
  `make_kl_fun()`, and the Hill model case study.

library(shiny)
library(dplyr)
library(DT)
library(ggplot2)
library(shinydashboard)
library(shinyalert)
library(plotly)
library(tidyverse)
library(hrbrthemes)
library(nleqslv)
library(shinyjs)


# Source dopt ----
source("dopt.R")

ui <- dashboardPage(
  dashboardHeader(title = "Design Calculator"),
  dashboardSidebar(
    sidebarMenu(id="sidebar",
                tags$head(tags$style(HTML(
                  '.myClass {
                font-size: 20px;
                line-height: 50px;
                text-align: left;
                font-family: "Helvetica Neue",Helvetica,Arial,sans-serif;
                padding: 0 15px;
                overflow: hidden;
                color: white;
                }
                '))),
                tags$script(HTML('
                               $(document).ready(function() {
                               $("header").find("nav").append(\'<div id="pageHeader" class="myClass"></div>\');
                               })
                               '))
                ,
                useShinyjs(),
                menuItem("Presentation", tabName = "presentation", icon = icon("info")),
                menuItem("Optimal design", tabName = "optdes", icon = icon("calculator")),
                menuItem("Custom design", tabName = "custom", icon = icon("clipboard")),
                # menuItem("Efficiency of industry designs", tabName = "compare", icon = icon("book")),
                # menuItem("Help", tabName = "help", icon = icon("question")),
                menuItem("Contact", tabName = "contact", icon = icon("envelope"))
    ),
    hr(),
    conditionalPanel(condition="input.sidebar == 'optdes'",
        selectizeInput(
          'variance', 'Variance Structure', choices = c("Homoscedastic", "Heteroscedastic"),
          options = list(
            placeholder = 'Type the desired variance',
            onInitialize = I('function() { this.setValue(""); }')
          )
        ),
        selectizeInput(
          'criteria', 'Optimality Criteria', choices = c("D-Optimality", "A-Optimality", "Ds-Optimality", "I-Optimality"),
          options = list(
            placeholder = 'Type the desired criteria',
            onInitialize = I('function() { this.setValue(""); }')
          )
        ),
        selectizeInput(
          'substance', 'Substance', choices = c("Water", "Acetic Acid", "Benzene", "Carbon Dioxide"#, "Methane"
                                                , "Naphtalene"),
          options = list(
            placeholder = 'Optionally choose a substance',
            onInitialize = I('function() { this.setValue(""); }')
          )
        ),
        sliderInput("design_sp", "Design Space",
                    min = -200, max = 400, value = c(1, 100)
        ),
        numericInput('a_par', 'Initial value for parameter a', value = 8.07131, step = 0.01
        ),
        numericInput('b_par', 'Initial value for parameter b', value = 1730.63, step = 0.01
        ),
        numericInput('c_par', 'Initial value for parameter c', value = 233.426, step = 0.01
        ),
        conditionalPanel("input.criteria == 'I-Optimality'",
                         sliderInput("int_region", "Region of Interest",
                                     min = -200, max = 400, value = c(80, 120)),
                         radioButtons("radio_integrate", label = h3("Distribution of choice"),
                                      choices = list("Uniform" = 1, "Triangular" = 2),
                                      selected = 1)
        ),
        conditionalPanel("input.radio_integrate == 2 && input.criteria == 'I-Optimality'",
                         uiOutput("vertexChoice")
        ),
        conditionalPanel("input.criteria == 'Ds-Optimality'",
                         checkboxGroupInput("intPars", "Parameters of Interest:",
                                            c("A" = 1,
                                              "B" = 2,
                                              "C" = 3))
        ),
        conditionalPanel("input.criteria == 'Ds-Optimality' || input.criteria == 'I-Optimality' || input.criteria == 'A-Optimality'",
                         checkboxInput("checkAlgoC", "Show convergence of algorithm?", FALSE)),
        actionButton("calc_opt_design", "Calculate Optimal Design"
        )
    ),
    #sliderInput("int_region", "Region of Interest",
    #            min = 1, max = 374, value = c(1, 100)
    #),
    #tags$p("Only for I-Optimality"
    #),
    conditionalPanel(condition="input.sidebar == 'custom'",
        numericInput('points_add', '1. Points', value = 0, min = 1, max = 374, step = 1
        ),
        conditionalPanel('input.points_add != 0',
                         numericInput('weights_add', '2. Weights', value = 0, min = 0, max = 1, step = 0.01)
        ),
        actionButton("add", "Add point"
        ),
        actionButton("remove", "Remove point"
        ),
        actionButton("both_designs", "Calculate efficiency"
        )
    # ),
    # conditionalPanel(condition="input.sidebar == 'compare'",
    #    conditionalPanel(condition="input.criteria == 'D-Optimality' || input.criteria == 'Ds-Optimality'",
    #      selectizeInput(
    #        'family', 'Choose a family of designs', choices = c("Equidistant", "Arithmetic", "Geometric"
    #        ),
    #        options = list(
    #          placeholder = 'Type the desired family',
    #          onInitialize = I('function() { this.setValue(""); }')
    #        )
    #      )
    #    ),
    #    conditionalPanel(condition="input.criteria == 'A-Optimality' || input.criteria == 'I-Optimality'",
    #                     selectizeInput(
    #                       'family2', 'Choose a family of designs', choices = c("Equidistant"#, "Arithmetic", "Geometric"
    #                       ),
    #                       options = list(
    #                         placeholder = 'Type the desired family',
    #                         onInitialize = I('function() { this.setValue(""); }')
    #                       )
    #                     )
    #    ),
    #    sliderInput("ind_points", "Points of the design",
    #                min = 3, max = 20, value = 5
    #    ),
    #    actionButton("calc_ind_eff", "Calculate Efficiencies"
    #    )
    )
  ),
  dashboardBody(
    useShinyalert(),
    tabItems(
      tabItem(tabName = "presentation", fluidPage(withMathJax(includeMarkdown("presentacion.Rmd")))),
      tabItem(tabName = "optdes",
              fluidRow(
                valueBox(value = "Optimum Design"#tags$p("Optimum Design", style = "font-size: 75%;")
                         , "For the selected criterion", icon = icon("check"), color = "yellow"),
                valueBoxOutput("op_sensitivity")
              ),
              fluidRow(
                box(title = "Optimum design",
                    solidHeader = T,
                    width = 4,
                    collapsible = T,
                    div(DT::DTOutput("opt_df"), style = "font-size: 120%;")
                ),
                box(title = "Sensitivity Function", solidHeader = T,
                    width = 8, collapsible = T,
                    plotlyOutput("sens_opt")
                )
              ), # row
              uiOutput('conditional_selection')
      ),
      tabItem(tabName = "custom",
              fluidRow(
                valueBoxOutput("new_design"),
                valueBoxOutput("new_sensitivity"),
                valueBoxOutput("efficiency")
              ), # row
              fluidRow(
                box(title = "New design",
                    solidHeader = T,
                    width = 4,
                    collapsible = T,
                    div(DT::DTOutput("custom_des"), style = "font-size: 120%;")
                ),
                box(title = "New sensitivity", solidHeader = T,
                    width = 8, collapsible = T,
                    plotlyOutput("sens_new")
                )
              )
      ),
    #   tabItem(tabName = "compare",
    # # Row --
    # fluidRow(
    #   valueBoxOutput("industry_des"),
    #   valueBoxOutput("industry_sens"),
    #   valueBoxOutput("efficiency_ind")
    # ),
    # # Row --fas
    # fluidRow(
    #   box(title = "Industry design",
    #       solidHeader = T,
    #       width = 4,
    #       collapsible = T,
    #       div(DT::DTOutput("ind_df"), style = "font-size: 120%;")
    #   ),
    #   box(title = "Sensitivity Function", solidHeader = T,
    #       width = 8, collapsible = T,
    #       plotlyOutput("sens_ind")
    #   )
    # )
    # ),
      # tabItem(tabName = "help", fluidPage(withMathJax(includeMarkdown("help.Rmd")))),
      tabItem(tabName = "contact", fluidPage(withMathJax(includeMarkdown("contact.Rmd"))))
      )
    )
  )


# Define server logic required to draw a histogram
server <- function(input, output, session) {

  # make reactive to store points
  ing_df <- shiny::reactiveValues()

  # Tm<-86;
  # TM<-218;
  ##Par?metros
  # p1<-8.07763 ;
  # p2<-2603.26 ;
  # p3<-282.769 ;
  # Update Nominal Values and Space Design based on Substance
  observeEvent(input$substance, {
    updateSliderInput(session, "design_sp", value = c(switch (input$substance,
                                                          "Water" = 1,
                                                          "Acetic Acid" = 17,
                                                          "Benzene" = 8,
                                                          "Carbon Dioxide" = -62,
                                                          #"Methane" = -181,
                                                          "Naphtalene" = 86
                                                      ), switch (input$substance,
                                                          "Water" = 100,
                                                          "Acetic Acid" = 118,
                                                          "Benzene" = 80,
                                                          "Carbon Dioxide" = 31,
                                                          #"Methane" = -163,
                                                          "Naphtalene" = 218
                                                      )
                      )
    )
    updateSliderInput(session, "int_region", value = c(switch (input$substance,
                                                          "Water" = 80,
                                                          "Acetic Acid" = 110,
                                                          "Benzene" = 75,
                                                          "Carbon Dioxide" = 20,
                                                          #"Methane" = -170,
                                                          "Naphtalene" = 200
                                                      ), switch (input$substance,
                                                          "Water" = 100,
                                                          "Acetic Acid" = 118,
                                                          "Benzene" = 80,
                                                          "Carbon Dioxide" = 31,
                                                          #"Methane" = -140,
                                                          "Naphtalene" = 218
                                                      )
                      )
    )
    updateNumericInput(session, "a_par", value = switch (input$substance,
                                                         "Water" = 8.07131,
                                                         "Acetic Acid" = 7.5596,
                                                         "Benzene" = 6.87987,
                                                         "Carbon Dioxide" = 7.5322,
                                                         #"Methane" = 6.34159,
                                                         "Naphtalene" = 8.07763
    ))
    updateNumericInput(session, "b_par", value =switch (input$substance,
                                                        "Water" = 1730.63,
                                                        "Acetic Acid" = 1644.05,
                                                        "Benzene" = 1196.76,
                                                        "Carbon Dioxide" = 835.06,
                                                        #"Methane" = 342.22,
                                                        "Naphtalene" = 2603.26
    ))
    updateNumericInput(session, "c_par", value = switch (input$substance,
                                                         "Water" = 233.426,
                                                         "Acetic Acid" = 233.524,
                                                         "Benzene" = 219.161,
                                                         "Carbon Dioxide" = 268.223,
                                                         #"Methane" = 260.221,
                                                         "Naphtalene" = 282.769
    ))
  })

  # Update the input dataframe
  observeEvent(input$remove, {
    isolate(ing_df$design<-ing_df$design[-(nrow(ing_df$design)),])
  })

  # Remove
  observeEvent(input$add, {
    isolate(ing_df$design[nrow(ing_df$design) + 1,] <- as.list(c(input$points_add,
                                                                 input$weights_add)
    ))

    # update choices
    updateNumericInput(session, 'weights_add', '2. Weights', 0)
    updateNumericInput(session, 'points_add', '1. Points', value = 0)
  })

  # Render custom design
  output$custom_des <- DT::renderDataTable({
    datatable(arrange(ing_df$design, Point), rownames=F, options = list(pageLength = 5)) %>% formatRound(c(1,2), 3)
  })

  # Industrial des
  output$ind_df <- DT::renderDataTable({
    datatable(arrange(ing_df$ind_des, Point), rownames=F, options = list(pageLength = 10)) %>% formatRound(c(1,2), 3)
  })

  # Render optimum design
  output$opt_df <- DT::renderDataTable({
    datatable(arrange(ing_df$opt_design, Point), rownames=F, options = list(pageLength = 5)) %>% formatRound(c(1,2), 3)
  })

  # Optimum design
  ing_df$opt_design <- data.frame("Point" = numeric(),
                                  "Weight" = numeric())

  # Custom design
  ing_df$design <- data.frame("Point" = numeric(),
                              "Weight" = numeric())

  # Convergence of the Algorithm
  ing_df$alg_conv <- data.frame("criteria" = numeric(),
                                "step" = numeric())

  # Industry design
  # ing_df$ind_des <- data.frame("Point" = numeric(),
  #                              "Weight" = numeric())


  # Render I-Opt triang center
  output$vertexChoice <- renderUI({
    sliderInput("vertex", "Vertex of the triangule",
                min = input$int_region[[1]], max = input$int_region[[2]], value = (input$int_region[[2]]+input$int_region[[1]])/2)
  })


  observeEvent(input$calc_opt_design, {
    if(!input$criteria %in% c("D-Optimality", "Ds-Optimality", "A-Optimality", "I-Optimality") |
       !input$variance %in% c("Homoscedastic", "Heteroscedastic")){
      shinyalert("Enter a valid criterion and variance!", type = "error")
    }
    else if(identical(input$criteria, "Ds-Optimality") & length(input$intPars) < 1){
      shinyalert("Select the parameters of interest", type = "warning")
    }
    else if(identical(input$criteria, "Ds-Optimality") & length(input$intPars) > 2){
      shinyalert("You cannot select all three parameters", type = "error")
    }
    else {
      if (identical(input$criteria, "I-Optimality")){
        if(input$radio_integrate == 1)
          B <- IntMatAntoine(input$variance, input$a_par, input$b_par, input$c_par, input$int_region[[1]], input$int_region[[2]], 0, option = "Unif")
        else if(input$radio_integrate == 2)
          B <- IntMatAntoine(input$variance, input$a_par, input$b_par, input$c_par, input$int_region[[1]], input$int_region[[2]], input$vertex, option = "Triang")
        else
          B <- diag(3)
      }
      else
        B <- diag(3)
      if(identical(input$criteria, "D-Optimality")){
        ing_df$opt_design <- doptAntoine(input$variance, input$a_par, input$b_par, input$c_par, input$design_sp[[1]], input$design_sp[[2]])
        line_top <- 3
      } else {
        des <- data.frame(Point = c(input$design_sp[[1]], (input$design_sp[[1]]+input$design_sp[[2]])/2, input$design_sp[[2]]), Weight = c(1/3, 1/3, 1/3))
        outputWF <- WF(input$variance, input$criteria, input$a_par, input$b_par, input$c_par, des, as.numeric(input$intPars), B, min = input$design_sp[[1]], max = input$design_sp[[2]], grid.length = 1000, joinThresh = 9, deleteThresh = 0.01, k = 3, s = length(as.numeric(input$intPars)), 1/2, 0.00001)
        ing_df$opt_design <- outputWF$optdes
        ing_df$conv_plot <- ggplot(data = outputWF$convergence, aes(x=step, y=criteria)) +
          geom_line( color="coral1") +
          #geom_point() +
          theme_ipsum()
        if(identical(input$criteria, "Ds-Optimality"))
          line_top <- length(input$intPars)
        else
          line_top <- crit(input$criteria, dmatrixAntoine(input$variance, input$a_par, input$b_par, input$c_par, ing_df$opt_design), k = 3, s = length(intPars), intPars = as.numeric(input$intPars), matB = B)
      }
      mat <- dmatrixAntoine(input$variance, input$a_par, input$b_par, input$c_par, ing_df$opt_design)
      sens_opt <- sens(input$variance, input$criteria, input$a_par, input$b_par, input$c_par, mat, as.numeric(input$intPars), B)

      x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
      y_val <- purrr::map_dbl(x_val, sens_opt)

      p <- ggplot() + theme_ipsum()

      ing_df$sens_opt <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  line_top, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "Temperature (ºC)", y = "")
    }
  })



  # Render Graph and Box for Algo convergence if asked for

  observeEvent(input$checkAlgoC,{
    if(input$checkAlgoC == TRUE){
      output$conditional_selection <- renderUI({
        div(fluidRow(
          valueBox(value = "Convergence"#tags$p("Optimum Design", style = "font-size: 75%;")
                   , "Value of the Criteria along the steps", icon = icon("check"), color = "navy"),
          valueBoxOutput("conv_box")
        ),
        box(title = "Convergence Plot", solidHeader = T,
            width = 12, collapsible = T,
            plotlyOutput("conv_plot")
        )
        )
      })
    }
    else{
      output$conditional_selection <- NULL
    }
  })



  # Efficiency of the new design
  # Optimum design
  # ing_df$eff <- ""

  observeEvent(input$both_designs, {
    criterion <- input$criteria
    compute <- TRUE
    length1 <- nrow(ing_df$design)
    ing_df$design <- ing_df$design %>%
      group_by(Point) %>%
      summarise_all(funs(sum))

    if(length1 != nrow(ing_df$design)){
      shinyalert("", "There are repeated points in the design, they have been added together"
                 , type = "warning"
                 , timer = 1500)
    }

    if(length(ing_df$design[["Point"]]) < 3)
    {
      shinyalert("The design is singular!", type = "error")
      compute <- FALSE
    }
    else if(0 %in% ing_df$design[["Weight"]])
    {
      compute <- FALSE
      shinyalert("Error", "There are points without weight!", type = "error")
    }
    else if(max(ing_df$design[["Point"]]) > input$design_sp[[2]] || min(ing_df$design[["Point"]]) < input$design_sp[[1]])
    {
      compute <- FALSE
      shinyalert("Error", "There are design points outside the design space!", type = "error")
    }
    else if(!near(1, sum(ing_df$design[["Weight"]])))
    {
      shinyalert("", "The weights don't add up to 1, it'll be corrected"
                 # , do you want us to fix that for you?"
                 , type = "warning"
                 # , showCancelButton = TRUE, showConfirmButton = TRUE, callbackR = mycallback
                 , timer = 1500)
      # if (TRUE) {
      ing_df$design[["Weight"]] <- ing_df$design[["Weight"]]/sum(ing_df$design[["Weight"]])
      # }
      # else {
      #   compute <- FALSE
      # }
    }
    if(isTRUE(compute)) {
      if(!input$criteria %in% c("D-Optimality", "Ds-Optimality", "A-Optimality", "I-Optimality") |
         !input$variance %in% c("Homoscedastic", "Heteroscedastic")){
        shinyalert("Enter a valid criterion and variance!", type = "error")
      }
      else if(identical(input$criteria, "Ds-Optimality") & length(input$intPars) < 1){
        shinyalert("Select the parameters of interest", type = "warning")
      }
      else if(identical(input$criteria, "Ds-Optimality") & length(input$intPars) > 2){
        shinyalert("You cannot select all three parameters", type = "error")
      }
      else {
        if (identical(input$criteria, "I-Optimality")){
          if(input$radio_integrate == 1)
            B <- IntMatAntoine(input$variance, input$a_par, input$b_par, input$c_par, input$int_region[[1]], input$int_region[[2]], 0, option = "Unif")
          else if(input$radio_integrate == 2)
            B <- IntMatAntoine(input$variance, input$a_par, input$b_par, input$c_par, input$int_region[[1]], input$int_region[[2]], input$vertex, option = "Triang")
          else
            B <- diag(3)
        }
        else
          B <- diag(3)
        if(identical(input$criteria, "D-Optimality")){
          ing_df$opt_design <- doptAntoine(input$variance, input$a_par, input$b_par, input$c_par, input$design_sp[[1]], input$design_sp[[2]])
          line_top <- 3
          line_top_opt <- 3
        } else {
          des <- data.frame(Point = c(input$design_sp[[1]], (input$design_sp[[1]]+input$design_sp[[2]])/2, input$design_sp[[2]]), Weight = c(1/3, 1/3, 1/3))
          outputWF <- WF(input$variance, input$criteria, input$a_par, input$b_par, input$c_par, des, as.numeric(input$intPars), B, min = input$design_sp[[1]], max = input$design_sp[[2]], grid.length = 1000, joinThresh = 9, deleteThresh = 0.01, k = 3, s = length(as.numeric(input$intPars)), 1/2, 0.00001)
          ing_df$opt_design <- outputWF$optdes
          ing_df$conv_plot <- ggplot(data = outputWF$convergence, aes(x=step, y=criteria)) +
            geom_line( color="coral1") +
            #geom_point() +
            theme_ipsum()
          if(identical(input$criteria, "Ds-Optimality")){
            line_top <- length(input$intPars)
            line_top_opt <- length(input$intPars)
          }
          else {
            line_top <- crit(input$criteria, dmatrixAntoine(input$variance, input$a_par, input$b_par, input$c_par, ing_df$design), k = 3, s = length(intPars), intPars = as.numeric(input$intPars), matB = B)
            line_top_opt <- crit(input$criteria, dmatrixAntoine(input$variance, input$a_par, input$b_par, input$c_par, ing_df$opt_design), k = 3, s = length(intPars), intPars = as.numeric(input$intPars), matB = B)
          }

        }
        mat_opt <- dmatrixAntoine(input$variance, input$a_par, input$b_par, input$c_par, ing_df$opt_design)
        mat <- dmatrixAntoine(input$variance, input$a_par, input$b_par, input$c_par, ing_df$design)
        ing_df$eff <- eff(input$criteria, mat, mat_opt, k = 3, s = length(input$intPars), intPars = as.numeric(input$intPars), matB = B)
        sens_opt <- sens(input$variance, input$criteria, input$a_par, input$b_par, input$c_par, mat_opt, as.numeric(input$intPars), B)
        sens_new <- sens(input$variance, input$criteria, input$a_par, input$b_par, input$c_par, mat, as.numeric(input$intPars), B)

        x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)

        y_val_opt <- purrr::map_dbl(x_val, sens_opt)
        y_val_new <- purrr::map_dbl(x_val, sens_new)

        p <- ggplot() + theme_ipsum()

        ing_df$sens_opt <- p + geom_line(mapping = aes(x = x_val, y = y_val_opt), color = "steelblue3") + geom_hline(yintercept =  line_top_opt, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "Temperature (ºC)", y = "")
        ing_df$sens_new <- p + geom_line(mapping = aes(x = x_val, y = y_val_new), color = "steelblue3") + geom_hline(yintercept =  line_top, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "Temperature (ºC)", y = "")
      }
    }
  })

  # Efficiency of the industry designs
  # observeEvent(input$calc_ind_eff, {
  #   calculate <- T
  #   B <- diag(3)
  #   if(identical(input$criteria, "D-Optimality")) {
  #     ing_df$opt_design <- doptAntoine(input$variance, input$a_par, input$b_par, input$c_par, input$design_sp[[1]], input$design_sp[[2]])
  #     opt_mat <- dmatrixAntoine(input$variance, input$a_par, input$b_par, input$c_par, ing_df$opt_design)
  #     line_top <- 3
  #   }
  #   else if(input$criteria %in% c("Ds-Optimality", "I-Optimality", "A-Optimality")){
  #     if (identical(input$criteria, "I-Optimality")){
  #       if(input$radio_integrate == 1)
  #         B <- IntMatAntoine(input$variance, input$a_par, input$b_par, input$c_par, input$int_region[[1]], input$int_region[[2]], 0, option = "Unif")
  #       else if(input$radio_integrate == 2)
  #         B <- IntMatAntoine(input$variance, input$a_par, input$b_par, input$c_par, input$int_region[[1]], input$int_region[[2]], input$vertex, option = "Triang")
  #     }
  #     des <- data.frame(Point = c(input$design_sp[[1]], (input$design_sp[[1]]+input$design_sp[[2]])/2, input$design_sp[[2]]), Weight = c(1/3, 1/3, 1/3))
  #     outputWF <- WF(input$variance, input$criteria, input$a_par, input$b_par, input$c_par, des, as.numeric(input$intPars), B, min = input$design_sp[[1]], max = input$design_sp[[2]], grid.length = 1000, joinThresh = 9, deleteThresh = 0.01, k = 3, s = length(as.numeric(input$intPars)), 1/2, 0.00001)
  #     ing_df$opt_design <- outputWF$optdes
  #     opt_mat <- dmatrixAntoine(input$variance, input$a_par, input$b_par, input$c_par, ing_df$opt_design)
  #   }
  #   else{
  #     shinyalert("Error", "Enter a valid criterion!", type = "error")
  #     calculate <- F
  #   }
  #   if(input$criteria %in% c("D-Optimality", "Ds-Optimality")){
  #     if(identical(input$family, "Equidistant")){
  #       ing_df$ind_des <- uniform_design(input$ind_points, input$design_sp[[1]], input$design_sp[[2]])
  #     }
  #     else if(identical(input$family, "Arithmetic")){
  #       out <- tryCatch(
  #         {
  #           arithmetic_design(as.numeric(input$ind_points), as.numeric(input$design_sp[[1]]), as.numeric(input$design_sp[[2]]), opt_mat, input$criteria, input$a_par, input$b_par, input$c_par, k = 3, s = length(input$intPars), intPars = as.numeric(input$intPars), matB = B)
  #         }, error=function(e) {
  #           cat(paste("in err handler\n"))
  #           shinyalert("Error", "The problem was computationally singular", type = "error")
  #           # Choose a return value in case of error
  #           return(data.frame("Point" = numeric(),
  #                             "Weight" = numeric()))
  #         }, warning=function(w) {
  #           cat(paste("in warn handler\n"))
  #           shinyalert("Error", "Error", type = "error")
  #           # Choose a return value in case of error
  #           return(data.frame("Point" = numeric(),
  #                             "Weight" = numeric()))
  #         }, finally={
  #           # message("I'm here")
  #         })
  #       ing_df$ind_des <- out
  #     }
  #     else if(identical(input$family, "Geometric")){
  #       out <- tryCatch(
  #         {
  #           geometric_design(input$variance, as.numeric(input$ind_points), as.numeric(input$design_sp[[1]]), as.numeric(input$design_sp[[2]]), opt_mat, input$criteria, input$a_par, input$b_par, input$c_par, k = 3, s = length(input$intPars), intPars = as.numeric(input$intPars), matB = B)
  #         }, error=function(e) {
  #           cat(paste("in err handler\n"))
  #           shinyalert("Error", "The problem was computationally singular", type = "error")
  #           # Choose a return value in case of error
  #           return(data.frame("Point" = numeric(),
  #                             "Weight" = numeric()))
  #         }, warning=function(w) {
  #           cat(paste("in warn handler\n"))
  #           shinyalert("Error", "Error", type = "error")
  #           # Choose a return value in case of error
  #           return(data.frame("Point" = numeric(),
  #                             "Weight" = numeric()))
  #         }, finally={
  #           # message("I'm here")
  #         })
  #
  #       ing_df$ind_des <- out
  #     }
  #     else{
  #       shinyalert("Error", "Choose a family of designs!", type = "error")
  #       calculate <- F
  #     }
  #   }
  #   else{
  #     if(identical(input$family2, "Equidistant")){
  #       ing_df$ind_des <- uniform_design(input$ind_points, input$design_sp[[1]], input$design_sp[[2]])
  #     }
  #     else{
  #       shinyalert("Error", "Choose a valid family of designs for this criterion!", type = "error")
  #       calculate <- F
  #     }
  #   }
  #   if(calculate & nrow(ing_df$ind_des)>0){
  #     if(identical(input$criteria, "Ds-Optimality")){
  #       line_top <- length(input$intPars)
  #     }
  #     else if(input$criteria %in% c("I-Optimality", "A-Optimality")){
  #       line_top <- crit(input$criteria, dmatrixAntoine(input$variance, input$a_par, input$b_par, input$c_par, ing_df$opt_design), k = 3, s = length(intPars), intPars = as.numeric(input$intPars), matB = B)
  #     }
  #     ind_mat <- dmatrixAntoine(input$variance, input$a_par, input$b_par, input$c_par, ing_df$ind_des)
  #     ing_df$effind <- eff(input$criteria, ind_mat, opt_mat, k = 3, s = length(input$intPars), intPars = as.numeric(input$intPars), matB = B)
  #
  #     sens_new <- sens(input$variance, input$criteria, input$a_par, input$b_par, input$c_par, ind_mat, as.numeric(input$intPars), B)
  #
  #     p <- ggplot() + theme_ipsum()
  #     new_des_data <- data.frame("Point" = ing_df$ind_des[["Point"]], "Weight" = ing_df$ind_des[["Weight"]], "Value" = rep(0, length(ing_df$ind_des[["Point"]])))
  #
  #     x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
  #     y_val <- purrr::map_dbl(x_val, sens_new)
  #
  #     ing_df$sens_ind <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  line_top, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "Temperature (ºC)", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))
  #   }
  #
  # })

  # Compute Effinciency
  efficiency <- reactive({
    if(is.numeric(ing_df$eff))
      round(ing_df$eff*100, 1)
    else
      ing_df$eff
  })

  # Compute Effinciency industry
  # efficiency_indus <- reactive({
  #   if(is.numeric(ing_df$effind))
  #     round(ing_df$effind*100, 1)
  #   else
  #     ing_df$effind
  # })

  # Plot sens
  ing_df$sens_opt <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()

  ing_df$sens_new <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()

  # ing_df$sens_ind <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()

  output$sens_opt <- renderPlotly({
    ggplotly(ing_df$sens_opt)
  })

  output$sens_new <- renderPlotly({
    ggplotly(ing_df$sens_new)
  })

  # output$sens_ind <- renderPlotly({
  #   ggplotly(ing_df$sens_ind)
  # })

  # Plot Conv
  ing_df$conv_plot <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()

  output$conv_plot <- renderPlotly({
    ggplotly(ing_df$conv_plot)
  })

  # value boxes
  output$op_sensitivity <- renderValueBox({
    valueBox(
      paste0("Sensitivity Function"#, output$efficiency
      ), "For the optimum design", icon = icon("bar-chart-o"),
      color = "purple"
    )
  })
  output$new_design <- renderValueBox({
    valueBox(
      paste0("Input Design"
      ), "By the user", icon = icon("clipboard-list"),
      color = "aqua"
    )
  })
  output$new_sensitivity <- renderValueBox({
    valueBox(
      paste0("Sensitivity Function"
      ), "For the input design", icon = icon("bar-chart-o"),
      color = "green"
    )
  })
  output$efficiency <- renderValueBox({
    valueBox(
      paste0("Efficiency ", efficiency(), "%" #input$count
      ), "For the input design", icon = icon("divide"),
      color = "red"
    )
  })

  # output$efficiency_ind <- renderValueBox({
  #   valueBox(
  #     paste0("Efficiency ", efficiency_indus(), "%" #input$count
  #     ), "For the input design", icon = icon("divide"),
  #     color = "red"
  #   )
  # })


  observeEvent(input$criteria, {
    header <- input$criteria

    # you can use any other dynamic content you like
    shinyjs::html("pageHeader", header)
  })
}

# Run the application
shinyApp(ui = ui, server = server)

# General Antoine

# Cálculo de la traza de una matriz
tr <- function(m) {
  return(sum(diag(m)))
}


#### HOMOCEDASTICO ----------------------------------------------------------

# Calcular matrix de un diseño para la ecuación de Antoine
dmatrixAntoineHom <- function(a, b, c, design) {
  a <- a*log(10)
  b <- b*log(10)

  m11 <- 0
  m12 <- 0
  m13 <- 0
  m21 <- 0
  m22 <- 0
  m23 <- 0
  m31 <- 0
  m32 <- 0
  m33 <- 0

  for(i in seq_along(design$Weight)){
    m11 <- m11 + exp(a - b/(c + design$Point[[i]]))*exp(a - b/(c + design$Point[[i]]))*design$Weight[[i]]
    m12 <- m12 + exp(a - b/(c + design$Point[[i]]))*(-(exp(a - b/(c + design$Point[[i]]))/(c + design$Point[[i]])))*design$Weight[[i]]
    m13 <- m13 + exp(a - b/(c + design$Point[[i]]))*(b*exp(a - b/(c + design$Point[[i]])))/(c + design$Point[[i]])^2*design$Weight[[i]]
    m21 <- m12
    m22 <- m22 + (-(exp(a - b/(c + design$Point[[i]]))/(c + design$Point[[i]])))*(-(exp(a - b/(c + design$Point[[i]]))/(c + design$Point[[i]])))*design$Weight[[i]]
    m23 <- m23 + (-(exp(a - b/(c + design$Point[[i]]))/(c + design$Point[[i]])))*(b*exp(a - b/(c + design$Point[[i]])))/(c + design$Point[[i]])^2*design$Weight[[i]]
    m31 <- m13
    m32 <- m23
    m33 <- m33 + (b*exp(a - b/(c + design$Point[[i]])))/(c + design$Point[[i]])^2*(b*exp(a - b/(c + design$Point[[i]])))/(c + design$Point[[i]])^2*design$Weight[[i]]
  }

  return(matrix(c(m11, m12, m13, m21, m22, m23, m31, m32, m33), nrow = 3, ncol = 3, byrow = TRUE))
}



# design <- data.frame(points = c(1, 50.5, 100), weights = c(1/3, 1/3, 1/3))

# (design2 <- doptAntoine(8.07131, 730.63, 233.426, 1, 100))
# matInit2 <- dmatrixAntoine(a = 8.07131, b = 730.63, c = 233.426, design = design2)

# solve(matInit2)

# matInit <- dmatrixAntoine(a = 8.07131, b = 1730.63, c = 233.426, design = design)


# Función para encontrar el x donde se alcanza el máximo de una función en una rejilla dada, evaluándolo en la misma
findmax <- function(sens, min, max, grid.length) {
  if(min <= max){
    grid <- seq(min, max, length.out = grid.length)
  }
  else {
    grid <- seq(max, min, length.out = grid.length)
  }
  xmax <- grid[which.max(map(grid, sens))]
  return(xmax)
}

# findmax(sens, 1, 100, 1000)


# Función para encontrar el mínimo de una función en una rejilla dada, evaluándolo en la misma
findminval <- function(sens, min, max, grid.length) {
  if(min <= max){
    grid <- seq(min, max, length.out = grid.length)
  }
  else {
    grid <- seq(max, min, length.out = grid.length)
  }
  minval <- min(map(grid, sens))
  return(minval)
}

# findmax(sens, 1, 100, 1000)



# Función para actualizar un diseño añadiendo un nuevo punto (o peso al mismo si está cerca de otro ya en el diseño)
updateDesign <- function(design, xmax, newalpha, delta){
  design$Weight <- design$Weight*(1-newalpha)
  absdiff <- abs(design$Point - xmax) < delta
  if (any(absdiff))
  {
    pos <- min(which(absdiff == TRUE))
    design$Point[[pos]] <- (design$Point[[pos]]*design$Weight[[pos]] + xmax*newalpha)/(design$Weight[[pos]]+newalpha)
    design$Weight[[pos]] <- design$Weight[[pos]]+newalpha
  }
  else
  {
    design[nrow(design) + 1,] <- c(xmax, newalpha)
  }
  return(design)
}

# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/4, 1/4, 1/2))

# updateDesign(des, 50, 0.6)

# Actualizar un diseño sin añadir puntos
updateDesignTotal <- function(design, delta){
  updated <- FALSE
  finished <- FALSE
  while(!finished) {
    for(i in 1:(length(design$Point)-1)) {
      absdiff <- abs(design$Point[-seq(1, i)] - design$Point[i]) < delta
      if (any(absdiff)){
        updated <- TRUE
        design <- updateDesign(design[-i,], design$Point[i], design$Weight[i], delta)
        break
      }
    }
    finished <- !updated
    updated <- FALSE
  }
  return(design)
}


# Función para eliminar los puntos con menos de un determinado peso, delta
deletePoints <- function(design, delta) {
  updatedDesign <- design[design$Weight > delta, ]
  updatedDesign$Weight <- updatedDesign$Weight/sum(updatedDesign$Weight)
  return(updatedDesign)
}

# des <- data.frame(Point = c(1, 50.5, 100, 5, 30, 80), Weight = c(1/4, 1/4, 1/4, 1/200, 1/100, 1/500))

# deletePoints(des, 0.005)






# --------------------------- D-Optimalidad ----------------------------------

# Función de sensibilidad para D-Optimalidad de la ecuación de Antoine
dsensAntoine <- function(a, b, c, mat) {
  a <- a*log(10)
  b <- b*log(10)
  invMat <- solve(mat)
  sensX <- function(x) {
    # return(exp(a - b/(c + x))*exp(a - b/(c + x))*desMat[[1,1]]+exp(a - b/(c + x))*(-(exp(a - b/(c + x))/(c + x)))*desMat[[1,2]]+exp(a - b/(c + x))*(b*exp(a - b/(c + x)))/(c + x)^2*desMat[[1,3]]+
    #          exp(a - b/(c + x))*(-(exp(a - b/(c + x))/(c + x)))*desMat[[2,1]]+(-(exp(a - b/(c + x))/(c + x)))*(-(exp(a - b/(c + x))/(c + x)))*desMat[[2,2]]+(-(exp(a - b/(c + x))/(c + x)))*(b*exp(a - b/(c + x)))/(c + x)^2*desMat[[2,3]]+
    #          (b*exp(a - b/(c + x)))/(c + x)^2*exp(a - b/(c + x))*desMat[[3,1]]+(b*exp(a - b/(c + x)))/(c + x)^2*(-(exp(a - b/(c + x))/(c + x)))*desMat[[3,2]]+(b*exp(a - b/(c + x)))/(c + x)^2*(b*exp(a - b/(c + x)))/(c + x)^2*desMat[[3,3]])
    return(exp(a - b/(c + x))*(invMat[[1,1]]*exp(a - b/(c + x)) + (invMat[[3,1]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,1]]*exp(a - b/(c + x)))/(c + x)) -
             (exp(a - b/(c + x))*(invMat[[1,2]]*exp(a - b/(c + x)) + (invMat[[3,2]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,2]]*exp(a - b/(c + x)))/(c + x)))/(c + x) +
             (b*exp(a - b/(c + x))*(invMat[[1,3]]*exp(a - b/(c + x)) + (invMat[[3,3]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,3]]*exp(a - b/(c + x)))/(c + x)))/(c + x)^2)
  }
  return(sensX)
}


# optdes <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))

# desMat <- dmatrixAntoine(8.07131, 1730.63, 233.426, optdes)

# sens <- dsensAntoine(8.07131, 1730.63, 233.426, desMat)

# sens(100)

# Cálculo Analítico del diseño D-Óptimo para la ecuación de Antoine
doptAntoineHom <- function(A, B, C, min, max){
  A <- log(10)*A
  B <- log(10)*B

  if(round(min - max, 3) == 0){
    opdes <- data.frame(Point = c(min), Weight = c(1))
    return(opdes)
  }

  T1 <- (-3*C^3+2*(B)^2*max-6*C^2*max-3*B*(C^2-max^2)-3*C*max^2-sqrt(3)*sqrt((B)^2*(C+max)^4))/(2*(B)^2+6*B*(C+max)+3*(C+max)^2)

  if(min <= T1)
  {
    T2 <- (-3*C^3+2*(B)^2*max-6*C^2*max-3*B*(C^2-max^2)-3*C*max^2+sqrt(3)*sqrt((B)^2*(C+max)^4))/(2*(B)^2+6*B*(C+max)+3*(C+max)^2)
    opdes <- data.frame(Point = c(T1, T2, max), Weight = c(1/3, 1/3, 1/3))
  }
  else
  {
    T2 <- (1/(2*(-B - 2*C - max - min)))*(2*C^2 - B*max - B*min - 2*max*min - sqrt((-2*C^2 + B*max + B*min + 2*max*min)^2 - 4*(-B - 2*C - max - min)*(C^2*max + C^2*min - B*max*min + 2*C*max*min)))
    opdes <- data.frame(Point = c(min, T2, max), Weight = c(1/3, 1/3, 1/3))
  }

  return(opdes)
}
# A <- 18.58487808693377
# B <- 1682.3377464942398
# C <- 233.426
#
# min <- 1
# max <- 100
# (1/(2*(-B - 2*C - max - min)))*(2*C^2 - B*max - B*min - 2*max*min - sqrt((-2*C^2 + B*max + B*min + 2*max*min)^2 - 4*(-B - 2*C - max - min)*(C^2*max + C^2*min - B*max*min + 2*C*max*min)))


# typeof(doptAntoine(A = 8.07131, B = 1730.63, C = 233.426, 99, 374))

# class(doptAntoine(A = 8.07131, B = 1730.63, C = 233.426, 99, 374))
# doptAntoine(A = 8.07131, B = 1730.63, C = 233.426, 1, 40)

# doptAntoine(A = 8.14019, B = 1810.94, C = 244.485, 1, 40)



# Cálculo de D-Eficiencia de la matriz 1 respecto a la matriz 2
deff <- function(mat1, mat2, k) {
  return((det(mat1)/det(mat2))^(1/k))
}

# dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$design)
#
# 5	0.375
# 35	0.3125
# 70	0.3125
#
# deff(dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$design),
#      dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$opt_design), 3)

# Valor del criterio de D-Optimalidad para la matriz
dcrit <- function(mat, k) {
  return((1/det(mat))^(1/k))
}

# dcrit(matInit, 3)


# Función para actualizar los pesos de un diseño para el algoritmo WF
updateWeights <- function(design, sens, k, delta) {
  weights <- design$Weight*(map_dbl(design$Point, sens)/k)^delta
  # print(length(design$Weight))
  # print(length((map_dbl(des$Point, sens)/k)^delta))
  return(weights/sum(weights))
}

# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/4, 1/3, 1/2))

# desMat <- dmatrixAntoine(8.07131, 1730.63, 233.426, des)

# sens <- dsensAntoine(8.07131, 1730.63, 233.426, desMat)

# (des$Weight <- updateWeights(des, sens, 3, 1/2))



# Algoritmo WF con multiplicativo para los pesos para D-Opt
WFMult <- function(A, B, C, initDes, min, max, grid.length, joinThresh, deleteThresh, k, deltaW, tol) {
  critVal <- numeric(2122)
  index <- 1
  for(i in 1:21) {
    M <- dmatrixAntoineHom(A, B, C, initDes)
    critVal[index] <- dcrit(M, k)
    index <- index + 1
    sens <- dsensAntoine(A, B, C, M)
    xmax <- findmax(sens, min, max, grid.length)
    if (((sens(xmax) - k)/k) < tol) {
      break
    }
    initDes <- updateDesign(initDes, xmax, 1/(i+k), joinThresh)
    iter <- 1
    maxiter <- 100
    stopw <- FALSE
    while(!stopw) {
      weightsInit <- initDes$Weight
      M <- dmatrixAntoineHom(A, B, C, initDes)
      critVal[index] <- dcrit(M, k)
      index <- index + 1
      sens <- dsensAntoine(A, B, C, M)
      initDes$Weight <- updateWeights(initDes, sens, k, deltaW)
      stopw <- any(max(abs(weightsInit - initDes$Weight) < tol)) || iter >= maxiter
      iter <- iter + 1
    }
    initDes <- deletePoints(initDes, deleteThresh)
  }
  critVal[index] <- dcrit(M, k)
  critVal <- critVal[1:(length(critVal)-sum(critVal == 0))]
  conv <- data.frame("criteria" = critVal, "step" = seq(1, length(critVal), 1))
  return(list("optdes" = initDes, "convergence" = conv))
}

# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/4, 1/3, 1/2))
# system.time(for(i in 1:100) {WFMult(8.07131, 1730.63, 233.426, des, 1, 100, 1000, 8, 0.07, 3, 5/6, 0.0001)})


# --------------------------- Ds-Optimalidad ----------------------------------

# Función de sensibilidad para D-Optimalidad de la ecuación de Antoine
DSsensAntoine <- function(a, b, c, mat, intPars) {
  a <- a*log(10)
  b <- b*log(10)
  invMat <- solve(mat)
  invMat2 <- solve(mat[-intPars, -intPars])
  if(all(intPars == c(1,2))){
    sensX <- function(x) {
      return(exp(a - b/(c + x))*(invMat[[1,1]]*exp(a - b/(c + x)) + (invMat[[3,1]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,1]]*exp(a - b/(c + x)))/(c + x)) -
               (exp(a - b/(c + x))*(invMat[[1,2]]*exp(a - b/(c + x)) + (invMat[[3,2]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,2]]*exp(a - b/(c + x)))/(c + x)))/(c + x) +
               (b*exp(a - b/(c + x))*(invMat[[1,3]]*exp(a - b/(c + x)) + (invMat[[3,3]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,3]]*exp(a - b/(c + x)))/(c + x)))/(c + x)^2
             - (invMat2[1,1]*b^2*exp(2*a - 2*b/(c + x)))/(c + x)^4)
    }
  }
  else if (all(intPars == c(1,3))) {
    sensX <- function(x) {
      return(exp(a - b/(c + x))*(invMat[[1,1]]*exp(a - b/(c + x)) + (invMat[[3,1]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,1]]*exp(a - b/(c + x)))/(c + x)) -
               (exp(a - b/(c + x))*(invMat[[1,2]]*exp(a - b/(c + x)) + (invMat[[3,2]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,2]]*exp(a - b/(c + x)))/(c + x)))/(c + x) +
               (b*exp(a - b/(c + x))*(invMat[[1,3]]*exp(a - b/(c + x)) + (invMat[[3,3]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,3]]*exp(a - b/(c + x)))/(c + x)))/(c + x)^2
             - exp(2*a-2*b/(c+x))*invMat2[1,1]/(c+x)^2)
    }
  }
  else if (all(intPars == c(2,3))) {
    sensX <- function(x) {
      return(exp(a - b/(c + x))*(invMat[[1,1]]*exp(a - b/(c + x)) + (invMat[[3,1]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,1]]*exp(a - b/(c + x)))/(c + x)) -
               (exp(a - b/(c + x))*(invMat[[1,2]]*exp(a - b/(c + x)) + (invMat[[3,2]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,2]]*exp(a - b/(c + x)))/(c + x)))/(c + x) +
               (b*exp(a - b/(c + x))*(invMat[[1,3]]*exp(a - b/(c + x)) + (invMat[[3,3]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,3]]*exp(a - b/(c + x)))/(c + x)))/(c + x)^2
             -exp(2*a-2*b/(c+x))*invMat2[1,1])
    }
  }
  else if (all(intPars == c(1))) {
    sensX <- function(x) {
      return(exp(a - b/(c + x))*(invMat[[1,1]]*exp(a - b/(c + x)) + (invMat[[3,1]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,1]]*exp(a - b/(c + x)))/(c + x)) -
               (exp(a - b/(c + x))*(invMat[[1,2]]*exp(a - b/(c + x)) + (invMat[[3,2]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,2]]*exp(a - b/(c + x)))/(c + x)))/(c + x) +
               (b*exp(a - b/(c + x))*(invMat[[1,3]]*exp(a - b/(c + x)) + (invMat[[3,3]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,3]]*exp(a - b/(c + x)))/(c + x)))/(c + x)^2
             - exp(2*a-2*b/(c+x))*(invMat2[2,2]*b^2+(c+x)*(-invMat2[1,2]*b-invMat2[2,1]*b+invMat2[1,1]*(c+x)))/(c+x)^4)
    }
  }
  else if (all(intPars == c(2))) {
    sensX <- function(x) {
      return(exp(a - b/(c + x))*(invMat[[1,1]]*exp(a - b/(c + x)) + (invMat[[3,1]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,1]]*exp(a - b/(c + x)))/(c + x)) -
               (exp(a - b/(c + x))*(invMat[[1,2]]*exp(a - b/(c + x)) + (invMat[[3,2]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,2]]*exp(a - b/(c + x)))/(c + x)))/(c + x) +
               (b*exp(a - b/(c + x))*(invMat[[1,3]]*exp(a - b/(c + x)) + (invMat[[3,3]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,3]]*exp(a - b/(c + x)))/(c + x)))/(c + x)^2
             - exp(2*a-2*b/(c+x))*(invMat2[2,2]*b^2+(c+x)^2*(invMat2[1,2]*b+invMat2[2,1]*b+invMat2[1,1]*(c+x)^2))/(c+x)^4)
    }
  }
  else if (all(intPars == c(3))) {
    sensX <- function(x) {
      return(exp(a - b/(c + x))*(invMat[[1,1]]*exp(a - b/(c + x)) + (invMat[[3,1]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,1]]*exp(a - b/(c + x)))/(c + x)) -
               (exp(a - b/(c + x))*(invMat[[1,2]]*exp(a - b/(c + x)) + (invMat[[3,2]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,2]]*exp(a - b/(c + x)))/(c + x)))/(c + x) +
               (b*exp(a - b/(c + x))*(invMat[[1,3]]*exp(a - b/(c + x)) + (invMat[[3,3]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,3]]*exp(a - b/(c + x)))/(c + x)))/(c + x)^2
             - exp(2*a-2*b/(c+x))*(invMat2[2,2]+(c+x)*(-invMat2[1,2]-invMat2[2,1]+invMat2[1,1]*(c+x)))/(c+x)^2)
    }
  }
  else {
    sensX <- NULL
  }
  return(sensX)
}

# optdes <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))

# desMat <- dmatrixAntoine(8.07131, 1730.63, 233.426, optdes)

# sens <- DSsensAntoine(8.07131, 1730.63, 233.426, desMat, c(1))

# sens(50)



# Cálculo de D-Eficiencia de la matriz 1 respecto a la matriz 2
dseff <- function(mat1, mat2, s, intPars) {
  if(length(mat1[-intPars, -intPars]) == 1){
    return((mat2[-intPars, -intPars]/det(mat2)/(mat1[-intPars, -intPars]/det(mat1)))^(1/s))
  }
  else {
    return((det(mat2[-intPars, -intPars])/det(mat2)/(det(mat1[-intPars, -intPars])/det(mat1)))^(1/s))
  }
}


# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))
# optdes <- DSWFMult(8.07131, 1730.63, 233.426, des, c(1,2), 1, 100, 1000, 8, 0.02, 3, 1/2, 0.00001)
# des2 <- data.frame(Point = c(34, 78, 90), Weight = c(1/3, 1/3, 1/3))
# dseff(dmatrixAntoine(8.07131, 1730.63, 233.426, des2),
#      dmatrixAntoine(8.07131, 1730.63, 233.426, optdes), 2, c(1, 2))



# Valor del criterio de D-Optimalidad para la matriz
dscrit <- function(mat, s, intPars) {
  if(length(mat[-intPars, -intPars]) == 1){
    return((mat[-intPars, -intPars]/det(mat))^(1/s))
  }
  else{
    return((det(mat[-intPars, -intPars])/det(mat))^(1/s))
  }
}

# dcrit(matInit, 3)


# Función para actualizar los pesos de un diseño para el algoritmo WF
updateWeightsDS <- function(design, sens, s, delta) {
  weights <- design$Weight*(map_dbl(design$Point, sens)/s)^delta
  # print(length(design$Weight))
  # print(length((map_dbl(des$Point, sens)/k)^delta))
  return(weights)
}

# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))
#
# desMat <- dmatrixAntoineHet(8.07131, 1730.63, 233.426, des)
#
# sens <- DSsensAntoineHet(8.07131, 1730.63, 233.426, desMat, c(1,2))
#
# (des$Weight <- updateWeightsDS(des, sens, 2, 1/2))



DSWFMult <- function(A, B, C, initDes, intPar, min, max, grid.length, joinThresh, deleteThresh, s, deltaW, tol) {
  critVal <- numeric(2122)
  index <- 1
  for(i in 1:21) {
    M <- dmatrixAntoineHom(A, B, C, initDes)
    critVal[index] <- dscrit(M, s, intPar)
    index <- index + 1
    sens <- DSsensAntoine(A, B, C, M, intPar)
    xmax <- findmax(sens, min, max, grid.length)
    if (((sens(xmax) - s)/s) < tol) {
      break
    }
    initDes <- updateDesign(initDes, xmax, 1/(i+3), joinThresh)
    iter <- 1
    maxiter <- 100
    stopw <- FALSE
    while(!stopw) {
      weightsInit <- initDes$Weight
      critVal[index] <- dscrit(M, s, intPar)
      index <- index + 1
      M <- dmatrixAntoineHom(A, B, C, initDes)
      sens <- DSsensAntoine(A, B, C, M, intPar)
      initDes$Weight <- updateWeightsDS(initDes, sens, length(intPar), deltaW)
      stopw <- max(abs(weightsInit - initDes$Weight)) < tol || iter >= maxiter
      iter <- iter + 1
    }
    initDes <- deletePoints(initDes, deleteThresh)
    if(i%%5 == 0)
      initDes <- updateDesignTotal(initDes, 8)
  }
  critVal[index] <- dscrit(M, s, intPar)
  critVal <- critVal[1:(length(critVal)-sum(critVal == 0))]
  conv <- data.frame("criteria" = critVal, "step" = seq(1, length(critVal), 1))
  return(list("optdes" = initDes, "convergence" = conv))
}

# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))
# system.time(for(i in 1:100) {DSWFMult(8.07131, 1730.63, 233.426, des, c(1,2), 1, 100, 1000, 8, 0.02, 3, 1/2, 0.00001)})
# optdes <- DSWFMult(8.07131, 1730.63, 233.426, des, c(1,2), 1, 100, 1000, 8, 0.02, 3, 1/2, 0.00001)
# optdes$optdes
# M <- dmatrixAntoine(8.07131, 1730.63, 233.426, optdes$optdes)
# sens <- DSsensAntoine(8.07131, 1730.63, 233.426, M, c(1,2))


# p <- ggplot(data = data.frame(x = 0), mapping = aes(x = x))
# p + stat_function(fun = sens) + stat_function(fun = function(x) 2, col = "blue") + xlim(1, 100) + labs(x = "Temperature (ºC)", y = "Pressure")

# --------------------------- A- e I-Optimalidad ----------------------------------

# Calcula la integral de la matriz de información en el rango de interés en función de una distribución dada
IntMatAntoineHom <- function(a, b, c, minReg, maxReg, vertex, option = "Unif") {
  a <- a*log(10)
  b <- b*log(10)

  m11 <- 0
  m12 <- 0
  m13 <- 0
  m21 <- 0
  m22 <- 0
  m23 <- 0
  m31 <- 0
  m32 <- 0
  m33 <- 0

  if(option == "Unif" && maxReg > minReg)
  {
    f11 <- function(x) {exp(a - b/(c + x))*exp(a - b/(c + x))/(maxReg - minReg)}
    f12 <- function(x) {exp(a - b/(c + x))*(-(exp(a - b/(c + x))/(c + x)))/(maxReg - minReg)}
    f13 <- function(x) {exp(a - b/(c + x))*(b*exp(a - b/(c + x)))/(c + x)^2/(maxReg - minReg)}
    f22 <- function(x) {(-(exp(a - b/(c + x))/(c + x)))*(-(exp(a - b/(c + x))/(c + x)))/(maxReg - minReg)}
    f23 <- function(x) {(-(exp(a - b/(c + x))/(c + x)))*(b*exp(a - b/(c + x)))/(c + x)^2/(maxReg - minReg)}
    f33 <- function(x) {(b*exp(a - b/(c + x)))/(c + x)^2*(b*exp(a - b/(c + x)))/(c + x)^2/(maxReg - minReg)}

    m11 <- integrate(f11, lower = minReg, upper = maxReg)$value
    m12 <- integrate(f12, lower = minReg, upper = maxReg)$value
    m13 <- integrate(f13, lower = minReg, upper = maxReg)$value
    m21 <- m12
    m22 <- integrate(f22, lower = minReg, upper = maxReg)$value
    m23 <- integrate(f23, lower = minReg, upper = maxReg)$value
    m31 <- m13
    m32 <- m23
    m33 <- integrate(f33, lower = minReg, upper = maxReg)$value
  }
  else if(option == "Triang")
  {
    f111 <- function(x) {exp(a - b/(c + x))*exp(a - b/(c + x))*(2*(x - minReg)/((maxReg - minReg)*(vertex - minReg)))}
    f112 <- function(x) {exp(a - b/(c + x))*exp(a - b/(c + x))*(2*(maxReg - x)/((maxReg - minReg)*(maxReg - vertex)))}
    f121 <- function(x) {exp(a - b/(c + x))*(-(exp(a - b/(c + x))/(c + x)))*(2*(x - minReg)/((maxReg - minReg)*(vertex - minReg)))}
    f122 <- function(x) {exp(a - b/(c + x))*(-(exp(a - b/(c + x))/(c + x)))*(2*(maxReg - x)/((maxReg - minReg)*(maxReg - vertex)))}
    f131 <- function(x) {exp(a - b/(c + x))*(b*exp(a - b/(c + x)))/(c + x)^2*(2*(x - minReg)/((maxReg - minReg)*(vertex - minReg)))}
    f132 <- function(x) {exp(a - b/(c + x))*(b*exp(a - b/(c + x)))/(c + x)^2*(2*(maxReg - x)/((maxReg - minReg)*(maxReg - vertex)))}
    f221 <- function(x) {(-(exp(a - b/(c + x))/(c + x)))*(-(exp(a - b/(c + x))/(c + x)))*(2*(x - minReg)/((maxReg - minReg)*(vertex - minReg)))}
    f222 <- function(x) {(-(exp(a - b/(c + x))/(c + x)))*(-(exp(a - b/(c + x))/(c + x)))*(2*(maxReg - x)/((maxReg - minReg)*(maxReg - vertex)))}
    f231 <- function(x) {(-(exp(a - b/(c + x))/(c + x)))*(b*exp(a - b/(c + x)))/(c + x)^2*(2*(x - minReg)/((maxReg - minReg)*(vertex - minReg)))}
    f232 <- function(x) {(-(exp(a - b/(c + x))/(c + x)))*(b*exp(a - b/(c + x)))/(c + x)^2*(2*(maxReg - x)/((maxReg - minReg)*(maxReg - vertex)))}
    f331 <- function(x) {(b*exp(a - b/(c + x)))/(c + x)^2*(b*exp(a - b/(c + x)))/(c + x)^2*(2*(x - minReg)/((maxReg - minReg)*(vertex - minReg)))}
    f332 <- function(x) {(b*exp(a - b/(c + x)))/(c + x)^2*(b*exp(a - b/(c + x)))/(c + x)^2*(2*(maxReg - x)/((maxReg - minReg)*(maxReg - vertex)))}

    if(vertex > minReg & maxReg > vertex)
    {
      m11 <- integrate(f111, lower = minReg, upper = vertex)$value + integrate(f112, lower = vertex, upper = maxReg)$value
      m12 <- integrate(f121, lower = minReg, upper = vertex)$value + integrate(f122, lower = vertex, upper = maxReg)$value
      m13 <- integrate(f131, lower = minReg, upper = vertex)$value + integrate(f132, lower = vertex, upper = maxReg)$value
      m21 <- m12
      m22 <- integrate(f221, lower = minReg, upper = vertex)$value + integrate(f222, lower = vertex, upper = maxReg)$value
      m23 <- integrate(f231, lower = minReg, upper = vertex)$value + integrate(f232, lower = vertex, upper = maxReg)$value
      m31 <- m13
      m32 <- m23
      m33 <- integrate(f331, lower = minReg, upper = vertex)$value + integrate(f332, lower = vertex, upper = maxReg)$value
    }
    else if(near(vertex, maxReg))
    {
      m11 <- integrate(f111, lower = minReg, upper = vertex)$value
      m12 <- integrate(f121, lower = minReg, upper = vertex)$value
      m13 <- integrate(f131, lower = minReg, upper = vertex)$value
      m21 <- m12
      m22 <- integrate(f221, lower = minReg, upper = vertex)$value
      m23 <- integrate(f231, lower = minReg, upper = vertex)$value
      m31 <- m13
      m32 <- m23
      m33 <- integrate(f331, lower = minReg, upper = vertex)$value
    }
    else
    {
      m11 <- integrate(f112, lower = vertex, upper = maxReg)$value
      m12 <- integrate(f122, lower = vertex, upper = maxReg)$value
      m13 <- integrate(f132, lower = vertex, upper = maxReg)$value
      m21 <- m12
      m22 <- integrate(f222, lower = vertex, upper = maxReg)$value
      m23 <- integrate(f232, lower = vertex, upper = maxReg)$value
      m31 <- m13
      m32 <- m23
      m33 <- integrate(f332, lower = vertex, upper = maxReg)$value
    }

  }



  return(matrix(c(m11, m12, m13, m21, m22, m23, m31, m32, m33), nrow = 3, ncol = 3, byrow = TRUE))
}


# IntMatAntoine(8.07131, 1730.63, 233.426, 80, 120, 0, option = "Unif")

# IntMatAntoine(8.07131, 1730.63, 233.426, 98, 102, 100, option = "Triang")

# library(tidyverse)
# IntMatAntoine(8.07131, 1730.63, 233.426, 98, 100, 100, option = "Triang")


# Función de sensibilidad para I-Optimalidad de la ecuación de Antoine
IsensAntoine <- function(a, b, c, mat, matB = diag(3)) {
  a <- a*log(10)
  b <- b*log(10)
  invMat <- solve(mat)
  sensMat <- solve(mat) %*% matB %*% solve(mat)
  sensX <- function(x) {
  return(exp(a - b/(c + x))*(sensMat[[1,1]]*exp(a - b/(c + x)) + (sensMat[[3,1]]*b*exp(a - b/(c + x)))/(c + x)^2 - (sensMat[[2,1]]*exp(a - b/(c + x)))/(c + x)) -
             (exp(a - b/(c + x))*(sensMat[[1,2]]*exp(a - b/(c + x)) + (sensMat[[3,2]]*b*exp(a - b/(c + x)))/(c + x)^2 - (sensMat[[2,2]]*exp(a - b/(c + x)))/(c + x)))/(c + x) +
             (b*exp(a - b/(c + x))*(sensMat[[1,3]]*exp(a - b/(c + x)) + (sensMat[[3,3]]*b*exp(a - b/(c + x)))/(c + x)^2 - (sensMat[[2,3]]*exp(a - b/(c + x)))/(c + x)))/(c + x)^2)
  }
  return(sensX)
}

# optdes <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))
#
# desMat <- dmatrixAntoine(8.07131, 1730.63, 233.426, optdes)
#
# sens <- IsensAntoine(8.07131, 1730.63, 233.426, desMat)
#
# sens(70)/10^8

# B <- IntMatAntoine(8.07131, 1730.63, 233.426, 80, 120, 0, option = "Unif")
#
# optdes <- data.frame(Point = c(100, 83.9316, 33.2345), Weight = c(0.359629, 0.377516, 0.262233))
#
# desMat <- dmatrixAntoine(8.07131, 1730.63, 233.426, optdes)
#
# sens <- IsensAntoine(8.07131, 1730.63, 233.426, desMat, B)
#
# sens(70)
#
# crit <- icrit(desMat, B)
#
# p <- ggplot(data = data.frame(x = 0), mapping = aes(x = x))
# p + stat_function(fun = sens) + stat_function(fun = function(x) crit, col = "blue") + xlim(1, 100) + labs(x = "Temperature (ºC)", y = "Pressure")




# Cálculo de D-Eficiencia de la matriz 1 respecto a la matriz 2
ieff <- function(mat1, mat2, matB = diag(3)) {
  return(tr(matB %*% solve(mat2))/tr(matB %*% solve(mat1)))
}

# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))
# optdes <- IWFMult(8.07131, 1730.63, 233.426, des, diag(3),1, 100, 1000, 5, 0.01, 1/2, 0.00001)
# des2 <- data.frame(Point = c(34, 78, 90), Weight = c(1/3, 1/3, 1/3))
# ieff(dmatrixAntoine(8.07131, 1730.63, 233.426, des2),
#       dmatrixAntoine(8.07131, 1730.63, 233.426, optdes), diag(3))



# Valor del criterio de D-Optimalidad para la matriz
icrit <- function(mat, matB = diag(3)) {
  return(tr(matB %*% solve(mat)))
}

# icrit(matInit, diag(3))


# Función para actualizar los pesos de un diseño para el algoritmo WF
updateWeightsI <- function(design, sens, crit, delta) {
  weights <- design$Weight*(map_dbl(design$Point, sens)/crit)^delta
  # print(length(design$Weight))
  # print(length((map_dbl(des$Point, sens)/k)^delta))
  return(weights)
}

# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/4, 1/3, 1/2))

# desMat <- dmatrixAntoine(8.07131, 1730.63, 233.426, des)

# sens <- dsensAntoine(8.07131, 1730.63, 233.426, desMat)

# (des$Weight <- updateWeights(des, sens, 3, 1/2))



IWFMult <- function(A, B, C, initDes, matB = diag(3), min, max, grid.length, joinThresh, deleteThresh, deltaW, tol) {
  critVal <- numeric(2122)
  index <- 1
  for(i in 1:21) {
    M <- dmatrixAntoineHom(A, B, C, initDes)
    critVal[index] <- icrit(M, matB)
    index <- index + 1
    sens <- IsensAntoine(A, B, C, M, matB)
    xmax <- findmax(sens, min, max, grid.length)
    if (((sens(xmax) - critVal[index-1])/critVal[index-1]) < tol) {
      break
    }
    initDes <- updateDesign(initDes, xmax, 1/(i+3), joinThresh)
    iter <- 1
    maxiter <- 100
    stopw <- FALSE
    while(!stopw) {
      weightsInit <- initDes$Weight
      M <- dmatrixAntoineHom(A, B, C, initDes)
      crit <- icrit(M, matB)
      critVal[index] <- crit
      index <- index + 1
      sens <- IsensAntoine(A, B, C, M, matB)
      initDes$Weight <- updateWeightsI(initDes, sens, crit, deltaW)
      stopw <- max(abs(weightsInit - initDes$Weight)) < tol || iter >= maxiter
      iter <- iter + 1
    }
    initDes <- deletePoints(initDes, deleteThresh)
    if(i%%5 == 0)
      initDes <- updateDesignTotal(initDes, 8)
  }
  M <- dmatrixAntoineHom(A, B, C, initDes)
  critVal[index] <- icrit(M, matB)
  critVal <- critVal[1:(length(critVal)-sum(critVal == 0))]
  conv <- data.frame("criteria" = critVal, "step" = seq(1, length(critVal), 1))
  return(list("optdes" = initDes, "convergence" = conv))
}

# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))
# matUnif <- IntMatAntoineHom(8.07131, 1730.63, 233.426, 70, 100, 100, option = "Unif")
# matTriang <- IntMatAntoineHom(8.07131, 1730.63, 233.426, 70, 100, 100, option = "Triang")
# optdes <- IWFMult(8.07131, 1730.63, 233.426, des, matUnif, 1, 100, 1000, 9, 0.01, 1/2, 0.00001)
# optdes <- IWFMult(8.07131, 1730.63, 233.426, des, matTriang, 1, 100, 1000, 9, 0.01, 1/2, 0.00001)

# system.time(for(i in 1:100) {IWFMult(8.07131, 1730.63, 233.426, des, diag(3), 1, 100, 1000, 9, 0.01, 1/2, 0.00001)})
# optdes$optdes
# optdes$convergence
# M <- dmatrixAntoine(8.07131, 1730.63, 233.426, optdes)
# sens <- IsensAntoine(8.07131, 1730.63, 233.426, M)
# crit <- icrit(M)

# p <- ggplot(data = data.frame(x = 0), mapping = aes(x = x))
# p + stat_function(fun = sens) + stat_function(fun = function(x) crit, col = "blue") + xlim(1, 100) + labs(x = "Temperature (ºC)", y = "Pressure")


# ______________________________________________________

# López-Ríos

# Calculate boundaries for given delta
delta_bound <- function(delta, k, sens_min){
  min <- -(-1 + delta)*((-1 + delta - sens_min*delta)/(-1 + delta))^(1/k)
  max <- -(-1 + delta)*((-1 + delta - k*delta)/(-1 + delta))^(1/k)
  return(c(min, max))
}

# delta_bound(1/4, 3, -2.50965)


# NO SE PUEDE "FÁCIL"
# Calculate boundary for given efficiency
# efficiency_bound <- function(deff, k, sens_min){
#   min <- -(-1 + delta)*((-1 + delta - sens_min*delta)/(-1 + delta))^(1/k)
#   max <- -(-1 + delta)*((-1 + delta - k*delta)/(-1 + delta))^(1/k)
#   return(c(min, max))
# }



# Calculate sens value for given delta and efficiency
sens_val_to_add <- function(deff, delta, k){
  return((1 - delta)/delta*((deff/(1 - delta))^k - 1))
}

# sens_val_to_add(0.9, 1/4, 3)


# Elimina puntos duplicados
update_sequence <- function(points, tol){
  i <- 1
  imax <- (length(points)-1)
  while(i < imax) {
    absdiff <- c(rep(T, i), abs(points[-seq(1, i)] - points[i]) > tol)
    points <- points[absdiff]
    i <- i+1
    imax <- (length(points)-1)
  }
  return(points)
}




# Calculate crosspoints
crosspoints <- function(deff, delta, k, sens, gridlength, tol, xmin, xmax){
  val <- sens_val_to_add(deff, delta, k)

  sensfix <- function(x){
    return(sens(x) - val)
  }

  sols <- seq(xmin, xmax, length.out = gridlength) %>%
    map(nleqslv, fn = sensfix) %>%
    map(function(x) x$x) %>%
    unlist

  # Eliminar duplicados
  sols_upd <- update_sequence(sols, tol)

  # Quedarnos con puntos dentro del espacio de diseño
  sols_upd <- sols_upd[sols_upd >= 1 & sols_upd <= 100]

  # Eliminar los que no son soluciones
  sols_upd <- sols_upd[abs(map_dbl(sols_upd, sens) - val) < tol]


  return(sols_upd)
}

# optdes <- doptAntoine(A = 8.07131, B = 1730.63, C = 233.426, 1, 100)

# desMat <- dmatrixAntoine(8.07131, 1730.63, 233.426, optdes)

# sens <- dsensAntoine(8.07131, 1730.63, 233.426, desMat)

# sols_new <- crosspoints(0.9, 1/4, 3, sens, 1000, 10^(-3), 1, 100)




# Update design given crosspoints and delta
add_points <- function(points, delta, design){
  new_points <- data.frame("Point" = points, "Weight" = rep(delta/length(points), times = length(points)))
  design["Weight"] <- design["Weight"]*(1-delta)
  return(rbind(design, new_points))
}

# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))
# add_points(c(20, 70, 90), 1/4, des)


#-----------------------------


# Uniform design
# uniform_design <- function(n, xmin, xmax){
#   unif_des <- data.frame("Point" = seq(xmin,xmax,length.out = n), "Weight" = rep(1/n, times = n))
#   return(unif_des)
# }

# Arithmetic design
# arithmetic_design <- function(Variance, n, xmin, xmax, opt_mat, Criterion, a = 0, b = 0, c = 0, k = 3, s = 2, intPars = 2, matB = diag(3)){
#   arithdes_opt <- function(r){
#     arith_des <- data.frame("Point" = seq(r,xmax,length.out = n), "Weight" = rep(1/n, times = n))
#     arith_mat <- dmatrixAntoine(Variance, a, b, c, arith_des)
#     return(eff(Criterion, opt_mat, arith_mat, k, s, intPars, matB))
#   }
#
#   r_val <- round(nloptr::direct(fn = arithdes_opt,lower = xmin, upper = xmax, control=list(xtol_rel=1e-24, maxeval=1000))$par, 3)
#
#   return(data.frame("Point" = seq(r_val,xmax,length.out = n), "Weight" = rep(1/n, times = n)))
# }

# opt_des <- doptCuad(1, 100)
# dopt_mat <- dmatrixCuad(opt_des)
#
# arithmetic_design(5, 1, 100, dopt_mat, "Cuadratic")


# Geometric design
# geometric_design <- function(Variance, n, xmin, xmax, opt_mat, Criterion, a = 0, b = 0, c = 0, k = 3, s = 2, intPars = 2, matB = diag(3)){
#
#   geodes_opt <- function(r){
#     geom_des <- data.frame("Point" = (xmax-xmin)*r^(0:(n-1))+xmin, "Weight" = rep(1/n, times = n))
#     geom_mat <- dmatrixAntoine(Variance, a, b, c, geom_des)
#     return(eff(Criterion, opt_mat, geom_mat, k, s, intPars, matB))
#   }
#
#   r_val <- round(nloptr::direct(fn = geodes_opt,lower = 0, upper = 1, control=list(xtol_rel=1e-8, maxeval=1000))$par, 3)
#
#   return(data.frame("Point" = (xmax-xmin)*r_val^(0:(n-1))+xmin, "Weight" = rep(1/n, times = n)))
# }





#### HETEROCEDASTICO ----------------------------------------------------------

# Calcular matrix de un diseño para la ecuación de Antoine
dmatrixAntoineHet <- function(a, b, c, design) {
  b <- b*log(10)

  m11 <- 0
  m12 <- 0
  m13 <- 0
  m21 <- 0
  m22 <- 0
  m23 <- 0
  m31 <- 0
  m32 <- 0
  m33 <- 0

  for(i in seq_along(design$Weight)){
    m11 <- m11 + design$Weight[[i]]
    m12 <- m12 + -1/(c + design$Point[[i]])*design$Weight[[i]]
    m13 <- m13 + b/(c + design$Point[[i]])^2*design$Weight[[i]]
    m21 <- m12
    m22 <- m22 + 1/(c + design$Point[[i]])^2*design$Weight[[i]]
    m23 <- m23 + -b/(c + design$Point[[i]])^3*design$Weight[[i]]
    m31 <- m13
    m32 <- m23
    m33 <- m33 + b^2/(c + design$Point[[i]])^4*design$Weight[[i]]
  }

  return(matrix(c(m11, m12, m13, m21, m22, m23, m31, m32, m33), nrow = 3, ncol = 3, byrow = TRUE))
}



# design <- data.frame(points = c(1, 50.5, 100), weights = c(1/3, 1/3, 1/3))

# (design2 <- doptAntoine(8.07131, 730.63, 233.426, 1, 100))
# matInit2 <- dmatrixAntoine(a = 8.07131, b = 730.63, c = 233.426, design = design2)

# solve(matInit2)

# matInit <- dmatrixAntoine(a = 8.07131, b = 1730.63, c = 233.426, design = design)




# --------------------------- D-Optimalidad ----------------------------------

# Función de sensibilidad para D-Optimalidad de la ecuación de Antoine
dsensAntoineHet <- function(a, b, c, mat) {
  a <- a*log(10)
  b <- b*log(10)
  invMat <- solve(mat)
  sensX <- function(x) {
    return(invMat[[1,1]] + invMat[[3,1]]*b/(c + x)^2 - (invMat[[2,1]]/(c + x)) -
             (invMat[[1,2]] + invMat[[3,2]]*b/(c + x)^2 - invMat[[2,2]]/(c + x))/(c + x) +
             b*(invMat[[1,3]] + invMat[[3,3]]*b/(c + x)^2 - invMat[[2,3]]/(c + x))/(c + x)^2)
  }
  return(sensX)
}


# optdes <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))
#
# desMat <- dmatrixAntoineHet(8.07131, 1730.63, 233.426, optdes)
#
# sens <- dsensAntoineHet(8.07131, 1730.63, 233.426, desMat)
#
# sens(1)

# Cálculo Analítico del diseño D-Óptimo para la ecuación de Antoine
doptAntoineHet <- function(A, B, C, min, max){
  A <- log(10)*A
  B <- log(10)*B

  if(round(min - max, 3) == 0){
    opdes <- data.frame(Point = c(min), Weight = c(1))
    return(opdes)
  }
  T2 <- (C*max+C*min+2*min*max)/(2*C+max+min)
  opdes <- data.frame(Point = c(min, T2, max), Weight = c(1/3, 1/3, 1/3))

  return(opdes)
}
# A <- 18.58487808693377
# B <- 1682.3377464942398
# C <- 233.426
#
# min <- 1
# max <- 100


# class(doptAntoine(A = 8.07131, B = 1730.63, C = 233.426, 99, 374))
# doptAntoineHet(A = 8.07131, B = 1730.63, C = 233.426, 1, 100)

# doptAntoine(A = 8.14019, B = 1810.94, C = 244.485, 1, 40)




# Algoritmo WF con multiplicativo para los pesos para D-Opt
WFMultHet <- function(A, B, C, initDes, min, max, grid.length, joinThresh, deleteThresh, k, deltaW, tol) {
  critVal <- numeric(2122)
  index <- 1
  for(i in 1:21) {
    M <- dmatrixAntoineHet(A, B, C, initDes)
    critVal[index] <- dcrit(M, k)
    index <- index + 1
    sens <- dsensAntoineHet(A, B, C, M)
    xmax <- findmax(sens, min, max, grid.length)
    if (((sens(xmax) - k)/k) < tol) {
      break
    }
    initDes <- updateDesign(initDes, xmax, 1/(i+k), joinThresh)
    iter <- 1
    maxiter <- 100
    stopw <- FALSE
    while(!stopw) {
      weightsInit <- initDes$Weight
      M <- dmatrixAntoineHet(A, B, C, initDes)
      critVal[index] <- dcrit(M, k)
      index <- index + 1
      sens <- dsensAntoineHet(A, B, C, M)
      initDes$Weight <- updateWeights(initDes, sens, k, deltaW)
      stopw <- any(max(abs(weightsInit - initDes$Weight) < tol)) || iter >= maxiter
      iter <- iter + 1
    }
    initDes <- deletePoints(initDes, deleteThresh)
  }
  critVal[index] <- dcrit(M, k)
  critVal <- critVal[1:(length(critVal)-sum(critVal == 0))]
  conv <- data.frame("criteria" = critVal, "step" = seq(1, length(critVal), 1))
  return(list("optdes" = initDes, "convergence" = conv))
}

# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/4, 1/3, 1/2))
# system.time(for(i in 1:100) {WFMult(8.07131, 1730.63, 233.426, des, 1, 100, 1000, 8, 0.07, 3, 5/6, 0.0001)})


# --------------------------- Ds-Optimalidad ----------------------------------

# Función de sensibilidad para Ds-Optimalidad de la ecuación de Antoine
DSsensAntoineHet <- function(a, b, c, mat, intPars) {
  a <- a*log(10)
  b <- b*log(10)
  invMat <- solve(mat)
  invMat2 <- solve(mat[-intPars, -intPars])
  if(all(intPars == c(1,2))){
    sensX <- function(x) {
      return(invMat[[1,1]] + invMat[[3,1]]*b/(c + x)^2 - (invMat[[2,1]]/(c + x)) -
               (invMat[[1,2]] + invMat[[3,2]]*b/(c + x)^2 - invMat[[2,2]]/(c + x))/(c + x) +
               b*(invMat[[1,3]] + invMat[[3,3]]*b/(c + x)^2 - invMat[[2,3]]/(c + x))/(c + x)^2 - invMat2[[1,1]]*b^2/(c+x)^4)
    }
  }
  else if (all(intPars == c(1,3))) {
    sensX <- function(x) {
      return(invMat[[1,1]] + invMat[[3,1]]*b/(c + x)^2 - (invMat[[2,1]]/(c + x)) -
               (invMat[[1,2]] + invMat[[3,2]]*b/(c + x)^2 - invMat[[2,2]]/(c + x))/(c + x) +
               b*(invMat[[1,3]] + invMat[[3,3]]*b/(c + x)^2 - invMat[[2,3]]/(c + x))/(c + x)^2 - invMat2[[1,1]]/(c+x)^2)
    }
  }
  else if (all(intPars == c(2,3))) {
    sensX <- function(x) {
      return(invMat[[1,1]] + invMat[[3,1]]*b/(c + x)^2 - (invMat[[2,1]]/(c + x)) -
               (invMat[[1,2]] + invMat[[3,2]]*b/(c + x)^2 - invMat[[2,2]]/(c + x))/(c + x) +
               b*(invMat[[1,3]] + invMat[[3,3]]*b/(c + x)^2 - invMat[[2,3]]/(c + x))/(c + x)^2 - invMat2[[1,1]])
    }
  }
  else if (all(intPars == c(1))) {
    sensX <- function(x) {
      return(invMat[[1,1]] + invMat[[3,1]]*b/(c + x)^2 - (invMat[[2,1]]/(c + x)) -
               (invMat[[1,2]] + invMat[[3,2]]*b/(c + x)^2 - invMat[[2,2]]/(c + x))/(c + x) +
               b*(invMat[[1,3]] + invMat[[3,3]]*b/(c + x)^2 - invMat[[2,3]]/(c + x))/(c + x)^2 + (b*invMat2[[1,2]]/(c+x)^2 - invMat2[[1,1]]/(c+x))/(c+x) - b*(b*invMat2[[2,2]]/(c+x)^2 - invMat2[[1,2]]/(c+x))/(c+x)^2)
    }
  }
  else if (all(intPars == c(2))) {
    sensX <- function(x) {
      return(invMat[[1,1]] + invMat[[3,1]]*b/(c + x)^2 - (invMat[[2,1]]/(c + x)) -
               (invMat[[1,2]] + invMat[[3,2]]*b/(c + x)^2 - invMat[[2,2]]/(c + x))/(c + x) +
               b*(invMat[[1,3]] + invMat[[3,3]]*b/(c + x)^2 - invMat[[2,3]]/(c + x))/(c + x)^2 - invMat2[[1,1]] - b*invMat2[[1,2]]/(c+x)^2 - b*(invMat2[[1,2]]+b*invMat2[[2,2]]/(c+x)^2)/(c+x)^2)
    }
  }
  else if (all(intPars == c(3))) {
    sensX <- function(x) {
      return(invMat[[1,1]] + invMat[[3,1]]*b/(c + x)^2 - (invMat[[2,1]]/(c + x)) -
               (invMat[[1,2]] + invMat[[3,2]]*b/(c + x)^2 - invMat[[2,2]]/(c + x))/(c + x) +
               b*(invMat[[1,3]] + invMat[[3,3]]*b/(c + x)^2 - invMat[[2,3]]/(c + x))/(c + x)^2 - invMat2[[1,1]] + invMat2[[1,2]]/(c+x) + (invMat2[[1,2]] - invMat2[[2,2]]/(c+x))/(c+x))
    }
  }
  else {
    sensX <- NULL
  }
  return(sensX)
}

# optdes <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))
#
# desMat <- dmatrixAntoineHet(8.07131, 1730.63, 233.426, optdes)
#
# sens <- DSsensAntoineHet(8.07131, 1730.63, 233.426, desMat, c(2))
#
# sens(50)



DSWFMultHet <- function(A, B, C, initDes, intPar, min, max, grid.length, joinThresh, deleteThresh, s, deltaW, tol) {
  critVal <- numeric(2122)
  index <- 1
  for(i in 1:21) {
    M <- dmatrixAntoineHet(A, B, C, initDes)
    critVal[index] <- dscrit(M, s, intPar)
    index <- index + 1
    sens <- DSsensAntoineHet(A, B, C, M, intPar)
    xmax <- findmax(sens, min, max, grid.length)
    if (((sens(xmax) - s)/s) < tol) {
      break
    }
    initDes <- updateDesign(initDes, xmax, 1/(i+3), joinThresh)
    iter <- 1
    maxiter <- 100
    stopw <- FALSE
    while(!stopw) {
      weightsInit <- initDes$Weight
      critVal[index] <- dscrit(M, s, intPar)
      index <- index + 1
      M <- dmatrixAntoineHet(A, B, C, initDes)
      sens <- DSsensAntoineHet(A, B, C, M, intPar)
      initDes$Weight <- updateWeightsDS(initDes, sens, length(intPar), deltaW)
      stopw <- max(abs(weightsInit - initDes$Weight)) < tol || iter >= maxiter
      iter <- iter + 1
    }
    initDes <- deletePoints(initDes, deleteThresh)
    if(i%%5 == 0)
      initDes <- updateDesignTotal(initDes, 8)
  }
  critVal[index] <- dscrit(M, s, intPar)
  critVal <- critVal[1:(length(critVal)-sum(critVal == 0))]
  conv <- data.frame("criteria" = critVal, "step" = seq(1, length(critVal), 1))
  return(list("optdes" = initDes, "convergence" = conv))
}

# initDes <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))
# system.time(for(i in 1:100) {DSWFMultHet(8.07131, 1730.63, 233.426, des, c(1,2), 1, 100, 1000, 8, 0.02, 3, 1/2, 0.00001)})
# optdes <- DSWFMultHet(8.07131, 1730.63, 233.426, des, c(1), 1, 100, 1000, 8, 0.05, 1, 5/6, 0.00001)

# arrange(optdes$optdes, Point)
# M <- dmatrixAntoineHet(8.07131, 1730.63, 233.426, optdes$optdes)
# sens <- DSsensAntoineHet(8.07131, 1730.63, 233.426, M, c(1))


# p <- ggplot(data = data.frame(x = 0), mapping = aes(x = x))
# p + stat_function(fun = sens) + stat_function(fun = function(x) 2, col = "blue") + xlim(1, 100) + labs(x = "Temperature (ºC)", y = "Pressure")

# --------------------------- A- e I-Optimalidad ----------------------------------

# Calcula la integral de la matriz de información en el rango de interés en función de una distribución dada
IntMatAntoineHet <- function(a, b, c, minReg, maxReg, vertex, option = "Unif") {
  a <- a*log(10)
  b <- b*log(10)

  m11 <- 0
  m12 <- 0
  m13 <- 0
  m21 <- 0
  m22 <- 0
  m23 <- 0
  m31 <- 0
  m32 <- 0
  m33 <- 0
  if(option == "Unif" && maxReg > minReg)
  {
    # f11 <- function(x) {1/(maxReg - minReg)}
    f12 <- function(x) {(-1/(c + x))/(maxReg - minReg)}
    f13 <- function(x) {b/(c + x)^2/(maxReg - minReg)}
    f22 <- function(x) {1/(c + x)^2/(maxReg - minReg)}
    f23 <- function(x) {-b/(c + x)^3/(maxReg - minReg)}
    f33 <- function(x) {b^2/(c + x)^4/(maxReg - minReg)}

    # m11 <- integrate(f11, lower = minReg, upper = maxReg)$value
    m11 <- 1
    m12 <- integrate(f12, lower = minReg, upper = maxReg)$value
    m13 <- integrate(f13, lower = minReg, upper = maxReg)$value
    m21 <- m12
    m22 <- integrate(f22, lower = minReg, upper = maxReg)$value
    m23 <- integrate(f23, lower = minReg, upper = maxReg)$value
    m31 <- m13
    m32 <- m23
    m33 <- integrate(f33, lower = minReg, upper = maxReg)$value
  }
  else if(option == "Triang")
  {
    # f111 <- function(x) {1*(2*(x - minReg)/((maxReg - minReg)*(vertex - minReg)))}
    # f112 <- function(x) {1*(2*(maxReg - x)/((maxReg - minReg)*(maxReg - vertex)))}
    f121 <- function(x) {(-1/(c + x))*(2*(x - minReg)/((maxReg - minReg)*(vertex - minReg)))}
    f122 <- function(x) {(-1/(c + x))*(2*(maxReg - x)/((maxReg - minReg)*(maxReg - vertex)))}
    f131 <- function(x) {b/(c + x)^2*(2*(x - minReg)/((maxReg - minReg)*(vertex - minReg)))}
    f132 <- function(x) {b/(c + x)^2*(2*(maxReg - x)/((maxReg - minReg)*(maxReg - vertex)))}
    f221 <- function(x) {1/(c + x)^2*(2*(x - minReg)/((maxReg - minReg)*(vertex - minReg)))}
    f222 <- function(x) {1/(c + x)^2*(2*(maxReg - x)/((maxReg - minReg)*(maxReg - vertex)))}
    f231 <- function(x) {-b/(c + x)^3*(2*(x - minReg)/((maxReg - minReg)*(vertex - minReg)))}
    f232 <- function(x) {-b/(c + x)^3*(2*(maxReg - x)/((maxReg - minReg)*(maxReg - vertex)))}
    f331 <- function(x) {b^2/(c + x)^4*(2*(x - minReg)/((maxReg - minReg)*(vertex - minReg)))}
    f332 <- function(x) {b^2/(c + x)^4*(2*(maxReg - x)/((maxReg - minReg)*(maxReg - vertex)))}

    if(vertex > minReg & maxReg > vertex)
    {
      # m11 <- integrate(f111, lower = minReg, upper = vertex)$value + integrate(f112, lower = vertex, upper = maxReg)$value
      m11 <- 1
      m12 <- integrate(f121, lower = minReg, upper = vertex)$value + integrate(f122, lower = vertex, upper = maxReg)$value
      m13 <- integrate(f131, lower = minReg, upper = vertex)$value + integrate(f132, lower = vertex, upper = maxReg)$value
      m21 <- m12
      m22 <- integrate(f221, lower = minReg, upper = vertex)$value + integrate(f222, lower = vertex, upper = maxReg)$value
      m23 <- integrate(f231, lower = minReg, upper = vertex)$value + integrate(f232, lower = vertex, upper = maxReg)$value
      m31 <- m13
      m32 <- m23
      m33 <- integrate(f331, lower = minReg, upper = vertex)$value + integrate(f332, lower = vertex, upper = maxReg)$value
    }
    else if(near(vertex, maxReg))
    {
      # m11 <- integrate(f111, lower = minReg, upper = vertex)$value
      m11 <- 1
      m12 <- integrate(f121, lower = minReg, upper = vertex)$value
      m13 <- integrate(f131, lower = minReg, upper = vertex)$value
      m21 <- m12
      m22 <- integrate(f221, lower = minReg, upper = vertex)$value
      m23 <- integrate(f231, lower = minReg, upper = vertex)$value
      m31 <- m13
      m32 <- m23
      m33 <- integrate(f331, lower = minReg, upper = vertex)$value
    }
    else
    {
      m11 <- 1
      # m11 <- integrate(f112, lower = vertex, upper = maxReg)$value
      m12 <- integrate(f122, lower = vertex, upper = maxReg)$value
      m13 <- integrate(f132, lower = vertex, upper = maxReg)$value
      m21 <- m12
      m22 <- integrate(f222, lower = vertex, upper = maxReg)$value
      m23 <- integrate(f232, lower = vertex, upper = maxReg)$value
      m31 <- m13
      m32 <- m23
      m33 <- integrate(f332, lower = vertex, upper = maxReg)$value
    }

  }



  return(matrix(c(m11, m12, m13, m21, m22, m23, m31, m32, m33), nrow = 3, ncol = 3, byrow = TRUE))
}


# IntMatAntoineHet(8.07131, 1730.63, 233.426, 80, 100, 0, option = "Unif")

# IntMatAntoine(8.07131, 1730.63, 233.426, 98, 102, 100, option = "Triang")

# library(tidyverse)
# IntMatAntoine(8.07131, 1730.63, 233.426, 98, 100, 100, option = "Triang")


# Función de sensibilidad para I-Optimalidad de la ecuación de Antoine
IsensAntoineHet <- function(a, b, c, mat, matB = diag(3)) {
  a <- a*log(10)
  b <- b*log(10)
  invMat <- solve(mat)
  sensMat <- solve(mat) %*% matB %*% solve(mat)
  sensX <- function(x) {
    return(sensMat[[1,1]] + sensMat[[3,1]]*b/(c + x)^2 - (sensMat[[2,1]]/(c + x)) -
             (sensMat[[1,2]] + sensMat[[3,2]]*b/(c + x)^2 - sensMat[[2,2]]/(c + x))/(c + x) +
             b*(sensMat[[1,3]] + sensMat[[3,3]]*b/(c + x)^2 - sensMat[[2,3]]/(c + x))/(c + x)^2)
  }
  return(sensX)
}

# optdes <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))
#
# desMat <- dmatrixAntoineHet(8.07131, 1730.63, 233.426, optdes)
#
# sens <- IsensAntoineHet(8.07131, 1730.63, 233.426, desMat)
#
# sens(1)/10^8

# B <- IntMatAntoine(8.07131, 1730.63, 233.426, 80, 120, 0, option = "Unif")
#
# optdes <- data.frame(Point = c(100, 83.9316, 33.2345), Weight = c(0.359629, 0.377516, 0.262233))
#
# desMat <- dmatrixAntoine(8.07131, 1730.63, 233.426, optdes)
#
# sens <- IsensAntoine(8.07131, 1730.63, 233.426, desMat, B)
#
# sens(70)
#
# crit <- icrit(desMat, B)
#
# p <- ggplot(data = data.frame(x = 0), mapping = aes(x = x))
# p + stat_function(fun = sens) + stat_function(fun = function(x) crit, col = "blue") + xlim(1, 100) + labs(x = "Temperature (ºC)", y = "Pressure")



IWFMultHet <- function(A, B, C, initDes, matB = diag(3), min, max, grid.length, joinThresh, deleteThresh, deltaW, tol) {
  critVal <- numeric(2122)
  index <- 1
  for(i in 1:21) {
    M <- dmatrixAntoineHet(A, B, C, initDes)
    critVal[index] <- icrit(M, matB)
    index <- index + 1
    sens <- IsensAntoineHet(A, B, C, M, matB)
    xmax <- findmax(sens, min, max, grid.length)
    if (((sens(xmax) - critVal[index-1])/critVal[index-1]) < tol) {
      break
    }
    initDes <- updateDesign(initDes, xmax, 1/(i+3), joinThresh)
    iter <- 1
    maxiter <- 100
    stopw <- FALSE
    while(!stopw) {
      weightsInit <- initDes$Weight
      M <- dmatrixAntoineHet(A, B, C, initDes)
      crit <- icrit(M, matB)
      critVal[index] <- crit
      index <- index + 1
      sens <- IsensAntoineHet(A, B, C, M, matB)
      initDes$Weight <- updateWeightsI(initDes, sens, crit, deltaW)
      initDes$Weight <- initDes$Weight/sum(initDes$Weight)
      stopw <- max(abs(weightsInit - initDes$Weight)) < tol || iter >= maxiter
      iter <- iter + 1
    }
    initDes <- deletePoints(initDes, deleteThresh)
    if(i%%5 == 0)
      initDes <- updateDesignTotal(initDes, 8)
  }
  M <- dmatrixAntoineHet(A, B, C, initDes)
  critVal[index] <- icrit(M, matB)
  critVal <- critVal[1:(length(critVal)-sum(critVal == 0))]
  conv <- data.frame("criteria" = critVal, "step" = seq(1, length(critVal), 1))
  return(list("optdes" = initDes, "convergence" = conv))
}

# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))
# matUnif <- IntMatAntoineHet(8.07131, 1730.63, 233.426, 80, 100, 100, option = "Unif")
# matTriang <- IntMatAntoineHet(8.07131, 1730.63, 233.426, 80, 100, 100, option = "Triang")
# optdes <- IWFMultHet(8.07131, 1730.63, 233.426, des, matUnif, 1, 100, 1000, 9, 0.01, 1/2, 0.00001)
# optdes1 <- IWFMultHet(8.07131, 1730.63, 233.426, des, matTriang, 1, 100, 1000, 9, 0.01, 1/2, 0.00001)

# optdes <- IWFMultHet(8.07131, 1730.63, 233.426, des, diag(3), 1, 100, 1000, 9, 0.01, 1/2, 0.00001)
# system.time(for(i in 1:100) {IWFMultHet(8.07131, 1730.63, 233.426, des, diag(3), 1, 100, 1000, 9, 0.01, 1/2, 0.00001)})
# optdes$optdes
# optdes1$optdes
# M <- dmatrixAntoineHet(8.07131, 1730.63, 233.426, optdes1$optdes)
# sens <- IsensAntoineHet(8.07131, 1730.63, 233.426, M, matTriang)
# crit <- icrit(M, matTriang)


# p <- ggplot(data = data.frame(x = 0), mapping = aes(x = x))
# p + stat_function(fun = sens) + stat_function(fun = function(x) crit, col = "blue") + xlim(1, 100) + labs(x = "Temperature (ºC)", y = "Pressure")

# optdes2 <- des <- data.frame(Point = c(1, 65.8262, 100), Weight = c(1/3, 1/3, 1/3))



####  Funciones generales
eff <- function(Criterion, mat1, mat2, k = 3, s = 2, intPars = 2, matB = diag(3)){
  if(identical(Criterion, "D-Optimality")){
    return(deff(mat1, mat2, k))
  }
  else if(identical(Criterion, "Ds-Optimality")){
    return(dseff(mat1, mat2, s, intPars))
  }
  else if(identical(Criterion, "A-Optimality")){
    return(ieff(mat1, mat2, matB))
  }
  else if(identical(Criterion, "I-Optimality")){
    return(ieff(mat1, mat2, matB))
  }
}

sens <- function(Variance, Criterion, a, b, c, mat, intPars = 0, matB = diag(3)){
  if(identical(Variance, "Homoscedastic"))
  {
        if(identical(Criterion, "D-Optimality")){
          return(dsensAntoine(a, b, c, mat))
        }
        else if(identical(Criterion, "Ds-Optimality")){
          return(DSsensAntoine(a, b, c, mat, intPars))
        }
        else if(identical(Criterion, "A-Optimality")){
          return(IsensAntoine(a, b, c, mat, matB))
        }
        else if(identical(Criterion, "I-Optimality")){
          return(IsensAntoine(a, b, c, mat, matB))
        }
  } else if(identical(Variance, "Heteroscedastic"))
  {
    if(identical(Criterion, "D-Optimality")){
      return(dsensAntoineHet(a, b, c, mat))
    }
    else if(identical(Criterion, "Ds-Optimality")){
      return(DSsensAntoineHet(a, b, c, mat, intPars))
    }
    else if(identical(Criterion, "A-Optimality")){
      return(IsensAntoineHet(a, b, c, mat, matB))
    }
    else if(identical(Criterion, "I-Optimality")){
      return(IsensAntoineHet(a, b, c, mat, matB))
    }
  }
}

crit <- function(Criterion, mat, k = 3, s = 2, intPars = 2, matB = diag(3)){
  if(identical(Criterion, "D-Optimality")){
    return(dcrit(mat, k))
  }
  else if(identical(Criterion, "Ds-Optimality")){
    return(dscrit(mat, s, intPars))
  }
  else if(identical(Criterion, "A-Optimality")){
    return(icrit(mat, matB))
  }
  else if(identical(Criterion, "I-Optimality")){
    return(icrit(mat, matB))
  }
}


WF <- function(Variance, Criterion, A, B, C, initDes, intPar = 0, matB = diag(3), min, max, grid.length, joinThresh, deleteThresh, k = 3, s = 2, deltaW, tol){
  if(identical(Variance, "Homoscedastic")){
    if(identical(Criterion, "D-Optimality")){
      return(WFMult(A, B, C, initDes, min, max, grid.length, joinThresh, deleteThresh, k, deltaW, tol))
    }
    else if(identical(Criterion, "Ds-Optimality")){
      return(DSWFMult(A, B, C, initDes, intPar, min, max, grid.length, joinThresh, deleteThresh, s, deltaW, tol))
    }
    else if(identical(Criterion, "A-Optimality")){
      return(IWFMult(A, B, C, initDes, matB, min, max, grid.length, joinThresh, deleteThresh, deltaW, tol))
    }
    else if(identical(Criterion, "I-Optimality")){
      return(IWFMult(A, B, C, initDes, matB, min, max, grid.length, joinThresh, deleteThresh, deltaW, tol))
    }
  } else if(identical(Variance, "Heteroscedastic")) {
    if(identical(Criterion, "D-Optimality")){
      return(WFMultHet(A, B, C, initDes, min, max, grid.length, joinThresh, deleteThresh, k, deltaW, tol))
    }
    else if(identical(Criterion, "Ds-Optimality")){
      return(DSWFMultHet(A, B, C, initDes, intPar, min, max, grid.length, joinThresh, deleteThresh, s, deltaW, tol))
    }
    else if(identical(Criterion, "A-Optimality")){
      return(IWFMultHet(A, B, C, initDes, matB, min, max, grid.length, joinThresh, deleteThresh, deltaW, tol))
    }
    else if(identical(Criterion, "I-Optimality")){
      return(IWFMultHet(A, B, C, initDes, matB, min, max, grid.length, joinThresh, deleteThresh, deltaW, tol))
    }
  }
}

dmatrixAntoine <- function(Variance, a, b, c, design) {
  if(identical(Variance, "Homoscedastic")){
    return(dmatrixAntoineHom(a, b, c, design))
  } else if(identical(Variance, "Heteroscedastic")){
    return(dmatrixAntoineHet(a, b, c, design))
  }
}

doptAntoine <- function(Variance, A, B, C, min, max){
  if(identical(Variance, "Homoscedastic")){
    return(doptAntoineHom(A, B, C, min, max))
  } else if(identical(Variance, "Heteroscedastic")){
    return(doptAntoineHet(A, B, C, min, max))
  }
}
# doptAntoine("Heteroscedastic", 8.07131, 1730.63, 233.426, 1, 100)

IntMatAntoine <- function(Variance, a, b, c, minReg, maxReg, vertex, option = "Unif") {
  if(identical(Variance, "Homoscedastic")){
    return(IntMatAntoineHom(a, b, c, minReg, maxReg, vertex, option))
  } else if(identical(Variance, "Heteroscedastic")){
    return(IntMatAntoineHet(a, b, c, minReg, maxReg, vertex, option))
  }
}

# eff <- function(Criterion, mat1, mat2, k = 3, s = 2, intPars = 2, matB = diag(3)){
#   if(identical(Criterion, "D-Optimality")){
#     return(deff(mat1, mat2, k))
#   }
#   else if(identical(Criterion, "Ds-Optimality")){
#     return(dseff(mat1, mat2, s, intPars))
#   }
#   else if(identical(Criterion, "A-Optimality")){
#     return(ieff(mat1, mat2, matB))
#   }
#   else if(identical(Criterion, "I-Optimality")){
#     return(ieff(mat1, mat2, matB))
#   }
# }
#
# sens <- function(Criterion, a, b, c, mat, intPars = 0, matB = diag(3)){
#   if(identical(Criterion, "D-Optimality")){
#     return(dsensAntoine(a, b, c, mat))
#   }
#   else if(identical(Criterion, "Ds-Optimality")){
#     return(DSsensAntoine(a, b, c, mat, intPars))
#   }
#   else if(identical(Criterion, "A-Optimality")){
#     return(IsensAntoine(a, b, c, mat, matB))
#   }
#   else if(identical(Criterion, "I-Optimality")){
#     return(IsensAntoine(a, b, c, mat, matB))
#   }
# }
#
# crit <- function(Criterion, mat, k = 3, s = 2, intPars = 2, matB = diag(3)){
#   if(identical(Criterion, "D-Optimality")){
#     return(dcrit(mat, k))
#   }
#   else if(identical(Criterion, "Ds-Optimality")){
#     return(dscrit(mat, s, intPars))
#   }
#   else if(identical(Criterion, "A-Optimality")){
#     return(icrit(mat, matB))
#   }
#   else if(identical(Criterion, "I-Optimality")){
#     return(icrit(mat, matB))
#   }
# }
#
#
# WF <- function(Criterion, A, B, C, initDes, intPar = 0, matB = diag(3), min, max, grid.length, joinThresh, deleteThresh, k = 3, s = 2, deltaW, tol){
#   if(identical(Criterion, "D-Optimality")){
#     return(WFMult(A, B, C, initDes, min, max, grid.length, joinThresh, deleteThresh, k, deltaW, tol))
#   }
#   else if(identical(Criterion, "Ds-Optimality")){
#     return(DSWFMult(A, B, C, initDes, intPar, min, max, grid.length, joinThresh, deleteThresh, s, deltaW, tol))
#   }
#   else if(identical(Criterion, "A-Optimality")){
#     return(IWFMult(A, B, C, initDes, matB, min, max, grid.length, joinThresh, deleteThresh, deltaW, tol))
#   }
#   else if(identical(Criterion, "I-Optimality")){
#     return(IWFMult(A, B, C, initDes, matB, min, max, grid.length, joinThresh, deleteThresh, deltaW, tol))
#   }
# }

getStart <- function(cross, min, max, val, sens_opt){
  if(length(cross)==1){
    if(round(cross[1]-min, 6)==0){
      if((val < sens_opt((max+cross[1])/2))){
        start <- F
      }
      else{
        start <- T
      }
    }
    else{
      if((val < sens_opt((min+cross[1])/2))){
        start <- T
      }
      else{
        start <- F
      }
    }
  }
  else if(val < sens_opt((cross[2]+cross[1])/2)){
    start <- F
  }
  else {
    start <- T
  }
  return(start)
}

getPar <- function(cross){
  return(length(cross)%%2 == 0)
}

getCross2 <- function(cross, min, max, start, par){
  if(par & start){
    cross2 <- c(min, cross, max)
  }
  else if(par & !start){
    cross2 <- cross
  }
  else if(!par & start){
    cross2 <- c(min, cross)
  }
  else{
    cross2 <- c(cross, max)
  }
  return(cross2)
}

# General Antoine

# Cálculo de la traza de una matriz
tr <- function(m) {
  return(sum(diag(m)))
}

# Calcular matrix de un diseño para la ecuación de Antoine
dmatrixAntoine <- function(a, b, c, design) {
  a <- a*log(10)
  b <- b*log(10)

  m11 <- 0
  m12 <- 0
  m13 <- 0
  m21 <- 0
  m22 <- 0
  m23 <- 0
  m31 <- 0
  m32 <- 0
  m33 <- 0

  for(i in seq_along(design$Weight)){
    m11 <- m11 + exp(a - b/(c + design$Point[[i]]))*exp(a - b/(c + design$Point[[i]]))*design$Weight[[i]]
    m12 <- m12 + exp(a - b/(c + design$Point[[i]]))*(-(exp(a - b/(c + design$Point[[i]]))/(c + design$Point[[i]])))*design$Weight[[i]]
    m13 <- m13 + exp(a - b/(c + design$Point[[i]]))*(b*exp(a - b/(c + design$Point[[i]])))/(c + design$Point[[i]])^2*design$Weight[[i]]
    m21 <- m12
    m22 <- m22 + (-(exp(a - b/(c + design$Point[[i]]))/(c + design$Point[[i]])))*(-(exp(a - b/(c + design$Point[[i]]))/(c + design$Point[[i]])))*design$Weight[[i]]
    m23 <- m23 + (-(exp(a - b/(c + design$Point[[i]]))/(c + design$Point[[i]])))*(b*exp(a - b/(c + design$Point[[i]])))/(c + design$Point[[i]])^2*design$Weight[[i]]
    m31 <- m13
    m32 <- m23
    m33 <- m33 + (b*exp(a - b/(c + design$Point[[i]])))/(c + design$Point[[i]])^2*(b*exp(a - b/(c + design$Point[[i]])))/(c + design$Point[[i]])^2*design$Weight[[i]]
  }

  return(matrix(c(m11, m12, m13, m21, m22, m23, m31, m32, m33), nrow = 3, ncol = 3, byrow = TRUE))
}



# design <- data.frame(points = c(1, 50.5, 100), weights = c(1/3, 1/3, 1/3))

# (design2 <- doptAntoine(8.07131, 730.63, 233.426, 1, 100))
# matInit2 <- dmatrixAntoine(a = 8.07131, b = 730.63, c = 233.426, design = design2)

# solve(matInit2)

# matInit <- dmatrixAntoine(a = 8.07131, b = 1730.63, c = 233.426, design = design)


# Función para encontrar el x donde se alcanza el máximo de una función en una rejilla dada, evaluándolo en la misma
findmax <- function(sens, min, max, grid.length) {
  if(min <= max){
    grid <- seq(min, max, length.out = grid.length)
  }
  else {
    grid <- seq(max, min, length.out = grid.length)
  }
  xmax <- grid[which.max(map(grid, sens))]
  return(xmax)
}

# findmax(sens, 1, 100, 1000)


# Función para encontrar el mínimo de una función en una rejilla dada, evaluándolo en la misma
findminval <- function(sens, min, max, grid.length) {
  if(min <= max){
    grid <- seq(min, max, length.out = grid.length)
  }
  else {
    grid <- seq(max, min, length.out = grid.length)
  }
  minval <- min(map_dbl(grid, sens))
  return(minval)
}


# Función para encontrar el mínimo de una función en una rejilla dada, evaluándolo en la misma
findmaxval <- function(sens, min, max, grid.length) {
  if(min <= max){
    grid <- seq(min, max, length.out = grid.length)
  }
  else {
    grid <- seq(max, min, length.out = grid.length)
  }
  maxval <- max(map_dbl(grid, sens))
  return(maxval)
}


# findmax(sens, 1, 100, 1000)



# Función para actualizar un diseño añadiendo un nuevo punto (o peso al mismo si está cerca de otro ya en el diseño)
updateDesign <- function(design, xmax, delta){
  absdiff <- abs(design$Point - xmax) < delta
  if (any(absdiff))
  {
    pos <- min(which(absdiff == TRUE))
    design$Point[[pos]] <- (design$Point[[pos]] + xmax)/2
  }
  else
  {
    design[nrow(design) + 1,] <- c(xmax, 1/(nrow(design) + 1))
  }
  design$Weight <- rep(1/nrow(design), nrow(design))
  return(design)
}

# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/4, 1/4, 1/2))

# updateDesign(des, 50, 0.6)

# Actualizar un diseño sin añadir puntos
updateDesignTotal <- function(design, delta){
  updated <- FALSE
  finished <- FALSE
  while(!finished) {
    for(i in 1:(length(design$Point)-1)) {
      absdiff <- abs(design$Point[-seq(1, i)] - design$Point[i]) < delta
      if (any(absdiff)){
        updated <- TRUE
        design <- updateDesign(design[-i,], design$Point[i], delta)
        break
      }
    }
    finished <- !updated
    updated <- FALSE
  }
  return(design)
}


# Función para eliminar los puntos con menos de un determinado peso, delta
deletePoints <- function(design, delta) {
  updatedDesign <- design[design$Weight > delta, ]
  updatedDesign$Weight <- updatedDesign$Weight/sum(updatedDesign$Weight)
  return(updatedDesign)
}

# des <- data.frame(Point = c(1, 50.5, 100, 5, 30, 80), Weight = c(1/4, 1/4, 1/4, 1/200, 1/100, 1/500))

# deletePoints(des, 0.005)






# --------------------------- D-Optimalidad ----------------------------------

# Función de sensibilidad para D-Optimalidad de la ecuación de Antoine
dsensAntoine <- function(a, b, c, mat) {
  a <- a*log(10)
  b <- b*log(10)
  invMat <- solve(mat)
  sensX <- function(x) {
    # return(exp(a - b/(c + x))*exp(a - b/(c + x))*desMat[[1,1]]+exp(a - b/(c + x))*(-(exp(a - b/(c + x))/(c + x)))*desMat[[1,2]]+exp(a - b/(c + x))*(b*exp(a - b/(c + x)))/(c + x)^2*desMat[[1,3]]+
    #          exp(a - b/(c + x))*(-(exp(a - b/(c + x))/(c + x)))*desMat[[2,1]]+(-(exp(a - b/(c + x))/(c + x)))*(-(exp(a - b/(c + x))/(c + x)))*desMat[[2,2]]+(-(exp(a - b/(c + x))/(c + x)))*(b*exp(a - b/(c + x)))/(c + x)^2*desMat[[2,3]]+
    #          (b*exp(a - b/(c + x)))/(c + x)^2*exp(a - b/(c + x))*desMat[[3,1]]+(b*exp(a - b/(c + x)))/(c + x)^2*(-(exp(a - b/(c + x))/(c + x)))*desMat[[3,2]]+(b*exp(a - b/(c + x)))/(c + x)^2*(b*exp(a - b/(c + x)))/(c + x)^2*desMat[[3,3]])
    return(exp(a - b/(c + x))*(invMat[[1,1]]*exp(a - b/(c + x)) + (invMat[[3,1]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,1]]*exp(a - b/(c + x)))/(c + x)) -
             (exp(a - b/(c + x))*(invMat[[1,2]]*exp(a - b/(c + x)) + (invMat[[3,2]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,2]]*exp(a - b/(c + x)))/(c + x)))/(c + x) +
             (b*exp(a - b/(c + x))*(invMat[[1,3]]*exp(a - b/(c + x)) + (invMat[[3,3]]*b*exp(a - b/(c + x)))/(c + x)^2 - (invMat[[2,3]]*exp(a - b/(c + x)))/(c + x)))/(c + x)^2)
  }
  return(sensX)
}


# optdes <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))

# desMat <- dmatrixAntoine(8.07131, 1730.63, 233.426, optdes)

# sens <- dsensAntoine(8.07131, 1730.63, 233.426, desMat)

# sens(100)

# Cálculo Analítico del diseño D-Óptimo para la ecuación de Antoine
doptAntoine <- function(A, B, C, min, max){
  A <- log(10)*A
  B <- log(10)*B

  if(round(min - max, 3) == 0){
    opdes <- data.frame(Point = c(min), Weight = c(1))
    return(opdes)
  }

  T1 <- (-3*C^3+2*(B)^2*max-6*C^2*max-3*B*(C^2-max^2)-3*C*max^2-sqrt(3)*sqrt((B)^2*(C+max)^4))/(2*(B)^2+6*B*(C+max)+3*(C+max)^2)

  if(min <= T1)
  {
    T2 <- (-3*C^3+2*(B)^2*max-6*C^2*max-3*B*(C^2-max^2)-3*C*max^2+sqrt(3)*sqrt((B)^2*(C+max)^4))/(2*(B)^2+6*B*(C+max)+3*(C+max)^2)
    opdes <- data.frame(Point = c(T1, T2, max), Weight = c(1/3, 1/3, 1/3))
  }
  else
  {
    T2 <- (1/(2*(-B - 2*C - max - min)))*(2*C^2 - B*max - B*min - 2*max*min - sqrt((-2*C^2 + B*max + B*min + 2*max*min)^2 - 4*(-B - 2*C - max - min)*(C^2*max + C^2*min - B*max*min + 2*C*max*min)))
    opdes <- data.frame(Point = c(min, T2, max), Weight = c(1/3, 1/3, 1/3))
  }

  return(opdes)
}
# A <- 18.58487808693377
# B <- 1682.3377464942398
# C <- 233.426
#
# min <- 1
# max <- 100
# (1/(2*(-B - 2*C - max - min)))*(2*C^2 - B*max - B*min - 2*max*min - sqrt((-2*C^2 + B*max + B*min + 2*max*min)^2 - 4*(-B - 2*C - max - min)*(C^2*max + C^2*min - B*max*min + 2*C*max*min)))


# typeof(doptAntoine(A = 8.07131, B = 1730.63, C = 233.426, 99, 374))

# class(doptAntoine(A = 8.07131, B = 1730.63, C = 233.426, 99, 374))
# doptAntoine(A = 8.07131, B = 1730.63, C = 233.426, 1, 40)

# doptAntoine(A = 8.14019, B = 1810.94, C = 244.485, 1, 40)



# Cálculo de D-Eficiencia de la matriz 1 respecto a la matriz 2
deff <- function(mat1, mat2, k) {
  return((det(mat1)/det(mat2))^(1/k))
}

# dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$design)
#
# 5	0.375
# 35	0.3125
# 70	0.3125
#
# deff(dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$design),
#      dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$opt_design), 3)

# Valor del criterio de D-Optimalidad para la matriz
dcrit <- function(mat, k) {
  return((1/det(mat))^(1/k))
}

# dcrit(matInit, 3)


# Función para actualizar los pesos de un diseño para el algoritmo WF
updateWeights <- function(design, sens, k, delta) {
  weights <- design$Weight*(map_dbl(design$Point, sens)/k)^delta
  # print(length(design$Weight))
  # print(length((map_dbl(des$Point, sens)/k)^delta))
  return(weights/sum(weights))
}

# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/4, 1/3, 1/2))

# desMat <- dmatrixAntoine(8.07131, 1730.63, 233.426, des)

# sens <- dsensAntoine(8.07131, 1730.63, 233.426, desMat)

# (des$Weight <- updateWeights(des, sens, 3, 1/2))




# ______________________________________________________

# López-Ríos

# Calculate boundaries for given delta
delta_bound <- function(delta, k, sens_min, sens_max = Inf){
  if(identical(sens_max, Inf)) {
    sens_max <- k
  }
  min <- -(-1 + delta)*((-1 + delta - sens_min*delta)/(-1 + delta))^(1/k)
  max <- -(-1 + delta)*((-1 + delta - sens_max*delta)/(-1 + delta))^(1/k)
  return(c(min, max))
}

# delta_bound(1/4, 3, -2.50965)


# Calculate delta for given efficiency


# NO SE PUEDE "FÁCIL"
# Calculate boundary for given efficiency
# efficiency_bound <- function(deff, k, sens_min){
#   min <- -(-1 + delta)*((-1 + delta - sens_min*delta)/(-1 + delta))^(1/k)
#   max <- -(-1 + delta)*((-1 + delta - k*delta)/(-1 + delta))^(1/k)
#   return(c(min, max))
# }



# Calculate sens value for given delta and efficiency
sens_val_to_add <- function(deff, delta, k){
  return((1 - delta)/delta*((deff/(1 - delta))^k - 1))
}

# sens_val_to_add(0.9, 1/4, 3)


# Elimina puntos duplicados
update_sequence <- function(points, tol){
  i <- 1
  imax <- (length(points)-1)
  while(i < imax) {
    absdiff <- c(rep(T, i), abs(points[-seq(1, i)] - points[i]) > tol)
    points <- points[absdiff]
    i <- i+1
    imax <- (length(points)-1)
  }
  return(points)
}




# Calculate crosspoints
crosspoints <- function(deff, delta, k, sens, gridlength, tol, xmin, xmax){
  val <- sens_val_to_add(deff, delta, k)

  sensfix <- function(x){
    return(sens(x) - val)
  }

  sols <- seq(xmin, xmax, length.out = gridlength) %>%
    map(nleqslv, fn = sensfix) %>%
    map(function(x) x$x) %>%
    unlist

  # Eliminar duplicados
  sols_upd <- update_sequence(sols, tol)

  # Quedarnos con puntos dentro del espacio de diseño
  sols_upd <- sols_upd[sols_upd >= xmin & sols_upd <= xmax]

  # Eliminar los que no son soluciones
  sols_upd <- sols_upd[abs(map_dbl(sols_upd, sens) - val) < tol]


  return(sort(sols_upd))
}

# optdes <- doptAntoine(A = 8.07131, B = 1730.63, C = 233.426, 1, 100)

# desMat <- dmatrixAntoine(8.07131, 1730.63, 233.426, optdes)

# sens <- dsensAntoine(8.07131, 1730.63, 233.426, desMat)

# sols_new <- crosspoints(0.9, 1/4, 3, sens, 1000, 10^(-3), 1, 100)



# Update design given crosspoints and delta
add_points <- function(points, delta, design){
  new_points <- data.frame("Point" = points, "Weight" = rep(delta/length(points), times = length(points)))
  design["Weight"] <- design["Weight"]*(1-delta)
  return(rbind(design, new_points))
}

# des <- data.frame(Point = c(1, 50.5, 100), Weight = c(1/3, 1/3, 1/3))
# add_points(c(20, 70, 90), 1/4, des)


# Uniform design
# uniform_design <- function(n, xmin, xmax){
#   unif_des <- data.frame("Point" = seq(xmin,xmax,length.out = n), "Weight" = rep(1/n, times = n))
#   return(unif_des)
# }

# Arithmetic design
# arithmetic_design <- function(n, xmin, xmax, dopt_mat, model, a = 0, b = 0, c = 0){
#   if(identical(model, "Cuadratic")){
#     arithdes_opt <- function(r){
#       arith_des <- data.frame("Point" = seq(r,xmax,length.out = n), "Weight" = rep(1/n, times = n))
#       arith_mat <- dmatrixCuad(arith_des)
#       return(deff(dopt_mat,arith_mat, 3))
#     }
#   } else if(identical(model, "Antoine Equation")){
#     arithdes_opt <- function(r){
#       arith_des <- data.frame("Point" = seq(r,xmax,length.out = n), "Weight" = rep(1/n, times = n))
#       arith_mat <- dmatrixAntoine(a, b, c, arith_des)
#       return(deff(dopt_mat,arith_mat, 3))
#     }
#   } else if(identical(model, "Michaelis-Menten")){
#     arithdes_opt <- function(r){
#       arith_des <- data.frame("Point" = seq(r,xmax,length.out = n), "Weight" = rep(1/n, times = n))
#       arith_mat <- dmatrixMM(a, b, arith_des)
#       return(deff(dopt_mat,arith_mat, 2))
#     }
#   }

  # grid <- seq(0, 1, length.out = 101)[1:100]
  #
  # map_dbl(grid, arithdes_opt)

  #if(criterion %in% c("D-Optimality", "Ds-Optimality")){
    # r_val <- round(nloptr::direct(fn = arithdes_opt,lower = xmin, upper = xmax-(xmax-xmin)/10, control=list(xtol_rel=1e-8, maxeval=1000))$par, 3)
  # }
  # else{
  #   seq(xmin, xmax, length.out = 1000)[which.max(map(seq(xmin, xmax-(xmax-xmin)/5, length.out = 1000), arithdes_opt))]
  # }

#   return(data.frame("Point" = seq(r_val,xmax,length.out = n), "Weight" = rep(1/n, times = n)))
# }

# opt_des <- doptCuad(1, 100)
# dopt_mat <- dmatrixCuad(opt_des)
#
# arithmetic_design(5, 1, 100, dopt_mat, "Cuadratic")

# opt_des <- doptMM(227.27, 43.73, 0, 10)
# dopt_mat <- dmatrixMM(227.27, 43.73, opt_des)
#
# arithmetic_design(5, 0, 10, dopt_mat, "Michaelis-Menten", 227.27, 43.73)



# Geometric design
# geometric_design <- function(n, xmin, xmax, dopt_mat, model, a = 0, b = 0, c = 0){
#   if(identical(model, "Cuadratic")){
#     geodes_opt <- function(r){
#       geom_des <- data.frame("Point" = (xmax-xmin)*r^(0:(n-1))+xmin, "Weight" = rep(1/n, times = n))
#       geom_mat <- dmatrixCuad(geom_des)
#       return(deff(dopt_mat,geom_mat, 3))
#     }
#   } else if(identical(model, "Antoine Equation")){
#     geodes_opt <- function(r){
#       geom_des <- data.frame("Point" = (xmax-xmin)*r^(0:(n-1))+xmin, "Weight" = rep(1/n, times = n))
#       geom_mat <- dmatrixAntoine(a, b, c, geom_des)
#       return(deff(dopt_mat,geom_mat, 3))
#     }
#   } else if(identical(model, "Michaelis-Menten")){
#     geodes_opt <- function(r){
#       geom_des <- data.frame("Point" = (xmax-xmin)*r^(0:(n-1))+xmin, "Weight" = rep(1/n, times = n))
#       geom_mat <- dmatrixMM(a, b, geom_des)
#       return(deff(dopt_mat,geom_mat, 2))
#     }
#   }
#
#   r_val <- round(nloptr::direct(fn = geodes_opt,lower = 0, upper = 1, control=list(xtol_rel=1e-8, maxeval=1000))$par, 3)
#
#   return(data.frame("Point" = (xmax-xmin)*r_val^(0:(n-1))+xmin, "Weight" = rep(1/n, times = n)) %>% arrange(Point))
# }

# opt_des <- doptCuad(1, 100)
# dopt_mat <- dmatrixCuad(opt_des)
#
#
# geometric_design(5, 1, 100, dopt_mat, "Cuadratic")




# -----------------------------------------------------------------------------------------------

# MODELO CUADRÁTICO


# Calcular matrix de un diseño
dmatrixCuad <- function(design) {

  m11 <- 0
  m12 <- 0
  m13 <- 0
  m21 <- 0
  m22 <- 0
  m23 <- 0
  m31 <- 0
  m32 <- 0
  m33 <- 0

  for(i in seq_along(design$Weight)){
    m11 <- m11 + design$Weight[[i]]
    m12 <- m12 + design$Point[[i]]*design$Weight[[i]]
    m13 <- m13 + design$Point[[i]]^2*design$Weight[[i]]
    m21 <- m12
    m22 <- m13
    m23 <- m23 + design$Point[[i]]^3*design$Weight[[i]]
    m31 <- m13
    m32 <- m23
    m33 <- m33 +  design$Point[[i]]^4*design$Weight[[i]]
  }

  return(matrix(c(m11, m12, m13, m21, m22, m23, m31, m32, m33), nrow = 3, ncol = 3, byrow = TRUE))
}




# --------------------------- D-Optimalidad ----------------------------------

# Función de sensibilidad para D-Optimalidad
dsensCuad <- function(mat) {
  invMat <- solve(mat)
  sensX <- function(x) {
    return(invMat[[1,1]]+x*(invMat[[2,1]]+invMat[[1,2]])+x^2*(invMat[[1,3]]+invMat[[2,2]]+invMat[[3,1]])+x^3*(invMat[[2,3]]+invMat[[3,2]])+x^4*invMat[[3,3]])
  }
  return(sensX)
}



# Cálculo Analítico del diseño D-Óptimo
doptCuad <- function(min, max){
  opdes <- data.frame(Point = c(min, (min+max)/2, max), Weight = c(1/3, 1/3, 1/3))

  return(opdes)
}

# doptCuad(0,1)


# -----------------------------------------------------------------------------------------------

# Michaelis-Menten


# Calcular matrix de un diseño
dmatrixMM <- function(K, V, design) {

  m11 <- 0
  m12 <- 0
  m21 <- 0
  m22 <- 0


  for(i in seq_along(design$Weight)){
    m11 <- m11 + (design$Point[[i]]^2/(K + design$Point[[i]])^2)*design$Weight[[i]]
    m12 <- m12 + (-((V * design$Point[[i]]^2)/(K + design$Point[[i]])^3) )*design$Weight[[i]]
    m21 <- m12
    m22 <- m22 + ((V^2 * design$Point[[i]]^2)/(K + design$Point[[i]])^4)*design$Weight[[i]]
  }

  return(matrix(c(m11, m12, m21, m22), nrow = 2, ncol = 2, byrow = TRUE))
}

# matMM <- dmatrixMM(227.27, 43.73, optdes)




# --------------------------- D-Optimalidad ----------------------------------


# Función de sensibilidad para D-Optimalidad
dsensMM <- function(K, V, mat) {
  invMat <- solve(mat)
  sensX <- function(x) {
    return((x *(-((invMat[[2, 1]] * V * x) / (K + x) ^ 2 ) + (invMat[[1, 1]] * x) / (K + x))) / (K + x) - (V * x *(-((invMat[[2, 2]] * V * x) / (K + x) ^ 2) + (invMat[[1, 2]] * x) / (K + x))) / (K + x) ^ 2)
  }
  return(sensX)
}

# sensM <- dsensMM(227.27, 43.73, matMM)
#
# sensM(10)

# Cálculo Analítico del diseño D-Óptimo
doptMM <- function(K, V, min, max){
  b <- max/K
  s <- b/(2+b)
  opdes <- data.frame(Point = c(s*K, max), Weight = c(1/2, 1/2))

  return(opdes)
}

# optdes <- doptMM(227.27, 43.73, 0, 10)


# -----------------------------------------------------------------------------------------------

# MODELO CUADRÁTICO HETEROCEDÁSTICO


# Calcular matrix de un diseño
dmatrixCuadHet <- function(order, design) {

  m11 <- 0
  m12 <- 0
  m13 <- 0
  m21 <- 0
  m22 <- 0
  m23 <- 0
  m31 <- 0
  m32 <- 0
  m33 <- 0

  for(i in seq_along(design$Weight)){
    m11 <- m11 + design$Weight[[i]] * exp(-design$Point[[i]]) * design$Point[[i]]^(1+order)
    m12 <- m12 + design$Point[[i]]*design$Weight[[i]] * exp(-design$Point[[i]]) * design$Point[[i]]^(1+order)
    m13 <- m13 + design$Point[[i]]^2*design$Weight[[i]] * exp(-design$Point[[i]]) * design$Point[[i]]^(1+order)
    m21 <- m12
    m22 <- m13
    m23 <- m23 + design$Point[[i]]^3*design$Weight[[i]] * exp(-design$Point[[i]]) * design$Point[[i]]^(1+order)
    m31 <- m13
    m32 <- m23
    m33 <- m33 +  design$Point[[i]]^4*design$Weight[[i]] * exp(-design$Point[[i]]) * design$Point[[i]]^(1+order)
  }

  return(matrix(c(m11, m12, m13, m21, m22, m23, m31, m32, m33), nrow = 3, ncol = 3, byrow = TRUE))
}




# --------------------------- D-Optimalidad ----------------------------------

# Función de sensibilidad para D-Optimalidad
dsensCuadHet <- function(order, mat) {
  invMat <- solve(mat)
  sensX <- function(x) {
    return(exp(-x) * x^(1+order)*(invMat[[1,1]]+x*(invMat[[2,1]]+invMat[[1,2]])+x^2*(invMat[[1,3]]+invMat[[2,2]]+invMat[[3,1]])+x^3*(invMat[[2,3]]+invMat[[3,2]])+x^4*invMat[[3,3]]))
  }
  return(sensX)
}



# Cálculo Analítico del diseño D-Óptimo
doptCuadHet <- function(order){
  opdes <- data.frame(Point = solve(glaguerre.polynomials(3, order, FALSE)[[4]]), Weight = c(1/3, 1/3, 1/3))

  return(opdes)
}

# opt_des <- doptCuadHet(0.5)
# M <- dmatrixCuadHet(0.5, opt_des)
# sens <- dsensCuadHet(0.5, M)
# sens(7.0328990)

library(optedr)

# Fixtures -------------------------------------------------------------------
# k = 2 model, used in other test files for consistency
model_2p    <- y ~ a * exp(-b / x)
params_2p   <- c("a", "b")
par_vals_2p <- c(1, 1500)
dspace_2p   <- c(212, 422)
# Three-point design, not optimal — used as init_design throughout
init_des_2p <- data.frame("Point"  = c(220, 300, 400),
                          "Weight" = c(1/3, 1/3, 1/3))
alpha_val <- 0.25

# delta_val = 0.85 is expected to lie comfortably within the efficiency range
# [findminval(eff_fun,...), findmaxval(eff_fun,...)] for this model with alpha=0.25, k=2.
# eff_fun = 0.75 * (1 + sens_1(x)/3)^0.5, minimum ~0.75, maximum well above 1.
# Adjust this constant if a test fails with "outside the valid range".
DELTA_D <- 0.85

# Ds model
model_ds    <- y ~ th0 * exp(x / th1)
params_ds   <- c("th0", "th1")
par_vals_ds <- c(10.4963, -3.2940)
dspace_ds   <- c(0.94, 30)
init_des_ds <- data.frame("Point"  = c(1, 10, 28),
                          "Weight" = c(1/3, 1/3, 1/3))
DELTA_DS <- 0.85


# get_augment_region — validation errors -------------------------------------

test_that("get_augment_region errors without delta_val in non-interactive context", {
  expect_error(
    get_augment_region("D-Optimality", init_des_2p, alpha_val,
                       model_2p, params_2p, par_vals_2p, dspace_2p, FALSE),
    "Supply delta_val"
  )
})

test_that("get_augment_region errors on invalid criterion", {
  expect_error(
    get_augment_region("E-Optimality", init_des_2p, alpha_val,
                       model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                       delta_val = DELTA_D),
    "Invalid criterion"
  )
})

test_that("get_augment_region warns when delta_val is below valid range", {
  expect_warning(
    tryCatch(
      suppressMessages(
        get_augment_region("D-Optimality", init_des_2p, alpha_val,
                           model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                           delta_val = 0)
      ),
      error = function(e) NULL  # subsequent computation may fail; only check warning
    ),
    "below the minimum"
  )
})

test_that("get_augment_region errors when delta_val is above valid range", {
  expect_error(
    get_augment_region("D-Optimality", init_des_2p, alpha_val,
                       model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                       delta_val = 100),
    "maximum achievable"
  )
})


# get_augment_region — output properties ------------------------------------

test_that("get_augment_region D-Optimality returns an augment_region with numeric region", {
  region <- suppressMessages(
    get_augment_region("D-Optimality", init_des_2p, alpha_val,
                       model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                       delta_val = DELTA_D)
  )
  expect_s3_class(region, "augment_region")
  r <- region$region
  expect_true(is.numeric(r))
  expect_gte(length(r), 2L)
  expect_equal(length(r) %% 2L, 0L)
  expect_true(all(r >= dspace_2p[1] & r <= dspace_2p[2]))
  for (i in seq_len(length(r) / 2L))
    expect_lt(r[2*i - 1], r[2*i])
})

test_that("get_augment_region Ds-Optimality returns an augment_region with numeric region", {
  region <- suppressMessages(
    get_augment_region("Ds-Optimality", init_des_ds, alpha_val,
                       model_ds, params_ds, par_vals_ds, dspace_ds, FALSE,
                       par_int = c(1L), delta_val = DELTA_DS)
  )
  expect_s3_class(region, "augment_region")
  r <- region$region
  expect_true(is.numeric(r))
  expect_equal(length(r) %% 2L, 0L)
  expect_true(all(r >= dspace_ds[1] & r <= dspace_ds[2]))
})


# augment_design — validation errors ----------------------------------------

test_that("augment_design errors without delta_val in non-interactive context", {
  expect_error(
    augment_design("D-Optimality", init_des_2p, alpha_val,
                   model_2p, params_2p, par_vals_2p, dspace_2p, FALSE),
    "Supply delta_val"
  )
})

test_that("augment_design errors without new_points in non-interactive context", {
  expect_error(
    augment_design("D-Optimality", init_des_2p, alpha_val,
                   model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                   delta_val = DELTA_D),
    "Supply new_points"
  )
})

test_that("augment_design errors on invalid criterion", {
  new_pts <- data.frame(Point = 300, Weight = 1)
  expect_error(
    augment_design("Z-Optimality", init_des_2p, alpha_val,
                   model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                   delta_val = DELTA_D, new_points = new_pts),
    "Invalid criterion"
  )
})

test_that("augment_design warns when delta_val is below valid range", {
  new_pts <- data.frame(Point = 300, Weight = 1)
  expect_warning(
    tryCatch(
      suppressMessages(
        augment_design("D-Optimality", init_des_2p, alpha_val,
                       model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                       delta_val = 0, new_points = new_pts)
      ),
      error = function(e) NULL
    ),
    "below the minimum"
  )
})


# new_points validation ------------------------------------------------------

test_that("new_points must be a data.frame with Point and Weight columns", {
  expect_error(
    augment_design("D-Optimality", init_des_2p, alpha_val,
                   model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                   delta_val = DELTA_D,
                   new_points = data.frame(Punto = 300, Weight = 1)),
    "'Point' and 'Weight'"
  )
  expect_error(
    augment_design("D-Optimality", init_des_2p, alpha_val,
                   model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                   delta_val = DELTA_D,
                   new_points = data.frame(Point = 300, Peso = 1)),
    "'Point' and 'Weight'"
  )
})

test_that("new_points weights must be strictly positive", {
  expect_error(
    augment_design("D-Optimality", init_des_2p, alpha_val,
                   model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                   delta_val = DELTA_D,
                   new_points = data.frame(Point = 300, Weight = -1)),
    "positive"
  )
  expect_error(
    augment_design("D-Optimality", init_des_2p, alpha_val,
                   model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                   delta_val = DELTA_D,
                   new_points = data.frame(Point = 300, Weight = 0)),
    "positive"
  )
})

test_that("new_points must lie within the design space", {
  expect_error(
    augment_design("D-Optimality", init_des_2p, alpha_val,
                   model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                   delta_val = DELTA_D,
                   new_points = data.frame(Point = 500, Weight = 1)),
    "outside the design space"
  )
  expect_error(
    augment_design("D-Optimality", init_des_2p, alpha_val,
                   model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                   delta_val = DELTA_D,
                   new_points = data.frame(Point = 100, Weight = 1)),
    "outside the design space"
  )
})

test_that("new_points outside candidate region are rejected", {
  region <- suppressMessages(
    get_augment_region("D-Optimality", init_des_2p, alpha_val,
                       model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                       delta_val = DELTA_D)
  )
  # Find a point inside the design space but outside the candidate region
  n_regions <- length(region$region) / 2L
  outside_pt <- NULL
  for (pt in seq(dspace_2p[1], dspace_2p[2], length.out = 100)) {
    in_reg <- any(vapply(seq_len(n_regions),
                         function(i) pt >= region$region[2*i-1] & pt <= region$region[2*i],
                         logical(1)))
    if (!in_reg) { outside_pt <- pt; break }
  }
  skip_if(is.null(outside_pt),
          "All sampled points happen to lie in the candidate region — skip")
  expect_error(
    augment_design("D-Optimality", init_des_2p, alpha_val,
                   model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                   delta_val = DELTA_D,
                   new_points = data.frame(Point = outside_pt, Weight = 1)),
    "candidate region"
  )
})


# augment_design — output correctness ----------------------------------------

test_that("augment_design D-Optimality: output has correct structure and sums to 1", {
  region <- suppressMessages(
    get_augment_region("D-Optimality", init_des_2p, alpha_val,
                       model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                       delta_val = DELTA_D)
  )
  new_pt  <- (region$region[1] + region$region[2]) / 2
  new_pts <- data.frame(Point = new_pt, Weight = 1)

  result <- suppressMessages(
    augment_design("D-Optimality", init_des_2p, alpha_val,
                   model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                   delta_val = DELTA_D, new_points = new_pts)
  )

  expect_s3_class(result, "data.frame")
  expect_named(result, c("Point", "Weight"))
  expect_equal(nrow(result), nrow(init_des_2p) + 1L)
  expect_equal(sum(result$Weight), 1, tolerance = 1e-10)
  expect_true(all(result$Weight > 0))
  expect_true(all(result$Point >= dspace_2p[1] & result$Point <= dspace_2p[2]))
})

test_that("augment_design D-Optimality: existing weights scaled by (1 - alpha), new weight = alpha", {
  region <- suppressMessages(
    get_augment_region("D-Optimality", init_des_2p, alpha_val,
                       model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                       delta_val = DELTA_D)
  )
  new_pt  <- (region$region[1] + region$region[2]) / 2
  new_pts <- data.frame(Point = new_pt, Weight = 1)

  result <- suppressMessages(
    augment_design("D-Optimality", init_des_2p, alpha_val,
                   model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                   delta_val = DELTA_D, new_points = new_pts)
  )

  orig_rows <- match(init_des_2p$Point, result$Point)
  expect_equal(result$Weight[orig_rows],
               init_des_2p$Weight * (1 - alpha_val),
               tolerance = 1e-10)
  new_row <- match(new_pt, result$Point)
  expect_equal(result$Weight[new_row], alpha_val, tolerance = 1e-10)
})

test_that("augment_design with empty new_points returns init_design unchanged", {
  result <- suppressMessages(
    augment_design("D-Optimality", init_des_2p, alpha_val,
                   model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                   delta_val = DELTA_D,
                   new_points = data.frame(Point = double(), Weight = double()))
  )
  expect_equal(result$Point,  init_des_2p$Point)
  expect_equal(result$Weight, init_des_2p$Weight)
})

test_that("augment_design with multiple new_points adds them all", {
  region <- suppressMessages(
    get_augment_region("D-Optimality", init_des_2p, alpha_val,
                       model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                       delta_val = DELTA_D)
  )
  n_regions <- length(region$region) / 2L
  skip_if(n_regions < 2L, "Need at least 2 candidate regions for this test")
  new_pts <- data.frame(
    Point  = c((region$region[1] + region$region[2]) / 2,
               (region$region[3] + region$region[4]) / 2),
    Weight = c(0.5, 0.5)
  )
  result <- suppressMessages(
    augment_design("D-Optimality", init_des_2p, alpha_val,
                   model_2p, params_2p, par_vals_2p, dspace_2p, FALSE,
                   delta_val = DELTA_D, new_points = new_pts)
  )
  expect_equal(nrow(result), nrow(init_des_2p) + 2L)
  expect_equal(sum(result$Weight), 1, tolerance = 1e-10)
})

test_that("augment_design Ds-Optimality: output has correct structure and sums to 1", {
  region <- suppressMessages(
    get_augment_region("Ds-Optimality", init_des_ds, alpha_val,
                       model_ds, params_ds, par_vals_ds, dspace_ds, FALSE,
                       par_int = c(1L), delta_val = DELTA_DS)
  )
  new_pt  <- (region$region[1] + region$region[2]) / 2
  new_pts <- data.frame(Point = new_pt, Weight = 1)

  result <- suppressMessages(
    augment_design("Ds-Optimality", init_des_ds, alpha_val,
                   model_ds, params_ds, par_vals_ds, dspace_ds, FALSE,
                   par_int = c(1L), delta_val = DELTA_DS, new_points = new_pts)
  )

  expect_s3_class(result, "data.frame")
  expect_named(result, c("Point", "Weight"))
  expect_equal(sum(result$Weight), 1, tolerance = 1e-10)
  expect_true(all(result$Weight > 0))
})

library(shiny)
library(dplyr)
library(DT)
library(ggplot2)
library(shinydashboard)
library(shinyalert)
library(plotly)
library(tidyverse)
library(hrbrthemes)
library(nleqslv)
library(markdown)
library(knitr)
library(shinyjs)
library(orthopolynom)


# Source dopt ----
source("dopt.R")

# Auxiliar functions -----
source("auxfuncs.R")


# UI -------
ui <- shinydashboard::dashboardPage(

    # Header ---------
    shinydashboard::dashboardHeader(title = "Design Builder"
                    # , tags$li(class = "dropdown", tags$p("Antoine"))
                    ),

    # Sidebar --------
    shinydashboard::dashboardSidebar(
        shinydashboard::sidebarMenu(id="sidebar",
            tags$head(tags$style(HTML(
                            '.myClass {
                font-size: 20px;
                line-height: 50px;
                text-align: left;
                font-family: "Helvetica Neue",Helvetica,Arial,sans-serif;
                padding: 0 15px;
                overflow: hidden;
                color: white;
                }
                '))),
                        tags$script(HTML('
                               $(document).ready(function() {
                               $("header").find("nav").append(\'<div id="pageHeader" class="myClass"></div>\');
                               })
                               '))
            ,
            shinyjs::useShinyjs(),
            shinydashboard::menuItem("Presentation", tabName = "presentation", icon = icon("info")),
            shinydashboard::menuItem("Set up & Optimal design", tabName = "optdes", icon = icon("calculator")),
            shinydashboard::menuItem("Custom design", tabName = "custom", icon = icon("clipboard"),
                     shinydashboard::menuSubItem("Design Proposal", tabName = "givedes"),
                     shinydashboard::menuSubItem("Restricted design", tabName = "restricteddes")),
            shinydashboard::menuItem("Your design", tabName = "your", icon = icon("clipboard"),
                     shinydashboard::menuSubItem("Build your design", tabName = "builddes"),
                     shinydashboard::menuSubItem("Enhance your design", tabName = "enhacedes")
            )
            # menuItem("Efficiency of industry designs", tabName = "compare", icon = icon("book")),
            # menuItem("Help", tabName = "help", icon = icon("question")),
            # menuItem("Contact", tabName = "contact", icon = icon("envelope"))
        ),
        shiny::hr(),
        shiny::conditionalPanel(condition="input.sidebar == 'optdes'",
            shiny::selectizeInput(
            'model',  shiny::HTML(paste("Inputs:",'<br/>',"Choose a model")), choices = c("Antoine Equation", "Quadratic", "Quadratic Heteroscedastic", "Michaelis-Menten"#, "A-Optimality", "Ds-Optimality", "I-Optimality"
                                                           ),
            options = list(
                placeholder = 'Type the desired model',
                onInitialize = I('function() { this.setValue(""); }')
            )
            ),
            shiny::conditionalPanel("input.model == 'Antoine Equation' || input.model == 'Quadratic'",
                shiny::sliderInput("design_sp", "Design Space",
                        min = -100, max = 374, value = c(1, 100)
                )
            ),
            shiny::conditionalPanel("input.model == 'Michaelis-Menten'",
                             shiny::sliderInput("design_sp_MM", "Design Space: [0, value]",
                                         min = 0.01, max = 100, value = 10
                             )
            ),
            shiny::conditionalPanel("input.model == 'Antoine Equation'",
                             shiny::numericInput('a_par', 'Initial value for parameter a', value = 8.07131, step = 0.01
                             ),
                             shiny::numericInput('b_par', 'Initial value for parameter b', value = 1730.63, step = 0.01
                             ),
                             shiny::numericInput('c_par', 'Initial value for parameter c', value = 233.426, step = 0.01
                             )
            ),
            shiny::conditionalPanel("input.model == 'Michaelis-Menten'",
                             shiny::numericInput('k_par', 'Initial value for parameter K', value = 227.27, step = 0.01
                             ),
                             shiny::numericInput('v_par', 'Initial value for parameter V', value = 43.73, step = 0.01
                             )
            ),
            shiny::conditionalPanel("input.model == 'Quadratic Heteroscedastic'",
                             shiny::sliderInput('order', 'Order of the polynomial', min = -0.99, max = 1, value = 0.5
                             ),
                             shiny::sliderInput("design_sp_CH", "Design Space: [0, value]",
                                         min = 8, max = 20, value = 10
                             )
            ),
            shiny::actionButton("calc_opt_design", "Calculate Optimal Design"
        )),
        # conditionalPanel(condition="input.sidebar == 'compare'",
        #                  selectizeInput(
        #                      'family', HTML(paste("Inputs:",'<br/>',"Choose a family of designs")), choices = c("Equidistant", "Arithmetic", "Geometric"#, "A-Optimality", "Ds-Optimality", "I-Optimality"
        #                      ),
        #                      options = list(
        #                          placeholder = 'Type the desired family',
        #                          onInitialize = I('function() { this.setValue(""); }')
        #                      )
        #                  ),
        #                  sliderInput("ind_points", "Points of the design",
        #                              min = 3, max = 20, value = 5
        #                  ),
        #                  actionButton("calc_ind_eff", "Calculate Efficiencies"
        #                  )
        # ),
        shiny::conditionalPanel(condition="input.sidebar == 'givedes' || input.sidebar =='restricteddes'",
            shiny::sliderInput("delta", shiny::HTML(paste("Inputs:",'<br/>',shiny::withMathJax("Weight of the new point(s), alpha"))),
                        min = 0, max = 0.99, value = 0
            ),
            shiny::uiOutput("eff"),
            shiny::actionButton(inputId = "give_points", "Region of Candidate Points"
            )
        ),
        shiny::conditionalPanel("input.sidebar == 'builddes'",
            shiny::numericInput('points_add',  shiny::HTML(paste("Inputs:",'<br/>',"1. Point")), value = -101, min = -100, max = 374, step = 1
            ),
            shiny::conditionalPanel('input.points_add != -101',
                            shiny::numericInput('weights_add', '2. Weight', value = 0, min = 0, max = 1, step = 0.01)
            ),
            shiny::actionButton("add", "Add point"
            ),
            shiny::actionButton("remove", "Remove last point"
            ),
            shiny::actionButton("remove_all", "Remove all points"
            ),
            shiny::actionButton("both_designs", "Calculate efficiency"
            )
            ),
        shiny::conditionalPanel("input.sidebar == 'restricteddes'",
        shiny::conditionalPanel("output.show_rest == 1",
                 shiny::numericInput('points_add_opt', '1. Point', value = -101, min = -100, max = 374, step = 1
                 ),
                 shiny::conditionalPanel('input.points_add_opt != -101',
                                shiny::numericInput('weights_add_opt', '2. Weight', value = 0, min = 0, max = 1, step = 0.01)
                 ),
                 shiny::actionButton("add_opt", "Add point"
                 ),
                 shiny::actionButton("remove_opt", "Remove last point"
                 ),
                 shiny::actionButton("remove_opt_all", "Remove all points"
                 ),
                 shiny::actionButton("with_opt", "Calculate efficiency"
                 )
             )
        ),
        shiny::conditionalPanel(condition="input.sidebar == 'enhacedes'",
                 shiny::sliderInput("deltaenh",  shiny::HTML(paste("Inputs:",'<br/>',"Weight of the new point(s)")),
                             min = 0, max = 0.99, value = 0
                 ),
                 shiny::uiOutput("effenh"),
                 shiny::actionButton("enhace_points", "Region of Candidate Points"
                 ),
                 shiny::conditionalPanel("output.show_enh == 1",
                     shiny::numericInput('points_add_enh', '1. Point', value = -101, min = -100, max = 374, step = 1
                     ),
                     shiny::conditionalPanel('input.points_add_enh != -101',
                                    shiny::numericInput('weights_add_enh', '2. Weight', value = 0, min = 0, max = 1, step = 0.01)
                     ),
                     shiny::actionButton("add_enh", "Add point"
                     ),
                     shiny::actionButton("remove_enh", "Remove last point"
                     ),
                     shiny::actionButton("remove_enh_all", "Remove all points"
                     ),
                     shiny::actionButton("enhace_design", "Calculate efficiency"
                     )
                 )
        ),
        shiny::conditionalPanel("input.sidebar == 'givedes'",
            #span("Test", style="color:white"),
            shiny::actionButton("stand_design", "Calculate design"
        ))),

    # Body ----------
    shinydashboard::dashboardBody(
        shinyalert::useShinyalert(),
        shinydashboard::tabItems(
            # Presentation
            shinydashboard::tabItem(tabName = "presentation",
                    shiny::fluidPage(shiny::withMathJax(shiny::includeMarkdown("presentacion.Rmd")))),
            # Optimal Design
            shinydashboard::tabItem(tabName = "optdes",
                    # Row --
                    shiny::fluidRow(
                        shinydashboard::valueBox(value = "Optimum Design"#tags$p("Optimum Design", style = "font-size: 75%;")
                                 , "For the selected criterion", icon = icon("check"), color = "yellow"),
                        shinydashboard::valueBoxOutput("op_sensitivity")
                    ),
                    # Row --
                    shiny::fluidRow(
                        shinydashboard::box(title = "Optimum design",
                            solidHeader = T,
                            width = 4,
                            collapsible = T,
                            div(DT::DTOutput("opt_df"), style = "font-size: 120%;")
                        ),
                        shinydashboard::box(title = "Sensitivity Function", solidHeader = T,
                            width = 8, collapsible = T,
                            plotlyOutput("sens_opt")
                        )
                    )
            ),
            # Custom design
            tabItem(tabName = "givedes",
                    # Row --
                    fluidRow(
                        valueBox(value = "Given Design"#tags$p("Optimum Design", style = "font-size: 75%;")
                                 , "With crosspoints", icon = icon("clipboard-list"), color = "aqua"),
                        valueBox(value = "Sensitivity Function"#tags$p("Optimum Design", style = "font-size: 75%;")
                                 , "For the given design", icon = icon("bar-chart-o"), color = "green"),
                        valueBoxOutput("efficiency")
                    ),
                    # Row --
                    fluidRow(
                        box(title = "New design",
                            solidHeader = T,
                            width = 4,
                            collapsible = T,
                            div(DT::DTOutput("givedes"), style = "font-size: 120%;")
                        ),
                        box(title = "New sensitivity", solidHeader = T,
                            width = 8, collapsible = T,
                            plotlyOutput("sens_new")
                        )
                    )
                    # ,
                    # Row --
                    # fluidRow(
                    #     valueBox(value = "Region for the points"#tags$p("Optimum Design", style = "font-size: 75%;")
                    #              , "Points at the extremes", icon = icon("bar-chart-o"), color = "yellow"),
                    #     box(title = "Feasible region 2", solidHeader = T,
                    #         width = 8, collapsible = T,
                    #         plotlyOutput("feas_regions")
                    #     )
                    # )
            ),
            tabItem(tabName = "restricteddes",
                    # Row --
                    fluidRow(
                        valueBox(value = "Design from the Optimum"#tags$p("Optimum Design", style = "font-size: 75%;")
                                 , "Chosen points", icon = icon("clipboard-list"), color = "aqua"),
                        valueBox(value = "Sensitivity Function"#tags$p("Optimum Design", style = "font-size: 75%;")
                                 , "For the added design", icon = icon("bar-chart-o"), color = "green"),
                        valueBoxOutput("efficiencyresdes")
                    ),
                    # Row --
                    fluidRow(
                        box(title = "New design",
                            solidHeader = T,
                            width = 4,
                            collapsible = T,
                            div(DT::DTOutput("woptdes"), style = "font-size: 120%;")
                        ),
                        box(title = "New sensitivity", solidHeader = T,
                            width = 8, collapsible = T,
                            plotlyOutput("sens_res")
                        )
                    )
                    # ,
                    # Row --
                    # fluidRow(
                    #     valueBox(value = "Feasible region"#tags$p("Optimum Design", style = "font-size: 75%;")
                    #              , "To add points", icon = icon("bar-chart-o"), color = "yellow"),
                    #     box(title = "Feasible region", solidHeader = T,
                    #         width = 8, collapsible = T,
                    #         plotlyOutput("feas_regions_res")
                    #     )
                    # )
            ),
            tabItem(tabName = "builddes",
                    # Row --
                    fluidRow(
                        valueBox(value = "Full custom design"#tags$p("Optimum Design", style = "font-size: 75%;")
                                 , "Chosen by the user", icon = icon("clipboard-list"), color = "aqua"),
                        valueBox(value = "Sensitivity Function"#tags$p("Optimum Design", style = "font-size: 75%;")
                                 , "For the built design", icon = icon("bar-chart-o"), color = "green"),
                        valueBoxOutput("efficiencybuildes")
                    ),
                    # Row --
                    fluidRow(
                        box(title = "New design",
                            solidHeader = T,
                            width = 4,
                            collapsible = T,
                            div(DT::DTOutput("buildes"), style = "font-size: 120%;")
                        ),
                        box(title = "New sensitivity", solidHeader = T,
                            width = 8, collapsible = T,
                            plotlyOutput("sens_build")
                        )
                    )
            ),
            tabItem(tabName = "enhacedes",
                    # Row --
                    fluidRow(
                        valueBox(value = "Design from the built design"#tags$p("Optimum Design", style = "font-size: 75%;")
                                 , "Chosen points", icon = icon("clipboard-list"), color = "aqua"),
                        valueBox(value = "Sensitivity Function"#tags$p("Optimum Design", style = "font-size: 75%;")
                                 , "For the enhanced design", icon = icon("bar-chart-o"), color = "green"),
                        valueBoxOutput("efficiencyenhdes")
                    ),
                    # Row --
                    fluidRow(
                        box(title = "New design",
                            solidHeader = T,
                            width = 4,
                            collapsible = T,
                            div(DT::DTOutput("enhdes"), style = "font-size: 120%;")
                        ),
                        box(title = "New sensitivity", solidHeader = T,
                            width = 8, collapsible = T,
                            plotlyOutput("sens_enh")
                        )
                    )
                    # ,
                    # Row --
                    # fluidRow(
                    #     valueBox(value = "Feasible region"#tags$p("Optimum Design", style = "font-size: 75%;")
                    #              , "To add points to the built design", icon = icon("bar-chart-o"), color = "yellow"),
                    #     box(title = "Feasible region", solidHeader = T,
                    #         width = 8, collapsible = T,
                    #         plotlyOutput("feas_regions_enh")
                    #     )
                    # )
            ),
            # tabItem(tabName = "compare",
            #         fluidRow(
            #             # Row --
            #             fluidRow(
            #                 valueBoxOutput("industry_des"),
            #                 valueBoxOutput("industry_sens"),
            #                 valueBoxOutput("efficiency_ind")
            #             ),
            #             # Row --
            #             fluidRow(
            #                 box(title = "Industry design",
            #                     solidHeader = T,
            #                     width = 4,
            #                     collapsible = T,
            #                     div(DT::DTOutput("ind_df"), style = "font-size: 120%;")
            #                 ),
            #                 box(title = "Sensitivity Function", solidHeader = T,
            #                     width = 8, collapsible = T,
            #                     plotlyOutput("sens_ind")
            #                 )
            #             )
            #         )
            # ),
            # tabItem(tabName = "help", fluidPage(withMathJax(includeMarkdown("help.Rmd")))),
            tabItem(tabName = "contact", fluidPage(withMathJax(includeMarkdown("contact.Rmd"))))
        )
    )
)

# Define server logic required to draw a histogram
server <- function(input, output, session) {

    # make reactive to store points
    ing_df <- shiny::reactiveValues()

    # Diseño en blanco que el usuario va construyendo
    ing_df$design <- data.frame("Point" = numeric(),
                                "Weight" = numeric())

    # dataset "standard", diseño añadiendo al óptimo los puntos de cortes con igula peso
    ing_df$design_std <- data.frame("Point" = numeric(),
                                "Weight" = numeric())

    # Punto a añadir al óptimo
    ing_df$points_to_add <- data.frame("Point" = numeric(),
                                      "Weight" = numeric())

    # Data frame con el óptimo valiendo (1-alpha)
    ing_df$weighted_opt <- data.frame("Point" = numeric(),
                                       "Weight" = numeric())

    # Optimum design
    ing_df$opt_design <- data.frame("Point" = numeric(),
                                    "Weight" = numeric())

    # Diseño final al partir del óptimo y añadir puntos
    ing_df$design_w_opt <- data.frame("Point" = numeric(),
                                    "Weight" = numeric())

    # Diseño industrial
    ing_df$ind_des <- data.frame("Point" = numeric(),
                                      "Weight" = numeric())

    # Punto a añadir al construido
    ing_df$points_to_add_build <- data.frame("Point" = numeric(),
                                       "Weight" = numeric())

    # Diseño final al partir del construido valiendo (1-alpha)
    ing_df$weighted_build <- data.frame("Point" = numeric(),
                                     "Weight" = numeric())

    # Diseño final al partir del construido y añadir puntos
    ing_df$enhaced_des <- data.frame("Point" = numeric(),
                                      "Weight" = numeric())

    # Se ha calculado el diseño óptimo?
    ing_df$calculated_design <- F

    # Se ha calculado un diseño propio?
    ing_df$built_design <- F

    # Límites de eficiencia en funcion de delta
    ing_df$eff_limit <- c(0, 1)

    # Límites de eficiencia en funcion de delta para enhace design
    ing_df$eff_limit_enh <- c(0, 1)

    # Regiones factibles añadir óptimo
    ing_df$fac_reg_opt <- vector("numeric")

    # Regiones factibles añadir custom
    ing_df$fac_reg_build <- vector("numeric")


    # Update weighted_opt
    ing_df$weighted_opt <- reactive({
        if(nrow(ing_df$opt_design > 0)){
            data.frame("Point" = ing_df$opt_design[["Point"]], "Weight" = ing_df$opt_design[["Weight"]]*(1-input$delta))
        }
        else{
            data.frame("Point" = numeric(),
                       "Weight" = numeric())
        }
    })

    # Update weighted_build
    weighted_build <- reactive({
        if(nrow(ing_df$design > 0)){
            data.frame("Point" = ing_df$design[["Point"]], "Weight" = ing_df$design[["Weight"]]*(1-input$deltaenh))
        }
        else{
            data.frame("Point" = numeric(),
                       "Weight" = numeric())
        }
    })


    # Update the input dataframe
    observeEvent(input$remove, {
        ing_df$design<-ing_df$design[-(nrow(ing_df$design)),]
        ing_df$fac_reg_build <- vector("numeric")
        ing_df$points_to_add_build <- data.frame("Point" = numeric(), "Weight" = numeric())
    })

    observeEvent(input$remove_all, {
        ing_df$design <- data.frame("Point" = numeric(), "Weight" = numeric())
        ing_df$fac_reg_build <- vector("numeric")
        ing_df$points_to_add_build <- data.frame("Point" = numeric(), "Weight" = numeric())
    })

    observeEvent(input$add, {
        if(input$weights_add <= 0){
            shinyalert("The weight must be greater than 0", type = "error")
        }
        else if((input$model %in% c('Antoine Equation', 'Quadratic') & between(input$points_add, input$design_sp[[1]], input$design_sp[[2]])) ||
                (input$model == 'Quadratic Heteroscedastic' & between(input$points_add, 0, input$design_sp_CH)) ||
                (input$model == 'Michaelis-Menten' & between(input$points_add, 0, input$design_sp_MM))){
            ing_df$design[nrow(ing_df$design) + 1,] <- c(input$points_add,
                                                         input$weights_add
            )

            # update choices
            updateNumericInput(session, 'weights_add', '2. Weights', 0)
            updateNumericInput(session, 'points_add', '1. Points', value = -101)
        }
        else{
            shinyalert("The point is out of the space of the design", type = "error")
        }
    })


    # Update the input dataframe for the design with (1-alpha) opt
    observeEvent(input$remove_opt, {
        ing_df$points_to_add<-ing_df$points_to_add[-(nrow(ing_df$points_to_add)),]
        ing_df$design_w_opt <- bind_rows(ing_df$weighted_opt(), ing_df$points_to_add)
    })

    observeEvent(input$remove_opt_all, {
        ing_df$points_to_add<-data.frame("Point" = numeric(), "Weight" = numeric())
        ing_df$design_w_opt <- bind_rows(ing_df$weighted_opt(), ing_df$points_to_add)
    })

    observeEvent(input$add_opt, {
        if(length(ing_df$fac_reg_opt) == 0){
            shinyalert("Calculate the Region of Candidate Points", type = "error")
        }
        else{
            in_region <- F
            for(i in 1:(length(ing_df$fac_reg_opt)/2)){
                if(between(input$points_add_opt, ing_df$fac_reg_opt[[2*i-1]], ing_df$fac_reg_opt[[2*i]])){
                    in_region <- T
                    break
                }
            }
            if(input$weights_add_opt <= 0){
                shinyalert("The weight must be greater than 0", type = "error")
            }
            else if(in_region){
                ing_df$points_to_add[nrow(ing_df$points_to_add) + 1,] <- c(input$points_add_opt,
                                                             input$weights_add_opt
                )
                ing_df$design_w_opt <- bind_rows(ing_df$weighted_opt(), ing_df$points_to_add)

                # update choices
                updateNumericInput(session, 'weights_add_opt', '2. Weights', 0)
                updateNumericInput(session, 'points_add_opt', '1. Points', value = -101)
            }
            else{
                shinyalert("The point is out of the allowed region", type = "error")
            }
        }
    })

    # Update the input dataframe for enhaced design
    observeEvent(input$remove_enh, {
        ing_df$points_to_add_build<-ing_df$points_to_add_build[-(nrow(ing_df$points_to_add_build)),]
        ing_df$enhaced_des <- bind_rows(weighted_build(), ing_df$points_to_add_build)
    })

    observeEvent(input$remove_enh_all, {
        ing_df$points_to_add_build <- data.frame("Point" = numeric(), "Weight" = numeric())
        ing_df$enhaced_des <- bind_rows(weighted_build(), ing_df$points_to_add_build)
    })

    observeEvent(input$add_enh, {
        if(length(ing_df$fac_reg_build) == 0){
            shinyalert("Calculate the region of candidate points", type = "error")
        }
        else{
            in_region <- F
            for(i in 1:(length(ing_df$fac_reg_build)/2)){
                if(between(input$points_add_enh, ing_df$fac_reg_build[[2*i-1]], ing_df$fac_reg_build[[2*i]])){
                    in_region <- T
                    break
                }
            }
            if(input$weights_add_enh <= 0){
                shinyalert("The weight must be greater than 0", type = "error")
            }
            else if(in_region){
                ing_df$points_to_add_build[nrow(ing_df$points_to_add_build) + 1,] <- c(input$points_add_enh,
                                                                                       input$weights_add_enh
                )
                ing_df$enhaced_des <- bind_rows(weighted_build(), ing_df$points_to_add_build)

                # update choices
                updateNumericInput(session, 'weights_add_enh', '2. Weights', 0)
                updateNumericInput(session, 'points_add_enh', '1. Points', value = -101)
            }
            else{
                shinyalert("The point is out of the allowed region", type = "error")
            }
        }
    })

    # Delta slider
    observeEvent(input$delta, {
        if(ing_df$calculated_design == TRUE){
            if(identical(input$model, "Antoine Equation")){
                sens_opt <- dsensAntoine(input$a_par, input$b_par, input$c_par, dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$opt_design))
                sens_min <- findminval(sens_opt, input$design_sp[[1]], input$design_sp[[2]], 1000)
                bounds<- delta_bound(input$delta, 3, sens_min)
                ing_df$eff_limit <- c(ceiling(bounds[1]*100)/100,floor(bounds[2]*100)/100)
            }
            else if(identical(input$model, "Quadratic")){
                sens_opt <- dsensCuad(dmatrixCuad(ing_df$opt_design))
                sens_min <- findminval(sens_opt, input$design_sp[[1]], input$design_sp[[2]], 1000)
                bounds<- delta_bound(input$delta, 3, sens_min)
                ing_df$eff_limit <- c(ceiling(bounds[1]*100)/100,floor(bounds[2]*100)/100)
            }
            else if(identical(input$model, "Michaelis-Menten")){
                sens_opt <- dsensMM(input$k_par, input$v_par, dmatrixMM(input$k_par, input$v_par, ing_df$opt_design))
                sens_min <- findminval(sens_opt, 0, input$design_sp_MM, 1000)
                bounds<- delta_bound(input$delta, 2, sens_min)
                ing_df$eff_limit <- c(ceiling(bounds[1]*100)/100,floor(bounds[2]*100)/100)
            }
            else if(identical(input$model, "Quadratic Heteroscedastic")){
                sens_opt <- dsensCuadHet(input$order, dmatrixCuadHet(input$order, ing_df$opt_design))
                sens_min <- findminval(sens_opt, 0, input$design_sp_CH, 1000)
                bounds<- delta_bound(input$delta, 3, sens_min)
                ing_df$eff_limit <- c(ceiling(bounds[1]*100)/100,floor(bounds[2]*100)/100)
            }
        }
    })

    # Delta enh slider
    observeEvent(input$deltaenh, {
        if(ing_df$built_design == FALSE)
        {

        } else if((nrow(ing_df$design) < 3 && !identical(input$model, "Michaelis-Menten")) || nrow(ing_df$design) < 2 || !is.numeric(ing_df$effbuild)){
            shinyalert("Build a regular design and evaluate it!", type = "error")
        }
        else{
            if(identical(input$model, "Antoine Equation")){
                sens_des <- dsensAntoine(input$a_par, input$b_par, input$c_par, dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$design))
                sens_min <- findminval(sens_des, input$design_sp[[1]], input$design_sp[[2]], 1000)
                sens_max <- findmaxval(sens_des, input$design_sp[[1]], input$design_sp[[2]], 1000)
                bounds_enh <- delta_bound(input$deltaenh, 3, sens_min, sens_max)#*ing_df$effbuild
                ing_df$eff_limit_enh <- c(ceiling(bounds_enh[1]*100)/100,floor(bounds_enh[2]*100)/100)
            }
            else if(identical(input$model, "Quadratic")){
                sens_des <- dsensCuad(dmatrixCuad(ing_df$design))
                sens_min <- findminval(sens_des, input$design_sp[[1]], input$design_sp[[2]], 1000)
                sens_max <- findmaxval(sens_des, input$design_sp[[1]], input$design_sp[[2]], 1000)
                bounds_enh <- delta_bound(input$deltaenh, 3, sens_min, sens_max)#*ing_df$effbuild
                ing_df$eff_limit_enh <- c(ceiling(bounds_enh[1]*100)/100,floor(bounds_enh[2]*100)/100)
            }
            else if(identical(input$model, "Michaelis-Menten")){
                sens_des <- dsensMM(input$k_par, input$v_par, dmatrixMM(input$k_par, input$v_par, ing_df$design))
                sens_min <- findminval(sens_des, 0, input$design_sp_MM, 1000)
                sens_max <- findmaxval(sens_des, 0, input$design_sp_MM, 1000)
                bounds_enh <- delta_bound(input$deltaenh, 2, sens_min, sens_max)#*ing_df$effbuild
                ing_df$eff_limit_enh <- c(ceiling(bounds_enh[1]*100)/100,floor(bounds_enh[2]*100)/100)
            }
            else if(identical(input$model, "Quadratic Heteroscedastic")){
                sens_des <- dsensCuadHet(input$order, dmatrixCuadHet(input$order, ing_df$design))
                sens_min <- findminval(sens_des, 0, input$design_sp_CH, 1000)
                sens_max <- findmaxval(sens_des, 0, input$design_sp_CH, 1000)
                bounds_enh <- delta_bound(input$deltaenh, 3, sens_min, sens_max)#*ing_df$effbuild
                ing_df$eff_limit_enh <- c(ceiling(bounds_enh[1]*100)/100,floor(bounds_enh[2]*100)/100)
            }
            else{
                shinyalert("Enter a valid model!", type = "error")
            }
        }
    }, ignoreInit = TRUE)

    # Render eff slider
    output$eff <- renderUI({
        sliderInput("deff", "Possible efficiency, delta", min=ing_df$eff_limit[1], max=ing_df$eff_limit[2], value = ing_df$eff_limit[1] + ing_df$eff_limit[2]/2 - ing_df$eff_limit[1]/2)
    })

    # Render effenh slider
    output$effenh <- renderUI({
        sliderInput("deffenh", "Possible relative efficiency", min=ing_df$eff_limit_enh[1], max=ing_df$eff_limit_enh[2], value = ing_df$eff_limit_enh[1] + ing_df$eff_limit_enh[2]/2 - ing_df$eff_limit_enh[1]/2)
    })


    # Build Des
    output$buildes <- DT::renderDataTable({
        datatable(ing_df$design, rownames=F, options = list(paging = FALSE, searching = FALSE)) %>% formatRound(c(1,2), 3)
    })

    # Give des
    output$givedes <- DT::renderDataTable({
        datatable(ing_df$design_std, rownames=F, options = list(paging = FALSE, searching = FALSE)) %>% formatRound(c(1,2), 3)
    })

    # Restricted des
    output$woptdes <- DT::renderDataTable({
        datatable(ing_df$design_w_opt, rownames=F, options = list(paging = FALSE, searching = FALSE)) %>% formatRound(c(1,2), 3)
    })

    # Enhaced des
    output$enhdes <- DT::renderDataTable({
        datatable(ing_df$enhaced_des, rownames=F, options = list(paging = FALSE, searching = FALSE)) %>% formatRound(c(1,2), 3)
    })

    # Industrial des
    output$ind_df <- DT::renderDataTable({
        datatable(ing_df$ind_des, rownames=F, options = list(paging = FALSE, searching = FALSE)) %>% formatRound(c(1,2), 3)
    })

    # Show/hide add points
    output$show_rest <- reactive({
        length(ing_df$fac_reg_opt) != 0
    })

    outputOptions(output, "show_rest", suspendWhenHidden = FALSE)

    # Show/hide add points enh
    output$show_enh <- reactive({
        length(ing_df$fac_reg_build) != 0
    })

    outputOptions(output, "show_enh", suspendWhenHidden = FALSE)



    # Calculate optimal design
    observeEvent(input$calc_opt_design, {
        model <- input$model
        if(identical(model, "Antoine Equation")){

            restart()
            ing_df$opt_design <- doptAntoine(input$a_par, input$b_par, input$c_par, input$design_sp[[1]], input$design_sp[[2]])
            sens_opt <- dsensAntoine(input$a_par, input$b_par, input$c_par, dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$opt_design))
            p <- ggplot() + theme_ipsum()

            x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
            y_val <- purrr::map_dbl(x_val, sens_opt)

            ing_df$sens_opt <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "Temperature (ºC)", y = "")
            ing_df$calculated_design <- T
            ing_df$points_to_add <- data.frame("Point" = numeric(), "Weight" = numeric())
            ing_df$design_w_opt <- bind_rows(ing_df$weighted_opt(), ing_df$points_to_add)
        }
        else if(identical(model, "Quadratic")){

            restart()
            ing_df$opt_design <- doptCuad(input$design_sp[[1]], input$design_sp[[2]])
            sens_opt <- dsensCuad(dmatrixCuad(ing_df$opt_design))
            p <- ggplot() + theme_ipsum()

            x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
            y_val <- purrr::map_dbl(x_val, sens_opt)

            ing_df$sens_opt <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "Points", y = "Value")
            ing_df$calculated_design <- T
            ing_df$points_to_add <- data.frame("Point" = numeric(), "Weight" = numeric())
            ing_df$design_w_opt <- bind_rows(ing_df$weighted_opt(), ing_df$points_to_add)
        }
        else if(identical(model, "Michaelis-Menten")){

            restart()
            ing_df$opt_design <- doptMM(input$k_par, input$v_par, 0, input$design_sp_MM)
            sens_opt <- dsensMM(input$k_par, input$v_par, dmatrixMM(input$k_par, input$v_par, ing_df$opt_design))
            p <- ggplot() + theme_ipsum()

            x_val <- seq(0, input$design_sp_MM, length.out = 10000)
            y_val <- purrr::map_dbl(x_val, sens_opt)

            ing_df$sens_opt <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  2, color = "goldenrod3") + xlim(0, input$design_sp_MM) + labs(x = "Points", y = "Value")
            ing_df$calculated_design <- T
            ing_df$points_to_add <- data.frame("Point" = numeric(), "Weight" = numeric())
            ing_df$design_w_opt <- bind_rows(ing_df$weighted_opt(), ing_df$points_to_add)
        }
        else if(identical(model, "Quadratic Heteroscedastic")){

            restart()
            ing_df$opt_design <- doptCuadHet(input$order)
            sens_opt <- dsensCuadHet(input$order, dmatrixCuadHet(input$order, ing_df$opt_design))
            p <- ggplot() + theme_ipsum()

            x_val <- seq(0, input$design_sp_CH, length.out = 10000)
            y_val <- purrr::map_dbl(x_val, sens_opt)

            ing_df$sens_opt <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(0, input$design_sp_CH) + labs(x = "Points", y = "Value")
            ing_df$calculated_design <- T
            ing_df$points_to_add <- data.frame("Point" = numeric(), "Weight" = numeric())
            ing_df$design_w_opt <- bind_rows(ing_df$weighted_opt(), ing_df$points_to_add)
        }
        else{
            shinyalert("Enter a valid model!", type = "error")
        }
    })

    # Render optimal design dataframe
    output$opt_df <- DT::renderDataTable({
        datatable(ing_df$opt_design, rownames=F, options = list(paging = FALSE, searching = FALSE), filter = 'none') %>% formatRound(c(1,2), 3)
    })


    # Show feasible points for given values
    observeEvent(input$give_points,{
        if(nrow(ing_df$opt_design) < 1){
            shinyalert("You need to calculate the optimal design first", type = "error")
        }
        else {
            if(identical(input$model, "Antoine Equation")){
                sens_opt <- dsensAntoine(input$a_par, input$b_par, input$c_par, dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$opt_design))
                sens_min <- findminval(sens_opt, input$design_sp[[1]], input$design_sp[[2]], 1000)
                bounds <- delta_bound(input$delta, 3, sens_min)
                if(ing_df$eff_limit[2] == 1){
                    shinyalert("Choose weight for the new points", type = "error")
                }
                else{
                    # Puntos de corte y valor del corte
                    cross <- sort(crosspoints(input$deff, input$delta, 3, sens_opt, 1000, 10^(-3), input$design_sp[[1]], input$design_sp[[2]]))
                    val <- sens_val_to_add(input$deff, input$delta, 3)
                    # Obtener start y par para tener las regiones
                    start <- getStart(cross, input$design_sp[[1]], input$design_sp[[2]], val, sens_opt)
                    par <- getPar(cross)
                    ing_df$fac_reg_opt <- getCross2(cross, input$design_sp[[1]], input$design_sp[[2]], start, par)

                    x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
                    eff <- function(x){
                        return((1-input$delta)*(1+input$delta*sens_opt(x)/(1-input$delta))^(1/3))
                    }
                    y_val <- purrr::map_dbl(x_val, eff)

                    p <- ggplot() +  geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") +
                        geom_hline(yintercept =  eff(cross[1]), color = "goldenrod3") +
                        xlim(input$design_sp[[1]], input$design_sp[[2]]) +
                        labs(x = "x", y = "Efficiency")+ theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
                                                                                      panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))

                    efficiency <- eff(cross[1])
                    for(i in 1:(length(ing_df$fac_reg_opt)/2)){
                        loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_opt[2*i-1],",xend=",ing_df$fac_reg_opt[2*i],",y=efficiency,yend=efficiency), size = 1.5, color = 'green3')", sep="")
                        p <- p + eval(parse(text=loop_input))
                    }



                    values <- map_dbl(cross, eff)
                    cutoffpoints <- data.frame("x_value" = cross, "eff" = values)

                    p <- p + geom_point(data = cutoffpoints, aes(x = x_value, y = eff), shape = 16, size = 2, color = "firebrick3")

                    p <- p + geom_segment(aes(x = input$design_sp[[1]], xend = input$design_sp[[1]], y = bounds[1], yend = bounds[2]), col = "mediumpurple2", size = 1.5)

                    ing_df$sens_new <- p
                    ing_df$sens_res <- p
                }
            }
            else if(identical(input$model, "Quadratic")){
                sens_opt <- dsensCuad( dmatrixCuad(ing_df$opt_design))
                sens_min <- findminval(sens_opt, input$design_sp[[1]], input$design_sp[[2]], 1000)
                bounds <- delta_bound(input$delta, 3, sens_min)
                if(ing_df$eff_limit[2] == 1){
                    shinyalert("Choose weight for the new points", type = "error")
                }
                else{
                    # Puntos de corte y valor del corte
                    cross <- sort(crosspoints(input$deff, input$delta, 3, sens_opt, 1000, 10^(-3), input$design_sp[[1]], input$design_sp[[2]]))
                    val <- sens_val_to_add(input$deff, input$delta, 3)
                    # Obtener start y par para tener las regiones
                    start <- getStart(cross, input$design_sp[[1]], input$design_sp[[2]], val, sens_opt)
                    par <- getPar(cross)
                    ing_df$fac_reg_opt <- getCross2(cross, input$design_sp[[1]], input$design_sp[[2]], start, par)

                    x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
                    eff <- function(x){
                        return((1-input$delta)*(1+input$delta*sens_opt(x)/(1-input$delta))^(1/3))
                    }
                    y_val <- purrr::map_dbl(x_val, eff)

                    p <- ggplot() +  geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") +
                        geom_hline(yintercept =  eff(cross[1]), color = "goldenrod3") +
                        xlim(input$design_sp[[1]], input$design_sp[[2]]) +
                        labs(x = "x", y = "Efficiency")+ theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
                                                                            panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))

                    efficiency <- eff(cross[1])
                    for(i in 1:(length(ing_df$fac_reg_opt)/2)){
                        loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_opt[2*i-1],",xend=",ing_df$fac_reg_opt[2*i],",y=efficiency,yend=efficiency), size = 1.5, color = 'green3')", sep="")
                        p <- p + eval(parse(text=loop_input))
                    }



                    values <- map_dbl(cross, eff)
                    cutoffpoints <- data.frame("x_value" = cross, "eff" = values)

                    p <- p + geom_point(data = cutoffpoints, aes(x = x_value, y = eff), shape = 16, size = 2, color = "firebrick3")

                    p <- p + geom_segment(aes(x = input$design_sp[[1]], xend = input$design_sp[[1]], y = bounds[1], yend = bounds[2]), col = "mediumpurple2", size = 1.5)

                    ing_df$sens_new <- p
                    ing_df$sens_res <- p
                }
            } else if(identical(input$model, "Michaelis-Menten")){
                sens_opt <- dsensMM(input$k_par, input$v_par, dmatrixMM(input$k_par, input$v_par, ing_df$opt_design))
                sens_min <- findminval(sens_opt, 0, input$design_sp_MM, 1000)
                bounds <- delta_bound(input$delta, 2, sens_min)
                if(ing_df$eff_limit[2] == 1){
                    shinyalert("Choose weight for the new points", type = "error")
                }
                else{
                    # Puntos de corte y valor del corte
                    cross <- sort(crosspoints(input$deff, input$delta, 2, sens_opt, 1000, 10^(-3), 0, input$design_sp_MM))
                    val <- sens_val_to_add(input$deff, input$delta, 2)
                    # Obtener start y par para tener las regiones
                    start <- getStart(cross, 0, input$design_sp_MM, val, sens_opt)
                    par <- getPar(cross)
                    ing_df$fac_reg_opt <- getCross2(cross, 0, input$design_sp_MM, start, par)

                    x_val <- seq(0, input$design_sp_MM, length.out = 10000)
                    eff <- function(x){
                        return((1-input$delta)*(1+input$delta*sens_opt(x)/(1-input$delta))^(1/2))
                    }
                    y_val <- purrr::map_dbl(x_val, eff)

                    p <- ggplot() +  geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") +
                        geom_hline(yintercept =  eff(cross[1]), color = "goldenrod3") +
                        xlim(0, input$design_sp_MM) +
                        labs(x = "x", y = "Efficiency")+ theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
                                                                            panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))

                    efficiency <- eff(cross[1])
                    for(i in 1:(length(ing_df$fac_reg_opt)/2)){
                        loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_opt[2*i-1],",xend=",ing_df$fac_reg_opt[2*i],",y=efficiency,yend=efficiency), size = 1.5, color = 'green3')", sep="")
                        p <- p + eval(parse(text=loop_input))
                    }



                    values <- map_dbl(cross, eff)
                    cutoffpoints <- data.frame("x_value" = cross, "eff" = values)

                    p <- p + geom_point(data = cutoffpoints, aes(x = x_value, y = eff), shape = 16, size = 2, color = "firebrick3")

                    p <- p + geom_segment(aes(x = 0, xend = 0, y = bounds[1], yend = bounds[2]), col = "mediumpurple2", size = 1.5)

                    ing_df$sens_new <- p
                    ing_df$sens_res <- p
                }
            } else if(identical(input$model, "Quadratic Heteroscedastic")){
                sens_opt <- dsensCuadHet(input$order, dmatrixCuadHet(input$order, ing_df$opt_design))
                sens_min <- findminval(sens_opt, 0, input$design_sp_CH, 1000)
                bounds <- delta_bound(input$delta, 3, sens_min)
                if(ing_df$eff_limit[2] == 1){
                    shinyalert("Choose weight for the new points", type = "error")
                }
                else{
                    # Puntos de corte y valor del corte
                    cross <- sort(crosspoints(input$deff, input$delta, 3, sens_opt, 1000, 10^(-3), 0, input$design_sp_CH))
                    val <- sens_val_to_add(input$deff, input$delta, 3)
                    # Obtener start y par para tener las regiones
                    start <- getStart(cross, 0, input$design_sp_CH, val, sens_opt)
                    par <- getPar(cross)
                    ing_df$fac_reg_opt <- getCross2(cross, 0, input$design_sp_CH, start, par)

                    x_val <- seq(0, input$design_sp_CH, length.out = 10000)
                    eff <- function(x){
                        return((1-input$delta)*(1+input$delta*sens_opt(x)/(1-input$delta))^(1/3))
                    }
                    y_val <- purrr::map_dbl(x_val, eff)

                    p <- ggplot() +  geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") +
                        geom_hline(yintercept =  eff(cross[1]), color = "goldenrod3") +
                        xlim(0, input$design_sp_CH) +
                        labs(x = "x", y = "Efficiency")+ theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
                                                                            panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))

                    efficiency <- eff(cross[1])
                    for(i in 1:(length(ing_df$fac_reg_opt)/2)){
                        loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_opt[2*i-1],",xend=",ing_df$fac_reg_opt[2*i],",y=efficiency,yend=efficiency), size = 1.5, color = 'green3')", sep="")
                        p <- p + eval(parse(text=loop_input))
                    }



                    values <- map_dbl(cross, eff)
                    cutoffpoints <- data.frame("x_value" = cross, "eff" = values)

                    p <- p + geom_point(data = cutoffpoints, aes(x = x_value, y = eff), shape = 16, size = 2, color = "firebrick3")

                    p <- p + geom_segment(aes(x = 0, xend = 0, y = bounds[1], yend = bounds[2]), col = "mediumpurple2", size = 1.5)

                    ing_df$sens_new <- p
                    ing_df$sens_res <- p
                }
            }
            else{
                shinyalert("Enter a valid model!", type = "error")
            }
        }
    })


    # Show feasible points for build design
    observeEvent(input$enhace_points,{
        if(nrow(ing_df$opt_design) < 1){
            shinyalert("You need to calculate the optimal design first", type = "error")
        }
        if((nrow(ing_df$design) < 3 && !identical(input$model, "Michaelis-Menten")) || nrow(ing_df$design) < 2 || !is.numeric(ing_df$effbuild)){
            shinyalert("You need to build a design and evaluate it", type = "error")
        }
        else {
            model <- input$model
            if(identical(model, "Antoine Equation")){
                sens_des <- dsensAntoine(input$a_par, input$b_par, input$c_par, dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$design))
                sens_min <- findminval(sens_des, input$design_sp[[1]], input$design_sp[[2]], 1000)
                sens_max <- findmaxval(sens_des, input$design_sp[[1]], input$design_sp[[2]], 1000)
                bounds <- delta_bound(input$deltaenh, 3, sens_min, sens_max)
                if(ing_df$eff_limit_enh[2] == 1){
                    shinyalert("Choose weight for the new points", type = "error")
                }
                else{
                    # Puntos de corte y valor del corte
                    cross <- sort(crosspoints(floor(input$deffenh#/ing_df$effbuild
                                                    *10000)/10000, input$deltaenh, 3, sens_des, 1000, 10^(-3), input$design_sp[[1]], input$design_sp[[2]]))
                    val <- sens_val_to_add(input$deffenh#/ing_df$effbuild
                                           , input$deltaenh, 3)
                    # Obtener start y par para tener las regiones
                    start <- getStart(cross, input$design_sp[[1]], input$design_sp[[2]], val, sens_des)
                    par <- getPar(cross)
                    ing_df$fac_reg_build <- getCross2(cross, input$design_sp[[1]], input$design_sp[[2]], start, par)

                    x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
                    eff <- function(x){
                        return((1-input$deltaenh)*(1+input$deltaenh*sens_des(x)/(1-input$deltaenh))^(1/3))
                    }
                    y_val <- purrr::map_dbl(x_val, eff)

                    p <- ggplot() +  geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") +
                        geom_hline(yintercept =  eff(cross[1]), color = "goldenrod3") +
                        xlim(input$design_sp[[1]], input$design_sp[[2]]) +
                        labs(x = "x", y = "Efficiency")+ theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
                                                                            panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))

                    efficiency <- eff(cross[1])
                    for(i in 1:(length(ing_df$fac_reg_build)/2)){
                        loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_build[2*i-1],",xend=",ing_df$fac_reg_build[2*i],",y=efficiency,yend=efficiency), size = 1.5, color = 'green3')", sep="")
                        p <- p + eval(parse(text=loop_input))
                    }



                    values <- map_dbl(cross, eff)
                    cutoffpoints <- data.frame("x_value" = cross, "eff" = values)

                    p <- p + geom_point(data = cutoffpoints, aes(x = x_value, y = eff), shape = 16, size = 2, color = "firebrick3")

                    p <- p + geom_segment(aes(x = input$design_sp[[1]], xend = input$design_sp[[1]], y = bounds[1], yend = bounds[2]), col = "mediumpurple2", size = 1.5)

                    ing_df$sens_enh <- p
                }
            }
            else if(identical(model, "Quadratic")){
                sens_des <- dsensCuad( dmatrixCuad(ing_df$design))
                sens_min <- findminval(sens_des, input$design_sp[[1]], input$design_sp[[2]], 1000)
                sens_max <- findmaxval(sens_des, input$design_sp[[1]], input$design_sp[[2]], 1000)
                bounds <- delta_bound(input$deltaenh, 3, sens_min, sens_max)
                if(ing_df$eff_limit_enh[2] == 1){
                    shinyalert("Choose weight for the new points", type = "error")
                }
                else{
                    # Puntos de corte y valor del corte
                    cross <- sort(crosspoints(floor(input$deffenh#/ing_df$effbuild
                                                    *10000)/10000, input$deltaenh, 3, sens_des, 1000, 10^(-3), input$design_sp[[1]], input$design_sp[[2]]))
                    val <- sens_val_to_add(input$deffenh#/ing_df$effbuild
                                           , input$deltaenh, 3)
                    # Obtener start y par para tener las regiones
                    start <- getStart(cross, input$design_sp[[1]], input$design_sp[[2]], val, sens_des)
                    par <- getPar(cross)

                    ing_df$fac_reg_build <- getCross2(cross, input$design_sp[[1]], input$design_sp[[2]], start, par)

                    x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
                    eff <- function(x){
                        return((1-input$deltaenh)*(1+input$deltaenh*sens_des(x)/(1-input$deltaenh))^(1/3))
                    }
                    y_val <- purrr::map_dbl(x_val, eff)

                    p <- ggplot() +  geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") +
                        geom_hline(yintercept =  eff(cross[1]), color = "goldenrod3") +
                        xlim(input$design_sp[[1]], input$design_sp[[2]]) +
                        labs(x = "x", y = "Efficiency")+ theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
                                                                            panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))

                    efficiency <- eff(cross[1])
                    for(i in 1:(length(ing_df$fac_reg_build)/2)){
                        loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_build[2*i-1],",xend=",ing_df$fac_reg_build[2*i],",y=efficiency,yend=efficiency), size = 1.5, color = 'green3')", sep="")
                        p <- p + eval(parse(text=loop_input))
                    }



                    values <- map_dbl(cross, eff)
                    cutoffpoints <- data.frame("x_value" = cross, "eff" = values)

                    p <- p + geom_point(data = cutoffpoints, aes(x = x_value, y = eff), shape = 16, size = 2, color = "firebrick3")

                    p <- p + geom_segment(aes(x = input$design_sp[[1]], xend = input$design_sp[[1]], y = bounds[1], yend = bounds[2]), col = "mediumpurple2", size = 1.5)

                    ing_df$sens_enh <- p
                }
            } else if(identical(model, "Michaelis-Menten")){
                sens_des <- dsensMM(input$k_par, input$v_par, dmatrixMM(input$k_par, input$v_par, ing_df$design))
                sens_min <- findminval(sens_des, 0, input$design_sp_MM, 1000)
                sens_max <- findmaxval(sens_des, 0, input$design_sp_MM, 1000)
                bounds <- delta_bound(input$deltaenh, 2, sens_min, sens_max)
                if(ing_df$eff_limit_enh[2] == 1){
                    shinyalert("Choose weight for the new points", type = "error")
                }
                else{
                    # Puntos de corte y valor del corte
                    cross <- sort(crosspoints(floor(input$deffenh#/ing_df$effbuild
                                                    *10000)/10000, input$deltaenh, 2, sens_des, 1000, 10^(-3), 0, input$design_sp_MM))
                    val <- sens_val_to_add(input$deffenh#/ing_df$effbuild
                                           , input$deltaenh, 2)
                    # Obtener start y par para tener las regiones
                    start <- getStart(cross, 0, input$design_sp_MM, val, sens_des)
                    par <- getPar(cross)
                    ing_df$fac_reg_build <- getCross2(cross, 0, input$design_sp_MM, start, par)

                    x_val <- seq(0, input$design_sp_MM, length.out = 10000)
                    eff <- function(x){
                        return((1-input$deltaenh)*(1+input$deltaenh*sens_des(x)/(1-input$deltaenh))^(1/2))
                    }
                    y_val <- purrr::map_dbl(x_val, eff)

                    p <- ggplot() +  geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") +
                        geom_hline(yintercept =  eff(cross[1]), color = "goldenrod3") +
                        xlim(0, input$design_sp_MM) +
                        labs(x = "x", y = "Efficiency")+ theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
                                                                            panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))

                    efficiency <- eff(cross[1])
                    for(i in 1:(length(ing_df$fac_reg_build)/2)){
                        loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_build[2*i-1],",xend=",ing_df$fac_reg_build[2*i],",y=efficiency,yend=efficiency), size = 1.5, color = 'green3')", sep="")
                        p <- p + eval(parse(text=loop_input))
                    }



                    values <- map_dbl(cross, eff)
                    cutoffpoints <- data.frame("x_value" = cross, "eff" = values)

                    p <- p + geom_point(data = cutoffpoints, aes(x = x_value, y = eff), shape = 16, size = 2, color = "firebrick3")

                    p <- p + geom_segment(aes(x = 0, xend = 0, y = bounds[1], yend = bounds[2]), col = "mediumpurple2", size = 1.5)

                    ing_df$sens_enh <- p
                }
            } else if(identical(model, "Quadratic Heteroscedastic")){
                sens_des <- dsensCuadHet(input$order, dmatrixCuadHet(input$order, ing_df$design))
                sens_min <- findminval(sens_des, 0, input$design_sp_CH, 1000)
                sens_max <- findmaxval(sens_des, 0, input$design_sp_CH, 1000)
                bounds <- delta_bound(input$deltaenh, 3, sens_min, sens_max)
                if(ing_df$eff_limit_enh[2] == 1){
                    shinyalert("Choose weight for the new points", type = "error")
                }
                else{
                    # Puntos de corte y valor del corte
                    cross <- sort(crosspoints(floor(input$deffenh#/ing_df$effbuild
                                                    *10000)/10000, input$deltaenh, 3, sens_des, 1000, 10^(-3), 0, input$design_sp_CH))
                    val <- sens_val_to_add(input$deffenh#/ing_df$effbuild
                                           , input$deltaenh, 3)
                    # Obtener start y par para tener las regiones
                    start <- getStart(cross, 0, input$design_sp_CH, val, sens_des)
                    par <- getPar(cross)

                    ing_df$fac_reg_build <- getCross2(cross, 0, input$design_sp_CH, start, par)

                    x_val <- seq(0, input$design_sp_CH, length.out = 10000)
                    eff <- function(x){
                        return((1-input$deltaenh)*(1+input$deltaenh*sens_des(x)/(1-input$deltaenh))^(1/3))
                    }
                    y_val <- purrr::map_dbl(x_val, eff)

                    p <- ggplot() +  geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") +
                        geom_hline(yintercept =  eff(cross[1]), color = "goldenrod3") +
                        xlim(0, input$design_sp_CH) +
                        labs(x = "x", y = "Efficiency")+ theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
                                                                            panel.grid.minor = element_blank(), axis.line = element_line(colour = "black"))

                    efficiency <- eff(cross[1])
                    for(i in 1:(length(ing_df$fac_reg_build)/2)){
                        loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_build[2*i-1],",xend=",ing_df$fac_reg_build[2*i],",y=efficiency,yend=efficiency), size = 1.5, color = 'green3')", sep="")
                        p <- p + eval(parse(text=loop_input))
                    }



                    values <- map_dbl(cross, eff)
                    cutoffpoints <- data.frame("x_value" = cross, "eff" = values)

                    p <- p + geom_point(data = cutoffpoints, aes(x = x_value, y = eff), shape = 16, size = 2, color = "firebrick3")

                    p <- p + geom_segment(aes(x = 0, xend = 0, y = bounds[1], yend = bounds[2]), col = "mediumpurple2", size = 1.5)

                    ing_df$sens_enh <- p
                }
            }
            else{
                shinyalert("Enter a valid model!", type = "error")
            }
        }
    })



    # Calculate the design with the given points and calculate its efficiency and sensitivity function
    observeEvent(input$stand_design, {
        if(input$delta == 0){
            shinyalert("Choose weight for the new points", type = "error")
        }
        else{
            if(identical(input$model, "Antoine Equation")) {
                sens_opt <- dsensAntoine(input$a_par, input$b_par, input$c_par, dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$opt_design))
                sens_min <- findminval(sens_opt, input$design_sp[[1]], input$design_sp[[2]], 1000)
                bounds <- delta_bound(input$delta, 3, sens_min)
                if(input$deff < bounds[1] || input$deff > bounds[2]){
                    shinyalert("The value of the efficiency is not feasible", type = "error")
                }
                else{
                    # Puntos de corte y valor del corte
                    cross <- sort(crosspoints(input$deff, input$delta, 3, sens_opt, 1000, 10^(-3), input$design_sp[[1]], input$design_sp[[2]]))
                    val <- sens_val_to_add(input$deff, input$delta, 3)

                    # Obtener start y par para tener las regiones
                    start <- getStart(cross, input$design_sp[[1]], input$design_sp[[2]], val, sens_opt)
                    par <- getPar(cross)
                    ing_df$fac_reg_opt <- getCross2(cross, input$design_sp[[1]], input$design_sp[[2]], start, par)

                    ing_df$design_std <- add_points(cross, input$delta, ing_df$opt_design)
                    ing_df$eff <- deff(dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$design_std), dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$opt_design), 3)

                    # Obtener nueva función de sensibilidad, diseño y plot
                    sens_new <- dsensAntoine(input$a_par, input$b_par, input$c_par, dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$design_std))

                    new_des_data <- data.frame("Point" = ing_df$design_std[["Point"]], "Weight" = ing_df$design_std[["Weight"]], "Value" = rep(0, length(ing_df$design_std[["Point"]])))
                    p <- ggplot() + theme_ipsum()

                    x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
                    y_val <- purrr::map_dbl(x_val, sens_new)

                    ing_df$sens_new <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "Temperature (ºC)", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))

                    # Loop para pintar las regiones factibles
                    for(i in 1:(length(ing_df$fac_reg_opt)/2)){
                        loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_opt[2*i-1],",xend=",ing_df$fac_reg_opt[2*i],",y=0,yend=0), color = 'green3')", sep="")
                        ing_df$sens_new <- ing_df$sens_new + eval(parse(text=loop_input))
                    }
                }
            }
            else if (identical(input$model, "Quadratic")){
                sens_opt <- dsensCuad(dmatrixCuad(ing_df$opt_design))
                sens_min <- findminval(sens_opt, input$design_sp[[1]], input$design_sp[[2]], 1000)
                bounds <- delta_bound(input$delta, 3, sens_min)
                if(input$deff < bounds[1] || input$deff > bounds[2]){
                    shinyalert("The value of the efficiency is not feasible", type = "error")
                }
                else{
                    # Puntos de corte y valor del corte
                    cross <- crosspoints(input$deff, input$delta, 3, sens_opt, 1000, 10^(-3), input$design_sp[[1]], input$design_sp[[2]])
                    val <- sens_val_to_add(input$deff, input$delta, 3)

                    # Obtener start y par para tener las regiones
                    start <- getStart(cross, input$design_sp[[1]], input$design_sp[[2]], val, sens_opt)
                    par <- getPar(cross)
                    ing_df$fac_reg_opt <- getCross2(cross, input$design_sp[[1]], input$design_sp[[2]], start, par)


                    ing_df$design_std <- add_points(cross, input$delta, ing_df$opt_design)
                    ing_df$eff <- deff(dmatrixCuad(ing_df$design_std), dmatrixCuad(ing_df$opt_design), 3)

                    # Obtener nueva función de sensibilidad, diseño y plot
                    sens_new <- dsensCuad(dmatrixCuad(ing_df$design_std))

                    new_des_data <- data.frame("Point" = ing_df$design_std[["Point"]], "Weight" = ing_df$design_std[["Weight"]], "Value" = rep(0, length(ing_df$design_std[["Point"]])))

                    p <- ggplot() + theme_ipsum()

                    x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
                    y_val <- purrr::map_dbl(x_val, sens_new)

                    ing_df$sens_new <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "X", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))

                    # Loop para pintar las regiones factibles
                    for(i in 1:(length(ing_df$fac_reg_opt)/2)){
                        loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_opt[2*i-1],",xend=",ing_df$fac_reg_opt[2*i],",y=0,yend=0), color = 'green3')", sep="")
                        ing_df$sens_new <- ing_df$sens_new + eval(parse(text=loop_input))
                    }
                }
            } else if(identical(input$model, "Michaelis-Menten")) {
                sens_opt <- dsensMM(input$k_par, input$v_par, dmatrixMM(input$k_par, input$v_par, ing_df$opt_design))
                sens_min <- findminval(sens_opt, 0, input$design_sp_MM, 1000)
                bounds <- delta_bound(input$delta, 2, sens_min)
                if(input$deff < bounds[1] || input$deff > bounds[2]){
                    shinyalert("The value of the efficiency is not feasible", type = "error")
                }
                else{
                    # Puntos de corte y valor del corte
                    cross <- sort(crosspoints(input$deff, input$delta, 2, sens_opt, 1000, 10^(-3), 0, input$design_sp_MM))
                    val <- sens_val_to_add(input$deff, input$delta, 2)

                    # Obtener start y par para tener las regiones
                    start <- getStart(cross, 0, input$design_sp_MM, val, sens_opt)
                    par <- getPar(cross)
                    ing_df$fac_reg_opt <- getCross2(cross, 0, input$design_sp_MM, start, par)

                    ing_df$design_std <- add_points(cross, input$delta, ing_df$opt_design)
                    ing_df$eff <- deff(dmatrixMM(input$k_par, input$v_par, ing_df$design_std), dmatrixMM(input$k_par, input$v_par, ing_df$opt_design), 2)

                    # Obtener nueva función de sensibilidad, diseño y plot
                    sens_new <- dsensMM(input$k_par, input$v_par, dmatrixMM(input$k_par, input$v_par, ing_df$design_std))

                    new_des_data <- data.frame("Point" = ing_df$design_std[["Point"]], "Weight" = ing_df$design_std[["Weight"]], "Value" = rep(0, length(ing_df$design_std[["Point"]])))
                    p <- ggplot() + theme_ipsum()

                    x_val <- seq(0, input$design_sp_MM, length.out = 10000)
                    y_val <- purrr::map_dbl(x_val, sens_new)

                    ing_df$sens_new <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  2, color = "goldenrod3") + xlim(0, input$design_sp_MM) + labs(x = "X", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))

                    # Loop para pintar las regiones factibles
                    for(i in 1:(length(ing_df$fac_reg_opt)/2)){
                        loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_opt[2*i-1],",xend=",ing_df$fac_reg_opt[2*i],",y=0,yend=0), color = 'green3')", sep="")
                        ing_df$sens_new <- ing_df$sens_new + eval(parse(text=loop_input))
                    }
                }
            } else if (identical(input$model, "Quadratic Heteroscedastic")){
                sens_opt <- dsensCuadHet(input$order, dmatrixCuadHet(input$order, ing_df$opt_design))
                sens_min <- findminval(sens_opt, 0, input$design_sp_CH, 1000)
                bounds <- delta_bound(input$delta, 3, sens_min)
                if(input$deff < bounds[1] || input$deff > bounds[2]){
                    shinyalert("The value of the efficiency is not feasible", type = "error")
                }
                else{
                    # Puntos de corte y valor del corte
                    cross <- crosspoints(input$deff, input$delta, 3, sens_opt, 1000, 10^(-3), 0, input$design_sp_CH)
                    val <- sens_val_to_add(input$deff, input$delta, 3)

                    # Obtener start y par para tener las regiones
                    start <- getStart(cross, 0, input$design_sp_CH, val, sens_opt)
                    par <- getPar(cross)
                    ing_df$fac_reg_opt <- getCross2(cross, 0, input$design_sp_CH, start, par)


                    ing_df$design_std <- add_points(cross, input$delta, ing_df$opt_design)
                    ing_df$eff <- deff(dmatrixCuadHet(input$order, ing_df$design_std), dmatrixCuadHet(input$order, ing_df$opt_design), 3)

                    # Obtener nueva función de sensibilidad, diseño y plot
                    sens_new <- dsensCuadHet(input$order, dmatrixCuadHet(input$order, ing_df$design_std))

                    new_des_data <- data.frame("Point" = ing_df$design_std[["Point"]], "Weight" = ing_df$design_std[["Weight"]], "Value" = rep(0, length(ing_df$design_std[["Point"]])))

                    p <- ggplot() + theme_ipsum()

                    x_val <- seq(0, input$design_sp_CH, length.out = 10000)
                    y_val <- purrr::map_dbl(x_val, sens_new)

                    ing_df$sens_new <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(0, input$design_sp_CH) + labs(x = "X", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))

                    # Loop para pintar las regiones factibles
                    for(i in 1:(length(ing_df$fac_reg_opt)/2)){
                        loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_opt[2*i-1],",xend=",ing_df$fac_reg_opt[2*i],",y=0,yend=0), color = 'green3')", sep="")
                        ing_df$sens_new <- ing_df$sens_new + eval(parse(text=loop_input))
                    }
                }
            }
            else{
                shinyalert("Error", "Enter a valid model!", type = "error")
            }
        }
    })

    # Building the design from scratch
    observeEvent(input$both_designs, {
        compute <- TRUE
        length1 <- nrow(ing_df$design)
        ing_df$design <- ing_df$design %>%
            group_by(Point) %>%
            summarise_all(funs(sum))

        if(length1 != nrow(ing_df$design)){
            shinyalert("", "There are repeated points in the design, they have been added together"
                       , type = "warning"
                       , timer = 1500)
        }

        if((length(ing_df$design[["Point"]]) < 3 && !identical(input$model, "Michaelis-Menten")) || length(ing_df$design[["Point"]]) < 2)
        {
            shinyalert("The design is singular!", type = "error")
            compute <- FALSE
        }
        else if(0 %in% ing_df$design[["Weight"]])
        {
            compute <- FALSE
            shinyalert("Error", "There are points without weight!", type = "error")
        }
        else if(((max(ing_df$design[["Point"]]) > input$design_sp[[2]] || min(ing_df$design[["Point"]]) < input$design_sp[[1]]) && !(input$model %in% c("Michaelis-Menten", "Quadratic Heteroscedastic"))) || ((max(ing_df$design[["Point"]]) > input$design_sp_MM || min(ing_df$design[["Point"]]) < 0) && identical(input$model, "Michaelis-Menten")) || ((max(ing_df$design[["Point"]]) > input$design_sp_CH || min(ing_df$design[["Point"]]) < 0) && identical(input$model, "Quadratic Heteroscedastic")))
        {
            compute <- FALSE
            shinyalert("Error", "There are design points outside the design space!", type = "error")
        }
        else if(!near(1, sum(ing_df$design[["Weight"]])))
        {
            shinyalert("", "The weights don't add up to 1, it'll be corrected"
                       # , do you want us to fix that for you?"
                       , type = "warning"
                       # , showCancelButton = TRUE, showConfirmButton = TRUE, callbackR = mycallback
                       , timer = 1500)
            ing_df$design[["Weight"]] <- ing_df$design[["Weight"]]/sum(ing_df$design[["Weight"]])
        }
        if(isTRUE(compute)) {
            if(identical(input$model, "Antoine Equation")) {
                ing_df$built_design <- TRUE
                ing_df$opt_design <- doptAntoine(input$a_par, input$b_par, input$c_par, input$design_sp[[1]], input$design_sp[[2]])
                ing_df$effbuild <- deff(dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$design), dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$opt_design), 3)
                ing_df$enhaced_des <- bind_rows(weighted_build(), ing_df$points_to_add_build)

                sens_opt <- dsensAntoine(input$a_par, input$b_par, input$c_par, dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$opt_design))
                sens_new <- dsensAntoine(input$a_par, input$b_par, input$c_par, dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$design))

                new_des_data <- data.frame("Point" = ing_df$design[["Point"]], "Weight" = ing_df$design[["Weight"]], "Value" = rep(0, length(ing_df$design[["Point"]])))
                p <- ggplot() + theme_ipsum()

                x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
                y_val <- purrr::map_dbl(x_val, sens_new)

                ing_df$sens_build <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3")  + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "Temperature (ºC)", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))
            }
            else if(identical(input$model, "Quadratic")) {
                ing_df$built_design <- TRUE
                ing_df$opt_design <- doptCuad(input$design_sp[[1]], input$design_sp[[2]])
                ing_df$effbuild <- deff(dmatrixCuad(ing_df$design), dmatrixCuad(ing_df$opt_design), 3)
                ing_df$enhaced_des <- bind_rows(weighted_build(), ing_df$points_to_add_build)

                sens_opt <- dsensCuad(dmatrixCuad(ing_df$opt_design))
                sens_new <- dsensCuad(dmatrixCuad(ing_df$design))

                new_des_data <- data.frame("Point" = ing_df$design[["Point"]], "Weight" = ing_df$design[["Weight"]], "Value" = rep(0, length(ing_df$design[["Point"]])))

                p <- ggplot() + theme_ipsum()

                x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
                y_val <- purrr::map_dbl(x_val, sens_new)

                ing_df$sens_build <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "X", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))
            } else if(identical(input$model, "Michaelis-Menten")) {
                ing_df$built_design <- TRUE
                ing_df$opt_design <- doptMM(input$k_par, input$v_par, 0, input$design_sp_MM)
                ing_df$effbuild <- deff(dmatrixMM(input$k_par, input$v_par, ing_df$design), dmatrixMM(input$k_par, input$v_par, ing_df$opt_design), 2)
                ing_df$enhaced_des <- bind_rows(weighted_build(), ing_df$points_to_add_build)

                sens_opt <- dsensMM(input$k_par, input$v_par, dmatrixMM(input$k_par, input$v_par, ing_df$opt_design))
                sens_new <- dsensMM(input$k_par, input$v_par, dmatrixMM(input$k_par, input$v_par, ing_df$design))

                new_des_data <- data.frame("Point" = ing_df$design[["Point"]], "Weight" = ing_df$design[["Weight"]], "Value" = rep(0, length(ing_df$design[["Point"]])))
                p <- ggplot() + theme_ipsum()

                x_val <- seq(0, input$design_sp_MM, length.out = 10000)
                y_val <- purrr::map_dbl(x_val, sens_new)

                ing_df$sens_build <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  2, color = "goldenrod3")  + xlim(0, input$design_sp_MM) + labs(x = "X", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))
            } else if(identical(input$model, "Quadratic Heteroscedastic")) {
                updateSliderInput(session, "design_sp",
                                  min = -100, max = 374, value = c(0, input$design_sp_CH))
                ing_df$built_design <- TRUE
                ing_df$opt_design <- doptCuadHet(input$order)
                ing_df$effbuild <- deff(dmatrixCuadHet(input$order, ing_df$design), dmatrixCuadHet(input$order, ing_df$opt_design), 3)
                ing_df$enhaced_des <- bind_rows(weighted_build(), ing_df$points_to_add_build)

                sens_opt <- dsensCuadHet(input$order, dmatrixCuadHet(input$order, ing_df$opt_design))
                sens_new <- dsensCuadHet(input$order, dmatrixCuadHet(input$order, ing_df$design))

                new_des_data <- data.frame("Point" = ing_df$design[["Point"]], "Weight" = ing_df$design[["Weight"]], "Value" = rep(0, length(ing_df$design[["Point"]])))

                p <- ggplot() + theme_ipsum()

                x_val <- seq(0, input$design_sp_CH, length.out = 10000)
                y_val <- purrr::map_dbl(x_val, sens_new)

                ing_df$sens_build <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(0, input$design_sp_CH) + labs(x = "X", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))
            }
            else{
                shinyalert("Error", "Enter a valid model!", type = "error")
            }
        }

    })


    # With restricted design from the optimum
    observeEvent(input$with_opt, {
        compute <- TRUE
        length1 <- nrow(ing_df$design_w_opt)
        ing_df$design_w_opt <- ing_df$design_w_opt %>%
            group_by(Point) %>%
            summarise_all(funs(sum))

        if(length1 != nrow(ing_df$design_w_opt)){
            shinyalert("", "There are repeated points in the design, they have been added together"
                       , type = "warning"
                       , timer = 1500)
        }

        if((length(ing_df$design_w_opt[["Point"]]) < 4 && !(input$model %in% c("Michaelis-Menten", "Quadratic Heteroscedastic"))) || length(ing_df$design_w_opt[["Point"]]) < 3)
        {
            shinyalert("You need to add points!", type = "error")
            compute <- FALSE
        }
        else if(0 %in% ing_df$design_w_opt[["Weight"]])
        {
            compute <- FALSE
            shinyalert("Error", "There are points without weight!", type = "error")
        }
        else if(((max(ing_df$design_w_opt[["Point"]]) > input$design_sp[[2]] || min(ing_df$design_w_opt[["Point"]]) < input$design_sp[[1]]) && !(input$model %in% c("Michaelis-Menten", "Quadratic Heteroscedastic"))) || ((max(ing_df$design_w_opt[["Point"]]) > input$design_sp_MM || min(ing_df$design_w_opt[["Point"]]) < 0) && identical(input$model, "Michaelis-Menten")) || ((max(ing_df$design_w_opt[["Point"]]) > input$design_sp_CH || min(ing_df$design_w_opt[["Point"]]) < 0) && identical(input$model, "Quadratic Heteroscedastic")))
        {
            compute <- FALSE
            shinyalert("Error", "There are design points outside the design space!", type = "error")
        }
        else if(!near(input$delta, sum(ing_df$points_to_add[["Weight"]])))
        {
            shinyalert("", "The weights don't add up to 1, it'll be corrected"
                       # , do you want us to fix that for you?"
                       , type = "warning"
                       # , showCancelButton = TRUE, showConfirmButton = TRUE, callbackR = mycallback
                       , timer = 1500)
            ing_df$points_to_add[["Weight"]] <- ing_df$points_to_add[["Weight"]]/sum(ing_df$points_to_add[["Weight"]])*input$delta
            ing_df$design_w_opt <- bind_rows(ing_df$weighted_opt(), ing_df$points_to_add)
        }
        if(isTRUE(compute)) {
            if(identical(input$model, "Antoine Equation")) {
                ing_df$opt_design <- doptAntoine(input$a_par, input$b_par, input$c_par, input$design_sp[[1]], input$design_sp[[2]])
                ing_df$effres <- deff(dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$design_w_opt), dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$opt_design), 3)

                sens_opt <- dsensAntoine(input$a_par, input$b_par, input$c_par, dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$opt_design))
                sens_new <- dsensAntoine(input$a_par, input$b_par, input$c_par, dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$design_w_opt))

                # Puntos de corte y valor del corte
                cross <- sort(crosspoints(input$deff, input$delta, 3, sens_opt, 1000, 10^(-3), input$design_sp[[1]], input$design_sp[[2]]))
                val <- sens_val_to_add(input$deff, input$delta, 3)
                # Obtener start y par para tener las regiones
                start <- getStart(cross, input$design_sp[[1]], input$design_sp[[2]], val, sens_opt)
                par <- getPar(cross)
                ing_df$fac_reg_opt <- getCross2(cross, input$design_sp[[1]], input$design_sp[[2]], start, par)

                new_des_data <- data.frame("Point" = ing_df$design_w_opt[["Point"]], "Weight" = ing_df$design_w_opt[["Weight"]], "Value" = rep(0, length(ing_df$design_w_opt[["Point"]])))

                p <- ggplot() + theme_ipsum()

                x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
                y_val <- purrr::map_dbl(x_val, sens_new)

                ing_df$sens_res <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "Temperature (ºC)", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))

                # Loop para pintar las regiones factibles
                for(i in 1:(length(ing_df$fac_reg_opt)/2)){
                    loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_opt[2*i-1],",xend=",ing_df$fac_reg_opt[2*i],",y=0,yend=0), color = 'green3')", sep="")
                    ing_df$sens_res <- ing_df$sens_res + eval(parse(text=loop_input))
                }
            }
            else if(identical(input$model, "Quadratic")) {
                ing_df$opt_design <- doptCuad(input$design_sp[[1]], input$design_sp[[2]])
                ing_df$effres <- deff(dmatrixCuad(ing_df$design_w_opt), dmatrixCuad(ing_df$opt_design), 3)

                sens_opt <- dsensCuad(dmatrixCuad(ing_df$opt_design))
                sens_new <- dsensCuad(dmatrixCuad(ing_df$design_w_opt))

                # Puntos de corte y valor del corte
                cross <- sort(crosspoints(input$deff, input$delta, 3, sens_opt, 1000, 10^(-3), input$design_sp[[1]], input$design_sp[[2]]))
                val <- sens_val_to_add(input$deff, input$delta, 3)
                # Obtener start y par para tener las regiones
                start <- getStart(cross, input$design_sp[[1]], input$design_sp[[2]], val, sens_opt)
                par <- getPar(cross)
                ing_df$fac_reg_opt <- getCross2(cross, input$design_sp[[1]], input$design_sp[[2]], start, par)

                new_des_data <- data.frame("Point" = ing_df$design_w_opt[["Point"]], "Weight" = ing_df$design_w_opt[["Weight"]], "Value" = rep(0, length(ing_df$design_w_opt[["Point"]])))
                p <- ggplot() + theme_ipsum()

                x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
                y_val <- purrr::map_dbl(x_val, sens_new)

                ing_df$sens_res <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "X", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))

                # Loop para pintar las regiones factibles
                for(i in 1:(length(ing_df$fac_reg_opt)/2)){
                    loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_opt[2*i-1],",xend=",ing_df$fac_reg_opt[2*i],",y=0,yend=0), color = 'green3')", sep="")
                    ing_df$sens_res <- ing_df$sens_res + eval(parse(text=loop_input))
                }
            } else if(identical(input$model, "Michaelis-Menten")) {
                ing_df$opt_design <- doptMM(input$k_par, input$v_par, 0, input$design_sp_MM)
                ing_df$effres <- deff(dmatrixMM(input$k_par, input$v_par, ing_df$design_w_opt), dmatrixMM(input$k_par, input$v_par, ing_df$opt_design), 2)

                sens_opt <- dsensMM(input$k_par, input$v_par, dmatrixMM(input$k_par, input$v_par, ing_df$opt_design))
                sens_new <- dsensMM(input$k_par, input$v_par, dmatrixMM(input$k_par, input$v_par, ing_df$design_w_opt))

                # Puntos de corte y valor del corte
                cross <- sort(crosspoints(input$deff, input$delta, 2, sens_opt, 1000, 10^(-3), 0, input$design_sp_MM))
                val <- sens_val_to_add(input$deff, input$delta, 2)
                # Obtener start y par para tener las regiones
                start <- getStart(cross, 0, input$design_sp_MM, val, sens_opt)
                par <- getPar(cross)
                ing_df$fac_reg_opt <- getCross2(cross, 0, input$design_sp_MM, start, par)

                new_des_data <- data.frame("Point" = ing_df$design_w_opt[["Point"]], "Weight" = ing_df$design_w_opt[["Weight"]], "Value" = rep(0, length(ing_df$design_w_opt[["Point"]])))

                p <- ggplot() + theme_ipsum()

                x_val <- seq(0, input$design_sp_MM, length.out = 10000)
                y_val <- purrr::map_dbl(x_val, sens_new)

                ing_df$sens_res <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  2, color = "goldenrod3") + xlim(0, input$design_sp_MM) + labs(x = "X", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))

                # Loop para pintar las regiones factibles
                for(i in 1:(length(ing_df$fac_reg_opt)/2)){
                    loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_opt[2*i-1],",xend=",ing_df$fac_reg_opt[2*i],",y=0,yend=0), color = 'green3')", sep="")
                    ing_df$sens_res <- ing_df$sens_res + eval(parse(text=loop_input))
                }
            } else if(identical(input$model, "Quadratic Heteroscedastic")) {
                ing_df$opt_design <- doptCuadHet(input$order)
                ing_df$effres <- deff(dmatrixCuadHet(input$order, ing_df$design_w_opt), dmatrixCuadHet(input$order, ing_df$opt_design), 3)

                sens_opt <- dsensCuadHet(input$order, dmatrixCuadHet(input$order, ing_df$opt_design))
                sens_new <- dsensCuadHet(input$order, dmatrixCuadHet(input$order, ing_df$design_w_opt))

                # Puntos de corte y valor del corte
                cross <- sort(crosspoints(input$deff, input$delta, 3, sens_opt, 1000, 10^(-3), 0, input$design_sp_CH))
                val <- sens_val_to_add(input$deff, input$delta, 3)
                # Obtener start y par para tener las regiones
                start <- getStart(cross, 0, input$design_sp_CH, val, sens_opt)
                par <- getPar(cross)
                ing_df$fac_reg_opt <- getCross2(cross, 0, input$design_sp_CH, start, par)

                new_des_data <- data.frame("Point" = ing_df$design_w_opt[["Point"]], "Weight" = ing_df$design_w_opt[["Weight"]], "Value" = rep(0, length(ing_df$design_w_opt[["Point"]])))
                p <- ggplot() + theme_ipsum()

                x_val <- seq(0, input$design_sp_CH, length.out = 10000)
                y_val <- purrr::map_dbl(x_val, sens_new)

                ing_df$sens_res <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(0, input$design_sp_CH) + labs(x = "X", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))

                # Loop para pintar las regiones factibles
                for(i in 1:(length(ing_df$fac_reg_opt)/2)){
                    loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_opt[2*i-1],",xend=",ing_df$fac_reg_opt[2*i],",y=0,yend=0), color = 'green3')", sep="")
                    ing_df$sens_res <- ing_df$sens_res + eval(parse(text=loop_input))
                }
            }
            else{
                shinyalert("Error", "Enter a valid model!", type = "error")
            }
        }

    })

    # With restricted design from the optimum
    observeEvent(input$enhace_design, {
        compute <- TRUE
        length1 <- nrow(ing_df$enhaced_des)
        ing_df$enhaced_des <- ing_df$enhaced_des %>%
            group_by(Point) %>%
            summarise_all(funs(sum))
        if((nrow(ing_df$design) < 3 && !(input$model %in% c("Michaelis-Menten", "Quadratic Heteroscedastic"))) || nrow(ing_df$design) < 2){
            shinyalert("Build a regular design!", type = "error")
            compute <- FALSE
        }
        if(length1 != nrow(ing_df$enhaced_des)){
            shinyalert("", "There are repeated points in the design, they have been added together"
                       , type = "warning"
                       , timer = 1500)
        }
        if(nrow(ing_df$enhaced_des) <= nrow(ing_df$design))
        {
            shinyalert("You need to add points!", type = "error")
            compute <- FALSE
        }
        else if(0 %in% ing_df$enhaced_des[["Weight"]])
        {
            compute <- FALSE
            shinyalert("Error", "There are points without weight!", type = "error")
        }
        else if(((max(ing_df$enhaced_des[["Point"]]) > input$design_sp[[2]] || min(ing_df$enhaced_des[["Point"]]) < input$design_sp[[1]]) && !(input$model %in% c("Michaelis-Menten", "Quadratic Heteroscedastic"))) || ((max(ing_df$enhaced_des[["Point"]]) > input$design_sp_MM || min(ing_df$enhaced_des[["Point"]]) < 0) && identical(input$model, "Michaelis-Menten")) || ((max(ing_df$enhaced_des[["Point"]]) > input$design_sp_CH || min(ing_df$enhaced_des[["Point"]]) < 0) && identical(input$model, "Quadratic Heteroscedastic")))
        {
            compute <- FALSE
            shinyalert("Error", "There are design points outside the design space!", type = "error")
        }
        else if(!near(input$deltaenh, sum(ing_df$points_to_add_build[["Weight"]])))
        {
            shinyalert("", "The weights don't add up to 1, it'll be corrected"
                       # , do you want us to fix that for you?"
                       , type = "warning"
                       # , showCancelButton = TRUE, showConfirmButton = TRUE, callbackR = mycallback
                       , timer = 1500)
            ing_df$points_to_add_build[["Weight"]] <- ing_df$points_to_add_build[["Weight"]]/sum(ing_df$points_to_add_build[["Weight"]])*input$deltaenh
            ing_df$enhaced_des <- bind_rows(weighted_build(), ing_df$points_to_add_build)
        }
        if(isTRUE(compute)) {
            if(identical(input$model, "Antoine Equation")) {
                ing_df$opt_design <- doptAntoine(input$a_par, input$b_par, input$c_par, input$design_sp[[1]], input$design_sp[[2]])
                ing_df$effenh <- deff(dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$enhaced_des), dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$opt_design), 3)

                sens_des <- dsensAntoine(input$a_par, input$b_par, input$c_par, dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$design))
                sens_new <- dsensAntoine(input$a_par, input$b_par, input$c_par, dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$enhaced_des))

                # Puntos de corte y valor del corte
                cross <- sort(crosspoints(floor(input$deffenh#/ing_df$effbuild
                                                *10000)/10000, input$deltaenh, 3, sens_des, 1000, 10^(-3), input$design_sp[[1]], input$design_sp[[2]]))
                val <- sens_val_to_add(input$deffenh#/ing_df$effbuild
                                       , input$deltaenh, 3)
                # Obtener start y par para tener las regiones
                start <- getStart(cross, input$design_sp[[1]], input$design_sp[[2]], val, sens_des)
                par <- getPar(cross)
                ing_df$fac_reg_build <- getCross2(cross, input$design_sp[[1]], input$design_sp[[2]], start, par)


                new_des_data <- data.frame("Point" = ing_df$enhaced_des[["Point"]], "Weight" = ing_df$enhaced_des[["Weight"]], "Value" = rep(0, length(ing_df$enhaced_des[["Point"]])))
                p <- ggplot() + theme_ipsum()

                x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
                y_val <- purrr::map_dbl(x_val, sens_new)

                ing_df$sens_enh <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "Temperature (ºC)", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))

                # Loop para pintar las regiones factibles
                for(i in 1:(length(ing_df$fac_reg_build)/2)){
                    loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_build[2*i-1],",xend=",ing_df$fac_reg_build[2*i],",y=0,yend=0), color = 'green3')", sep="")
                    ing_df$sens_enh <- ing_df$sens_enh + eval(parse(text=loop_input))
                }
            }
            else if(identical(input$model, "Quadratic")) {
                ing_df$opt_design <- doptCuad(input$design_sp[[1]], input$design_sp[[2]])
                ing_df$effenh <- deff(dmatrixCuad(ing_df$enhaced_des), dmatrixCuad(ing_df$opt_design), 3)

                sens_des <- dsensCuad(dmatrixCuad(ing_df$design))
                sens_new <- dsensCuad(dmatrixCuad(ing_df$enhaced_des))

                # Puntos de corte y valor del corte
                cross <- sort(crosspoints(floor(input$deffenh#/ing_df$effbuild
                                                *10000)/10000, input$deltaenh, 3, sens_des, 1000, 10^(-3), input$design_sp[[1]], input$design_sp[[2]]))
                val <- sens_val_to_add(input$deffenh#/ing_df$effbuild
                                       , input$deltaenh, 3)
                # Obtener start y par para tener las regiones
                start <- getStart(cross, input$design_sp[[1]], input$design_sp[[2]], val, sens_des)
                par <- getPar(cross)
                ing_df$fac_reg_build <- getCross2(cross, input$design_sp[[1]], input$design_sp[[2]], start, par)

                new_des_data <- data.frame("Point" = ing_df$enhaced_des[["Point"]], "Weight" = ing_df$enhaced_des[["Weight"]], "Value" = rep(0, length(ing_df$enhaced_des[["Point"]])))

                p <- ggplot() + theme_ipsum()

                x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
                y_val <- purrr::map_dbl(x_val, sens_new)

                ing_df$sens_enh <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "X", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))

                # Loop para pintar las regiones factibles
                for(i in 1:(length(ing_df$fac_reg_build)/2)){
                    loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_build[2*i-1],",xend=",ing_df$fac_reg_build[2*i],",y=0,yend=0), color = 'green3')", sep="")
                    ing_df$sens_enh <- ing_df$sens_enh + eval(parse(text=loop_input))
                }
            } else if(identical(input$model, "Michaelis-Menten")) {
                ing_df$opt_design <- doptMM(input$k_par, input$v_par,  0, input$design_sp_MM)
                ing_df$effenh <- deff(dmatrixMM(input$k_par, input$v_par,  ing_df$enhaced_des), dmatrixMM(input$k_par, input$v_par,  ing_df$opt_design), 2)

                sens_des <- dsensMM(input$k_par, input$v_par,  dmatrixMM(input$k_par, input$v_par,  ing_df$design))
                sens_new <- dsensMM(input$k_par, input$v_par,  dmatrixMM(input$k_par, input$v_par,  ing_df$enhaced_des))

                # Puntos de corte y valor del corte
                cross <- sort(crosspoints(floor(input$deffenh#/ing_df$effbuild
                                                *10000)/10000, input$deltaenh, 2, sens_des, 1000, 10^(-3), 0, input$design_sp_MM))
                val <- sens_val_to_add(input$deffenh#/ing_df$effbuild
                                       , input$deltaenh, 2)
                # Obtener start y par para tener las regiones
                start <- getStart(cross, 0, input$design_sp_MM, val, sens_des)
                par <- getPar(cross)
                ing_df$fac_reg_build <- getCross2(cross, 0, input$design_sp_MM, start, par)


                new_des_data <- data.frame("Point" = ing_df$enhaced_des[["Point"]], "Weight" = ing_df$enhaced_des[["Weight"]], "Value" = rep(0, length(ing_df$enhaced_des[["Point"]])))
                p <- ggplot() + theme_ipsum()

                x_val <- seq(0, input$design_sp_MM, length.out = 10000)
                y_val <- purrr::map_dbl(x_val, sens_new)

                ing_df$sens_enh <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  2, color = "goldenrod3") + xlim(0, input$design_sp_MM) + labs(x = "X", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))

                # Loop para pintar las regiones factibles
                for(i in 1:(length(ing_df$fac_reg_build)/2)){
                    loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_build[2*i-1],",xend=",ing_df$fac_reg_build[2*i],",y=0,yend=0), color = 'green3')", sep="")
                    ing_df$sens_enh <- ing_df$sens_enh + eval(parse(text=loop_input))
                }
            } else if(identical(input$model, "Quadratic Heteroscedastic")) {
                ing_df$opt_design <- doptCuadHet(input$order)
                ing_df$effenh <- deff(dmatrixCuadHet(input$order, ing_df$enhaced_des), dmatrixCuadHet(input$order, ing_df$opt_design), 3)

                sens_des <- dsensCuadHet(input$order, dmatrixCuadHet(input$order, ing_df$design))
                sens_new <- dsensCuadHet(input$order, dmatrixCuadHet(input$order, ing_df$enhaced_des))

                # Puntos de corte y valor del corte
                cross <- sort(crosspoints(floor(input$deffenh#/ing_df$effbuild
                                                *10000)/10000, input$deltaenh, 3, sens_des, 1000, 10^(-3), 0, input$design_sp_CH))
                val <- sens_val_to_add(input$deffenh#/ing_df$effbuild
                                       , input$deltaenh, 3)
                # Obtener start y par para tener las regiones
                start <- getStart(cross, 0, input$design_sp_CH, val, sens_des)
                par <- getPar(cross)
                ing_df$fac_reg_build <- getCross2(cross, 0, input$design_sp_CH, start, par)

                new_des_data <- data.frame("Point" = ing_df$enhaced_des[["Point"]], "Weight" = ing_df$enhaced_des[["Weight"]], "Value" = rep(0, length(ing_df$enhaced_des[["Point"]])))

                p <- ggplot() + theme_ipsum()

                x_val <- seq(0, input$design_sp_CH, length.out = 10000)
                y_val <- purrr::map_dbl(x_val, sens_new)

                ing_df$sens_enh <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(0, input$design_sp_CH) + labs(x = "X", y = "") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))

                # Loop para pintar las regiones factibles
                for(i in 1:(length(ing_df$fac_reg_build)/2)){
                    loop_input = paste("geom_segment(aes(x=",ing_df$fac_reg_build[2*i-1],",xend=",ing_df$fac_reg_build[2*i],",y=0,yend=0), color = 'green3')", sep="")
                    ing_df$sens_enh <- ing_df$sens_enh + eval(parse(text=loop_input))
                }
            }
            else{
                shinyalert("Error", "Enter a valid model!", type = "error")
            }
        }
    })


    # Efficiency of the industry designs
    # observeEvent(input$calc_ind_eff, {
    #     calculate <- T
    #     if(identical(input$model, "Antoine Equation")) {
    #         ing_df$opt_design <- doptAntoine(input$a_par, input$b_par, input$c_par, input$design_sp[[1]], input$design_sp[[2]])
    #         dopt_mat <- dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$opt_design)
    #
    #         if(identical(input$family, "Equidistant")){
    #             ing_df$ind_des <- uniform_design(input$ind_points, input$design_sp[[1]], input$design_sp[[2]])
    #         }
    #         else if(identical(input$family, "Arithmetic")){
    #             ing_df$ind_des <- arithmetic_design(as.numeric(input$ind_points), as.numeric(input$design_sp[[1]]), as.numeric(input$design_sp[[2]]), dopt_mat, input$model, input$a_par, input$b_par, input$c_par)
    #         }
    #         else if(identical(input$family, "Geometric")){
    #             ing_df$ind_des <- geometric_design(as.numeric(input$ind_points), as.numeric(input$design_sp[[1]]), as.numeric(input$design_sp[[2]]), dopt_mat, input$model, input$a_par, input$b_par, input$c_par)
    #         }
    #         else{
    #             shinyalert("Error", "Choose a family of designs!", type = "error")
    #             calculate <- F
    #         }
    #
    #         if(calculate){
    #             ing_df$effind <- deff(dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$ind_des), dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$opt_design), 3)
    #
    #             sens_opt <- dsensAntoine(input$a_par, input$b_par, input$c_par, dopt_mat)
    #             sens_new <- dsensAntoine(input$a_par, input$b_par, input$c_par, dmatrixAntoine(input$a_par, input$b_par, input$c_par, ing_df$ind_des))
    #
    #
    #             new_des_data <- data.frame("Point" = ing_df$ind_des[["Point"]], "Weight" = ing_df$ind_des[["Weight"]], "Value" = rep(0, length(ing_df$ind_des[["Point"]])))
    #             p <- ggplot() + theme_ipsum()
    #
    #             x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
    #             y_val <- purrr::map_dbl(x_val, sens_new)
    #
    #             ing_df$sens_ind <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "Temperature (ºC)", y = "Pressure") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))
    #         }
    #     }
    #     else if(identical(input$model, "Quadratic")) {
    #         ing_df$opt_design <- doptCuad(input$design_sp[[1]], input$design_sp[[2]])
    #         dopt_mat <- dmatrixCuad(ing_df$opt_design)
    #
    #         if(identical(input$family, "Equidistant")){
    #             ing_df$ind_des <- uniform_design(input$ind_points, input$design_sp[[1]], input$design_sp[[2]])
    #         }
    #         else if(identical(input$family, "Arithmetic")){
    #             ing_df$ind_des <- arithmetic_design(as.numeric(input$ind_points), as.numeric(input$design_sp[[1]]), as.numeric(input$design_sp[[2]]), dopt_mat, input$model)
    #         }
    #         else if(identical(input$family, "Geometric")){
    #             ing_df$ind_des <- geometric_design(as.numeric(input$ind_points), as.numeric(input$design_sp[[1]]), as.numeric(input$design_sp[[2]]), dopt_mat, input$model)
    #         }
    #         else{
    #             shinyalert("Error", "Choose a family of designs!", type = "error")
    #             calculate <- F
    #         }
    #
    #         if(calculate){
    #             ing_df$effind <- deff(dmatrixCuad(ing_df$ind_des), dopt_mat, 3)
    #
    #             sens_opt <- dsensCuad(dmatrixCuad(ing_df$opt_design))
    #             sens_new <- dsensCuad(dmatrixCuad(ing_df$ind_des))
    #
    #             new_des_data <- data.frame("Point" = ing_df$ind_des[["Point"]], "Weight" = ing_df$ind_des[["Weight"]], "Value" = rep(0, length(ing_df$ind_des[["Point"]])))
    #             p <- ggplot() + theme_ipsum()
    #
    #             x_val <- seq(input$design_sp[[1]], input$design_sp[[2]], length.out = 10000)
    #             y_val <- purrr::map_dbl(x_val, sens_new)
    #
    #             ing_df$sens_ind <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  3, color = "goldenrod3") + xlim(input$design_sp[[1]], input$design_sp[[2]]) + labs(x = "X", y = "Y") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))
    #         }
    #     }
    #     else if(identical(input$model, "Michaelis-Menten")) {
    #         ing_df$opt_design <- doptMM(input$k_par, input$v_par,  0, input$design_sp_MM)
    #         dopt_mat <- dmatrixMM(input$k_par, input$v_par,  ing_df$opt_design)
    #
    #         if(identical(input$family, "Equidistant")){
    #             ing_df$ind_des <- uniform_design(input$ind_points, 0, input$design_sp_MM)
    #         }
    #         else if(identical(input$family, "Arithmetic")){
    #             ing_df$ind_des <- arithmetic_design(as.numeric(input$ind_points), as.numeric(0), as.numeric(input$design_sp_MM), dopt_mat, input$model, input$k_par, input$v_par)
    #         }
    #         else if(identical(input$family, "Geometric")){
    #             ing_df$ind_des <- geometric_design(as.numeric(input$ind_points), as.numeric(0), as.numeric(input$design_sp_MM), dopt_mat, input$model, input$k_par, input$v_par)
    #         }
    #         else{
    #             shinyalert("Error", "Choose a family of designs!", type = "error")
    #             calculate <- F
    #         }
    #
    #         if(calculate){
    #             ing_df$effind <- deff(dmatrixMM(input$k_par, input$v_par,  ing_df$ind_des), dmatrixMM(input$k_par, input$v_par,  ing_df$opt_design), 2)
    #
    #             sens_opt <- dsensMM(input$k_par, input$v_par,  dopt_mat)
    #             sens_new <- dsensMM(input$k_par, input$v_par,  dmatrixMM(input$k_par, input$v_par,  ing_df$ind_des))
    #
    #
    #             new_des_data <- data.frame("Point" = ing_df$ind_des[["Point"]], "Weight" = ing_df$ind_des[["Weight"]], "Value" = rep(0, length(ing_df$ind_des[["Point"]])))
    #             p <- ggplot() + theme_ipsum()
    #
    #             x_val <- seq(0, input$design_sp_MM, length.out = 10000)
    #             y_val <- purrr::map_dbl(x_val, sens_new)
    #
    #             ing_df$sens_ind <- p + geom_line(mapping = aes(x = x_val, y = y_val), color = "steelblue3") + geom_hline(yintercept =  2, color = "goldenrod3") + xlim(0, input$design_sp_MM) + labs(x = "Temperature (ºC)", y = "Pressure") + geom_point(data = new_des_data, aes(x = Point, y = Value, size = Weight), shape = 16, color = "steelblue3") + scale_size_continuous(range = c(2, 4))
    #         }
    #     }
    #     else{
    #         shinyalert("Error", "Enter a valid model!", type = "error")
    #     }
    # })


    # Compute Effinciency givedes
    efficiency <- reactive({
        if(is.numeric(ing_df$eff))
            round(ing_df$eff*100, 1)
        else
            ing_df$eff
    })

    # Compute Effinciency restricteddes
    efficiencyres <- reactive({
        if(is.numeric(ing_df$effres))
            round(ing_df$effres*100, 1)
        else
            ing_df$effres
    })

    # Compute Effinciency builddes
    efficiencybuild <- reactive({
        if(is.numeric(ing_df$effbuild))
            round(ing_df$effbuild*100, 1)
        else
            ing_df$effbuild
    })

    # Compute Effinciency enhanced
    efficiencyenh <- reactive({
        if(is.numeric(ing_df$effenh))
            round(ing_df$effenh*100, 1)
        else
            ing_df$effenh
    })

    # Compute Effinciency industry
    # efficiency_indus <- reactive({
    #     if(is.numeric(ing_df$effind))
    #         round(ing_df$effind*100, 1)
    #     else
    #         ing_df$effind
    # })

    # Plot sens
    ing_df$sens_opt <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()

    output$sens_opt <- renderPlotly({
        ggplotly(ing_df$sens_opt)
    })

    # Plot sens give
    ing_df$sens_new <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()

    output$sens_new <- renderPlotly({
        ggplotly(ing_df$sens_new)
    })

    # Plot sens res
    ing_df$sens_res <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()

    output$sens_res <- renderPlotly({
        ggplotly(ing_df$sens_res)
    })

    #Plot sens build
    ing_df$sens_build <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()

    output$sens_build <- renderPlotly({
        ggplotly(ing_df$sens_build)
    })

    #Plot sens enh
    ing_df$sens_enh <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()

    output$sens_enh <- renderPlotly({
        ggplotly(ing_df$sens_enh)
    })

    #Plot sens ind
    # ing_df$sens_ind <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()
    #
    # output$sens_ind <- renderPlotly({
    #     ggplotly(ing_df$sens_ind)
    # })







    # value box optimal design senstitivy
    output$op_sensitivity <- renderValueBox({
        valueBox(
            paste0("Sensitivity Function"#, output$efficiency
            ), "For the optimum design", icon = icon("bar-chart-o"),
            color = "purple"
        )
    })

    # value box industry design
    # output$industry_des <- renderValueBox({
    #     valueBox(
    #         paste0("Sensitivity Function"#, output$efficiency
    #         ), "For the optimum design", icon = icon("bar-chart-o"),
    #         color = "yellow"
    #     )
    # })

    # value box industry design senstitivy
    # output$industry_sens <- renderValueBox({
    #     valueBox(
    #         paste0("Sensitivity Function"#, output$efficiency
    #         ), "For the industry design", icon = icon("bar-chart-o"),
    #         color = "purple"
    #     )
    # })

    # Value box for different efficiencies
    output$efficiency <- renderValueBox({
        valueBox(
            paste0("Efficiency ", efficiency(), "%" #input$count
            ), "For the input design", icon = icon("divide"),
            color = "red"
        )
    })

    output$efficiencyresdes <- renderValueBox({
        valueBox(
            paste0("Efficiency ", efficiencyres(), "%" #input$count
            ), "For the input design", icon = icon("divide"),
            color = "red"
        )
    })

    output$efficiencybuildes <- renderValueBox({
        valueBox(
            paste0("Efficiency ", efficiencybuild(), "%" #input$count
            ), "For the input design", icon = icon("divide"),
            color = "red"
        )
    })

    output$efficiencyenhdes <- renderValueBox({
        valueBox(
            paste0("Efficiency ", efficiencyenh(), "%" #input$count
            ), "With respect to the optimal design", icon = icon("divide"),
            color = "red"
        )
    })

    # output$efficiency_ind <- renderValueBox({
    #     valueBox(
    #         paste0("Efficiency ", efficiency_indus(), "%" #input$count
    #         ), "For the industry design", icon = icon("divide"),
    #         color = "red"
    #     )
    # })


    observeEvent(input$model, {
        header <- input$model

        # you can use any other dynamic content you like
        shinyjs::html("pageHeader", header)
    })


    restart <- function(){
        updateSliderInput(session, "delta",
                          min = 0, max = 0.99, value = 0)

        updateSliderInput(session, "deltaenh",
                          min = 0, max = 0.99, value = 0)

        # Diseño en blanco que el usuario va construyendo
        ing_df$design <- data.frame("Point" = numeric(),
                                    "Weight" = numeric())

        # dataset "standard", diseño añadiendo al óptimo los puntos de cortes con igula peso
        ing_df$design_std <- data.frame("Point" = numeric(),
                                        "Weight" = numeric())

        # Diseño final al partir del óptimo y añadir puntos
        # ing_df$ind_des <- data.frame("Point" = numeric(),
        #                              "Weight" = numeric())

        # Punto a añadir al construido
        ing_df$points_to_add_build <- data.frame("Point" = numeric(),
                                                 "Weight" = numeric())

        # Diseño final al partir del construido valiendo (1-alpha)
        ing_df$weighted_build <- data.frame("Point" = numeric(),
                                            "Weight" = numeric())

        # Diseño final al partir del construido y añadir puntos
        ing_df$enhaced_des <- data.frame("Point" = numeric(),
                                         "Weight" = numeric())

        # Se ha calculado el diseño óptimo?
        ing_df$calculated_design <- F

        # Se ha calculado un diseño propio?
        ing_df$built_design <- F

        # Límites de eficiencia en funcion de delta
        ing_df$eff_limit <- c(0, 1)

        # Límites de eficiencia en funcion de delta para enhace design
        ing_df$eff_limit_enh <- c(0, 1)

        # Regiones factibles añadir óptimo
        ing_df$fac_reg_opt <- vector("numeric")

        # Regiones factibles añadir custom
        ing_df$fac_reg_build <- vector("numeric")

        # Plot sens
        ing_df$sens_opt <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()


        # Plot sens give
        ing_df$sens_new <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()


        # Plot sens res
        ing_df$sens_res <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()


        #Plot sens build
        ing_df$sens_build <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()


        #Plot sens enh
        ing_df$sens_enh <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()


        #Plot sens ind
        # ing_df$sens_ind <- ggplot(data = data.frame(x = 0), mapping = aes(x = x)) + theme_ipsum()





    }

}

# Run the application
shinyApp(ui = ui, server = server)

# Packages required by each Shiny app beyond those already in Imports.
# Checked simultaneously so the user learns about all missing ones at once.
.shiny_needed <- function(app) {
  common <- c("shinydashboard", "shinyalert", "shinyjs", "plotly",
              "DT", "hrbrthemes", "markdown", "magrittr", "tidyverse")
  if (app == "augment") c(common, "orthopolynom") else common
}

.shiny_check <- function(app_name, url) {
  if (!interactive()) {
    message(app_name, "() requires an interactive R session. ",
            "Access the app online: ", url)
    return(invisible(FALSE))
  }
  missing_pkgs <- .shiny_needed(app_name)
  missing_pkgs <- missing_pkgs[
    !vapply(missing_pkgs, requireNamespace, logical(1L), quietly = TRUE)
  ]
  if (length(missing_pkgs) > 0) {
    stop(
      "The following packages are needed but not installed: ",
      paste(missing_pkgs, collapse = ", "), ".\n",
      "Install with:\n  install.packages(c(",
      paste0('"', missing_pkgs, '"', collapse = ", "), "))\n",
      "Or access the app online: ", url,
      call. = FALSE
    )
  }
  invisible(TRUE)
}


#' Shiny Optimal
#'
#' @description
#' Launches the demo Shiny application to calculate optimal designs for
#' Antoine's Equation. Requires an interactive R session; if called
#' non-interactively a message with the web URL is emitted instead.
#'
#' @export
#'
#' @examples
#' shiny_optimal()
shiny_optimal <- function() {
  url <- "https://kezrael.shinyapps.io/AntoineOptimal/"
  if (!isTRUE(.shiny_check("optimal", url))) return(invisible(NULL))
  appDir <- system.file("shiny-examples", "D-Optimality", package = "optedr")
  if (appDir == "")
    stop("Could not find the app directory. Try re-installing optedr.",
         call. = FALSE)
  shiny::runApp(appDir, display.mode = "normal")
}


#' Shiny D-augment
#'
#' @description
#' Launches the demo Shiny application to D-augment several pre-specified
#' models. Requires an interactive R session; if called non-interactively
#' a message with the web URL is emitted instead.
#'
#' @export
#'
#' @examples
#' shiny_augment()
shiny_augment <- function() {
  url <- "https://kezrael.shinyapps.io/AddPoints/"
  if (!isTRUE(.shiny_check("augment", url))) return(invisible(NULL))
  appDir <- system.file("shiny-examples", "AddPoints", package = "optedr")
  if (appDir == "")
    stop("Could not find the app directory. Try re-installing optedr.",
         call. = FALSE)
  shiny::runApp(appDir, display.mode = "normal")
}

# Optimality Criteria -----------------------

#' Master function for the criterion function
#'
#' @description
#' Depending on the criterion input, the function returns the output of the corresponding criterion function given
#' the information matrix.
#'
#' @param criterion character variable with the chosen optimality criterion. Can be one of the following:
#'   * 'D-Optimality'
#'   * 'Ds-Optimality'
#'   * 'A-Optimality'
#'   * 'I-Optimality'
#'   * 'L-Optimality'
#' @param M information matrix for which the criterion value wants to be calculated.
#' @param k numeric variable with the number of parameters of the model. Taken from the number of rows of the matrix if omitted.
#' @param par_int numeric vector with the index of the parameters of interest of the model. Only for "Ds-Optimality".
#' @param matB optional matrix of dimensions k x k, for I- and L-optimality.
#'
#' @return Numeric value of the optimality criterion for the information matrix.
crit <- function(criterion, M, k = 0, par_int = c(1), matB = NA) {
  if (identical(criterion, "D-Optimality")) {
    return(dcrit(M, k))
  }
  else if (identical(criterion, "Ds-Optimality")) {
    return(dscrit(M, par_int))
  }
  else if (identical(criterion, "A-Optimality")) {
    return(icrit(M, diag(k)))
  }
  else if (identical(criterion, "I-Optimality") || identical(criterion, "L-Optimality")) {
    return(icrit(M, matB))
  }
}


#' Criterion function for D-Optimality
#'
#' @description
#' Calculates the value of the D-Optimality criterion function, which follows the expression:
#' \deqn{\phi_D = \left(\frac{1}{|M|}\right)^{1/k}}
#'
#'
#' @param M information matrix for which the criterion value wants to be calculated.
#' @param k numeric variable with the number of parameters of the model. Taken from the number of rows of the matrix if omitted.
#'
#'
#' @return numeric value of the D-optimality criterion for the information matrix.
dcrit <- function(M, k) {
  if (k == 0) k <- nrow(M)
  return((1 / det(M))^(1 / k))
}


#' Criterion function for Ds-Optimality
#'
#' @description
#' Calculates the value of the Ds-Optimality criterion function, which follows the expression:
#' \deqn{\phi_{Ds} = \left(\frac{|M_{22}|}{|M|}\right)^{1/s}}
#'
#'
#' @param M information matrix for which the criterion value wants to be calculated.
#' @param par_int numeric vector with the index of the parameters of interest of the model.
#'
#'
#' @return Numeric value of the Ds-optimality criterion for the information matrix.
dscrit <- function(M, par_int) {
  if (length(M[-par_int, -par_int]) == 1) {
    return((M[-par_int, -par_int] / det(M))^(1 / length(par_int)))
  }
  else {
    return((det(M[-par_int, -par_int]) / det(M))^(1 / length(par_int)))
  }
}

#' Criterion function for I-Optimality and L-Optimality
#'
#' @description
#' Calculates the value of the I-Optimality criterion function, which follows the expression:
#' \deqn{\phi_I = Tr(M^{-1} \cdot B)}
#'
#'
#' @param M information matrix for which the criterion value wants to be calculated.
#' @param matB matrix of the integral of the information matrix over the interest region. Identity matrix for
#'   A-Optimality.
#'
#'
#' @return Numeric value of the I-optimality criterion for the information matrix.
icrit <- function(M, matB) {
  return(tr(matB %*% inv_spd(M)))
}


# Compound criterion value: weighted sum of individual criterion values.
# compound_specs: preprocessed list (each element has criterion, weight, k/par_int/matB).
ccrit <- function(compound_specs, M) {
  total <- 0
  for (spec in compound_specs) {
    phi <- if (identical(spec$criterion, "D-Optimality"))
             dcrit(M, spec$k)
           else if (identical(spec$criterion, "Ds-Optimality"))
             dscrit(M, spec$par_int)
           else
             icrit(M, spec$matB)
    total <- total + spec$weight * phi
  }
  total
}



# Efficiency-------- -----------------------


#' Efficiency between two Information Matrices
#'
#' @param criterion character variable with the chosen optimality criterion. Can be one of the following:
#'   * 'D-Optimality'
#'   * 'Ds-Optimality'
#'   * 'A-Optimality'
#'   * 'I-Optimality'
#'   * 'L-Optimality'
#' @param mat1 first information matrix, for the numerator.
#' @param mat2 second information matrix, for the denominator.
#' @param k number of parameters of the model. Taken from the number of rows of the matrix if omitted.
#' @param intPars numeric vector with the index of the parameters of interest of the model. Only for "Ds-Optimality".
#' @param matB matrix of the integral of the information matrix over the interest region. Only for "I-Optimality".
#'
#' @return Efficiency of first design with respect to the second design, as a decimal number.
eff <- function(criterion, mat1, mat2, k = 0, intPars = c(1), matB = NA) {
  if (identical(criterion, "D-Optimality")) {
    if (k == 0) k <- nrow(mat1)
    return((det(mat1) / det(mat2))^(1 / k))
  }
  else if (identical(criterion, "Ds-Optimality")) {
    if (length(mat1[-intPars, -intPars]) == 1) {
      return((mat2[-intPars, -intPars] / det(mat2) / (mat1[-intPars, -intPars] / det(mat1)))^(1 / length(intPars)))
    }
    else {
      return((det(mat2[-intPars, -intPars]) / det(mat2) / (det(mat1[-intPars, -intPars]) / det(mat1)))^(1 / length(intPars)))
    }
  }
  else if (identical(criterion, "A-Optimality")) {
    if (k == 0) k <- nrow(mat1)
    return(tr(diag(k) %*% inv_spd(mat2)) / tr(diag(k) %*% inv_spd(mat1)))
  }
  else if (identical(criterion, "I-Optimality") || identical(criterion, "L-Optimality")) {
    return(tr(matB %*% inv_spd(mat2)) / tr(matB %*% inv_spd(mat1)))
  }
}


#' Efficiency between optimal design and a user given design
#'
#' @description
#' Takes an optimal design provided from the function \code{opt_des} and a user given design and compares their
#' efficiency
#'
#' @seealso opt_des
#'
#' @param opt_des_obj an object given by the function \code{opt_des}.
#' @param design dataframe that represents the design. Must have two columns:
#'   * \code{Point} contains the support points of the design.
#'   * \code{Weight} contains the corresponding weights of the \code{Point}s.
#'
#' @return The efficiency as a value between 0 and 1
#' @export
#'
#' @examples
#' result <- opt_des("D-Optimality", y ~ a * exp(-b / x), c("a", "b"), c(1, 1500), c(212, 422))
#' design <- data.frame("Point" = c(220, 240, 400), "Weight" = c(1 / 3, 1 / 3, 1 / 3))
#' design_efficiency(design, result)
design_efficiency <- function(design, opt_des_obj) {
  if (!"optdes" %in% class(opt_des_obj)) {
    stop(crayon::red(cli::symbol$cross),
         " The second argument must be an optdes object returned by opt_des()",
         call. = FALSE)
  }

  # KL-Optimality: no information matrix; efficiency = KL(new) / KL(opt)
  if (identical(opt_des_obj$criterion, "KL-Optimality")) {
    kl_params <- attr(opt_des_obj, "hidden_value")
    if ("optdes" %in% class(design)) {
      des_df <- design$optdes
    } else if ("data.frame" %in% class(design)) {
      des_df <- design
    } else {
      stop(crayon::red(cli::symbol$cross),
           " The first argument must be a data.frame or an optdes object returned by opt_des()",
           call. = FALSE)
    }
    inner_new <- .kl_minopt(des_df, kl_params$kl_fun,
                             kl_params$beta2_star, kl_params$lower, kl_params$upper)
    eff <- inner_new$kl_val / opt_des_obj$crit_value
    message(crayon::blue(cli::symbol$info), " The efficiency of the design is ", eff * 100, "%")
    return(eff)
  }

  grad       <- attr(opt_des_obj, "gradient")
  dvars      <- attr(grad, "design_vars")
  if (is.null(dvars)) dvars <- "x"

  if ("optdes" %in% class(design)) {
    mat1 <- inf_mat(grad, design$optdes)
  } else if ("data.frame" %in% class(design)) {
    # Accept both "Point" (1D backward compat) and x1/x2/... column names
    design <- normalize_design_cols(design, dvars)
    missing_vars <- setdiff(dvars, names(design))
    if (length(missing_vars) > 0L || !"Weight" %in% names(design)) {
      expected <- if (length(dvars) == 1L && dvars == "x") "'Point' and 'Weight'" else
        paste0("'", paste(c(dvars, "Weight"), collapse = "', '"), "'")
      stop(crayon::red(cli::symbol$cross),
           " The design data.frame must have columns ", expected,
           call. = FALSE)
    }
    mat1 <- inf_mat(grad, design)
  } else {
    stop(crayon::red(cli::symbol$cross),
         " The first argument must be a data.frame or an optdes object returned by opt_des()",
         call. = FALSE)
  }
  mat2 <- inf_mat(grad, opt_des_obj$optdes)
  if (identical(opt_des_obj$criterion, "D-Optimality")) {
    eff <- (det(mat1) / det(mat2))^(1 / attr(opt_des_obj, "hidden_value"))
    message(crayon::blue(cli::symbol$info), " The efficiency of the design is ", eff * 100, "%")
    return(eff)
  }
  else if (identical(opt_des_obj$criterion, "Ds-Optimality")) {
    int_pars <- attr(opt_des_obj, "hidden_value")
    if (length(mat1[-int_pars, -int_pars]) == 1) {
      eff <- (mat2[-int_pars, -int_pars] / det(mat2) / (mat1[-int_pars, -int_pars] / det(mat1)))^(1 / length(int_pars))
      message(crayon::blue(cli::symbol$info), " The efficiency of the design is ", eff * 100, "%")
      return(eff)
    }
    else {
      eff <- (det(mat2[-int_pars, -int_pars]) / det(mat2) / (det(mat1[-int_pars, -int_pars]) / det(mat1)))^(1 / length(int_pars))
      message(crayon::blue(cli::symbol$info), " The efficiency of the design is ", eff * 100, "%")
      return(eff)
    }
  }
  else if (identical(opt_des_obj$criterion, "A-Optimality")  || identical(opt_des_obj$criterion, "I-Optimality")  || identical(opt_des_obj$criterion, "L-Optimality")) {
    eff <- tr(attr(opt_des_obj, "hidden_value") %*% inv_spd(mat2)) / tr(attr(opt_des_obj, "hidden_value") %*% inv_spd(mat1))
    message(crayon::blue(cli::symbol$info), " The efficiency of the design is ", eff * 100, "%")
    return(eff)
  }
  else if (identical(opt_des_obj$criterion, "Compound")) {
    specs <- attr(opt_des_obj, "hidden_value")   # compound_specs
    eff   <- ccrit(specs, mat2) / ccrit(specs, mat1)
    message(crayon::blue(cli::symbol$info), " The efficiency of the design is ", eff * 100, "%")
    return(eff)
  }
}

library(optedr)

test_that("efficiency works for D-Optimality", {
  resArr.D <- evaluate_promise(opt_des(
    "D-Optimality",
    model = y ~ a * exp(-b / x),
    parameters = c("a", "b"),
    par_values = c(1, 1500),
    design_space = c(212, 422)
  ))$result
  design <- data.frame("Point" = c(220, 240, 400), "Weight" = c(1/3, 1/3, 1/3))

  efficiency1 <- evaluate_promise(design_efficiency(design, resArr.D))
  expect_equal(round(efficiency1$result, 7), 0.3063763)
})

test_that("efficiency errors on wrong column names in first argument", {
  resArr.D <- evaluate_promise(opt_des(
    "D-Optimality",
    model = y ~ a * exp(-b / x),
    parameters = c("a", "b"),
    par_values = c(1, 1500),
    design_space = c(212, 422)
  ))$result
  design_bad <- data.frame("Puntos" = c(220, 240, 400), "Weight" = c(1/3, 1/3, 1/3))
  expect_error(design_efficiency(design_bad, resArr.D), "'Point' and 'Weight'")
})

test_that("efficiency errors when second argument is not an optdes object", {
  design <- data.frame("Point" = c(220, 240, 400), "Weight" = c(1/3, 1/3, 1/3))
  expect_error(
    design_efficiency(design, design),
    "second argument must be an optdes object"
  )
})

library(optedr)

# Shared fixtures -------------------------------------------------------
model  <- y ~ a * exp(-b / x)
params <- c("a", "b")
pvals  <- c(1, 1500)
dspace <- c(212, 422)
grad   <- optedr:::gradient(model, params, pvals)
# Suboptimal 3-point design used where we need something clearly worse than optimal
des      <- data.frame(Point = c(329.3, 422),       Weight = c(0.5, 0.5))
des_sub  <- data.frame(Point = c(220, 300, 400),    Weight = c(1/3, 1/3, 1/3))
M      <- optedr:::inf_mat(grad, des)

# dcrit() ---------------------------------------------------------------

test_that("dcrit returns a finite positive value for non-singular M", {
  val <- optedr:::dcrit(M, k = 2L)
  expect_true(is.finite(val))
  expect_gt(val, 0)
})

test_that("dcrit with k = 0 infers k from matrix dimensions", {
  expect_equal(optedr:::dcrit(M, k = 0L), optedr:::dcrit(M, k = 2L))
})

test_that("dcrit is minimised at the D-optimal design vs a suboptimal one", {
  res    <- evaluate_promise(opt_des("D-Optimality", model, params, pvals, dspace))$result
  M_opt  <- optedr:::inf_mat(grad, res$optdes)
  M_s    <- optedr:::inf_mat(grad, des_sub)
  expect_lte(optedr:::dcrit(M_opt, 2L), optedr:::dcrit(M_s, 2L))
})

test_that("dcrit is invariant to weight permutation", {
  d1 <- data.frame(Point = c(329.3, 422), Weight = c(0.3, 0.7))
  d2 <- data.frame(Point = c(422, 329.3), Weight = c(0.7, 0.3))
  expect_equal(
    optedr:::dcrit(optedr:::inf_mat(grad, d1), 2L),
    optedr:::dcrit(optedr:::inf_mat(grad, d2), 2L),
    tolerance = 1e-10
  )
})

# dscrit() --------------------------------------------------------------

test_that("dscrit returns a finite positive value", {
  val <- optedr:::dscrit(M, par_int = c(1L))
  expect_true(is.finite(val))
  expect_gt(val, 0)
})

test_that("dscrit handles 1x1 nuisance submatrix (scalar case)", {
  val_sc <- optedr:::dscrit(M, par_int = c(2L))
  expect_true(is.finite(val_sc))
  expect_gt(val_sc, 0)
})

# icrit() ---------------------------------------------------------------

test_that("icrit returns a finite positive value", {
  val <- optedr:::icrit(M, diag(2))
  expect_true(is.finite(val))
  expect_gt(val, 0)
})

test_that("icrit with identity matB equals tr(M^-1)", {
  expect_equal(
    optedr:::icrit(M, diag(2)),
    optedr:::tr(optedr:::inv_spd(M)),
    tolerance = 1e-10
  )
})

test_that("icrit is minimised at the A-optimal design", {
  res   <- evaluate_promise(opt_des("A-Optimality", model, params, pvals, dspace))$result
  M_opt <- optedr:::inf_mat(grad, res$optdes)
  expect_lte(optedr:::icrit(M_opt, diag(2)), optedr:::icrit(M, diag(2)))
})

# design_efficiency() ---------------------------------------------------

test_that("design_efficiency returns value in (0, 1] for D-Optimality", {
  res <- evaluate_promise(opt_des("D-Optimality", model, params, pvals, dspace))$result
  eff <- evaluate_promise(design_efficiency(des_sub, res))$result
  expect_gte(eff, 0)
  expect_lte(eff, 1)
})

test_that("design_efficiency returns 1 when comparing optimal design to itself", {
  res <- evaluate_promise(opt_des("D-Optimality", model, params, pvals, dspace))$result
  eff <- evaluate_promise(design_efficiency(res$optdes, res))$result
  expect_equal(eff, 1, tolerance = 0.001)
})

test_that("design_efficiency accepts an optdes object as first argument", {
  res <- evaluate_promise(opt_des("D-Optimality", model, params, pvals, dspace))$result
  eff <- evaluate_promise(design_efficiency(res, res))$result
  expect_equal(eff, 1, tolerance = 0.001)
})

test_that("design_efficiency works for Ds-Optimality", {
  res <- evaluate_promise(opt_des(
    "Ds-Optimality",
    y ~ th0 * exp(x / th1), c("th0", "th1"), c(10.4963, -3.2940), c(0.94, 30),
    par_int = c(1L)
  ))$result
  grad_ds <- optedr:::gradient(y ~ th0 * exp(x / th1), c("th0", "th1"), c(10.4963, -3.2940))
  des_ds  <- data.frame(Point = c(1, 15), Weight = c(0.5, 0.5))
  eff     <- evaluate_promise(design_efficiency(des_ds, res))$result
  expect_gte(eff, 0)
})

test_that("design_efficiency works for A-Optimality", {
  res <- evaluate_promise(opt_des("A-Optimality", model, params, pvals, dspace))$result
  eff <- evaluate_promise(design_efficiency(des, res))$result
  expect_gte(eff, 0)
})

test_that("design_efficiency works for I-Optimality", {
  res <- evaluate_promise(opt_des(
    "I-Optimality", model, params, pvals, dspace, reg_int = c(380, 422)
  ))$result
  eff <- evaluate_promise(design_efficiency(des, res))$result
  expect_gte(eff, 0)
})

test_that("design_efficiency errors when second arg is not an optdes object", {
  expect_error(design_efficiency(des, des), "second argument must be an optdes object")
})

test_that("design_efficiency errors on wrong column names in first arg", {
  res     <- evaluate_promise(opt_des("D-Optimality", model, params, pvals, dspace))$result
  bad_des <- data.frame(Punto = c(300, 400), Weight = c(0.5, 0.5))
  expect_error(design_efficiency(bad_des, res), "'Point' and 'Weight'")
})

library(optedr)

# Shared fixtures
model  <- y ~ a * exp(-b / x)
params <- c("a", "b")
pvals  <- c(1, 1500)
dspace <- c(212, 422)

compound_di <- list(
  list(criterion = "D-Optimality", weight = 0.7),
  list(criterion = "I-Optimality", weight = 0.3, reg_int = c(380, 422))
)
compound_da <- list(
  list(criterion = "D-Optimality", weight = 0.5),
  list(criterion = "A-Optimality", weight = 0.5)
)

local({
  res_di <<- evaluate_promise(
    opt_des("Compound", model, params, pvals, dspace, compound = compound_di)
  )$result
  res_da <<- evaluate_promise(
    opt_des("Compound", model, params, pvals, dspace, compound = compound_da)
  )$result
})


# ── Result structure ──────────────────────────────────────────────────────────

test_that("Compound result has class optdes", {
  expect_s3_class(res_di, "optdes")
})

test_that("Compound result has all required components", {
  expect_named(res_di,
               c("optdes", "convergence", "sens", "criterion", "crit_value", "atwood"),
               ignore.order = TRUE)
  expect_equal(res_di$criterion, "Compound")
})

test_that("Compound weights sum to 1", {
  expect_equal(sum(res_di$optdes$Weight), 1, tolerance = 1e-6)
})

test_that("Compound atwood is non-negative and <= 100", {
  expect_gte(res_di$atwood, 0)
  expect_lte(res_di$atwood, 100)
})

test_that("Compound design has Point and Weight columns (1D)", {
  expect_true(all(c("Point", "Weight") %in% names(res_di$optdes)))
})

test_that("Compound hidden_value stores compound_specs list", {
  specs <- attr(res_di, "hidden_value")
  expect_type(specs, "list")
  expect_length(specs, 2L)
  expect_true(all(sapply(specs, function(s) "weight" %in% names(s))))
})

test_that("Compound weights are normalised to sum 1 even when input doesn't", {
  res_unnorm <- evaluate_promise(
    opt_des("Compound", model, params, pvals, dspace,
            compound = list(
              list(criterion = "D-Optimality", weight = 7),
              list(criterion = "A-Optimality", weight = 3)
            ))
  )$result
  specs <- attr(res_unnorm, "hidden_value")
  expect_equal(sum(sapply(specs, `[[`, "weight")), 1, tolerance = 1e-10)
})


# ── Sub-criterion combinations ────────────────────────────────────────────────

test_that("D + A Compound converges with atwood > 0", {
  expect_s3_class(res_da, "optdes")
  expect_gte(res_da$atwood, 0)
})

test_that("D + Ds Compound works", {
  res_dds <- evaluate_promise(
    opt_des("Compound", model, params, pvals, dspace,
            compound = list(
              list(criterion = "D-Optimality",  weight = 0.6),
              list(criterion = "Ds-Optimality", weight = 0.4, par_int = c(1L))
            ))
  )$result
  expect_s3_class(res_dds, "optdes")
  expect_equal(sum(res_dds$optdes$Weight), 1, tolerance = 1e-6)
})

test_that("Three-criterion compound (D + A + I) works", {
  res_3 <- evaluate_promise(
    opt_des("Compound", model, params, pvals, dspace,
            compound = list(
              list(criterion = "D-Optimality", weight = 0.5),
              list(criterion = "A-Optimality", weight = 0.3),
              list(criterion = "I-Optimality", weight = 0.2, reg_int = c(380, 422))
            ))
  )$result
  expect_s3_class(res_3, "optdes")
  specs <- attr(res_3, "hidden_value")
  expect_length(specs, 3L)
})


# ── print / summary ───────────────────────────────────────────────────────────

test_that("print.optdes for Compound shows composition", {
  out <- capture.output(print(res_di))
  expect_true(any(grepl("Compound", out)))
  expect_true(any(grepl("0.70", out) | grepl("0.7", out)))
})

test_that("summary.optdes for Compound shows criterion list", {
  out <- capture.output(summary(res_di))
  expect_true(any(grepl("Compound criterion", out)))
  expect_true(any(grepl("D-Optimality", out)))
  expect_true(any(grepl("I-Optimality", out)))
})

test_that("plot.optdes for Compound returns a ggplot", {
  p <- plot(res_di)
  expect_s3_class(p, "ggplot")
})


# ── design_efficiency ─────────────────────────────────────────────────────────

test_that("design_efficiency works for Compound: optimal vs itself = 1", {
  eff <- evaluate_promise(design_efficiency(res_di, res_di))$result
  expect_equal(eff, 1, tolerance = 1e-6)
})

test_that("design_efficiency for Compound: suboptimal design < 1", {
  des_unif <- data.frame(
    Point  = seq(212, 422, length.out = 5),
    Weight = rep(0.2, 5)
  )
  eff <- evaluate_promise(design_efficiency(des_unif, res_di))$result
  expect_gte(eff, 0)
})


# ── Validation errors ─────────────────────────────────────────────────────────

test_that("Compound errors when compound list is NULL", {
  expect_error(
    opt_des("Compound", model, params, pvals, dspace),
    "compound list"
  )
})

test_that("Compound errors when compound list has only one element", {
  expect_error(
    opt_des("Compound", model, params, pvals, dspace,
            compound = list(list(criterion = "D-Optimality", weight = 1))),
    "compound list"
  )
})

test_that("Compound errors when a sub-criterion is unknown", {
  expect_error(
    opt_des("Compound", model, params, pvals, dspace,
            compound = list(
              list(criterion = "D-Optimality",  weight = 0.5),
              list(criterion = "E-Optimality",  weight = 0.5)
            )),
    "Unknown sub-criterion"
  )
})

test_that("Compound warns when weights don't sum to 1", {
  expect_message(
    opt_des("Compound", model, params, pvals, dspace,
            compound = list(
              list(criterion = "D-Optimality", weight = 7),
              list(criterion = "A-Optimality", weight = 3)
            )),
    "normalising"
  )
})

test_that("Compound normalises weights to sum 1 after warning", {
  r <- suppressMessages(
    opt_des("Compound", model, params, pvals, dspace,
            compound = list(
              list(criterion = "D-Optimality", weight = 7),
              list(criterion = "A-Optimality", weight = 3)
            ))
  )
  ws <- sapply(attr(r, "hidden_value"), `[[`, "weight")
  expect_equal(sum(ws), 1, tolerance = 1e-10)
  expect_equal(ws[1], 0.7, tolerance = 1e-10)
})

test_that("Compound warns on exact duplicate criterion", {
  expect_warning(
    suppressMessages(
      opt_des("Compound", model, params, pvals, dspace,
              compound = list(
                list(criterion = "D-Optimality", weight = 0.6),
                list(criterion = "D-Optimality", weight = 0.4)
              ))
    ),
    "Duplicate"
  )
})

test_that("Compound does NOT warn for Ds with different par_int", {
  warns <- character(0)
  withCallingHandlers(
    suppressMessages(
      opt_des("Compound", model, params, pvals, dspace,
              compound = list(
                list(criterion = "Ds-Optimality", weight = 0.5, par_int = c(1L)),
                list(criterion = "Ds-Optimality", weight = 0.5, par_int = c(2L))
              ))
    ),
    warning = function(w) {
      warns <<- c(warns, conditionMessage(w))
      invokeRestart("muffleWarning")
    }
  )
  dup_warns <- grep("Duplicate", warns, value = TRUE)
  expect_length(dup_warns, 0L)
})

test_that("Compound errors when L-Optimality component has no matB", {
  expect_error(
    opt_des("Compound", model, params, pvals, dspace,
            compound = list(
              list(criterion = "D-Optimality", weight = 0.5),
              list(criterion = "L-Optimality", weight = 0.5)
            )),
    "matB"
  )
})

# KL-Optimality -----------------------------------------------------------
# KL divergence for exponential-family GLMs:
#   KL(x, mu1, mu2) = (1/phi) * [b(t2) - b(t1) - (t2 - t1) * b'(t1)]
# where t = link(mu) is the canonical parameter.
# The inner optimisation finds rival parameters that MINIMISE KL (adversarial).
# The cocktail algorithm finds the design that MAXIMISES KL.


# Build closure: x_val -> mu  (reference model evaluated at nominal params)
.make_mu_eval <- function(model, char_vars, par_values) {
  design_vars <- detect_design_vars(model, char_vars)
  ext_vars    <- c(char_vars, design_vars)
  arglist     <- lapply(ext_vars, function(v) NULL)
  f  <- as.function(append(stats::setNames(arglist, ext_vars), quote({})))
  f1 <- stats::deriv(model, char_vars, f)
  fn <- function(x_val) as.numeric(do.call(f1, as.list(c(par_values, x_val))))
  attr(fn, "design_vars") <- design_vars
  fn
}


# Build closure: (x_val, beta2_val) -> mu  (rival model, parameterised)
.make_rival_eval <- function(rival_model, rival_params) {
  design_vars <- detect_design_vars(rival_model, rival_params)
  ext_vars    <- c(rival_params, design_vars)
  arglist     <- lapply(ext_vars, function(v) NULL)
  f  <- as.function(append(stats::setNames(arglist, ext_vars), quote({})))
  f1 <- stats::deriv(rival_model, rival_params, f)
  # Filter x_val to only the design vars the rival model uses (supports
  # rivals that ignore some of the design dimensions).
  function(x_val, beta2_val) {
    x_use <- if (!is.null(names(x_val))) x_val[design_vars] else x_val
    as.numeric(do.call(f1, as.list(c(beta2_val, x_use))))
  }
}


# KL divergence at a single point using the exponential-family cumulant formula.
.kl_at_point <- function(x, mu1_fn, rival_eval_fn, beta2_val, family, phi) {
  mu1 <- mu1_fn(x)
  mu2 <- rival_eval_fn(x, beta2_val)
  t1  <- family$link(mu1)
  t2  <- family$link(mu2)
  (family$b(t2) - family$b(t1) - (t2 - t1) * family$b_prime(t1)) / phi
}


# Integrated KL over a design given kl_fun(x, beta2).
.kl_integrated <- function(design, kl_fun, beta2) {
  dc <- coord_cols(design)
  sum(design$Weight * vapply(seq_len(nrow(design)), function(j) {
    x_j <- if (identical(dc, "Point")) design$Point[[j]]
            else stats::setNames(unlist(design[j, dc]), dc)
    kl_fun(x_j, beta2)
  }, numeric(1L)))
}


# Inner optimisation: find beta2 minimising integrated KL (adversarial rival).
.kl_minopt <- function(design, kl_fun, beta2_init, lower = -Inf, upper = Inf) {
  obj     <- function(b2) .kl_integrated(design, kl_fun, b2)
  bounded <- !all(is.infinite(lower)) || !all(is.infinite(upper))
  method  <- if (bounded) "L-BFGS-B" else "BFGS"
  res <- tryCatch(
    stats::optim(beta2_init, obj, method = method, lower = lower, upper = upper),
    error = function(e) stats::optim(beta2_init, obj, method = "Nelder-Mead")
  )
  list(beta2_star = res$par, kl_val = res$value)
}


#' Cocktail Algorithm for KL-Optimality
#'
#' @inherit WFMult return params
#' @param kl_fun function(x, beta2); returns the KL divergence at design point
#'   \code{x} given rival parameters \code{beta2}.
#' @param beta2_init numeric vector of initial rival parameter values.
#' @param lower lower bounds for rival parameters in the inner optimisation.
#' @param upper upper bounds for rival parameters in the inner optimisation.
#' @param kl_meta list with summary metadata (\code{type}, and optionally
#'   \code{family} and \code{phi} for the standard path).
#' @family cocktail algorithms
KLWFMult <- function(init_design, kl_fun, beta2_init, lower, upper,
                     design_space, grid.length,
                     join_thresh, delete_thresh,
                     delta_weights, tol, tol2, max_iter,
                     kl_meta = list(type = "kl_fun")) {
  dc        <- coord_cols(init_design)
  multi     <- is_multifactor(dc)
  maxiter_weights <- 100L
  crit_val  <- numeric(max_iter * (maxiter_weights + 1L) + 1L)
  index     <- 1L
  beta2_cur <- beta2_init
  cli::cli_progress_bar("Calculating optimal design")

  for (i in seq_len(max_iter)) {
    cli::cli_progress_update()

    inner     <- .kl_minopt(init_design, kl_fun, beta2_cur, lower, upper)
    beta2_cur <- inner$beta2_star
    crit_val[index] <- inner$kl_val
    index <- index + 1L

    sensKL <- local({ b2 <- beta2_cur; function(x) kl_fun(x, b2) })
    xmax   <- findmax(sensKL, design_space, grid.length)

    if ((as.numeric(sensKL(xmax)) - crit_val[index - 1L]) < tol2) {
      message("\n", crayon::blue(cli::symbol$info),
              " Stop condition reached: difference between sensitivity and criterion < ", tol2)
      break
    }
    init_design <- update_design(init_design, xmax, join_thresh, 1 / (index + 2))
    iter <- 1L; stopw <- FALSE
    while (!stopw) {
      weightsInit <- init_design$Weight
      inner     <- .kl_minopt(init_design, kl_fun, beta2_cur, lower, upper)
      beta2_cur <- inner$beta2_star
      crit      <- inner$kl_val
      crit_val[index] <- crit
      index <- index + 1L
      sensKL <- local({ b2 <- beta2_cur; function(x) kl_fun(x, b2) })
      init_design$Weight <- update_weightsI(init_design, sensKL, crit, delta_weights)
      stopw <- max(abs(weightsInit - init_design$Weight)) < tol || iter >= maxiter_weights
      iter  <- iter + 1L
    }
    init_design <- delete_points(init_design, delete_thresh)
    if (i %% 5 == 0) init_design <- update_design_total(init_design, join_thresh)
    if (i == max_iter)
      message("\n", crayon::blue(cli::symbol$info),
              " Stop condition not reached, max iterations performed")
  }

  cli::cli_progress_update(force = TRUE)
  base::cat("")

  inner     <- .kl_minopt(init_design, kl_fun, beta2_cur, lower, upper)
  beta2_cur <- inner$beta2_star
  crit_val[index] <- inner$kl_val
  crit_val <- crit_val[1L:(length(crit_val) - sum(crit_val == 0))]
  conv_plot <- plot_convergence(data.frame("criteria" = crit_val,
                                           "step"     = seq_along(crit_val)))

  init_design <- init_design[order(init_design[[dc[1L]]]), ]
  rownames(init_design) <- NULL

  sensKL        <- local({ b2 <- beta2_cur; function(x) kl_fun(x, b2) })
  xmax          <- findmax(sensKL, design_space, grid.length * 10L)
  kl_crit_final <- crit_val[length(crit_val)]
  atwood        <- kl_crit_final / as.numeric(sensKL(xmax)) * 100
  check_atwood(atwood)
  message(crayon::blue(cli::symbol$info), " The lower bound for efficiency is ", atwood, "%")

  plot_opt <- .make_sens_plot(multi, design_space, sensKL, dc, init_design, kl_crit_final)

  grad_stub <- function(x) NULL
  attr(grad_stub, "design_vars") <- dc

  l_return <- list("optdes"      = init_design,
                   "convergence" = conv_plot,
                   "sens"        = plot_opt,
                   "criterion"   = "KL-Optimality",
                   "crit_value"  = kl_crit_final,
                   "atwood"      = atwood)
  attr(l_return, "hidden_value") <- c(
    kl_meta,
    list(kl_fun     = kl_fun,
         beta2_star = beta2_cur,
         lower      = lower,
         upper      = upper)
  )
  attr(l_return, "gradient")      <- grad_stub
  attr(l_return, "design_space")  <- design_space
  attr(l_return, "crit_function") <- local({
    kl_fn_ <- kl_fun; b2_ <- beta2_cur; lo_ <- lower; up_ <- upper
    function(design) .kl_minopt(design, kl_fn_, b2_, lo_, up_)$kl_val
  })
  class(l_return) <- "optdes"
  l_return
}


#' Build a KL-divergence point function for use with opt_des()
#'
#' @description
#' Returns a \code{function(x, beta2)} computing the point KL divergence at
#' design point \code{x} given rival parameters \code{beta2}.  The result is
#' passed to \code{\link{opt_des}} via the \code{kl_fun} argument, allowing
#' discrimination between models with different family, dispersion, or mean
#' structure without having to derive the formula manually.
#'
#' Supported family pairs:
#' \itemize{
#'   \item Same family and same \code{phi}: any of
#'     \code{"Normal"}, \code{"Poisson"}, \code{"Binomial"}, \code{"Gamma"}.
#'     Uses the standard exponential-family cumulant formula.
#'   \item \code{"Normal"} vs \code{"Normal"} with \code{phi1 != phi2}:
#'     \eqn{KL = \frac{1}{2}\bigl[\log(\phi_2/\phi_1) + \phi_1/\phi_2 + (\mu_1-\mu_2)^2/\phi_2 - 1\bigr]}.
#'   \item \code{"Gamma"} vs \code{"Gamma"} with different shape
#'     (\eqn{k_i = 1/\phi_i}): closed form involving \code{digamma} and
#'     \code{lgamma}.
#' }
#' For other cross-family pairs provide \code{kl_fun} directly.
#'
#' @param family1 character; reference distribution (\code{"Normal"},
#'   \code{"Poisson"}, \code{"Binomial"} or \code{"Gamma"}).
#' @param model1 formula; reference model mean function.
#' @param params1 character vector; parameter names in \code{model1}.
#' @param par_values1 numeric vector; nominal values for the reference parameters.
#' @param family2 character; rival distribution (default: same as \code{family1}).
#' @param model2 formula; rival model mean function (default: same as \code{model1}).
#' @param params2 character vector; rival parameter names (optimised internally).
#'   Default: same as \code{params1}.
#' @param phi1 positive numeric; dispersion of the reference
#'   (\code{Normal}: variance; \code{Gamma}: \eqn{1/\text{shape}}).
#' @param phi2 positive numeric; dispersion of the rival (default: same as \code{phi1}).
#'
#' @return A function \code{function(x, beta2)} giving the point KL divergence.
#'   Works for both 1-D (\code{x} scalar) and multi-factor designs (\code{x}
#'   named numeric vector).
#' @export
#'
#' @examples
#' # Same family (Normal), different model structures
#' kl_fn <- make_kl_fun(
#'   "Normal",
#'   model1 = y ~ Vmax * x / (Km + x), params1 = c("Vmax", "Km"),
#'   par_values1 = c(2, 1),
#'   model2 = y ~ a * x, params2 = "a"
#' )
#' kl_fn(x = 1, beta2 = 0.5)
#'
#' \donttest{
#' # Normal vs Normal with different variance (phi2 = 4)
#' kl_fn2 <- make_kl_fun(
#'   "Normal",
#'   model1 = y ~ a * exp(-b * x), params1 = c("a", "b"),
#'   par_values1 = c(1, 0.5), phi1 = 1,
#'   family2 = "Normal",
#'   model2 = y ~ c * exp(-d * x), params2 = c("c", "d"), phi2 = 4
#' )
#' opt_des("KL-Optimality",
#'         model = y ~ a * exp(-b * x), parameters = c("a", "b"),
#'         par_values = c(1, 0.5), design_space = c(0, 4),
#'         kl_fun = kl_fn2, rival_pars = c(1, 1),
#'         rival_lower = c(0.5, 0.8), rival_upper = c(2, 1.5))
#' }
make_kl_fun <- function(family1, model1, params1, par_values1,
                        family2     = family1,
                        model2      = model1,
                        params2     = params1,
                        phi1        = 1,
                        phi2        = phi1) {

  if (!is.character(family1) || length(family1) != 1L)
    stop("'family1' must be a single character string.", call. = FALSE)
  if (!is.character(family2) || length(family2) != 1L)
    stop("'family2' must be a single character string.", call. = FALSE)
  if (!is.numeric(phi1) || phi1 <= 0)
    stop("'phi1' must be a positive number.", call. = FALSE)
  if (!is.numeric(phi2) || phi2 <= 0)
    stop("'phi2' must be a positive number.", call. = FALSE)

  mu1_fn        <- .make_mu_eval(model1, params1, par_values1)
  rival_eval_fn <- .make_rival_eval(model2, params2)

  same_fam <- identical(family1, family2)
  same_phi <- isTRUE(all.equal(phi1, phi2))

  if (same_fam && same_phi) {
    fam   <- make_glm_family(family1)
    phi1_ <- phi1
    return(function(x, beta2)
      .kl_at_point(x, mu1_fn, rival_eval_fn, beta2, fam, phi1_))
  }

  if (identical(family1, "Normal") && identical(family2, "Normal")) {
    # KL(N(mu1,phi1) || N(mu2,phi2)) = 0.5*(log(phi2/phi1) + phi1/phi2 + (mu1-mu2)^2/phi2 - 1)
    phi1_ <- phi1; phi2_ <- phi2
    return(function(x, beta2) {
      mu1 <- as.numeric(mu1_fn(x))
      mu2 <- as.numeric(rival_eval_fn(x, beta2))
      0.5 * (log(phi2_/phi1_) + phi1_/phi2_ + (mu1 - mu2)^2 / phi2_ - 1)
    })
  }

  if (identical(family1, "Gamma") && identical(family2, "Gamma")) {
    # KL(Gamma(mu1,k1) || Gamma(mu2,k2)), ki = 1/phi_i
    # = k2*log(k1/k2) + k2*log(mu2/mu1) + (k1-k2)*digamma(k1)
    #   - k1 + k2*mu1/mu2 - lgamma(k1) + lgamma(k2)
    k1    <- 1 / phi1; k2 <- 1 / phi2
    const <- k2 * log(k1/k2) + (k1 - k2) * digamma(k1) - k1 - lgamma(k1) + lgamma(k2)
    return(function(x, beta2) {
      mu1 <- as.numeric(mu1_fn(x))
      mu2 <- as.numeric(rival_eval_fn(x, beta2))
      const + k2 * log(mu2/mu1) + k2 * mu1/mu2
    })
  }

  stop("Family pair ('", family1, "', '", family2, "') is not supported by ",
       "make_kl_fun(). Provide kl_fun directly to opt_des().", call. = FALSE)
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  fig.width  = 6,
  fig.height = 4,
  out.width  = "90%"
)

## ----setup--------------------------------------------------------------------
library(optedr)

## ----kl-mm--------------------------------------------------------------------
result_kl_mm <- opt_des(
  "KL-Optimality",
  model        = y ~ Vmax * x / (Km + x),
  parameters   = c("Vmax", "Km"),
  par_values   = c(2, 1),
  design_space = c(0.1, 5),
  rival_model  = y ~ a * x,
  rival_params = c("a"),
  rival_pars   = c(1),
  rival_lower  = c(0.2),
  rival_upper  = c(2.5),
  family       = "Normal",
  phi          = 1
)
result_kl_mm

## ----kl-mm-summary------------------------------------------------------------
summary(result_kl_mm)

## ----kl-mm-plot, fig.cap = "KL sensitivity function: MM vs linear rival."-----
plot(result_kl_mm)

## ----kl-mm-eff----------------------------------------------------------------
design_unif <- data.frame(
  Point  = c(0.1, 1.3, 2.5, 3.8, 5.0),
  Weight = rep(1/5, 5)
)
eff_kl <- design_efficiency(design_unif, result_kl_mm)
cat("Efficiency of uniform design:", round(eff_kl * 100, 2), "%\n")

## ----kl-poisson---------------------------------------------------------------
result_kl_pois <- opt_des(
  "KL-Optimality",
  model        = y ~ exp(a - b * x),
  parameters   = c("a", "b"),
  par_values   = c(2, 0.5),
  design_space = c(0, 4),
  rival_pars   = c(2, 1.0),
  rival_lower  = c(1.5, 0.8),
  rival_upper  = c(2.5, 1.5),
  family       = "Poisson",
  phi          = 1
)
result_kl_pois
summary(result_kl_pois)

## ----kl-poisson-plot, fig.cap = "KL sensitivity for Poisson decay model."-----
plot(result_kl_pois)

## ----kl-poisson-rival---------------------------------------------------------
hv <- attr(result_kl_pois, "hidden_value")
cat("Optimal rival: a =", round(hv$beta2_star[1], 3),
    " b =", round(hv$beta2_star[2], 3), "\n")

## ----kl-variances-------------------------------------------------------------
kl_fn_var <- make_kl_fun(
  "Normal",
  model1      = y ~ a * exp(-b * x),
  params1     = c("a", "b"),
  par_values1 = c(1, 0.5),
  phi1        = 1,
  family2     = "Normal",
  model2      = y ~ c * exp(-d * x),
  params2     = c("c", "d"),
  phi2        = 4
)

result_kl_var <- opt_des(
  "KL-Optimality",
  model        = y ~ a * exp(-b * x),
  parameters   = c("a", "b"),
  par_values   = c(1, 0.5),
  design_space = c(0, 4),
  kl_fun       = kl_fn_var,
  rival_pars   = c(1, 1.0),
  rival_lower  = c(0.5, 0.8),
  rival_upper  = c(2.0, 1.5)
)
result_kl_var
summary(result_kl_var)

## ----kl-var-plot, fig.cap = "KL sensitivity: Normal phi=1 vs Normal phi=4."----
plot(result_kl_var)

## ----kl-2d--------------------------------------------------------------------
kl_fn_2d <- make_kl_fun(
  "Normal",
  model1      = y ~ Vmax * x1 * x2 / ((Km1 + x1) * (Km2 + x2)),
  params1     = c("Vmax", "Km1", "Km2"),
  par_values1 = c(1, 1, 1),
  model2      = y ~ alpha * x1,
  params2     = "alpha"
)

result_kl_2d <- opt_des(
  "KL-Optimality",
  model        = y ~ Vmax * x1 * x2 / ((Km1 + x1) * (Km2 + x2)),
  parameters   = c("Vmax", "Km1", "Km2"),
  par_values   = c(1, 1, 1),
  design_space = list(x1 = c(0.1, 5), x2 = c(0.1, 5)),
  kl_fun       = kl_fn_2d,
  rival_pars   = c(0.5),
  rival_lower  = c(0.05),
  rival_upper  = c(2.0)
)
result_kl_2d

## ----kl-2d-plot, fig.cap = "KL sensitivity heatmap: 2D MM vs linear rival."----
plot(result_kl_2d)

## ----kl-hill------------------------------------------------------------------
kl_fun_hill <- function(x, beta2) {
  sigma_sq <- beta2[1]
  eta <- (1.70 - 0.137) * (x / 111)^(-1.03) / (1 + (x / 111)^(-1.03)) + 0.137
  0.5 * (eta^2 / sigma_sq - 1 + log(sigma_sq / eta^2))
}

result_kl_hill <- opt_des(
  "KL-Optimality",
  model        = y ~ (Econ - b) * (x / IC50)^s / (1 + (x / IC50)^s) + b,
  parameters   = c("Econ", "b", "IC50", "s"),
  par_values   = c(1.70, 0.137, 111, -1.03),
  design_space = c(0.01, 1500),
  kl_fun       = kl_fun_hill,
  rival_pars   = c(1.0),
  rival_lower  = c(1e-4),
  rival_upper  = c(1e6)
)
result_kl_hill
summary(result_kl_hill)

## ----kl-hill-plot, fig.cap = "KL sensitivity for the Hill model (error structure discrimination)."----
plot(result_kl_hill)

## ----kl-hill-rival------------------------------------------------------------
hv_hill <- attr(result_kl_hill, "hidden_value")
cat("Optimal rival sigma_abs^2 =", round(hv_hill$beta2_star, 4), "\n")

# Infrastructure for auto-detecting design variables and supporting multi-factor designs.
# Single-factor models use the variable name "x"; multi-factor models use "x1", "x2", ...
# Detection is performed by inspecting the formula and excluding the response and parameters.


#' Detect design variables from a model formula
#'
#' @description
#' Inspects the formula to identify which symbols are design variables (as opposed to
#' parameters or the response).  Enforces the naming convention: \code{"x"} for
#' single-factor models and \code{"x1"}, \code{"x2"}, \ldots for multi-factor models.
#'
#' @param model A formula describing the model.
#' @param parameters Character vector of parameter names (excluded from design variables).
#'
#' @return Character vector of design variable names, sorted in numerical order for
#'   multi-factor models (e.g. \code{c("x1", "x2", "x10")}).
#'
detect_design_vars <- function(model, parameters) {
  response  <- as.character(model[[2L]])
  all_vars  <- all.vars(model)
  remaining <- setdiff(all_vars, c(response, parameters))

  has_x   <- "x" %in% remaining
  x_multi <- grep("^x[0-9]+$", remaining, value = TRUE)

  # Sort numerically: x1 < x2 < x10 (not lexicographic x1 < x10 < x2)
  if (length(x_multi) > 0L)
    x_multi <- x_multi[order(as.integer(sub("^x", "", x_multi)))]

  if (has_x && length(x_multi) > 0L)
    stop(
      "The model formula mixes the single-factor variable 'x' with multi-factor ",
      "variables 'x1', 'x2', ... - use one convention or the other.",
      call. = FALSE
    )

  if (!has_x && length(x_multi) == 0L)
    stop(
      "No design variable found in the formula. ",
      "Use 'x' for single-factor models or 'x1', 'x2', ... for multi-factor models.",
      call. = FALSE
    )

  if (has_x) "x" else x_multi
}


# Returns TRUE when design_vars has more than one element (multi-factor model).
is_multifactor <- function(design_vars) length(design_vars) > 1L


# Returns the coordinate column names of a design data.frame (everything except "Weight").
coord_cols <- function(design) setdiff(names(design), "Weight")


# Renames "Point" to the design variable name for single-factor user-supplied designs,
# enabling backward-compatible use of data.frame(Point, Weight).
normalize_design_cols <- function(design, design_vars) {
  if ("Point" %in% names(design) && length(design_vars) == 1L)
    names(design)[names(design) == "Point"] <- design_vars[1L]
  design
}


# Latin Hypercube Sample: n points in d dimensions.
# design_space: named list with one element c(min, max) per factor.
# Returns an n x d numeric matrix with column names matching names(design_space).
lhs_sample <- function(n, design_space) {
  d     <- length(design_space)
  nms   <- names(design_space)
  perms <- replicate(d, sample.int(n))                        # n x d stratification
  u     <- (perms - 1L + matrix(stats::runif(n * d), n, d)) / n
  for (j in seq_len(d))
    u[, j] <- design_space[[j]][1L] + u[, j] * diff(design_space[[j]])
  colnames(u) <- nms
  u
}


# Coerce a user-supplied design_space to a canonical named list.
# Accepts c(min, max) for 1D (for backward compat) or a named list for multi-factor.
# design_vars: character vector from detect_design_vars().
canonicalise_design_space <- function(design_space, design_vars) {
  if (is.list(design_space)) {
    if (!setequal(names(design_space), design_vars))
      stop(
        "names(design_space) must match the design variables detected in the formula (",
        paste(design_vars, collapse = ", "), ").",
        call. = FALSE
      )
    return(design_space[design_vars])          # ensure consistent ordering
  }
  # Numeric vector: only valid for single-factor models
  if (length(design_vars) > 1L)
    stop(
      "design_space must be a named list for multi-factor models, e.g. ",
      "list(x1 = c(0, 10), x2 = c(0, 5)).",
      call. = FALSE
    )
  if (!is.numeric(design_space) || length(design_space) != 2L)
    stop("design_space must be a numeric vector of length 2.", call. = FALSE)
  stats::setNames(list(design_space), design_vars)
}

library(optedr)

# ── Infrastructure helpers ────────────────────────────────────────────────────

test_that("detect_design_vars finds 'x' for single-factor models", {
  expect_equal(optedr:::detect_design_vars(y ~ a * exp(-b / x), c("a", "b")), "x")
})

test_that("detect_design_vars finds x1, x2 for two-factor models", {
  dvars <- optedr:::detect_design_vars(y ~ a * x1 + b * x2, c("a", "b"))
  expect_equal(dvars, c("x1", "x2"))
})

test_that("detect_design_vars sorts multi-factor variables numerically", {
  dvars <- optedr:::detect_design_vars(y ~ a * x1 + b * x2 + c * x10, c("a", "b", "c"))
  expect_equal(dvars, c("x1", "x2", "x10"))
})

test_that("detect_design_vars errors when x and x1/x2 are mixed", {
  expect_error(
    optedr:::detect_design_vars(y ~ a * x + b * x1, c("a", "b")),
    "mixes"
  )
})

test_that("detect_design_vars errors when no design variable is found", {
  expect_error(
    optedr:::detect_design_vars(y ~ a + b, c("a", "b")),
    "No design variable"
  )
})

test_that("canonicalise_design_space accepts numeric c(min,max) for 1D", {
  ds <- optedr:::canonicalise_design_space(c(0, 10), "x")
  expect_equal(ds, list(x = c(0, 10)))
})

test_that("canonicalise_design_space accepts named list for multi-factor", {
  inp <- list(x1 = c(0, 5), x2 = c(1, 8))
  ds  <- optedr:::canonicalise_design_space(inp, c("x1", "x2"))
  expect_equal(ds, list(x1 = c(0, 5), x2 = c(1, 8)))
})

test_that("canonicalise_design_space errors when list names do not match design vars", {
  expect_error(
    optedr:::canonicalise_design_space(list(z1 = c(0, 5), z2 = c(0, 5)), c("x1", "x2")),
    "names\\(design_space\\)"
  )
})

test_that("canonicalise_design_space errors when numeric vector used for multi-factor model", {
  expect_error(
    optedr:::canonicalise_design_space(c(0, 10), c("x1", "x2")),
    "named list"
  )
})

test_that("normalize_design_cols renames 'Point' to design variable for 1D", {
  d <- data.frame(Point = c(1, 2), Weight = c(0.5, 0.5))
  d2 <- optedr:::normalize_design_cols(d, "x")
  expect_true("x" %in% names(d2))
  expect_false("Point" %in% names(d2))
})

test_that("normalize_design_cols leaves multi-factor columns unchanged", {
  d <- data.frame(x1 = c(1, 2), x2 = c(3, 4), Weight = c(0.5, 0.5))
  d2 <- optedr:::normalize_design_cols(d, c("x1", "x2"))
  expect_equal(names(d2), c("x1", "x2", "Weight"))
})

test_that("lhs_sample returns matrix with correct dimensions and bounds", {
  ds  <- list(x1 = c(0, 5), x2 = c(10, 20))
  pts <- optedr:::lhs_sample(50L, ds)
  expect_equal(dim(pts), c(50L, 2L))
  expect_equal(colnames(pts), c("x1", "x2"))
  expect_true(all(pts[, "x1"] >= 0 & pts[, "x1"] <= 5))
  expect_true(all(pts[, "x2"] >= 10 & pts[, "x2"] <= 20))
})

test_that("is_multifactor correctly identifies single vs multi factor", {
  expect_false(optedr:::is_multifactor("x"))
  expect_true(optedr:::is_multifactor(c("x1", "x2")))
  expect_true(optedr:::is_multifactor(c("x1", "x2", "x3")))
})


# ── 2-factor opt_des (bisubstrate Michaelis-Menten) ──────────────────────────

# Shared fixture — kept fast with max_iter = 10
local({
  mm2d_res <<- evaluate_promise(opt_des(
    "D-Optimality",
    model       = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
    parameters  = c("Vmax", "K1", "K2"),
    par_values  = c(1, 1, 1),
    design_space = list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
    max_iter    = 10L
  ))$result
})

test_that("opt_des 2D result has class optdes", {
  expect_s3_class(mm2d_res, "optdes")
})

test_that("opt_des 2D result has all required components", {
  expect_named(mm2d_res,
               c("optdes", "convergence", "sens", "criterion", "crit_value", "atwood"),
               ignore.order = TRUE)
})

test_that("opt_des 2D design has correct columns (x1, x2, Weight)", {
  expect_true(all(c("x1", "x2", "Weight") %in% names(mm2d_res$optdes)))
  expect_false("Point" %in% names(mm2d_res$optdes))
})

test_that("opt_des 2D weights sum to 1", {
  expect_equal(sum(mm2d_res$optdes$Weight), 1, tolerance = 1e-6)
})

test_that("opt_des 2D support points lie within design_space", {
  des <- mm2d_res$optdes
  expect_true(all(des$x1 >= 0.1 & des$x1 <= 10))
  expect_true(all(des$x2 >= 0.1 & des$x2 <= 10))
})

test_that("opt_des 2D atwood is a non-negative number", {
  expect_true(is.numeric(mm2d_res$atwood))
  expect_gte(mm2d_res$atwood, 0)
})

test_that("opt_des 2D has at least 3 support points (>= num parameters)", {
  expect_gte(nrow(mm2d_res$optdes), 3L)
})

test_that("opt_des 2D gradient has design_vars attribute set to c('x1','x2')", {
  dvars <- attr(attr(mm2d_res, "gradient"), "design_vars")
  expect_equal(dvars, c("x1", "x2"))
})

test_that("opt_des 2D design_space attribute is stored on result", {
  ds <- attr(mm2d_res, "design_space")
  expect_equal(ds$x1, c(0.1, 10))
  expect_equal(ds$x2, c(0.1, 10))
})


# ── print / summary for multi-factor ─────────────────────────────────────────

test_that("print.optdes for 2D shows '2 factors' header", {
  out <- capture.output(print(mm2d_res))
  expect_true(any(grepl("2 factors", out, fixed = TRUE)))
})

test_that("summary.optdes for 2D shows design_space and criterion", {
  out <- capture.output(summary(mm2d_res))
  expect_true(any(grepl("Design space", out, fixed = TRUE)))
  expect_true(any(grepl("D-Optimality", out, fixed = TRUE)))
  expect_true(any(grepl("Atwood", out, fixed = TRUE)))
})

test_that("plot.optdes for 2D returns heatmap (ggplot)", {
  expect_s3_class(plot(mm2d_res), "ggplot")
})


# ── plot for d > 2 (pairs scatter matrix) ────────────────────────────────────

local({
  mm3d_res <<- evaluate_promise(opt_des(
    "D-Optimality",
    model        = y ~ Vmax * x1 * x2 * x3 / ((K1+x1) * (K2+x2) * (K3+x3)),
    parameters   = c("Vmax", "K1", "K2", "K3"),
    par_values   = c(1, 1, 1, 1),
    design_space = list(x1 = c(0.1, 10), x2 = c(0.1, 10), x3 = c(0.1, 10)),
    max_iter     = 5L
  ))$result
})

test_that("plot.optdes for 3D returns a ggplot", {
  expect_s3_class(plot(mm3d_res), "ggplot")
})

test_that("plot.optdes for 3D uses facet_grid with C(3,2)=3 xvar/yvar combos", {
  p <- plot(mm3d_res)
  expect_s3_class(p$facet, "FacetGrid")
  # 3 unique (xvar, yvar) pairs
  combos <- nrow(unique(p$data[, c("xvar","yvar")]))
  expect_equal(combos, 3L)
})

test_that("plot.optdes for 3D xvar/yvar are design variable names", {
  p <- plot(mm3d_res)
  expect_true(all(levels(p$data$xvar) %in% c("x1","x2","x3")))
  expect_true(all(levels(p$data$yvar) %in% c("x1","x2","x3")))
})

test_that("plot.optdes for 4D has C(4,2)=6 xvar/yvar combos", {
  r4d <- evaluate_promise(opt_des(
    "D-Optimality",
    y ~ a*x1 + b*x2 + c*x3 + d*x4,
    c("a","b","c","d"), c(1,1,1,1),
    list(x1=c(0,1), x2=c(0,1), x3=c(0,1), x4=c(0,1)),
    max_iter = 5L
  ))$result
  p <- plot(r4d)
  combos <- nrow(unique(p$data[, c("xvar","yvar")]))
  expect_equal(combos, 6L)
})


# ── design_efficiency for multi-factor ───────────────────────────────────────

test_that("design_efficiency accepts x1/x2 column names for 2D designs", {
  # Use the full optimal design with equal weights so M is non-singular
  des_eq <- mm2d_res$optdes
  des_eq$Weight <- rep(1 / nrow(des_eq), nrow(des_eq))
  eff <- evaluate_promise(design_efficiency(des_eq, mm2d_res))$result
  expect_true(is.numeric(eff))
  expect_gte(eff, 0)
  expect_lte(eff, 1 + 1e-4)   # allow small floating-point overshoot from equal-weight design
})

test_that("design_efficiency for 2D errors on wrong column names", {
  bad <- data.frame(z1 = c(1, 2), z2 = c(1, 2), Weight = c(0.5, 0.5))
  expect_error(design_efficiency(bad, mm2d_res), "'x1'")
})

test_that("design_efficiency efficiency of optimal design is 1 for 2D", {
  eff <- evaluate_promise(design_efficiency(mm2d_res, mm2d_res))$result
  expect_equal(eff, 1, tolerance = 1e-6)
})


# ── augment multi-factor ──────────────────────────────────────────────────────

local({
  init_aug <<- data.frame(x1 = c(1, 10), x2 = c(1, 10), Weight = c(0.5, 0.5))

  region_res <<- evaluate_promise(get_augment_region(
    "D-Optimality", init_aug, 0.25,
    y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
    c("Vmax", "K1", "K2"), c(1, 1, 1),
    list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
    calc_optimal_design = FALSE,
    delta_val = 0.85
  ))$result
})

test_that("get_augment_region 2D returns an augment_region with region and eff_fun", {
  expect_s3_class(region_res, "augment_region")
  expect_true("region"    %in% names(region_res))
  expect_true("eff_fun"   %in% names(region_res))
  expect_true("delta_val" %in% names(region_res))
  expect_equal(region_res$delta_val, 0.85)
})

test_that("get_augment_region 2D region has x1, x2 and efficiency columns", {
  cands <- region_res$region
  expect_true(all(c("x1", "x2", "efficiency") %in% names(cands)))
  expect_true(all(cands$efficiency >= 0.85))
})

test_that("get_augment_region 2D returns a ggplot for d=2", {
  expect_s3_class(region_res$plot, "ggplot")
})

test_that("get_augment_region 2D eff_fun is callable and returns scalar", {
  f <- region_res$eff_fun
  v <- f(c(x1 = 5, x2 = 5))
  expect_length(as.numeric(v), 1L)
  expect_true(is.finite(as.numeric(v)))
})

test_that("augment_design 2D adds new_points within candidate region", {
  init_aug2 <- data.frame(x1 = c(1, 10), x2 = c(1, 10), Weight = c(0.5, 0.5))
  # Pick a candidate point
  cands <- region_res$region
  pt    <- cands[1L, c("x1", "x2")]
  pt$Weight <- 1
  aug <- evaluate_promise(augment_design(
    "D-Optimality", init_aug2, 0.25,
    y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
    c("Vmax", "K1", "K2"), c(1, 1, 1),
    list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
    calc_optimal_design = FALSE,
    delta_val = 0.85,
    new_points = pt
  ))$result
  expect_s3_class(aug, "data.frame")
  expect_true(all(c("x1", "x2", "Weight") %in% names(aug)))
  expect_equal(sum(aug$Weight), 1, tolerance = 1e-6)
  expect_equal(nrow(aug), 3L)
})

test_that("augment_design 2D errors when new_point is outside candidate region", {
  init_bad <- data.frame(x1 = c(1, 10), x2 = c(1, 10), Weight = c(0.5, 0.5))
  bad_pt   <- data.frame(x1 = 0.11, x2 = 0.11, Weight = 1)  # very low efficiency
  expect_error(
    augment_design(
      "D-Optimality", init_bad, 0.25,
      y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
      c("Vmax", "K1", "K2"), c(1, 1, 1),
      list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
      calc_optimal_design = FALSE,
      delta_val = 0.85,
      new_points = bad_pt
    ),
    "outside the candidate region"
  )
})

test_that("augment_design Ds-Optimality works for multi-factor with valid new_points", {
  init_ds <- data.frame(x1 = c(0.8, 10, 5), x2 = c(10, 0.8, 5), Weight = rep(1/3, 3))
  reg_ds  <- evaluate_promise(get_augment_region(
    "Ds-Optimality", init_ds, 0.25,
    y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
    c("Vmax", "K1", "K2"), c(1, 1, 1),
    list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
    calc_optimal_design = FALSE,
    par_int = c(1), delta_val = 0.85
  ))$result
  expect_s3_class(reg_ds, "augment_region")
  expect_true("region" %in% names(reg_ds))
})


# ── A / I / L augment 2D ─────────────────────────────────────────────────────

local({
  init_ail <<- data.frame(x1 = c(0.8, 10, 5), x2 = c(10, 0.8, 5), Weight = rep(1/3, 3))
  model_ail <<- y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2))
  pars_ail  <<- c("Vmax", "K1", "K2")
  vals_ail  <<- c(1, 1, 1)
  ds_ail    <<- list(x1 = c(0.1, 10), x2 = c(0.1, 10))
})

test_that("get_augment_region A-Optimality 2D returns augment_region", {
  reg <- evaluate_promise(get_augment_region(
    "A-Optimality", init_ail, 0.25, model_ail, pars_ail, vals_ail, ds_ail,
    calc_optimal_design = FALSE, delta_val = 0.85
  ))$result
  expect_s3_class(reg, "augment_region")
  expect_gte(nrow(reg$region), 1L)
  expect_true(all(reg$region$efficiency >= 0.85))
})

test_that("augment_design A-Optimality 2D augments correctly", {
  reg <- evaluate_promise(get_augment_region(
    "A-Optimality", init_ail, 0.25, model_ail, pars_ail, vals_ail, ds_ail,
    calc_optimal_design = FALSE, delta_val = 0.85
  ))$result
  best <- reg$region[which.max(reg$region$efficiency), ]
  new_pt <- data.frame(x1 = best$x1, x2 = best$x2, Weight = 1)
  aug <- evaluate_promise(augment_design(
    "A-Optimality", init_ail, 0.25, model_ail, pars_ail, vals_ail, ds_ail,
    calc_optimal_design = FALSE, delta_val = 0.85, new_points = new_pt
  ))$result
  expect_s3_class(aug, "data.frame")
  expect_equal(sum(aug$Weight), 1, tolerance = 1e-6)
  expect_equal(nrow(aug), 4L)
})

test_that("get_augment_region I-Optimality 2D returns augment_region", {
  reg <- evaluate_promise(get_augment_region(
    "I-Optimality", init_ail, 0.25, model_ail, pars_ail, vals_ail, ds_ail,
    calc_optimal_design = FALSE, delta_val = 0.85
  ))$result
  expect_s3_class(reg, "augment_region")
  expect_gte(nrow(reg$region), 1L)
})

test_that("get_augment_region L-Optimality 2D returns augment_region", {
  matB_l <- diag(3)
  reg <- evaluate_promise(get_augment_region(
    "L-Optimality", init_ail, 0.25, model_ail, pars_ail, vals_ail, ds_ail,
    calc_optimal_design = FALSE, matB = matB_l, delta_val = 0.85
  ))$result
  expect_s3_class(reg, "augment_region")
  expect_gte(nrow(reg$region), 1L)
})

# ── efficient_round and combinatorial_round multi-factor ─────────────────────

test_that("efficient_round works for multi-factor designs", {
  des <- mm2d_res$optdes
  exact <- evaluate_promise(efficient_round(des, 12L))$result
  expect_equal(sum(exact$Weight), 12L)
  expect_true(all(c("x1", "x2", "Weight") %in% names(exact)))
})

test_that("combinatorial_round works for multi-factor designs", {
  rounded <- combinatorial_round(mm2d_res, 9L)
  expect_equal(sum(rounded$Weight), 9L)
  expect_true(all(c("x1", "x2", "Weight") %in% names(rounded)))
})

test_that("print.augment_region works for 1D and 2D", {
  # 1D
  init1d <- data.frame(Point = c(30, 60, 90), Weight = rep(1/3, 3))
  reg1d  <- evaluate_promise(get_augment_region(
    "D-Optimality", init1d, 0.25,
    y ~ 10^(a - b/(c + x)), c("a","b","c"), c(8.07131, 1730.63, 233.426),
    c(1, 100), FALSE, delta_val = 0.85
  ))$result
  expect_output(print(reg1d), "Intervals")

  # 2D
  expect_output(print(region_res), "candidate points")
})

# Gradient Vector ------------------------------------


#' Gradient function
#'
#' @description
#' Calculates the gradient function of a \code{model} with respect to the parameters, \code{char_vars}, evaluates
#' it at the provided \code{values} and returns the result as a function of the variable \code{x}.
#'
#' @param model formula describing the model, which must contain only \code{x}, the parameters defined in
#'   \code{char_vars} and the numerical operators.
#' @param char_vars character vector of the parameters of the model.
#' @param values numeric vector with the nominal values of the parameters in \code{char_vars}.
#' @param weight_fun optional function variable that represents the square of the structure of variance, in case of heteroscedastic variance of the response
#'
#' @return A function depending on \code{x} that's the gradient of the \code{model} with respect to \code{char_vars}
gradient <- function(model, char_vars, values, weight_fun = function(x) 1) {
  design_vars   <- detect_design_vars(model, char_vars)
  ext_char_vars <- c(char_vars, design_vars)
  arglist <- lapply(ext_char_vars, function(x) NULL)
  f  <- as.function(append(stats::setNames(arglist, ext_char_vars), quote({})))
  f1 <- stats::deriv(model, char_vars, f)
  # x_val: scalar for single-factor, or named numeric vector for multi-factor.
  # c(values, x_val) passes parameters positionally and design variables by name
  # via do.call — works for both cases without branching.
  f2 <- function(x_val) {
    attr(do.call(f1, as.list(c(values, x_val))), "gradient")
  }
  f3 <- function(x) f2(x) * weight_fun(x)
  attr(f3, "design_vars") <- design_vars
  f3
}


# Information Matrix ------------------------------------


#' Information Matrix
#'
#' @description
#' Given the gradient vector of a model in a single variable model and a design, calculates the information matrix.
#'
#' @param grad A function in a single variable that returns the partial derivatives vector of the model.
#' @param design A dataframe that represents the design. Must have two columns:
#'   * \code{Point} contains the support points of the design.
#'   * \code{Weight} contains the corresponding weights of the \code{Point}s.
#'
#' @return The information matrix of the design, a \eqn{k\times k} matrix where k is the length of the gradient.
inf_mat <- function(grad, design) {
  dvars <- attr(grad, "design_vars")
  if (is.null(dvars)) dvars <- "x"          # backward compat if attr absent
  design <- normalize_design_cols(design, dvars)

  if (!is_multifactor(dvars)) {
    # ── Single-factor path (current implementation) ──────────────────────────
    k     <- length(grad(design[[dvars]][[1L]]))
    F_mat <- vapply(design[[dvars]], function(x) as.vector(grad(x)), numeric(k))
  } else {
    # ── Multi-factor path ─────────────────────────────────────────────────────
    # Evaluate gradient at each design point (a named row vector)
    x0    <- unlist(design[1L, dvars])
    k     <- length(grad(x0))
    F_mat <- vapply(seq_len(nrow(design)), function(i) {
      as.vector(grad(unlist(design[i, dvars])))
    }, numeric(k))
  }

  if (!is.matrix(F_mat)) F_mat <- matrix(F_mat, nrow = 1L)
  # M = F diag(w) F^T = crossprod(sqrt(w) * F^T)  [LAPACK DSYRK]
  crossprod(sqrt(design$Weight) * t(F_mat))
}

# Sensibility Function ------------------------------------

#' Master function to calculate the sensitivity function
#'
#' @description
#' Calculates the sensitivity function given the desired \code{Criterion}, an information matrix and other
#' necessary values depending on the chosen criterion.
#'
#' @param Criterion character variable with the chosen optimality criterion. Can be one of the following:
#'   * 'D-Optimality'
#'   * 'Ds-Optimality'
#'   * 'A-Optimality'
#'   * 'I-Optimality'
#'   * 'L-Optimality'
#' @param grad A function in a single variable that returns the partial derivatives vector of the model.
#' @param M Information Matrix for the sensitivity function.
#' @param par_int Numeric vector of the indexes of the parameters of interest for Ds-Optimality.
#' @param matB Matrix resulting from the integration of the one-point Information Matrix along the interest
#'   region or lineal matrix for L-Optimality.
#'
#' @return The sensitivity function as a matrix of single variable.
sens <- function(Criterion, grad, M, par_int = c(1), matB = NA) {
  if (identical(Criterion, "D-Optimality")) {
    return(dsens(grad, M))
  }
  else if (identical(Criterion, "Ds-Optimality")) {
    return(dssens(grad, M, par_int))
  }
  else if (identical(Criterion, "A-Optimality")) {
    return(isens(grad, M, diag(nrow(M))))
  }
  else if (identical(Criterion, "I-Optimality") || identical(Criterion, "L-Optimality")) {
    return(isens(grad, M, matB))
  }
}


#' Sensitivity function for D-Optimality
#'
#' @description
#' Calculates the sensitivity function from the gradient vector and the Identity Matrix.
#'
#' @inherit sens params return
#'
dsens <- function(grad, M) {
  invMat <- inv_spd(M)
  sens_ret <- function(xval) {
    f_col <- as.matrix(grad(xval), nrow = 1, ncol = 3, byrow = TRUE, dimnames = NULL)
    return(f_col %*% invMat %*% t(f_col))
  }
}


#' Sensitivity function for Ds-Optimality
#'
#' @description
#' Calculates the sensitivity function from the gradient vector, the Identity Matrix and the parameters of
#' interest.
#'
#' @inherit sens params return
#'
dssens <- function(grad, M, par_int) {
  invMat <- inv_spd(M)
  if (length(M[-par_int, -par_int]) == 1) {
    invMat22 <- 1 / M[-par_int, -par_int]
  } else {
    invMat22 <- inv_spd(M[-par_int, -par_int])
  }
  sens_ret <- function(xval) {
    f_col <- as.matrix(grad(xval), nrow = 1, byrow = TRUE, dimnames = NULL)
    return(f_col %*% invMat %*% t(f_col) - f_col[-par_int] %*% invMat22 %*% as.matrix(f_col[-par_int], ncol = 1))
  }
}


#' Sensitivity function for I-Optimality
#'
#' @description
#' Calculates the sensitivity function from the gradient vector, the Information Matrix and the integral of the
#' one-point Identity Matrix over the interest region. If instead the identity matrix is used, it can be used
#' for A-Optimality.
#'
#' @inherit sens params return
#'
isens <- function(grad, M, matB) {
  invMat <- inv_spd(M)
  sens_ret <- function(xval) {
    f_col <- as.matrix(grad(xval), nrow = 1, ncol = 3, byrow = TRUE, dimnames = NULL)
    return(f_col %*% invMat %*% matB %*% invMat %*% t(f_col))
  }
}


# Compound sensitivity function: weighted sum of individual sensitivity functions.
# compound_specs: list of per-criterion specs (see opt_des compound parameter).
# Returns a closure that evaluates d_c(xval) = sum_i w_i * d_i(xval).
csens <- function(compound_specs, grad, M) {
  sens_fns <- lapply(compound_specs, function(spec) {
    if (identical(spec$criterion, "D-Optimality"))
      dsens(grad, M)
    else if (identical(spec$criterion, "Ds-Optimality"))
      dssens(grad, M, spec$par_int)
    else
      isens(grad, M, spec$matB)   # A, I, L
  })
  weights <- sapply(compound_specs, `[[`, "weight")
  function(xval) {
    sum(mapply(function(s, w) w * as.numeric(s(xval)), sens_fns, weights))
  }
}


# Equivalence Theorem threshold for the compound criterion:
# sum_i w_i * threshold_i, where threshold_i = k (D), s (Ds), tr(B M^{-1}) (A/I/L).
compound_threshold <- function(compound_specs, M) {
  total <- 0
  for (spec in compound_specs) {
    thr <- if (identical(spec$criterion, "D-Optimality"))
             spec$k
           else if (identical(spec$criterion, "Ds-Optimality"))
             length(spec$par_int)
           else
             icrit(M, spec$matB)
    total <- total + spec$weight * thr
  }
  total
}

library(optedr)

# Shared fixtures -------------------------------------------------------
model  <- y ~ a * exp(-b / x)
params <- c("a", "b")
pvals  <- c(1, 1500)
des    <- data.frame(Point = c(329.3, 422), Weight = c(0.5, 0.5))

# gradient() ------------------------------------------------------------

test_that("gradient returns a function", {
  grad <- optedr:::gradient(model, params, pvals)
  expect_true(is.function(grad))
})

test_that("gradient output has length k (number of parameters)", {
  grad <- optedr:::gradient(model, params, pvals)
  g    <- grad(300)
  expect_equal(length(g), 2L)
  expect_true(all(is.finite(g)))
})

test_that("gradient is scaled by weight_fun", {
  grad_1 <- optedr:::gradient(model, params, pvals)
  grad_2 <- optedr:::gradient(model, params, pvals, weight_fun = function(x) 2)
  expect_equal(as.numeric(grad_2(300)), 2 * as.numeric(grad_1(300)), tolerance = 1e-10)
})

test_that("gradient for 3-parameter model has length 3", {
  g3 <- optedr:::gradient(
    y ~ a * exp(-b / x) + c, c("a", "b", "c"), c(1, 1500, 0)
  )
  expect_equal(length(g3(300)), 3L)
})

# inf_mat() -------------------------------------------------------------

test_that("inf_mat returns a k x k symmetric PSD matrix", {
  grad <- optedr:::gradient(model, params, pvals)
  M    <- optedr:::inf_mat(grad, des)
  expect_equal(dim(M), c(2L, 2L))
  expect_equal(M, t(M), tolerance = 1e-10)
  expect_true(all(eigen(M)$values >= -1e-10))
})

test_that("inf_mat result does not depend on point ordering", {
  grad <- optedr:::gradient(model, params, pvals)
  d1   <- data.frame(Point = c(329.3, 422), Weight = c(0.5, 0.5))
  d2   <- data.frame(Point = c(422, 329.3), Weight = c(0.5, 0.5))
  expect_equal(optedr:::inf_mat(grad, d1), optedr:::inf_mat(grad, d2), tolerance = 1e-10)
})

test_that("inf_mat is additive: equal-weight 2-point = sum of 1-point matrices", {
  grad <- optedr:::gradient(model, params, pvals)
  d1   <- data.frame(Point = 329.3, Weight = 1)
  d2   <- data.frame(Point = 422,   Weight = 1)
  M1   <- optedr:::inf_mat(grad, d1)
  M2   <- optedr:::inf_mat(grad, d2)
  M12  <- optedr:::inf_mat(grad, data.frame(Point = c(329.3, 422), Weight = c(0.5, 0.5)))
  expect_equal(M12, 0.5 * M1 + 0.5 * M2, tolerance = 1e-10)
})

test_that("inf_mat works for a single-parameter model (1x1 output)", {
  g1 <- optedr:::gradient(y ~ a * x, c("a"), c(1))
  M  <- optedr:::inf_mat(g1, data.frame(Point = c(1, 2), Weight = c(0.5, 0.5)))
  expect_equal(dim(M), c(1L, 1L))
  expect_true(M[1, 1] > 0)
})

# dsens() ---------------------------------------------------------------

test_that("dsens returns a function giving non-negative values", {
  grad  <- optedr:::gradient(model, params, pvals)
  M     <- optedr:::inf_mat(grad, des)
  sens  <- optedr:::dsens(grad, M)
  expect_true(is.function(sens))
  expect_gte(as.numeric(sens(300)), 0)
})

test_that("dsens at D-optimal support points equals k (Equivalence Theorem)", {
  res  <- evaluate_promise(opt_des("D-Optimality", model, params, pvals, c(212, 422)))$result
  grad <- optedr:::gradient(model, params, pvals)
  M    <- optedr:::inf_mat(grad, res$optdes)
  sens <- optedr:::dsens(grad, M)
  for (x in res$optdes$Point) {
    expect_equal(as.numeric(sens(x)), 2, tolerance = 0.02)
  }
})

# dssens() --------------------------------------------------------------

test_that("dssens returns a function", {
  grad  <- optedr:::gradient(model, params, pvals)
  M     <- optedr:::inf_mat(grad, des)
  sens  <- optedr:::dssens(grad, M, par_int = c(1L))
  expect_true(is.function(sens))
  expect_true(is.finite(as.numeric(sens(300))))
})

test_that("dssens differs from dsens (Ds != D sensitivity)", {
  grad  <- optedr:::gradient(model, params, pvals)
  M     <- optedr:::inf_mat(grad, des)
  sd    <- optedr:::dsens(grad, M)
  sds   <- optedr:::dssens(grad, M, par_int = c(1L))
  expect_false(isTRUE(all.equal(as.numeric(sd(300)), as.numeric(sds(300)))))
})

# isens() ---------------------------------------------------------------

test_that("isens returns a function giving non-negative values", {
  grad  <- optedr:::gradient(model, params, pvals)
  M     <- optedr:::inf_mat(grad, des)
  sens  <- optedr:::isens(grad, M, diag(2))
  expect_true(is.function(sens))
  expect_gte(as.numeric(sens(300)), 0)
})

# sens() dispatcher -----------------------------------------------------

test_that("sens dispatches to the correct function for each criterion", {
  grad <- optedr:::gradient(model, params, pvals)
  M    <- optedr:::inf_mat(grad, des)
  for (crit in c("D-Optimality", "A-Optimality")) {
    fn <- optedr:::sens(crit, grad, M)
    expect_true(is.function(fn))
    expect_true(is.finite(as.numeric(fn(300))))
  }
  fn_ds <- optedr:::sens("Ds-Optimality", grad, M, par_int = c(1L))
  expect_true(is.function(fn_ds))
  fn_i  <- optedr:::sens("I-Optimality",  grad, M, matB = diag(2))
  expect_true(is.function(fn_i))
  fn_l  <- optedr:::sens("L-Optimality",  grad, M, matB = diag(2))
  expect_true(is.function(fn_l))
})

# Auxiliar function for algorithms --------------------------------------



#' Weight function per distribution
#'
#' @param model formula describing the model to use. Must use \code{x} as the design variable.
#' @param char_vars character vector with the parameters of the models, as written in the \code{formula}
#' @param values numeric vector with the parameters nominal values, in the same order as given in \code{parameters}.
#' @param distribution character variable specifying the probability distribution of the response. Can be one of the following:
#'   * 'Normal', for normal homoscedastic (default)
#'   * 'Gamma', which can be used for exponential or normal heteroscedastic with constant relative error
#'   * 'Poisson'
#'   * 'Logistic'
#'
#' @return one variable function that represents the square of the structure of variance, in case of heteroscedastic variance of the response.
#'
weight_function <- function(model, char_vars, values, distribution = "Normal") {
  # vars <- as.list(match.call())[-(1:2)]
  # char_vars <- sapply(vars, as.character)
  if(!(distribution %in% c("Poisson", "Gamma", "Logistic", #"log-normal",
                           "Normal"))){
    warning(crayon::yellow(cli::symbol$warning), " Not a valid distribution specified, using a normal homoscedastic")
    return(function(x) 1)
  }
  else if(distribution == "Normal"){
    return(function(x) 1)
  }
  else if(distribution == "Poisson"){
    cmd <- utils::tail(as.character(model),1)
    expres <- parse(text=cmd)
    lista <- values
    names(lista) <- char_vars
    f <- function(x_val) {
      exp(eval(expres, c(lista, list("x" = x_val)))/2)
    }
  }
  else if(distribution == "Gamma"){
    cmd <- utils::tail(as.character(model),1)
    expres <- parse(text=cmd)
    lista <- values
    names(lista) <- char_vars
    f <- function(x_val) {
      (eval(expres, c(lista, list("x" = x_val))))^(-1)
    }
  }
  else if(distribution == "Logistic"){
    cmd <- utils::tail(as.character(model),1)
    expres <- parse(text=cmd)
    lista <- values
    names(lista) <- char_vars
    f <- function(x_val) {
      sqrt(exp(-eval(expres, c(lista, list("x" = x_val))))/(1 + exp(-eval(expres, c(lista, list("x" = x_val)))))^2)
    }
  }
  else if(distribution == "Log-normal"){
    return(function(x) 1)
  }
  return(f)
}


#' Find Minimum Value
#'
#' @description
#' Searches the minimum of a function over a grid on the design space.
#'
#' @param sens A numeric function to evaluate (scalar for 1D, named numeric vector for multi-factor).
#' @param design_space Numeric vector \code{c(min, max)} for single-factor models, or a
#'   named list \code{list(x1 = c(min, max), ...)} for multi-factor models.
#' @param grid.length Number of grid points (1D) or LHS samples (multi-factor).
#'
#' @return The value of the minimum
findminval <- function(sens, design_space, grid.length) {
  if (is.numeric(design_space)) {
    # Backward-compat: numeric vector c(min, max) as used by augment functions
    lo <- min(design_space); hi <- max(design_space)
    return(min(purrr::map_dbl(seq(lo, hi, length.out = grid.length), sens)))
  }
  if (length(design_space) == 1L) {
    # Single-factor canonical list: list(x = c(min, max))
    bnds <- design_space[[1L]]
    lo <- min(bnds); hi <- max(bnds)
    return(min(purrr::map_dbl(seq(lo, hi, length.out = grid.length), sens)))
  }
  # Multi-factor: LHS grid
  pts <- lhs_sample(grid.length, design_space)
  min(apply(pts, 1L, function(x) as.numeric(sens(stats::setNames(x, names(design_space))))))
}

#' Find Maximum Value
#'
#' @description
#' Searches the maximum of a function over a grid on a given interval or design space.
#'
#' @param sens A numeric function to evaluate (scalar argument for 1D, named numeric
#'   vector for multi-factor).
#' @param design_space Numeric vector \code{c(min, max)} for single-factor models, or a
#'   named list \code{list(x1 = c(min, max), ...)} for multi-factor models.
#' @param grid.length Number of grid points (1D) or LHS samples (multi-factor).
#'
#' @return The value of the maximum
findmaxval <- function(sens, design_space, grid.length) {
  if (is.numeric(design_space)) {
    lo <- min(design_space); hi <- max(design_space)
    return(max(purrr::map_dbl(seq(lo, hi, length.out = grid.length), sens)))
  }
  if (length(design_space) == 1L) {
    bnds <- design_space[[1L]]
    lo <- min(bnds); hi <- max(bnds)
    return(max(purrr::map_dbl(seq(lo, hi, length.out = grid.length), sens)))
  }
  pts <- lhs_sample(grid.length, design_space)
  max(apply(pts, 1L, function(x) as.numeric(sens(stats::setNames(x, names(design_space))))))
}


#' Find Maximum
#'
#' @description
#' Searches the location of the maximum of a sensitivity function over the design space.
#' For single-factor models the search uses a regular grid followed by direct selection.
#' For multi-factor models a Latin Hypercube Sample is evaluated and then refined with
#' L-BFGS-B local optimisation from the \code{n_starts} best candidate points.
#'
#' @param sens Sensitivity function (scalar for 1D, named numeric vector for multi-factor).
#' @param design_space Numeric vector \code{c(min, max)} or named list.
#' @param grid.length Number of grid / LHS points for the initial sweep.
#' @param n_starts Number of local-optimisation restarts (multi-factor only).
#'
#' @return The design point (scalar or named numeric vector) at which \code{sens} is maximised.
findmax <- function(sens, design_space, grid.length, n_starts = 5L) {
  if (is.numeric(design_space)) {
    lo <- min(design_space); hi <- max(design_space)
    grid <- seq(lo, hi, length.out = grid.length)
    return(grid[which.max(purrr::map(grid, sens))])
  }
  if (length(design_space) == 1L) {
    # ── Single-factor (current behaviour) ────────────────────────────────────
    bnds <- design_space[[1L]]
    lo   <- min(bnds); hi <- max(bnds)
    grid <- seq(lo, hi, length.out = grid.length)
    return(grid[which.max(purrr::map(grid, sens))])
  }

  # ── Multi-factor ──────────────────────────────────────────────────────────
  d        <- length(design_space)
  n_lhs    <- max(grid.length, 20L^min(d, 3L))
  pts      <- lhs_sample(n_lhs, design_space)
  nms      <- names(design_space)
  sens_vec <- apply(pts, 1L,
                    function(x) as.numeric(sens(stats::setNames(x, nms))))

  # Refine the best n_starts candidates with L-BFGS-B
  n_starts  <- min(n_starts, n_lhs)
  top_idx   <- order(sens_vec, decreasing = TRUE)[seq_len(n_starts)]
  best_x    <- pts[top_idx[1L], ]
  best_val  <- sens_vec[top_idx[1L]]
  lower     <- sapply(design_space, `[`, 1L)
  upper     <- sapply(design_space, `[`, 2L)

  for (idx in top_idx) {
    opt <- tryCatch(
      stats::optim(
        par    = pts[idx, ],
        fn     = function(x) -as.numeric(sens(stats::setNames(x, nms))),
        method = "L-BFGS-B",
        lower  = lower,
        upper  = upper
      ),
      error = function(e) list(value = Inf, par = pts[idx, ])
    )
    if (-opt$value > best_val) { best_val <- -opt$value; best_x <- opt$par }
  }
  stats::setNames(best_x, nms)
}

#' Deletes duplicates points
#'
#' @description
#' Within a vector of points, deletes points that are close enough (less than the tol parameter).
#' Returns the points without the "duplicates"
#'
#' @param points Points to be updated
#' @param tol Tolerance for which two points are considered the same
#'
#' @return The points without duplicates
update_sequence <- function(points, tol){
  i <- 1
  imax <- (length(points)-1)
  while(i < imax) {
    absdiff <- c(rep(T, i), abs(points[-seq(1, i)] - points[i]) > tol)
    points <- points[absdiff]
    i <- i+1
    imax <- (length(points)-1)
  }
  return(points)
}


#' Find where the candidate points region starts
#'
#' @description
#' Given the crosspoints and the sensitivity function, this function
#' finds where the candidate points region starts, either on the extreme of
#' the space of the design or the first crosspoints
#'
#' @param cross Vector of crosspoints in the sensitivity function given an efficiency and weight
#' @param min Minimum of the space of the design
#' @param max Maximum of the space of the design
#' @param val Value of the sensitivity function at the crosspoints
#' @param sens_opt Sensitivity function
#'
#' @return True if the candidate points region starts on the minimum, False otherwise
#'
getStart <- function(cross, min, max, val, sens_opt){
  if(length(cross)==1){
    if(round(cross[1]-min, 6)==0){
      if((val < sens_opt((max+cross[1])/2))){
        start <- F
      }
      else{
        start <- T
      }
    }
    else{
      if((val < sens_opt((min+cross[1])/2))){
        start <- T
      }
      else{
        start <- F
      }
    }
  }
  else if(val < sens_opt((cross[2]+cross[1])/2)){
    start <- F
  }
  else {
    start <- T
  }
  return(start)
}


#' Parity of the crosspoints
#'
#' @description
#' Determines if the number of crosspoints is even or odd given the vector
#' of crosspoints
#'
#' @param cross Vector of crosspoints in the sensitivity function given an efficiency and weight
#'
#' @return True if the number of crosspoints is even, false otherwise
getPar <- function(cross){
  return(length(cross)%%2 == 0)
}

#' Give effective limits to candidate points region
#'
#' @description
#' Given the start of the candidates points region, the parity of the crosspoints
#' and the boundaries of the space of the design returns the effective limits of
#' the candidate points region. Those points, taken in pairs from the first to
#' the last delimit the region.
#'
#' @param cross Vector of crosspoints in the sensitivity function given an efficiency and weight
#' @param min Minimum of the space of the design
#' @param max Maximum of the space of the design
#' @param start Boolean that gives the effective start of the candidate points region
#' @param par Boolean with the parity of the region
#'
#' @return Vector of effective limits of the candidate points region. Taken
#' in pairs from the beginning delimit the region.
getCross2 <- function(cross, min, max, start, par){
  if(par & start){
    cross2 <- c(min, cross, max)
  }
  else if(par & !start){
    cross2 <- cross
  }
  else if(!par & start){
    cross2 <- c(min, cross)
  }
  else{
    cross2 <- c(cross, max)
  }
  return(cross2)
}




#' Update weight D-Optimality
#'
#' @description
#' Implementation of the weight update formula for D-Optimality used to optimize the weights of a design,
#' which is to be applied iteratively until no sizable changes happen.
#'
#' @param design Design to optimize the weights from. It's a dataframe with two columns:
#'   * \code{Point} contains the support points of the design.
#'   * \code{Weight} contains the corresponding weights of the \code{Point}s.
#' @param sens Sensibility function for the design and model.
#' @param k Number of parameters of the model.
#' @param delta A parameter of the algorithm that can be tuned. Must be \eqn{0< delta < 1}.
#'
#' @return returns the new weights of the design after one iteration.
update_weights <- function(design, sens, k, delta) {
  weights <- design$Weight * (.eval_sens_all(design, sens) / k)^delta
  w_sum   <- sum(weights)
  if (!is.finite(w_sum) || w_sum == 0)
    stop("Weight update produced non-finite values. ",
         "The model is likely not identifiable in all specified parameters -",
         "check for parameter redundancy (see warnings above).",
         call. = FALSE)
  return(weights / w_sum)
}


#' Update weight Ds-Optimality
#'
#' @description
#' Implementation of the weight update formula for Ds-Optimality used to optimize the weights of a design,
#' which is to be applied iteratively until no sizable changes happen.
#'
#'
#' @param s number of parameters of interest of the model
#' @inheritParams update_weights
#'
#' @return returns the new weights of the design after one iteration.
update_weightsDS <- function(design, sens, s, delta) {
  weights <- design$Weight * (.eval_sens_all(design, sens) / s)^delta
  w_sum   <- sum(weights)
  if (!is.finite(w_sum) || w_sum == 0)
    stop("Weight update produced non-finite values. ",
         "The model is likely not identifiable in all specified parameters -",
         "check for parameter redundancy (see warnings above).",
         call. = FALSE)
  return(weights / w_sum)
}

#' Update weight I-Optimality
#'
#' @description
#' Implementation of the weight update formula for I-Optimality used to optimize the weights of a design,
#' which is to be applied iteratively until no sizable changes happen. A-Optimality if instead of the
#' integral matrix the identity function is used.
#'
#'
#' @param crit Value of the criterion function for I-Optimality.
#' @inheritParams update_weights
#'
#' @return returns the new weights of the design after one iteration.
update_weightsI <- function(design, sens, crit, delta) {
  exponent <- function(a, pow) (abs(a)^pow) * sign(a)
  weights  <- design$Weight * exponent(.eval_sens_all(design, sens) / crit, delta)
  w_sum    <- sum(weights)
  if (!is.finite(w_sum) || w_sum == 0)
    stop("Weight update produced non-finite values. ",
         "The model is likely not identifiable in all specified parameters -",
         "check for parameter redundancy (see warnings above).",
         call. = FALSE)
  return(weights / w_sum)
}


#' Update Design with new point
#'
#' @description
#' Updates a design adding a new point to it. If the added point is closer than \code{delta} to an existing
#' point of the design, the two points are merged together as their arithmetic average. Then updates the weights
#' to be equal to all points of the design.
#'
#' @param design Design to update. It's a dataframe with two columns:
#'   * \code{Point} contains the support points of the design.
#'   * \code{Weight} contains the corresponding weights of the \code{Point}s.
#' @param xmax The point to add as a numeric value.
#' @param delta Threshold which defines how close the new point has to be to any of the existing ones in order to
#'   merge with them.
#' @param new_weight Number with the weight for the new point.
#'
#' @return The updated design.
update_design <- function(design, xmax, delta, new_weight) {
  dcols <- coord_cols(design)
  design$Weight <- design$Weight * (1 - new_weight)

  if (identical(dcols, "Point")) {
    # ── Single-factor (current behaviour) ──────────────────────────────────
    absdiff <- abs(design$Point - xmax) < delta
    if (any(absdiff)) {
      pos <- min(which(absdiff))
      design$Point[[pos]] <- (design$Point[[pos]] + xmax) / 2
      design$Weight[[pos]] <- design$Weight[[pos]] + new_weight
    } else {
      design[nrow(design) + 1L, ] <- c(xmax, new_weight)
    }
  } else {
    # ── Multi-factor: Euclidean distance ───────────────────────────────────
    dists <- apply(as.matrix(design[, dcols, drop = FALSE]), 1L, function(row)
      sqrt(sum((row - xmax[dcols])^2)))
    if (any(dists < delta)) {
      pos <- which.min(dists)
      design[pos, dcols] <- (unlist(design[pos, dcols]) + xmax[dcols]) / 2
      design$Weight[[pos]] <- design$Weight[[pos]] + new_weight
    } else {
      new_row        <- as.data.frame(as.list(xmax[dcols]))
      new_row$Weight <- new_weight
      design         <- rbind(design, new_row)
    }
  }
  design
}


#' Merge close points of a design
#'
#' @description
#' Takes a design and merge together all points that are closer between them than a certain threshold \code{delta}.
#'
#' @param design The design to update. It's a dataframe with two columns:
#'   * \code{Point} contains the support points of the design.
#'   * \code{Weight} contains the corresponding weights of the \code{Point}s.
#' @param delta Threshold which defines how close two points have to be to any of the existing ones in order to
#'   merge with them.
#'
#' @return The updated design.
update_design_total <- function(design, delta) {
  dcols    <- coord_cols(design)
  use_1d   <- identical(dcols, "Point")
  updated  <- FALSE
  finished <- FALSE
  while (!finished) {
    n <- nrow(design)
    if (n < 2L) break
    for (i in seq_len(n - 1L)) {
      if (use_1d) {
        absdiff <- abs(design$Point[-seq_len(i)] - design$Point[i]) < delta
      } else {
        rest <- as.matrix(design[-seq_len(i), dcols, drop = FALSE])
        curr <- unlist(design[i, dcols])
        absdiff <- apply(rest, 1L, function(row) sqrt(sum((row - curr)^2))) < delta
      }
      if (any(absdiff)) {
        updated  <- TRUE
        x_i      <- if (use_1d) design$Point[i] else unlist(design[i, dcols])
        design   <- update_design(design[-i, ], x_i, delta, design$Weight[i])
        break
      }
    }
    finished <- !updated
    updated  <- FALSE
  }
  design
}


#' Remove low weight points
#'
#' @description
#' Removes the points of a design with a weight lower than a threshold, \code{delta}, and distributes that weights
#' proportionally to the rest of the points.
#'
#' @param design The design from which to remove points as a dataframe with two columns:
#'   * \code{Point} contains the support points of the design.
#'   * \code{Weight} contains the corresponding weights of the \code{Point}s.
#' @param delta The threshold from which the points with such a weight or lower will be removed.
#'
#' @return The design without the removed points.
delete_points <- function(design, delta) {
  updatedDesign <- design[design$Weight > delta, ]
  updatedDesign$Weight <- updatedDesign$Weight / sum(updatedDesign$Weight)
  return(updatedDesign)
}

# General auxiliar functions --------------------------
# Cálculo de la traza de una matriz
#' Trace
#'
#' @description
#' Return the mathematical trace of a matrix, the sum of its diagonal elements.
#'
#'
#' @param M The matrix from which to calculate the trace.
#'
#' @return The trace of the matrix.
tr <- function(M) {
  return(sum(diag(M)))
}


# Evaluates a sensitivity function at every design point.
# Works for both single-factor (design has "Point" column, sens accepts scalar)
# and multi-factor (design has x1, x2, ..., sens accepts named vector).
.eval_sens_all <- function(design, sens) {
  dcols <- coord_cols(design)
  if (identical(dcols, "Point")) {
    purrr::map_dbl(design$Point, sens)
  } else {
    vapply(seq_len(nrow(design)), function(i)
      as.numeric(sens(unlist(design[i, dcols]))), numeric(1L))
  }
}

# Validates the Atwood efficiency bound and warns when it falls outside [0, 100],
# which indicates a degenerate information matrix (model not identifiable or design
# is singular). A bound > 100 % is the silent-failure signature of A/I/L-optimality
# applied to a non-identifiable model (pseudoinverse causes the ET condition to
# trigger prematurely).
check_atwood <- function(atwood) {
  val <- as.numeric(atwood)
  # Allow 0.01 percentage-point tolerance to absorb floating-point rounding
  # near the exact convergence boundary of 0 % and 100 %.
  if (!is.finite(val) || val < -0.01 || val > 100.01) {
    warning(sprintf(
      paste0("The Atwood efficiency bound is %.4g%%, which is outside [0, 100]. ",
             "This indicates a degenerate information matrix -the model is likely ",
             "not identifiable in all specified parameters. ",
             "Check for parameter redundancy (see any preceding warnings)."),
      val
    ), call. = FALSE)
  }
}

# Stable inverse for symmetric positive definite matrices (information matrices).
# Primary path: Cholesky (fast, exact for SPD).
# Fallback: Moore-Penrose pseudoinverse via SVD with a warning, for cases where
# the matrix is theoretically invertible but numerically non-PD (e.g., extreme
# parameter scales). Uses only base R -no extra dependencies.
inv_spd <- function(M) {
  result <- tryCatch(chol2inv(chol(M)), error = function(e) NULL)
  if (!is.null(result)) return(result)
  s <- svd(M)
  tol <- max(dim(M)) * max(s$d) * .Machine$double.eps
  d_inv <- ifelse(s$d > tol, 1 / s$d, 0)
  n_zero <- sum(d_inv == 0)
  if (n_zero > 0) {
    warning(sprintf(
      paste0("The information matrix has %d near-zero singular value(s) ",
             "(rank %d out of %d). The model likely has %d fewer identifiable ",
             "parameter(s) than specified -check for parameter redundancy or collinearity."),
      n_zero, nrow(M) - n_zero, nrow(M), n_zero
    ), call. = FALSE)
  } else {
    warning(
      "Information matrix is not positive definite; falling back to Moore-Penrose pseudoinverse. ",
      "Results may be unreliable. Check that the design has enough distinct support points ",
      "and that model parameters are on similar scales.",
      call. = FALSE
    )
  }
  s$v %*% diag(d_inv) %*% t(s$u)
}


# Heatmap of the sensitivity function for two-factor designs.
# Intended for d=2; returns a ggplot object with support points and ET contour overlaid.
plot_sens_2d <- function(design_space, sens_fn, design_points, criterion_value) {
  sens <- wlabel <- NULL   # avoid R CMD check NOTE on ggplot2 aes variables
  dvars   <- names(design_space)
  n_grid  <- 40L
  grid_df <- do.call(expand.grid, lapply(design_space, function(ds)
    seq(ds[1L], ds[2L], length.out = n_grid)))
  grid_df$sens <- apply(grid_df, 1L, function(x)
    as.numeric(sens_fn(stats::setNames(x, dvars))))

  label_df         <- design_points
  label_df$wlabel  <- paste0(round(label_df$Weight, 2))

  x1s <- rlang::sym(dvars[1L]); x2s <- rlang::sym(dvars[2L])
  p <- ggplot2::ggplot(grid_df, ggplot2::aes(x = !!x1s, y = !!x2s, fill = sens)) +
    ggplot2::geom_tile() +
    ggplot2::scale_fill_viridis_c(name = "Sensitivity")
  # Only draw the ET contour when criterion_value falls within the grid's
  # sensitivity range (can fall outside due to Monte Carlo noise in I-Optimality).
  sens_finite <- grid_df$sens[is.finite(grid_df$sens)]
  if (length(sens_finite) > 0L &&
      criterion_value >= min(sens_finite) && criterion_value <= max(sens_finite)) {
    p <- p + ggplot2::geom_contour(ggplot2::aes(z = sens, fill = NULL),
                                   breaks = criterion_value, colour = "white", linewidth = 0.7)
  }
  p +
    ggplot2::geom_point(data = label_df,
                        ggplot2::aes(x = !!x1s, y = !!x2s, fill = NULL),
                        colour = "red", size = 3.5, shape = 16) +
    ggplot2::geom_text(data = label_df,
                       ggplot2::aes(x = !!x1s, y = !!x2s, label = wlabel, fill = NULL),
                       colour = "white", size = 3, vjust = -1) +
    ggplot2::theme_bw() +
    ggplot2::labs(x       = dvars[1L],
                  y       = dvars[2L],
                  caption = paste("White contour: ET threshold =", round(criterion_value, 4)))
}


#' Plot sensitivity function
#'
#' @description
#' Plots the sensitivity function and the value of the Equivalence Theorem as an horizontal line, which helps
#' assess the optimality of the design of the given sensitivity function.
#'
#' @param min Minimum of the space of the design, used in the limits of the representation.
#' @param max Maximum of the space of the design, used in the limits of the representation.
#' @param sens_function A single variable function, the sensitivity function.
#' @param criterion_value A numeric value representing the other side of the inequality of the Equivalence Theorem.
#'
#' @return A `ggplot` object that represents the sensitivity function
plot_sens <- function(min, max, sens_function, criterion_value) {
  x <- y <- NULL
  grid <- seq(min, max, length.out = 10000)
  sens_grid <- purrr::map_dbl(grid, sens_function)

  sensibility <- ggplot2::ggplot(data = data.frame(x = grid, y = sens_grid), mapping = ggplot2::aes(x = x)) +
    ggplot2::theme_bw() +
    ggplot2::geom_line(mapping = ggplot2::aes(x = x, y = y), color = "steelblue3") +
    ggplot2::stat_function(fun = function(x) criterion_value, col = "goldenrod3") +
    ggplot2::xlim(min, max) +
    ggplot2::labs(x = "Design Space", y = "Sensitivity Function")
}

#' Plot Convergence of the algorithm
#'
#' @description
#' Plots the criterion value on each of the steps of the algorithm, both for optimizing weights and points,
#' against the total step number.
#'
#' @param convergence A dataframe with two columns:
#'   * \code{criteria} contains value of the criterion on each step.
#'   * \code{step} contains number of the step.
#'
#' @return A ggplot object with the \code{criteria} in the \code{y} axis and \code{step} in the \code{x} axis.
plot_convergence <- function(convergence) {
  step <- criteria <- NULL
  ggplot2::ggplot(data = convergence, ggplot2::aes(x = step, y = criteria)) +
    ggplot2::geom_line(color = "coral1") +
    ggplot2::labs(y = "criterion") +
    ggplot2::theme_bw()
}


#' Integrate IM
#'
#' @description
#' Integrates the information matrix over the region of interest to calculate matrix B to be used in I-Optimality
#' calculation.
#'
#' @param grad function of partial derivatives of the model.
#' @param k number of unknown parameters of the model.
#' @param reg_int optional numeric vector of two components with the bounds of the interest region for I-Optimality.
#'
#' @return The integrated information matrix.
integrate_reg_int <- function(grad, k, reg_int) {
  if (is.numeric(reg_int) && length(reg_int) == 2L) {
    # ── Single-factor: exact quadrature (current implementation) ─────────────
    matrix_int <- 0 * diag(k)
    for (i in seq_len(k)) {
      for (j in seq_len(k)) {
        if (j >= i) {
          int_part <- function(x_value) {
            purrr::map_dbl(x_value, function(x) {
              g <- grad(x)
              g[i] * g[j] / (reg_int[2L] - reg_int[1L])
            })
          }
          matrix_int[i, j] <- stats::integrate(int_part,
                                                lower = reg_int[1L],
                                                upper = reg_int[2L])$value
        } else {
          matrix_int[i, j] <- matrix_int[j, i]
        }
      }
    }
    return(matrix_int)
  }

  # ── Multi-factor: Monte Carlo integration ───────────────────────────────────
  # reg_int must be a named list with the same names as the design variables.
  if (!is.list(reg_int))
    stop("For multi-factor models, reg_int must be a named list, e.g. ",
         "list(x1 = c(0, 10), x2 = c(0, 5)).", call. = FALSE)
  dvars  <- attr(grad, "design_vars")
  if (is.null(dvars)) dvars <- names(reg_int)
  n_mc   <- 10000L
  pts    <- lhs_sample(n_mc, reg_int)           # n_mc x d matrix
  vol    <- prod(sapply(reg_int, diff))
  # Accumulate outer products: B = vol * E[g(x) g(x)^T]
  matrix_int <- Reduce("+", lapply(seq_len(n_mc), function(i) {
    g <- as.vector(grad(stats::setNames(pts[i, ], dvars)))
    outer(g, g)
  })) * (vol / n_mc)
  matrix_int
}


#' Summary function for optdes
#'
#' @param object An object of class \code{optdes}.
#' @param ... Possible extra arguments for the summary
#'
#' @export
#'
#' @examples
#' rri <- opt_des(criterion = "I-Optimality", model = y ~ a * exp(-b / x),
#'   parameters = c("a", "b"), par_values = c(1, 1500), design_space = c(212, 422),
#'   reg_int = c(380, 422))
#' summary(rri)
summary.optdes <- function(object, ...) {
  dvars <- attr(attr(object, "gradient"), "design_vars")
  multi <- is_multifactor(dvars)

  cat("Model:\n")
  print(attr(object, "model"))

  wf     <- attr(object, "weight_fun")
  wf_src <- attr(wf, "srcref")
  cat("Weight function:\n")
  if (!is.null(wf_src)) {
    print(wf_src)
  } else {
    cat(paste(deparse(wf), collapse = "\n"), "\n")
  }

  if (multi) {
    ds <- attr(object, "design_space")
    cat(sprintf("\nDesign space (%d factors):\n", length(dvars)))
    for (v in dvars)
      cat(sprintf("  %-5s [%.4g, %.4g]\n", paste0(v, ":"), ds[[v]][1L], ds[[v]][2L]))
  }

  if (identical(object$criterion, "Compound")) {
    specs <- attr(object, "hidden_value")
    cat("\nCompound criterion:\n")
    for (s in specs)
      cat(sprintf("  %.2f * %s\n", s$weight, s$criterion))
  }

  if (identical(object$criterion, "KL-Optimality")) {
    kl <- attr(object, "hidden_value")
    if (identical(kl$type, "family"))
      cat(sprintf("\nGLM family: %s  (phi = %g)\n", kl$family$name, kl$phi))
    else
      cat("\nKL function: user-supplied\n")
    cat(sprintf("Optimal rival parameters: %s\n",
                paste(round(kl$beta2_star, 4), collapse = ", ")))
  }

  cat(sprintf("\nOptimal design for %s", object$criterion))
  if (multi) cat(sprintf(" (%d support points)", nrow(object$optdes)))
  cat(":\n")
  print.data.frame(object$optdes, ...)
  cat(sprintf("\nMinimum efficiency (Atwood): %s%%", object$atwood))
  cat(sprintf("\nCriterion value:             %g\n", object$crit_value))
}


#' Print function for optdes
#'
#' @param x An object of class \code{optdes}.
#' @param ... Possible extra arguments for printing dataframes
#'
#' @export
#'
#' @examples
#' rri <- opt_des(criterion = "I-Optimality", model = y ~ a * exp(-b / x),
#'   parameters = c("a", "b"), par_values = c(1, 1500), design_space = c(212, 422),
#'   reg_int = c(380, 422))
#' print(rri)
print.optdes <- function(x, ...) {
  dvars <- attr(attr(x, "gradient"), "design_vars")
  if (identical(x$criterion, "Compound")) {
    specs <- attr(x, "hidden_value")
    label <- paste(sapply(specs, function(s)
      sprintf("%.2f*%s", s$weight, sub("-Optimality", "", s$criterion))),
      collapse = " + ")
    cat(sprintf("Optimal design for Compound (%s):\n", label))
  } else if (is_multifactor(dvars)) {
    cat(sprintf("Optimal design for %s (%d factors):\n", x$criterion, length(dvars)))
  } else {
    cat(sprintf("Optimal design for %s:\n", x$criterion))
  }
  print.data.frame(x$optdes, ...)
}



#' Plot function for optdes
#'
#' @description
#' For single-factor models, overlays the support points on the sensitivity function curve.
#' For two-factor models, shows a heatmap of the sensitivity function with the support
#' points overlaid and the Equivalence Theorem contour highlighted.
#' For models with more than two factors, shows a pairwise scatter matrix with one
#' panel per pair of design variables and point size proportional to weight.
#'
#' @param x An object of class \code{optdes}.
#' @param ... Possible extra arguments (currently unused).
#'
#' @export
#'
#' @examples
#' rri <- opt_des(criterion = "I-Optimality", model = y ~ a * exp(-b / x),
#'   parameters = c("a", "b"), par_values = c(1, 1500), design_space = c(212, 422),
#'   reg_int = c(380, 422))
#' plot(rri)
plot.optdes <- function(x, ...) {
  dcols <- coord_cols(x$optdes)

  if (identical(dcols, "Point")) {
    # ── Single-factor: sensitivity curve + support points ──────────────────
    Point <- Value <- Weight <- NULL
    # Place support points at the minimum of the sensitivity curve so they
    # appear inside the plot area (y=0 is often outside the visible range).
    sens_min <- min(x$sens$data$y, na.rm = TRUE)
    x$optdes[["Value"]]  <- rep(sens_min, nrow(x$optdes))
    x$optdes[["Weight"]] <- round(x$optdes[["Weight"]], 2)
    p <- x$sens +
      ggplot2::geom_point(data = x$optdes, ggplot2::aes(x = Point, y = Value),
                          size = 4, shape = 16, color = "darkgreen") +
      ggplot2::geom_text(data = x$optdes, ggplot2::aes(x = Point, y = Value, label = Weight),
                         hjust = 0.5, vjust = -0.6, size = 3.5) +
      ggplot2::labs(x = "Design Space", y = "Sensitivity Function")
    return(p)
  }

  if (length(dcols) == 2L) {
    # ── Two-factor: sensitivity heatmap with support points overlaid ───────
    # x$sens is pre-computed by the cocktail algorithm and already includes
    # the support points and ET contour — return it directly.
    return(x$sens)
  }

  # ── d > 2: pairwise scatter matrix ────────────────────────────────────
  return(.plot_pairs_optdes(x$optdes, x$criterion))
}


# Pairwise scatter matrix for designs with d > 2 factors.
# Shows all C(d,2) unique pairs as facets; point size proportional to weight.
.plot_pairs_optdes <- function(design, criterion_label) {
  wlabel <- xvar <- yvar <- x <- y <- Weight <- NULL  # avoid R CMD check NOTE on ggplot2 aes variables
  dvars <- coord_cols(design)
  d     <- length(dvars)

  pairs_list <- utils::combn(dvars, 2L, simplify = FALSE)
  long_df    <- do.call(rbind, lapply(pairs_list, function(p) {
    data.frame(
      xvar   = p[1],
      yvar   = p[2],
      x      = design[[p[1]]],
      y      = design[[p[2]]],
      Weight = design$Weight,
      wlabel = paste0(round(design$Weight, 2)),
      stringsAsFactors = FALSE
    )
  }))
  # Preserve variable ordering so strips read x1, x2, x3, ...
  long_df$xvar <- factor(long_df$xvar, levels = dvars)
  long_df$yvar <- factor(long_df$yvar, levels = dvars)

  ggplot2::ggplot(long_df, ggplot2::aes(x = x, y = y, size = Weight)) +
    ggplot2::geom_point(colour = "darkgreen", alpha = 0.8) +
    ggplot2::geom_text(ggplot2::aes(label = wlabel),
                       vjust = -1, size = 3, show.legend = FALSE) +
    ggplot2::facet_grid(yvar ~ xvar, scales = "free") +
    ggplot2::scale_size_continuous(limits = c(0, 1), range = c(1, 10)) +
    ggplot2::theme_bw() +
    ggplot2::labs(
      title   = paste(criterion_label, "optimal design"),
      x       = NULL,
      y       = NULL,
      caption = sprintf(
        "%d factors, %d support points. Column strip = x-axis variable, row strip = y-axis variable.",
        d, nrow(design))
    ) +
    ggplot2::theme(strip.background = ggplot2::element_rect(fill = "grey95"),
                   strip.text       = ggplot2::element_text(size = 9, face = "bold"))
}

library(optedr)

# Shared fixtures -------------------------------------------------------
model  <- y ~ a * exp(-b / x)
params <- c("a", "b")
pvals  <- c(1, 1500)
des    <- data.frame(Point = c(329.3, 422), Weight = c(0.5, 0.5))

# tr() ------------------------------------------------------------------

test_that("tr returns sum of diagonal", {
  expect_equal(optedr:::tr(diag(3)), 3)
  expect_equal(optedr:::tr(matrix(c(1, 2, 3, 4), 2, 2)), 5)  # 1 + 4
  expect_equal(optedr:::tr(matrix(0, 3, 3)), 0)
})

# inv_spd() -------------------------------------------------------------

test_that("inv_spd inverts a 2x2 SPD matrix", {
  M <- matrix(c(4, 2, 2, 3), 2, 2)
  expect_equal(M %*% optedr:::inv_spd(M), diag(2), tolerance = 1e-10)
})

test_that("inv_spd falls back to pseudoinverse for singular matrix and warns", {
  M_sing <- matrix(c(1, 1, 1, 1), 2, 2)  # rank 1
  expect_warning(optedr:::inv_spd(M_sing), "near-zero singular")
})

test_that("inv_spd pseudoinverse satisfies Moore-Penrose condition M M+ M = M", {
  M <- matrix(c(1, 1, 1, 1), 2, 2)
  P <- suppressWarnings(optedr:::inv_spd(M))
  expect_equal(M %*% P %*% M, M, tolerance = 1e-10)
})

# findminval / findmaxval / findmax -------------------------------------

test_that("findminval and findmaxval find global extrema on a grid", {
  f <- function(x) (x - 3)^2
  expect_equal(optedr:::findminval(f, c(0, 6), 1000),  0, tolerance = 0.01)
  expect_equal(optedr:::findmaxval(f, c(0, 6), 1000),  9, tolerance = 0.1)
})

test_that("findmax returns x location of maximum", {
  f <- function(x) -(x - 3)^2
  expect_equal(optedr:::findmax(f, c(0, 6), 1000), 3, tolerance = 0.01)
})

# check_atwood() --------------------------------------------------------

test_that("check_atwood warns when bound is outside [0, 100]", {
  expect_warning(optedr:::check_atwood(200),  "outside \\[0, 100\\]")
  expect_warning(optedr:::check_atwood(-5),   "outside \\[0, 100\\]")
  expect_warning(optedr:::check_atwood(Inf),  "outside \\[0, 100\\]")
  expect_warning(optedr:::check_atwood(NaN),  "outside \\[0, 100\\]")
})

test_that("check_atwood is silent for valid bounds", {
  expect_silent(optedr:::check_atwood(0))
  expect_silent(optedr:::check_atwood(99.9))
  expect_silent(optedr:::check_atwood(100))
})

# update_sequence() -----------------------------------------------------

test_that("update_sequence removes near-duplicate points", {
  pts    <- c(1.0, 1.001, 2.0, 2.0005, 3.0)
  result <- optedr:::update_sequence(pts, tol = 0.01)
  expect_equal(length(result), 3)
  expect_true(all(diff(result) > 0.01))
})

# delete_points() -------------------------------------------------------

test_that("delete_points removes low-weight points and renormalises", {
  d <- data.frame(Point = c(1, 2, 3), Weight = c(0.01, 0.60, 0.39))
  result <- optedr:::delete_points(d, delta = 0.02)
  expect_equal(nrow(result), 2)
  expect_equal(sum(result$Weight), 1, tolerance = 1e-10)
  expect_true(all(result$Weight > 0.02))
})

test_that("delete_points keeps all points when all weights > delta", {
  d <- data.frame(Point = c(1, 2), Weight = c(0.5, 0.5))
  expect_equal(nrow(optedr:::delete_points(d, 0.1)), 2)
})

# update_design() -------------------------------------------------------

test_that("update_design adds a new point when it is far from existing ones", {
  d <- data.frame(Point = c(1, 3), Weight = c(0.5, 0.5))
  result <- optedr:::update_design(d, xmax = 2, delta = 0.1, new_weight = 0.1)
  expect_equal(nrow(result), 3)
  expect_true(2 %in% result$Point)
})

test_that("update_design merges when new point is close to an existing one", {
  d <- data.frame(Point = c(1.0, 3.0), Weight = c(0.5, 0.5))
  result <- optedr:::update_design(d, xmax = 1.05, delta = 0.2, new_weight = 0.1)
  expect_equal(nrow(result), 2)   # merged, not added
})

# getPar / getCross2 ----------------------------------------------------

test_that("getPar returns TRUE for even number of crosspoints", {
  expect_true(optedr:::getPar(c(1, 2, 3, 4)))
  expect_false(optedr:::getPar(c(1, 2, 3)))
  expect_true(optedr:::getPar(numeric(0)))   # 0 is even
})

test_that("getCross2 returns correct region limits", {
  # even, start = FALSE: return cross as-is
  expect_equal(optedr:::getCross2(c(2, 4), 1, 5, start = FALSE, par = TRUE),  c(2, 4))
  # even, start = TRUE: prepend min, append max
  expect_equal(optedr:::getCross2(c(2, 4), 1, 5, start = TRUE,  par = TRUE),  c(1, 2, 4, 5))
  # odd, start = TRUE: prepend min
  expect_equal(optedr:::getCross2(c(3), 1, 5, start = TRUE,  par = FALSE), c(1, 3))
  # odd, start = FALSE: append max
  expect_equal(optedr:::getCross2(c(3), 1, 5, start = FALSE, par = FALSE), c(3, 5))
})

# integrate_reg_int() ---------------------------------------------------

test_that("integrate_reg_int returns a symmetric PSD matrix", {
  grad <- optedr:::gradient(model, params, pvals)
  B    <- optedr:::integrate_reg_int(grad, 2, c(380, 422))
  expect_equal(dim(B), c(2, 2))
  expect_equal(B, t(B), tolerance = 1e-10)
  expect_true(all(eigen(B)$values >= -1e-10))
})

# update_weights() ------------------------------------------------------

test_that("update_weights returns normalised positive weights", {
  grad  <- optedr:::gradient(model, params, pvals)
  M     <- optedr:::inf_mat(grad, des)
  sensM <- optedr:::dsens(grad, M)
  new_w <- optedr:::update_weights(des, sensM, k = 2L, delta = 0.5)
  expect_equal(sum(new_w), 1, tolerance = 1e-10)
  expect_true(all(new_w > 0))
})

test_that("update_weights errors on non-finite sensitivity (non-identifiable model)", {
  # Create a rank-deficient M whose pseudoinverse gives near-zero sensitivity
  # Use a degenerate design (two identical points) to force singular M
  d_sing <- data.frame(Point = c(300, 300, 400), Weight = c(1/3, 1/3, 1/3))
  grad   <- optedr:::gradient(model, params, pvals)
  M_sing <- optedr:::inf_mat(grad, d_sing)   # rank 1 (two identical points)
  sens_fn <- suppressWarnings(optedr:::dsens(grad, M_sing))
  # Verify sensitivity values are finite (pseudoinverse handles rank-1 case)
  val <- sens_fn(350)
  expect_true(is.finite(as.numeric(val)))
})

# weight_function() -----------------------------------------------------

test_that("weight_function returns constant 1 for Normal", {
  f <- optedr:::weight_function(model, params, pvals, distribution = "Normal")
  expect_equal(f(300), 1)
})

test_that("weight_function warns for unknown distribution", {
  expect_warning(
    optedr:::weight_function(model, params, pvals, distribution = "Binomial"),
    "Not a valid distribution"
  )
})

test_that("weight_function returns a non-trivial function for Gamma", {
  f <- optedr:::weight_function(model, params, pvals, distribution = "Gamma")
  expect_true(is.function(f))
  val <- f(300)
  expect_true(is.finite(val))
  expect_gt(val, 0)
})

#' Augment Design
#'
#' @description
#' Augments a design. The user gives an initial design for which he would like to add points
#' and specifies the weight of the new points. Then he is prompted to choose a
#' minimum efficiency. After that, the candidate points region is calculated
#' and the user can choose the points and weights to add.
#'
#' @param criterion character variable with the chosen optimality criterion. Can be one of the following:
#'   * 'D-Optimality'
#'   * 'Ds-Optimality'
#'   * 'A-Optimality'
#'   * 'I-Optimality'
#'   * 'L-Optimality'
#' @param init_design dataframe with "Point" and "Weight" columns that represents the initial design to augment
#' @param alpha combined weight of the new points
#' @param model formula that represents the model with x as the independent variable
#' @param parameters character vector with the unknown parameters of the model to estimate
#' @param par_values numeric vector with the initial values of the unknown parameters
#' @param design_space numeric vector with the limits of the space of the design
#' @param calc_optimal_design boolean parameter, if TRUE, the optimal design is calculated and efficiencies of the initial and augmented design are given
#' @param weight_fun optional one variable function that represents the square of the structure of variance, in case of heteroscedastic variance of the response
#' @param par_int optional numeric vector with the index of the \code{parameters} of interest for Ds-optimality.
#' @param matB optional matrix of dimensions k x k, for L-optimality.
#' @param distribution character specifying the probability distribution of the response. Can be one of the following:
#'   * 'Homoscedasticity'
#'   * 'Gamma', which can be used for exponential or normal heteroscedastic with constant relative error
#'   * 'Poisson'
#'   * 'Logistic'
#'   * 'Log-Normal' (work in progress)
#' @param delta_val optional numeric value for the minimum relative efficiency. If \code{NULL} (default),
#'   the user is prompted interactively. Providing this value enables non-interactive use.
#' @param new_points optional dataframe with \code{Point} and \code{Weight} columns specifying the points
#'   to add. All points must lie within the candidate region determined by \code{delta_val}. If \code{NULL}
#'   (default), the user is prompted interactively.
#' @param n_lhs integer number of LHS points used to visualise the candidate region for multi-factor
#'   models with \code{d > 2} (default 2000). Increase for smoother scatter plots.
#'
#' @return A dataframe that represents the augmented design
#' @export
#'
#' @examples
#' \donttest{
#' init_des <- data.frame("Point" = c(30, 60, 90), "Weight" = c(1/3, 1/3, 1/3))
#' region <- get_augment_region("D-Optimality", init_des, 0.25,
#'   y ~ 10^(a - b/(c + x)), c("a", "b", "c"),
#'   c(8.07131, 1730.63, 233.426), c(1, 100), FALSE, delta_val = 0.85)
#' new_pts <- data.frame(Point = mean(region$region[1:2]), Weight = 1)
#' augment_design("D-Optimality", init_des, 0.25, y ~ 10^(a - b/(c + x)),
#'   c("a", "b", "c"), c(8.07131, 1730.63, 233.426), c(1, 100), FALSE,
#'   delta_val = 0.85, new_points = new_pts)
#' }
augment_design <- function(criterion, init_design, alpha, model, parameters, par_values,
                           design_space, calc_optimal_design,
                           par_int = NA, matB = NULL, distribution = NA,
                           weight_fun = function(x) 1,
                           delta_val = NULL, new_points = NULL,
                           n_lhs = 2000L) {
  design_vars <- detect_design_vars(model, parameters)
  if (is_multifactor(design_vars)) {
    design_space <- canonicalise_design_space(design_space, design_vars)
    if (!is.na(distribution))
      weight_fun <- weight_function(model, parameters, par_values, distribution = distribution)
    grad <- gradient(model, parameters, par_values, weight_fun)
    k    <- length(parameters)
    if (criterion == "A-Optimality") matB <- diag(k)
    if (criterion == "I-Optimality") matB <- integrate_reg_int(grad, k, design_space)
    if (criterion == "Ds-Optimality") {
      grad22     <- gradient22(model, parameters, par_values, par_int, weight_fun)
      inf_mat_1  <- inf_mat(grad,   init_design)
      inf_mat_22 <- inf_mat(grad22, init_design)
      sens_1     <- dsens(grad,   inf_mat_1)
      sens_22    <- dsens(grad22, inf_mat_22)
      eff_fn     <- function(x) (1 - alpha) *
        ((1 + alpha * as.numeric(sens_1(x))   / (1 - alpha)) /
         (1 + alpha * as.numeric(sens_22(x))  / (1 - alpha)))^(1 / length(par_int))
    } else {
      eff_fn <- .aug_eff_fun(criterion, grad, init_design, alpha, k, matB)
    }
    delta_range <- c(findminval(eff_fn, design_space, 2000L),
                     findmaxval(eff_fn, design_space, 2000L))
    if (calc_optimal_design) {
      opt  <- opt_des(criterion, model, parameters, par_values, design_space,
                      matB = matB, weight_fun = weight_fun)
      eff1 <- eff(criterion, inf_mat(grad, init_design), inf_mat(grad, opt$optdes),
                  k = k, intPars = par_int, matB = matB) * 100
      message(crayon::blue(cli::symbol$info),
              " Efficiency of initial design: ", round(eff1, 2), "%")
    }
    delta_val <- ask_delta(delta_range, delta_val)
    if (is.null(delta_val)) return(NULL)
    if (length(design_vars) == 2L)
      plot(.plot_aug_heatmap_2d(design_space, eff_fn, delta_val, init_design))
    else
      plot(.plot_aug_pairs_candidates(design_space, eff_fn, delta_val, init_design, n_lhs))
    cands <- .sample_aug_candidates(eff_fn, design_space, delta_val, n_lhs)
    message(crayon::blue(cli::symbol$info), " ", nrow(cands),
            " candidate points with efficiency >= ", round(delta_val, 4),
            " (from LHS sample of ", n_lhs, ")")
    if (nrow(cands) > 0L) {
      cat("Sample of candidate points:\n")
      print(utils::head(cands[, c(design_vars, "efficiency"), drop = FALSE], 15L))
    }
    pts <- if (!is.null(new_points)) {
      .validate_new_points_mf(new_points, design_vars, design_space, eff_fn, delta_val)
    } else {
      .select_new_points_mf_interactive(design_vars, design_space, eff_fn, delta_val)
    }
    init_norm <- normalize_design_cols(init_design, design_vars)
    aug       <- add_design(init_norm, pts, alpha)
    if (calc_optimal_design) {
      eff2 <- eff(criterion, inf_mat(grad, aug), inf_mat(grad, opt$optdes),
                  k = k, matB = matB) * 100
      message(crayon::blue(cli::symbol$info),
              " Efficiency of augmented design: ", round(eff2, 2), "%")
    }
    return(aug)
  }
  if (!is.na(distribution)) {
    weight_fun <- weight_function(model, parameters, par_values, distribution = distribution)
  }
  if (criterion == "D-Optimality") {
    daugment_design(init_design, alpha, model, parameters, par_values, design_space,
                    calc_optimal_design, weight_fun, delta_val = delta_val, new_points = new_points)
  } else if (criterion == "A-Optimality") {
    laugment_design(init_design, alpha, model, parameters, par_values, design_space,
                    calc_optimal_design, matB = diag(length(parameters)), weight_fun,
                    delta_val = delta_val, new_points = new_points)
  } else if (criterion == "I-Optimality") {
    grad <- gradient(model, parameters, par_values, weight_fun)
    matB <- integrate_reg_int(grad, length(parameters), design_space)
    laugment_design(init_design, alpha, model, parameters, par_values, design_space,
                    calc_optimal_design, matB, weight_fun, delta_val = delta_val, new_points = new_points)
  } else if (criterion == "L-Optimality") {
    laugment_design(init_design, alpha, model, parameters, par_values, design_space,
                    calc_optimal_design, matB, weight_fun, delta_val = delta_val, new_points = new_points)
  } else if (criterion == "Ds-Optimality") {
    dsaugment_design(init_design, alpha, model, parameters, par_values, par_int, design_space,
                     calc_optimal_design, weight_fun, delta_val = delta_val, new_points = new_points)
  } else {
    stop("Invalid criterion. Choose from: D-Optimality, Ds-Optimality, A-Optimality, ",
         "I-Optimality, L-Optimality", call. = FALSE)
  }
}



#' Get Augment Regions
#'
#' @description
#' Given a model and criterion, calculates the candidate points region. The user gives an initial design for which
#' he would like to add points and specifies the weight of the new points. Then he is prompted
#' to choose a minimum efficiency. After that, the candidate points region is calculated.
#'
#' @param criterion character with the chosen optimality criterion. Can be one of the following:
#'   * 'D-Optimality'
#'   * 'Ds-Optimality'
#'   * 'A-Optimality'
#'   * 'I-Optimality'
#'   * 'L-Optimality'
#' @param init_design dataframe with "Point" and "Weight" columns that represents the initial design to augment
#' @param alpha combined weight of the new points
#' @param model formula that represent the model with x as the independent variable
#' @param parameters character vector with the unknown parameters of the model to estimate
#' @param par_values numeric vector with the initial values of the unknown parameters
#' @param design_space numeric vector with the limits of the space of the design
#' @param calc_optimal_design boolean parameter, if TRUE, the optimal design is calculated and efficiencies of the initial and augmented design are given
#' @param weight_fun optional one variable function that represents the square of the structure of variance, in case of heteroscedastic variance of the response
#' @param par_int optional numeric vector with the index of the \code{parameters} of interest for Ds-optimality.
#' @param matB optional matrix of dimensions k x k, for L-optimality.
#' @param distribution character specifying the probability distribution of the response. Can be one of the following:
#'   * 'Homoscedasticity'
#'   * 'Gamma', which can be used for exponential or normal heteroscedastic with constant relative error
#'   * 'Poisson'
#'   * 'Logistic'
#'   * 'Log-Normal' (work in progress)
#' @param delta_val optional numeric value for the minimum relative efficiency. If \code{NULL} (default),
#'   the user is prompted interactively. Providing this value enables non-interactive use.
#' @param n_lhs integer number of LHS points used to visualise and sample the candidate region for
#'   multi-factor models with \code{d > 2} (default 2000). Increase for smoother scatter plots.
#'
#' @return A vector of the points limiting the candidate points region
#' @export
#'
#' @examples
#' \donttest{
#' init_des <- data.frame("Point" = c(30, 60, 90), "Weight" = c(1/3, 1/3, 1/3))
#' get_augment_region("D-Optimality", init_des, 0.25, y ~ 10^(a - b/(c + x)),
#'   c("a", "b", "c"), c(8.07131, 1730.63, 233.426), c(1, 100), FALSE,
#'   delta_val = 0.85)
#' }
get_augment_region <- function(criterion, init_design, alpha, model, parameters, par_values,
                               design_space, calc_optimal_design,
                               par_int = NA, matB = NA, distribution = NA,
                               weight_fun = function(x) 1,
                               delta_val = NULL, n_lhs = 2000L) {
  design_vars <- detect_design_vars(model, parameters)
  if (is_multifactor(design_vars)) {
    design_space <- canonicalise_design_space(design_space, design_vars)
    if (!is.na(distribution))
      weight_fun <- weight_function(model, parameters, par_values, distribution = distribution)
    grad <- gradient(model, parameters, par_values, weight_fun)
    k    <- length(parameters)
    if (criterion == "A-Optimality") matB <- diag(k)
    if (criterion == "I-Optimality") matB <- integrate_reg_int(grad, k, design_space)
    if (criterion == "Ds-Optimality") {
      grad22     <- gradient22(model, parameters, par_values, par_int, weight_fun)
      inf_mat_1  <- inf_mat(grad,   init_design)
      inf_mat_22 <- inf_mat(grad22, init_design)
      sens_1     <- dsens(grad,   inf_mat_1)
      sens_22    <- dsens(grad22, inf_mat_22)
      eff_fn     <- function(x) (1 - alpha) *
        ((1 + alpha * as.numeric(sens_1(x))  / (1 - alpha)) /
         (1 + alpha * as.numeric(sens_22(x)) / (1 - alpha)))^(1 / length(par_int))
    } else {
      eff_fn <- .aug_eff_fun(criterion, grad, init_design, alpha, k, matB)
    }
    delta_range <- c(findminval(eff_fn, design_space, 2000L),
                     findmaxval(eff_fn, design_space, 2000L))
    if (calc_optimal_design) {
      opt  <- opt_des(criterion, model, parameters, par_values, design_space,
                      matB = if (criterion == "L-Optimality") matB else NULL,
                      weight_fun = weight_fun)
      eff1 <- eff(criterion, inf_mat(grad, init_design), inf_mat(grad, opt$optdes),
                  k = k, intPars = par_int, matB = matB) * 100
      message(crayon::blue(cli::symbol$info),
              " Efficiency of initial design: ", round(eff1, 2), "%")
    }
    delta_val <- ask_delta(delta_range, delta_val)
    if (is.null(delta_val)) return(NULL)
    p <- if (length(design_vars) == 2L)
           .plot_aug_heatmap_2d(design_space, eff_fn, delta_val, init_design)
         else
           .plot_aug_pairs_candidates(design_space, eff_fn, delta_val, init_design, n_lhs)
    plot(p)
    cands  <- .sample_aug_candidates(eff_fn, design_space, delta_val, n_lhs)
    message(crayon::blue(cli::symbol$info), " ", nrow(cands),
            " candidate points with efficiency >= ", round(delta_val, 4),
            " (from LHS sample of ", n_lhs, ")")
    return(invisible(.make_augment_region(cands, delta_val, eff_fn, design_vars, p)))
  }
  if (!is.na(distribution)) {
    weight_fun <- weight_function(model, parameters, par_values, distribution = distribution)
  }
  if (criterion == "D-Optimality") {
    get_daugment_region(init_design, alpha, model, parameters, par_values, design_space,
                        calc_optimal_design, weight_fun, delta_val = delta_val)
  } else if (criterion == "A-Optimality") {
    get_laugment_region(init_design, alpha, model, parameters, par_values, design_space,
                        calc_optimal_design, matB = diag(length(parameters)), weight_fun,
                        delta_val = delta_val)
  } else if (criterion == "I-Optimality") {
    grad <- gradient(model, parameters, par_values, weight_fun)
    matB <- integrate_reg_int(grad, length(parameters), design_space)
    get_laugment_region(init_design, alpha, model, parameters, par_values, design_space,
                        calc_optimal_design, matB, weight_fun, delta_val = delta_val)
  } else if (criterion == "L-Optimality") {
    get_laugment_region(init_design, alpha, model, parameters, par_values, design_space,
                        calc_optimal_design, matB, weight_fun, delta_val = delta_val)
  } else if (criterion == "Ds-Optimality") {
    get_dsaugment_region(init_design, alpha, model, parameters, par_values, par_int,
                         design_space, calc_optimal_design, weight_fun, delta_val = delta_val)
  } else {
    stop("Invalid criterion. Choose from: D-Optimality, Ds-Optimality, A-Optimality, ",
         "I-Optimality, L-Optimality", call. = FALSE)
  }
}


# --- Private helpers for interactive augmentation ---------------------------

# Prompts the user to choose a delta value within [delta_range[1], delta_range[2]].
# If delta_val is provided, validates it and returns it directly (non-interactive mode).
# Returns the chosen value, or NULL after 1000 failed interactive attempts.
ask_delta <- function(delta_range, delta_val = NULL) {
  if (!is.null(delta_val)) {
    if (delta_val > delta_range[[2]])
      stop("delta_val (", round(delta_val, 4), ") exceeds the maximum achievable efficiency (",
           round(delta_range[[2]], 4), "). No augmentation can reach this target.", call. = FALSE)
    if (delta_val < delta_range[[1]])
      warning("delta_val (", round(delta_val, 4), ") is below the minimum efficiency (",
              round(delta_range[[1]], 4), "). The entire design space is a candidate region.",
              call. = FALSE)
    return(delta_val)
  }
  if (!interactive())
    stop("Supply delta_val for non-interactive use.", call. = FALSE)
  val <- -Inf
  eval_count <- 0
  while (val < delta_range[[1]] || val > delta_range[[2]]) {
    val <- suppressWarnings(as.numeric(readline(prompt = paste(
      "Choose a value for the minimum relative efficiency between",
      crayon::magenta(ceiling(delta_range[[1]] * 100) / 100), "and",
      crayon::magenta(floor(delta_range[[2]] * 100) / 100), ": \n"
    ))))
    if (is.na(val)) {
      cat(crayon::red(cli::symbol$cross), "The efficiency must be a number")
      val <- -Inf
    } else if (val < delta_range[[1]] || val > delta_range[[2]]) {
      cat(crayon::red(cli::symbol$cross), "The efficiency must be in the given range")
    }
    eval_count <- eval_count + 1
    if (eval_count > 1000) return(NULL)
  }
  val
}

# Builds the augment region plot: efficiency curve, candidate region segments,
# crosspoints, and the delta range bar.
plot_augment_region <- function(eff_fun, cross, delta_range, cand_points_reg, design_space) {
  x_value <- eff <- NULL
  x_val <- seq(design_space[[1]], design_space[[2]], length.out = 10000)
  y_val <- purrr::map_dbl(x_val, eff_fun)
  efficiency <- eff_fun(cross[[1]])
  p <- ggplot2::ggplot() +
    ggplot2::geom_line(mapping = ggplot2::aes(x = x_val, y = y_val), color = "steelblue3") +
    ggplot2::geom_hline(yintercept = efficiency, color = "goldenrod3") +
    ggplot2::xlim(design_space[[1]], design_space[[2]]) +
    ggplot2::labs(x = "x", y = "Efficiency") +
    ggplot2::theme_bw() +
    ggplot2::theme(panel.border = ggplot2::element_blank(),
                   panel.grid.major = ggplot2::element_blank(),
                   panel.grid.minor = ggplot2::element_blank(),
                   axis.line = ggplot2::element_line(colour = "black"))
  for (i in 1:(length(cand_points_reg) / 2)) {
    p <- p + ggplot2::annotate("segment", x = cand_points_reg[2*i-1], xend = cand_points_reg[2*i],
                               y = efficiency, yend = efficiency, linewidth = 1.5, color = "green3")
  }
  cutoffpoints <- data.frame("x_value" = cross, "eff" = purrr::map_dbl(cross, eff_fun))
  p +
    ggplot2::geom_point(data = cutoffpoints, ggplot2::aes(x = x_value, y = eff),
                        shape = 16, size = 2, color = "firebrick3") +
    ggplot2::annotate("segment", x = design_space[[1]], xend = design_space[[1]],
                      y = delta_range[1], yend = delta_range[2],
                      colour = "mediumpurple2", linewidth = 1.5)
}

# Selects new points from the candidate region, either from new_points (non-interactive)
# or via readline prompts (interactive). Returns a data.frame with Point and Weight columns.
select_new_points <- function(cand_points_reg, design_space, new_points = NULL) {
  n_regions <- length(cand_points_reg) / 2
  in_region <- function(pt) any(vapply(seq_len(n_regions), function(i)
    pt >= cand_points_reg[2*i-1] & pt <= cand_points_reg[2*i], logical(1)))
  if (!is.null(new_points)) {
    if (!is.data.frame(new_points) || !identical(names(new_points), c("Point", "Weight")))
      stop("new_points must be a data.frame with 'Point' and 'Weight' columns.", call. = FALSE)
    if (any(new_points$Weight <= 0))
      stop("All weights in new_points must be positive.", call. = FALSE)
    out_space <- new_points$Point < design_space[[1]] | new_points$Point > design_space[[2]]
    if (any(out_space))
      stop("Points outside the design space [", design_space[[1]], ", ", design_space[[2]], "]: ",
           paste(round(new_points$Point[out_space], 4), collapse = ", "), call. = FALSE)
    bad <- !vapply(new_points$Point, in_region, logical(1))
    if (any(bad))
      stop("Points outside the candidate region for the chosen efficiency: ",
           paste(round(new_points$Point[bad], 4), collapse = ", "), call. = FALSE)
    return(new_points)
  }
  if (!interactive())
    stop("Supply new_points for non-interactive use.", call. = FALSE)
  cutoff_text <- paste(
    vapply(seq_len(n_regions), function(i)
      paste0("[", ceiling(cand_points_reg[2*i-1] * 100) / 100, "-",
             floor(cand_points_reg[2*i] * 100) / 100, "]"),
      character(1)),
    collapse = ", ")
  result <- data.frame(Point = double(), Weight = double())
  point_to_add <- -1
  while (!is.na(point_to_add)) {
    cat("The region(s) are ", crayon::green(cutoff_text))
    point_to_add <- suppressWarnings(as.numeric(
      readline(prompt = "Choose a point to add or enter another character to finish: \n")
    ))
    if (!is.na(point_to_add)) {
      if (in_region(point_to_add)) {
        weight_ok <- FALSE
        while (!weight_ok) {
          weight_to_add <- suppressWarnings(as.numeric(
            readline(prompt = "Choose the weight of the point: \n")
          ))
          if (is.na(weight_to_add)) {
            cat(crayon::red(cli::symbol$cross), "The weight must be a positive number")
          } else if (weight_to_add <= 0) {
            cat(crayon::red(cli::symbol$cross), "The weight must be positive")
          } else {
            result[nrow(result) + 1, ] <- c(point_to_add, weight_to_add)
            weight_ok <- TRUE
          }
        }
      } else {
        cat(crayon::red(cli::symbol$cross), "The point is outside the candidate points region \n")
      }
    }
  }
  result
}


# --- Criterion-specific augment functions -----------------------------------

#' D-Augment Design
#'
#' @description
#' D-Augments a design. The user gives an initial design for which he would like to add points
#' and specifies the weight of the new points. Then he is prompted to choose a
#' minimum efficiency. After that, the candidate points region is calculated
#' and the user can choose the points and weights to add.
#'
#' @inherit augment_design return params examples
#'
#' @family augment designs
#'
daugment_design <- function(init_design, alpha, model, parameters, par_values, design_space,
                            calc_optimal_design, weight_fun = function(x) 1,
                            delta_val = NULL, new_points = NULL) {
  grad <- gradient(model, parameters, par_values, weight_fun)
  inf_mat_1 <- inf_mat(grad, init_design)
  sens_1 <- dsens(grad, inf_mat_1)
  eff_fun <- function(x) (1 - alpha) * (1 + alpha * sens_1(x) / (1 - alpha))^(1 / length(parameters))
  delta_range <- c(findminval(eff_fun, design_space, 10000),
                   findmaxval(eff_fun, design_space, 10000))
  if (calc_optimal_design) {
    optimal_design <- opt_des("D-Optimality", model, parameters, par_values, design_space, weight_fun = weight_fun)
    inf_mat_opt <- inf_mat(grad, optimal_design$optdes)
    eff_1 <- (det(inf_mat_1) / det(inf_mat_opt))^(1 / length(parameters)) * 100
    message(crayon::blue(cli::symbol$info), " The efficiency of the initial design is ", round(eff_1, digits = 2), "%")
  }
  delta_val <- ask_delta(delta_range, delta_val)
  if (is.null(delta_val)) return(NULL)
  val_to_add <- (1 - alpha) / alpha * ((delta_val / (1 - alpha))^length(parameters) - 1)
  cross <- sort(crosspoints(val_to_add, sens_1, 10000, 10^(-3), design_space[[1]], design_space[[2]]))
  cand_points_reg <- getCross2(cross, design_space[[1]], design_space[[2]],
                               getStart(cross, design_space[[1]], design_space[[2]], val_to_add, sens_1),
                               getPar(cross))
  plot(plot_augment_region(eff_fun, cross, delta_range, cand_points_reg, design_space))
  pts <- select_new_points(cand_points_reg, design_space, new_points)
  aug_design <- if (nrow(pts) > 0) add_design(init_design, pts, alpha) else init_design
  if (calc_optimal_design) {
    inf_mat_aug <- inf_mat(grad, aug_design)
    eff_2 <- (det(inf_mat_aug) / det(inf_mat_opt))^(1 / length(parameters)) * 100
    message(crayon::blue(cli::symbol$info), " The efficiency of the augmented design is ", round(eff_2, digits = 2), "%")
  }
  aug_design
}


#' L-Augment Design
#'
#' @description
#' L-Augments a design. The user gives an initial design for which he would like to add points
#' and specifies the weight of the new points. Then he is prompted to choose a
#' minimum efficiency. After that, the candidate points region is calculated
#' and the user can choose the points and weights to add.
#'
#' @inherit augment_design params
#'
#' @return A dataframe that represents the L-augmented design
#'
#' @examples
#' \dontrun{
#' init_des <- data.frame("Point" = c(30, 60, 90), "Weight" = c(1/3, 1/3, 1/3))
#' augment_design("I-Optimality", init_des, 0.25, y ~ 10^(a-b/(c+x)), c("a","b","c"),
#'   c(8.07131,  1730.63, 233.426), c(1, 100), TRUE)
#' }
#'
#' @family augment designs
#'
laugment_design <- function(init_design, alpha, model, parameters, par_values, design_space,
                            calc_optimal_design, matB, weight_fun = function(x) 1,
                            delta_val = NULL, new_points = NULL) {
  grad <- gradient(model, parameters, par_values, weight_fun)
  inf_mat_1 <- inf_mat(grad, init_design)
  dsens_1 <- dsens(grad, inf_mat_1)
  isens_1 <- isens(grad, inf_mat_1, matB)
  crit_1 <- icrit(inf_mat_1, matB)
  eff_fun <- function(x) (1 - alpha) * (crit_1 / (crit_1 - alpha * isens_1(x) / (1 - alpha + alpha * dsens_1(x))))
  delta_range <- c(findminval(eff_fun, design_space, 10000),
                   findmaxval(eff_fun, design_space, 10000))
  if (calc_optimal_design) {
    optimal_design <- opt_des("L-Optimality", model, parameters, par_values, design_space, matB = matB, weight_fun = weight_fun)
    inf_mat_opt <- inf_mat(grad, optimal_design$optdes)
    eff_1 <- (tr(matB %*% inv_spd(inf_mat_opt)) / tr(matB %*% inv_spd(inf_mat_1))) * 100
    message(crayon::blue(cli::symbol$info), " The efficiency of the initial design is ", round(eff_1, digits = 2), "%")
  }
  delta_val <- ask_delta(delta_range, delta_val)
  if (is.null(delta_val)) return(NULL)
  cross <- sort(crosspoints(delta_val, eff_fun, 10000, 10^(-3), design_space[[1]], design_space[[2]]))
  cand_points_reg <- getCross2(cross, design_space[[1]], design_space[[2]],
                               getStart(cross, design_space[[1]], design_space[[2]], delta_val, eff_fun),
                               getPar(cross))
  plot(plot_augment_region(eff_fun, cross, delta_range, cand_points_reg, design_space))
  pts <- select_new_points(cand_points_reg, design_space, new_points)
  aug_design <- if (nrow(pts) > 0) add_design(init_design, pts, alpha) else init_design
  if (calc_optimal_design) {
    inf_mat_aug <- inf_mat(grad, aug_design)
    eff_2 <- (tr(matB %*% inv_spd(inf_mat_opt)) / tr(matB %*% inv_spd(inf_mat_aug))) * 100
    message(crayon::blue(cli::symbol$info), " The efficiency of the augmented design is ", round(eff_2, digits = 2), "%")
  }
  aug_design
}


#' Ds-Augment Design
#'
#' @description
#' Ds-Augments a design. The user gives an initial design for which he would like to add points
#' and specifies the weight of the new points. Then he is prompted to choose a
#' minimum efficiency. After that, the candidate points region is calculated
#' and the user can choose the points and weights to add.
#'
#' @inherit augment_design params
#'
#' @return A dataframe that represents the Ds-augmented design
#'
#' @examples
#' \dontrun{
#' init_des <- data.frame("Point" = c(30, 60, 90), "Weight" = c(1/3, 1/3, 1/3))
#' augment_design("Ds-Optimality", init_des, 0.25, y ~ 10^(a-b/(c+x)), c("a","b","c"),
#'   c(8.07131,  1730.63, 233.426), c(1, 100), par_int = c(1), TRUE)
#' }
#'
#' @family augment designs
#'
dsaugment_design <- function(init_design, alpha, model, parameters, par_values, par_int,
                             design_space, calc_optimal_design, weight_fun = function(x) 1,
                             delta_val = NULL, new_points = NULL) {
  grad <- gradient(model, parameters, par_values, weight_fun)
  grad22 <- gradient22(model, parameters, par_values, par_int, weight_fun)
  inf_mat_1 <- inf_mat(grad, init_design)
  inf_mat_22 <- inf_mat(grad22, init_design)
  sens_1 <- dsens(grad, inf_mat_1)
  sens_22 <- dsens(grad22, inf_mat_22)
  dseff <- function(x) (1 - alpha) * ((1 + alpha * sens_1(x) / (1 - alpha)) / (1 + alpha * sens_22(x) / (1 - alpha)))^(1 / length(par_int))
  delta_range <- c(findminval(dseff, design_space, 10000),
                   findmaxval(dseff, design_space, 10000))
  if (calc_optimal_design) {
    optimal_design <- opt_des("Ds-Optimality", model, parameters, par_values, design_space, par_int = par_int, weight_fun = weight_fun)
    inf_mat_opt <- inf_mat(grad, optimal_design$optdes)
    if (length(inf_mat_1[-par_int, -par_int]) == 1) {
      eff_1 <- (inf_mat_opt[-par_int, -par_int] / det(inf_mat_opt) / (inf_mat_1[-par_int, -par_int] / det(inf_mat_1)))^(1 / length(par_int)) * 100
    } else {
      eff_1 <- (det(inf_mat_opt[-par_int, -par_int]) / det(inf_mat_opt) / (det(inf_mat_1[-par_int, -par_int]) / det(inf_mat_1)))^(1 / length(par_int)) * 100
    }
    message(crayon::blue(cli::symbol$info), " The efficiency of the initial design is ", round(eff_1, digits = 2), "%")
  }
  delta_val <- ask_delta(delta_range, delta_val)
  if (is.null(delta_val)) return(NULL)
  cross <- sort(crosspoints(delta_val, dseff, 10000, 10^(-3), design_space[[1]], design_space[[2]]))
  cand_points_reg <- getCross2(cross, design_space[[1]], design_space[[2]],
                               getStart(cross, design_space[[1]], design_space[[2]], delta_val, dseff),
                               getPar(cross))
  plot(plot_augment_region(dseff, cross, delta_range, cand_points_reg, design_space))
  pts <- select_new_points(cand_points_reg, design_space, new_points)
  aug_design <- if (nrow(pts) > 0) add_design(init_design, pts, alpha) else init_design
  if (calc_optimal_design) {
    inf_mat_aug <- inf_mat(grad, aug_design)
    if (length(inf_mat_aug[-par_int, -par_int]) == 1) {
      eff_2 <- (inf_mat_opt[-par_int, -par_int] / det(inf_mat_opt) / (inf_mat_aug[-par_int, -par_int] / det(inf_mat_aug)))^(1 / length(par_int)) * 100
    } else {
      eff_2 <- (det(inf_mat_opt[-par_int, -par_int]) / det(inf_mat_opt) / (det(inf_mat_aug[-par_int, -par_int]) / det(inf_mat_aug)))^(1 / length(par_int)) * 100
    }
    message(crayon::blue(cli::symbol$info), " The efficiency of the augmented design is ", round(eff_2, digits = 2), "%")
  }
  aug_design
}


#' Get D-augment region
#'
#' @description
#' Given a model, calculates the candidate points region for D-Optimality. The user gives an initial
#' design for which he would like to add points and specifies the weight of the new points. Then he
#' is prompted to choose a minimum efficiency. After that, the candidate points region is calculated.
#'
#' @inherit get_augment_region return params examples
#'
#' @family augment regions
#'
get_daugment_region <- function(init_design, alpha, model, parameters, par_values, design_space,
                                calc_optimal_design, weight_fun = function(x) 1,
                                delta_val = NULL) {
  grad <- gradient(model, parameters, par_values, weight_fun)
  inf_mat_1 <- inf_mat(grad, init_design)
  sens_1 <- dsens(grad, inf_mat_1)
  eff_fun <- function(x) (1 - alpha) * (1 + alpha * sens_1(x) / (1 - alpha))^(1 / length(parameters))
  delta_range <- c(findminval(eff_fun, design_space, 10000),
                   findmaxval(eff_fun, design_space, 10000))
  if (calc_optimal_design) {
    optimal_design <- opt_des("D-Optimality", model, parameters, par_values, design_space, weight_fun = weight_fun)
    inf_mat_opt <- inf_mat(grad, optimal_design$optdes)
    eff_1 <- (det(inf_mat_1) / det(inf_mat_opt))^(1 / length(parameters)) * 100
    message(crayon::blue(cli::symbol$info), " The efficiency of the initial design is ", round(eff_1, digits = 2), "%")
  }
  delta_val <- ask_delta(delta_range, delta_val)
  if (is.null(delta_val)) return(NULL)
  val_to_add <- (1 - alpha) / alpha * ((delta_val / (1 - alpha))^length(parameters) - 1)
  cross <- sort(crosspoints(val_to_add, sens_1, 10000, 10^(-3), design_space[[1]], design_space[[2]]))
  cand_points_reg <- getCross2(cross, design_space[[1]], design_space[[2]],
                               getStart(cross, design_space[[1]], design_space[[2]], val_to_add, sens_1),
                               getPar(cross))
  p <- plot_augment_region(eff_fun, cross, delta_range, cand_points_reg, design_space)
  plot(p)
  .make_augment_region(cand_points_reg, delta_val, eff_fun, "x", p)
}


#' Get L-augment region
#'
#' @description
#' Given a model, calculates the candidate points region for L-Optimality. The user gives an initial
#' design for which he would like to add points and specifies the weight of the new points. Then he
#' is prompted to choose a minimum efficiency. After that, the candidate points region is calculated.
#'
#' @inherit get_augment_region return params examples
#'
#' @family augment region
#'
get_laugment_region <- function(init_design, alpha, model, parameters, par_values, design_space,
                                calc_optimal_design, matB, weight_fun = function(x) 1,
                                delta_val = NULL) {
  grad <- gradient(model, parameters, par_values, weight_fun)
  inf_mat_1 <- inf_mat(grad, init_design)
  dsens_1 <- dsens(grad, inf_mat_1)
  isens_1 <- isens(grad, inf_mat_1, matB)
  crit_1 <- icrit(inf_mat_1, matB)
  eff_fun <- function(x) (1 - alpha) * (crit_1 / (crit_1 - alpha * isens_1(x) / (1 - alpha + alpha * dsens_1(x))))
  delta_range <- c(findminval(eff_fun, design_space, 10000),
                   findmaxval(eff_fun, design_space, 10000))
  if (calc_optimal_design) {
    optimal_design <- opt_des("I-Optimality", model, parameters, par_values, design_space, matB = matB, weight_fun = weight_fun)
    inf_mat_opt <- inf_mat(grad, optimal_design$optdes)
    eff_1 <- (tr(matB %*% inv_spd(inf_mat_opt)) / tr(matB %*% inv_spd(inf_mat_1))) * 100
    message(crayon::blue(cli::symbol$info), " The efficiency of the initial design is ", round(eff_1, digits = 2), "%")
  }
  delta_val <- ask_delta(delta_range, delta_val)
  if (is.null(delta_val)) return(NULL)
  cross <- sort(crosspoints(delta_val, eff_fun, 10000, 10^(-3), design_space[[1]], design_space[[2]]))
  cand_points_reg <- getCross2(cross, design_space[[1]], design_space[[2]],
                               getStart(cross, design_space[[1]], design_space[[2]], delta_val, eff_fun),
                               getPar(cross))
  p <- plot_augment_region(eff_fun, cross, delta_range, cand_points_reg, design_space)
  plot(p)
  .make_augment_region(cand_points_reg, delta_val, eff_fun, "x", p)
}


#' Get Ds-augment region
#'
#' @description
#' Given a model, calculates the candidate points region for Ds-Optimality. The user gives an initial
#' design for which he would like to add points and specifies the weight of the new points. Then he
#' is prompted to choose a minimum efficiency. After that, the candidate points region is calculated.
#'
#' @inherit get_augment_region return params examples
#'
#' @family augment region
#'
get_dsaugment_region <- function(init_design, alpha, model, parameters, par_values, par_int,
                                 design_space, calc_optimal_design, weight_fun = function(x) 1,
                                 delta_val = NULL) {
  grad <- gradient(model, parameters, par_values, weight_fun)
  grad22 <- gradient22(model, parameters, par_values, par_int, weight_fun)
  inf_mat_1 <- inf_mat(grad, init_design)
  inf_mat_22 <- inf_mat(grad22, init_design)
  sens_1 <- dsens(grad, inf_mat_1)
  sens_22 <- dsens(grad22, inf_mat_22)
  dseff <- function(x) (1 - alpha) * ((1 + alpha * sens_1(x) / (1 - alpha)) / (1 + alpha * sens_22(x) / (1 - alpha)))^(1 / length(par_int))
  delta_range <- c(findminval(dseff, design_space, 10000),
                   findmaxval(dseff, design_space, 10000))
  if (calc_optimal_design) {
    optimal_design <- opt_des("Ds-Optimality", model, parameters, par_values, design_space, par_int = par_int, weight_fun = weight_fun)
    inf_mat_opt <- inf_mat(grad, optimal_design$optdes)
    if (length(inf_mat_1[-par_int, -par_int]) == 1) {
      eff_1 <- (inf_mat_opt[-par_int, -par_int] / det(inf_mat_opt) / (inf_mat_1[-par_int, -par_int] / det(inf_mat_1)))^(1 / length(par_int)) * 100
    } else {
      eff_1 <- (det(inf_mat_opt[-par_int, -par_int]) / det(inf_mat_opt) / (det(inf_mat_1[-par_int, -par_int]) / det(inf_mat_1)))^(1 / length(par_int)) * 100
    }
    message(crayon::blue(cli::symbol$info), " The efficiency of the initial design is ", round(eff_1, digits = 2), "%")
  }
  delta_val <- ask_delta(delta_range, delta_val)
  if (is.null(delta_val)) return(NULL)
  cross <- sort(crosspoints(delta_val, dseff, 10000, 10^(-3), design_space[[1]], design_space[[2]]))
  cand_points_reg <- getCross2(cross, design_space[[1]], design_space[[2]],
                               getStart(cross, design_space[[1]], design_space[[2]], delta_val, dseff),
                               getPar(cross))
  p <- plot_augment_region(dseff, cross, delta_range, cand_points_reg, design_space)
  plot(p)
  .make_augment_region(cand_points_reg, delta_val, dseff, "x", p)
}


# --- Unified augment_region object ------------------------------------------

# Constructor for the "augment_region" S3 class.
# region: numeric crosspoints vector (1D) or data.frame of candidates (multi-factor)
# design_vars: "x" for 1D or c("x1","x2",...) for multi-factor
.make_augment_region <- function(region, delta_val, eff_fun, design_vars, plot = NULL) {
  structure(
    list(region      = region,
         delta_val   = delta_val,
         eff_fun     = eff_fun,
         design_vars = design_vars,
         plot        = plot),
    class = "augment_region"
  )
}

#' Print method for augment_region objects
#'
#' @param x An object of class \code{augment_region} returned by \code{get_augment_region}.
#' @param ... Unused.
#' @export
print.augment_region <- function(x, ...) {
  cat(sprintf("Augment candidate region  (delta = %.4f)\n", x$delta_val))
  if (is.numeric(x$region)) {
    # 1D: format as intervals
    n_int <- length(x$region) / 2L
    if (n_int == 0L) {
      cat("  No candidate intervals found.\n")
    } else {
      intervals <- vapply(seq_len(n_int), function(i)
        sprintf("[%.4g, %.4g]", x$region[2L*i-1L], x$region[2L*i]), character(1L))
      cat("  Intervals:", paste(intervals, collapse = ", "), "\n")
    }
  } else {
    # Multi-factor: summarise candidate data frame
    d <- length(x$design_vars)
    cat(sprintf("  %d candidate points  (%d factor%s)\n",
                nrow(x$region), d, if (d > 1L) "s" else ""))
    if (nrow(x$region) > 0L) {
      cat(sprintf("  Efficiency range: [%.4f, %.4f]\n",
                  min(x$region$efficiency), max(x$region$efficiency)))
    }
  }
  invisible(x)
}


# --- Multi-factor augment helpers -------------------------------------------

# Build the augment efficiency function for D / A / I / L criteria.
# Returns a scalar function eff(x) where x is a scalar (1D) or named vector (multi-factor).
.aug_eff_fun <- function(criterion, grad, init_design, alpha, k, matB = NULL) {
  M1 <- inf_mat(grad, init_design)
  if (identical(criterion, "D-Optimality")) {
    s1 <- dsens(grad, M1)
    function(x) (1 - alpha) * (1 + alpha * as.numeric(s1(x)) / (1 - alpha))^(1/k)
  } else {
    ds1   <- dsens(grad, M1)
    is1   <- isens(grad, M1, matB)
    crit1 <- icrit(M1, matB)
    function(x) {
      dv <- as.numeric(ds1(x)); iv <- as.numeric(is1(x))
      (1 - alpha) * crit1 / (crit1 - alpha * iv / (1 - alpha + alpha * dv))
    }
  }
}


# Heatmap of the augment efficiency function for d=2 design spaces.
# Returns a ggplot; the candidate region (efficiency >= delta_val) is above the white contour.
.plot_aug_heatmap_2d <- function(design_space, eff_fn, delta_val, init_design) {
  efficiency <- NULL   # avoid R CMD check NOTE on ggplot2 aes variable
  dvars   <- names(design_space)
  n_grid  <- 40L
  grid_df <- do.call(expand.grid, lapply(design_space, function(ds)
    seq(ds[1L], ds[2L], length.out = n_grid)))
  grid_df$efficiency <- apply(grid_df, 1L, function(x)
    as.numeric(eff_fn(stats::setNames(x, dvars))))

  dc  <- intersect(dvars, names(init_design))
  x1s <- rlang::sym(dvars[1L]); x2s <- rlang::sym(dvars[2L])
  ggplot2::ggplot(grid_df, ggplot2::aes(x = !!x1s, y = !!x2s, fill = efficiency)) +
    ggplot2::geom_tile() +
    ggplot2::scale_fill_viridis_c(name = "Efficiency", option = "plasma") +
    ggplot2::geom_contour(ggplot2::aes(z = efficiency, fill = NULL),
                          breaks = delta_val, colour = "white", linewidth = 0.7) +
    ggplot2::geom_point(data = init_design[, dc, drop = FALSE],
                        ggplot2::aes(x = !!x1s, y = !!x2s, fill = NULL),
                        colour = "cyan", size = 3.5, shape = 17) +
    ggplot2::theme_bw() +
    ggplot2::labs(x = dvars[1L], y = dvars[2L],
                  caption = sprintf(
                    "White contour: candidate region boundary (delta = %.4f). Triangles: current design.",
                    delta_val))
}


# Pairwise scatter matrix of the augment candidate region for d > 2.
# Samples n_lhs points, splits into candidate / non-candidate, overlays init_design.
.plot_aug_pairs_candidates <- function(design_space, eff_fn, delta_val, init_design,
                                       n_lhs = 2000L) {
  type <- xvar <- yvar <- x <- y <- NULL  # avoid R CMD check NOTE
  dvars   <- names(design_space)
  pts     <- lhs_sample(n_lhs, design_space)
  eff_vec <- apply(pts, 1L, function(x)
    as.numeric(eff_fn(stats::setNames(x, dvars))))
  eff_finite <- ifelse(is.finite(eff_vec), eff_vec, -Inf)

  sample_df      <- as.data.frame(pts)
  sample_df$type <- ifelse(eff_finite >= delta_val, "Candidate", "Non-candidate")

  dc <- intersect(dvars, names(init_design))
  init_df      <- init_design[, dc, drop = FALSE]
  init_df$type <- "Current design"

  pairs_list <- utils::combn(dvars, 2L, simplify = FALSE)
  long_df <- do.call(rbind, lapply(pairs_list, function(p) {
    rbind(
      data.frame(xvar = p[1], yvar = p[2],
                 x = sample_df[[p[1]]], y = sample_df[[p[2]]],
                 type = sample_df$type,  stringsAsFactors = FALSE),
      data.frame(xvar = p[1], yvar = p[2],
                 x = init_df[[p[1]]],   y = init_df[[p[2]]],
                 type = init_df$type,    stringsAsFactors = FALSE)
    )
  }))
  long_df$xvar <- factor(long_df$xvar, levels = dvars)
  long_df$yvar <- factor(long_df$yvar, levels = dvars)
  long_df$type <- factor(long_df$type,
                          levels = c("Candidate", "Non-candidate", "Current design"))

  ggplot2::ggplot(long_df,
                  ggplot2::aes(x = x, y = y,
                               colour = type, size = type,
                               alpha  = type, shape = type)) +
    ggplot2::geom_point() +
    ggplot2::scale_colour_manual(
      values = c("Candidate" = "steelblue3",
                 "Non-candidate" = "grey75",
                 "Current design" = "firebrick3")) +
    ggplot2::scale_size_manual(
      values = c("Candidate" = 1.5, "Non-candidate" = 0.4, "Current design" = 3)) +
    ggplot2::scale_alpha_manual(
      values = c("Candidate" = 0.7, "Non-candidate" = 0.25, "Current design" = 1)) +
    ggplot2::scale_shape_manual(
      values = c("Candidate" = 16, "Non-candidate" = 16, "Current design" = 17)) +
    ggplot2::facet_grid(yvar ~ xvar, scales = "free") +
    ggplot2::theme_bw() +
    ggplot2::labs(
      title   = sprintf("Augment candidate region (delta = %.4f)", delta_val),
      x = NULL, y = NULL,
      colour = NULL, size = NULL, alpha = NULL, shape = NULL,
      caption = "Blue: candidates (efficiency >= delta). Triangles: current design."
    ) +
    ggplot2::theme(strip.background = ggplot2::element_rect(fill = "grey95"),
                   legend.position  = "bottom")
}


# Sample candidate points from the design space via LHS and return those
# with efficiency >= delta_val.  Includes an 'efficiency' column.
.sample_aug_candidates <- function(eff_fn, design_space, delta_val, n_lhs = 5000L) {
  dvars   <- names(design_space)
  pts     <- lhs_sample(n_lhs, design_space)
  eff_vec <- apply(pts, 1L, function(x) as.numeric(eff_fn(stats::setNames(x, dvars))))
  keep    <- eff_vec >= delta_val
  cands   <- as.data.frame(pts[keep, , drop = FALSE])
  cands$efficiency <- eff_vec[keep]
  cands
}


# Interactive point selection for multi-factor augment.
# Shows the candidate list, then prompts for each coordinate and weight in a loop.
.select_new_points_mf_interactive <- function(design_vars, design_space, eff_fn, delta_val) {
  if (!interactive())
    stop("Supply new_points for non-interactive use.", call. = FALSE)
  result <- data.frame(matrix(ncol = length(design_vars) + 1L, nrow = 0L))
  names(result) <- c(design_vars, "Weight")
  repeat {
    cat(crayon::blue(cli::symbol$info),
        " Enter coordinates for a new point, or press Enter with no value to finish.\n")
    coords <- numeric(length(design_vars))
    valid  <- TRUE
    for (v in design_vars) {
      raw <- readline(prompt = paste0("  ", v, " [", design_space[[v]][1L],
                                      ", ", design_space[[v]][2L], "]: "))
      if (nchar(trimws(raw)) == 0L) { valid <- FALSE; break }
      val <- suppressWarnings(as.numeric(raw))
      if (is.na(val) || val < design_space[[v]][1L] || val > design_space[[v]][2L]) {
        cat(crayon::red(cli::symbol$cross), " Value out of range or not numeric. Point skipped.\n")
        valid <- FALSE; break
      }
      coords[match(v, design_vars)] <- val
    }
    if (!valid) break
    names(coords) <- design_vars
    e <- as.numeric(eff_fn(coords))
    if (e < delta_val) {
      cat(crayon::red(cli::symbol$cross),
          sprintf(" Efficiency %.4f < delta_val (%.4f). Point outside candidate region.\n",
                  e, delta_val))
      next
    }
    cat(crayon::green(cli::symbol$tick),
        sprintf(" Efficiency: %.4f >= %.4f\n", e, delta_val))
    w_raw <- readline(prompt = "  Weight (positive number): ")
    w     <- suppressWarnings(as.numeric(w_raw))
    if (is.na(w) || w <= 0) {
      cat(crayon::red(cli::symbol$cross), " Weight must be a positive number. Point skipped.\n")
      next
    }
    new_row            <- as.data.frame(as.list(c(coords, Weight = w)))
    names(new_row)     <- c(design_vars, "Weight")
    result             <- rbind(result, new_row)
  }
  result
}


# Validate a multi-factor new_points data.frame:
# - must have all design variable columns + Weight
# - all weights positive
# - all points within design_space
# - efficiency of each point >= delta_val
# Returns new_points (possibly with Weight normalised to sum = 1).
.validate_new_points_mf <- function(new_points, design_vars, design_space, eff_fn, delta_val) {
  expected <- c(design_vars, "Weight")
  missing  <- setdiff(expected, names(new_points))
  if (length(missing) > 0L)
    stop("new_points must have columns: ", paste(expected, collapse = ", "),
         " (missing: ", paste(missing, collapse = ", "), ")", call. = FALSE)
  if (any(new_points$Weight <= 0))
    stop("All weights in new_points must be positive.", call. = FALSE)
  for (i in seq_len(nrow(new_points))) {
    pt <- unlist(new_points[i, design_vars])
    for (v in design_vars) {
      if (pt[[v]] < design_space[[v]][1L] || pt[[v]] > design_space[[v]][2L])
        stop("Row ", i, ": coordinate ", v, " = ", round(pt[[v]], 4),
             " is outside design space [", design_space[[v]][1L], ", ",
             design_space[[v]][2L], "].", call. = FALSE)
    }
    e <- as.numeric(eff_fn(pt))
    if (e < delta_val)
      stop("Row ", i, ": efficiency ", round(e, 4), " < delta_val (", round(delta_val, 4),
           "). Point is outside the candidate region.", call. = FALSE)
  }
  new_points
}


# --- Other private functions ------------------------------------------------

#' Gradient function for a subset of variables
#'
#' @description
#' Calculates the gradient function of a \code{model} with respect to a subset of the parameters given in
#' \code{par_int}, \code{char_vars}, evaluates it at the provided \code{values} and returns the result as
#' a function of the variable \code{x}.
#'
#' @param model formula describing the model, which must contain only \code{x}, the parameters defined in
#'   \code{char_vars} and the numerical operators.
#' @param char_vars character vector of the parameters of the model.
#' @param values numeric vector with the nominal values of the parameters in \code{char_vars}.
#' @param par_int vector of indexes indicating the subset of variables to omit in the calculation of the gradient.
#' @param weight_fun optional one variable function that represents the square of the structure of variance, in case of heteroscedastic variance of the response
#'
#' @return A function depending on \code{x} that's the gradient of the \code{model} with respect to \code{char_vars}
gradient22 <- function(model, char_vars, values, par_int, weight_fun = function(x) 1) {
  design_vars   <- detect_design_vars(model, char_vars)
  ext_char_vars <- c(char_vars, design_vars)
  arglist <- lapply(ext_char_vars, function(x) NULL)
  f  <- as.function(append(stats::setNames(arglist, ext_char_vars), quote({})))
  f1 <- stats::deriv(model, char_vars[-par_int], f)
  f2 <- function(x_val) {
    attr(do.call(f1, as.list(c(values, x_val))), "gradient")
  }
  f3 <- function(x) f2(x) * weight_fun(x)
  attr(f3, "design_vars") <- design_vars
  f3
}


#' Calculate crosspoints
#'
#' @description
#' Given the parameters for augmenting a design, this function calculates the
#' crosspoints in the efficiency function that delimit the candidate points
#' region
#'
#' @param val Efficiency value to solve in the curve relationing the space of the design and efficiency of new design
#' @param sens Sensitivity function of the design for the model
#' @param gridlength Number of points in the initial grid used to bracket roots
#' @param tol Tolerance for root refinement passed to \code{uniroot}
#' @param xmin Minimum of the space of the design
#' @param xmax Maximum of the space of the design
#'
#' @return A numeric vector of crosspoints that define the candidate points region
#'
crosspoints <- function(val, sens, gridlength, tol, xmin, xmax) {
  f <- function(x) sens(x) - val
  grid <- seq(xmin, xmax, length.out = gridlength)
  fvals <- purrr::map_dbl(grid, f)
  # Brackets: consecutive grid cells where the function changes sign
  brackets <- which(fvals[-gridlength] * fvals[-1] < 0)
  if (length(brackets) == 0) return(numeric(0))
  # Refine each bracket to the exact root with uniroot
  roots <- vapply(brackets, function(i) {
    tryCatch(
      stats::uniroot(f, lower = grid[i], upper = grid[i + 1L], tol = tol)$root,
      error = function(e) NA_real_
    )
  }, numeric(1))
  sort(roots[!is.na(roots)])
}


#' Update design given crosspoints and alpha
#'
#' @description
#' Given a set of points, a weight and the design, the function adds these points
#' to the new design with uniform weight, and combined weight alpha
#'
#' @param points Points to be added to the design
#' @param alpha Combined weight of the new points to be added to the design
#' @param design A design as a dataframe with "Point" and "Weight" columns
#'
#' @return A design as a dataframe with "Point" and "Weight" columns that is the
#' addition of the design and the new points
add_points <- function(points, alpha, design){
  new_points <- data.frame("Point" = points, "Weight" = rep(alpha/length(points), times = length(points)))
  design["Weight"] <- design["Weight"]*(1-alpha)
  return(rbind(design, new_points))
}


#' Add two designs
#'
#' @param design_1 A dataframe with 'Point' and 'Weight' as columns that represent the first design to add
#' @param design_2 A dataframe with 'Point' and 'Weight' as columns that represent the second design to add
#' @param alpha Weight of the first design
#'
#' @return A design as a dataframe with the weighted addition of the two designs
add_design <- function(design_1, design_2, alpha){
  design_2[, c("Weight")] <- design_2[, c("Weight")] / sum(design_2[, c("Weight")]) * alpha
  design_1[, c("Weight")] <- design_1[, c("Weight")] * (1 - alpha)
  return(rbind(design_1, design_2))
}

# GLM family objects for KL-Optimality ----------------------------
#
# Each family is a list with:
#   b(theta)        cumulant generating function
#   b_prime(theta)  mean: mu = b'(theta)
#   b_dbl(theta)    variance function: V(mu) = phi * b''(theta)
#   link(mu)        canonical link: theta = link(mu)
#   link_inv(eta)   inverse canonical link: mu = link_inv(eta)
#   name            character identifier
#
# The model formula in opt_des() gives the MEAN mu(x, theta) directly.
# KL-optimality converts mu -> canonical theta via link(), then applies
# the cumulant-based KL formula.


#' GLM family specification for KL-Optimality
#'
#' @description
#' Returns a GLM family object for use with \code{criterion = "KL-Optimality"}.
#' The family encodes the cumulant generating function and canonical link needed
#' to compute the Kullback-Leibler divergence between two distributions.
#'
#' @param name character; one of \code{"Normal"}, \code{"Poisson"},
#'   \code{"Binomial"} or \code{"Gamma"}.
#'
#' @return A named list with elements \code{b}, \code{b_prime}, \code{b_dbl},
#'   \code{link}, \code{link_inv} and \code{name}.
#' @export
#'
#' @examples
#' make_glm_family("Poisson")
#' make_glm_family("Normal")
make_glm_family <- function(name) {
  switch(name,
    Normal = list(
      name     = "Normal",
      b        = function(theta) theta^2 / 2,
      b_prime  = function(theta) theta,               # mu = theta
      b_dbl    = function(theta) rep(1, length(theta)),
      link     = function(mu) mu,                     # identity (canonical)
      link_inv = function(eta) eta
    ),
    Poisson = list(
      name     = "Poisson",
      b        = function(theta) exp(theta),
      b_prime  = function(theta) exp(theta),          # mu = exp(theta)
      b_dbl    = function(theta) exp(theta),          # V(mu) = mu
      link     = function(mu) log(pmax(mu, .Machine$double.eps)),
      link_inv = function(eta) exp(eta)
    ),
    Binomial = list(
      name     = "Binomial",
      b        = function(theta) log(1 + exp(theta)),
      b_prime  = function(theta) {
        e <- exp(theta); e / (1 + e)                  # mu = logistic(theta)
      },
      b_dbl    = function(theta) {
        p <- exp(theta) / (1 + exp(theta)); p * (1 - p)
      },
      link     = function(mu) {
        mu <- pmax(pmin(mu, 1 - .Machine$double.eps), .Machine$double.eps)
        log(mu / (1 - mu))                            # logit (canonical)
      },
      link_inv = function(eta) exp(eta) / (1 + exp(eta))
    ),
    Gamma = list(
      name     = "Gamma",
      b        = function(theta) -log(-theta),        # theta < 0
      b_prime  = function(theta) -1 / theta,          # mu = -1/theta > 0
      b_dbl    = function(theta) 1 / theta^2,         # V(mu) = mu^2
      link     = function(mu) -1 / pmax(mu, .Machine$double.eps),
      link_inv = function(eta) -1 / eta               # eta must be < 0
    ),
    stop("Unknown GLM family '", name,
         "'. Choose from: Normal, Poisson, Binomial, Gamma.", call. = FALSE)
  )
}

library(optedr)

# ---------------------------------------------------------------------------
# make_glm_family
# ---------------------------------------------------------------------------

test_that("make_glm_family returns correct structure for all families", {
  for (fam in c("Normal", "Poisson", "Binomial", "Gamma")) {
    f <- make_glm_family(fam)
    expect_equal(f$name, fam)
    expect_true(is.function(f$b))
    expect_true(is.function(f$b_prime))
    expect_true(is.function(f$b_dbl))
    expect_true(is.function(f$link))
    expect_true(is.function(f$link_inv))
  }
})

test_that("make_glm_family throws on unknown family", {
  expect_error(make_glm_family("Beta"), "Unknown GLM family")
})

test_that("Normal family: canonical link is identity", {
  f <- make_glm_family("Normal")
  expect_equal(f$link(3), 3)
  expect_equal(f$link_inv(3), 3)
  expect_equal(f$b_prime(2), 2)    # mu = theta for Normal
})

test_that("Poisson family: canonical link is log", {
  f <- make_glm_family("Poisson")
  expect_equal(f$link(1), 0, tolerance = 1e-10)   # log(1) = 0
  expect_equal(f$link_inv(0), 1, tolerance = 1e-10)
  expect_equal(f$b_prime(0), 1, tolerance = 1e-10) # exp(0) = 1
})

test_that("Binomial family: canonical link is logit, b_prime is p*(1-p)", {
  f <- make_glm_family("Binomial")
  expect_equal(f$link(0.5), 0, tolerance = 1e-10)   # logit(0.5) = 0
  expect_equal(f$link_inv(0), 0.5, tolerance = 1e-10)
  expect_equal(f$b_dbl(0), 0.25, tolerance = 1e-10)  # p*(1-p) at p=0.5
})

test_that("Gamma family: canonical link is negative inverse", {
  f <- make_glm_family("Gamma")
  expect_equal(f$link(2), -0.5, tolerance = 1e-10)   # -1/mu
  expect_equal(f$link_inv(-0.5), 2, tolerance = 1e-10)
  expect_equal(f$b_prime(-1), 1, tolerance = 1e-10)   # -1/theta at theta=-1
})

test_that("KL formula is zero when both models agree (link symmetry)", {
  # For any family, b(t) - b(t) - (t - t)*b'(t) == 0
  for (fam in c("Normal", "Poisson", "Binomial")) {
    f  <- make_glm_family(fam)
    mu <- if (fam == "Binomial") 0.5 else 1
    t  <- f$link(mu)
    kl <- f$b(t) - f$b(t) - (t - t) * f$b_prime(t)
    expect_equal(kl, 0, tolerance = 1e-12, label = paste(fam, "self-KL"))
  }
})


# ---------------------------------------------------------------------------
# Shared fixtures: discriminate quadratic (reference) vs linear (rival)
# under Normal family.
# Reference: mu1(x) = a * x^2,  a = 1
# Rival:     mu2(x) = b * x,    b optimised by inner step
# Design space: [1, 5]
# ---------------------------------------------------------------------------

local({
  kl_res <<- evaluate_promise(opt_des(
    "KL-Optimality",
    model        = y ~ a * x^2,
    parameters   = c("a"),
    par_values   = c(1),
    design_space = c(1, 5),
    rival_model  = y ~ b * x,
    rival_params = c("b"),
    rival_pars   = c(3),
    family       = "Normal",
    phi          = 1,
    max_iter     = 12L
  ))$result
})


# ---------------------------------------------------------------------------
# Structure tests
# ---------------------------------------------------------------------------

test_that("opt_des KL-Optimality returns optdes object with correct criterion", {
  expect_s3_class(kl_res, "optdes")
  expect_equal(kl_res$criterion, "KL-Optimality")
  expect_named(kl_res,
               c("optdes", "convergence", "sens", "criterion", "crit_value", "atwood"),
               ignore.order = TRUE)
})

test_that("opt_des KL-Optimality design is valid", {
  des <- kl_res$optdes
  expect_named(des, c("Point", "Weight"))
  expect_true(all(des$Point >= 1 & des$Point <= 5))
  expect_equal(sum(des$Weight), 1, tolerance = 1e-6)
  expect_gte(nrow(des), 1L)
})

test_that("opt_des KL-Optimality criterion value is positive", {
  expect_gt(kl_res$crit_value, 0)
})

test_that("opt_des KL-Optimality Atwood bound is in valid range", {
  expect_gte(kl_res$atwood, 0)
  expect_lte(kl_res$atwood, 101)
})

test_that("opt_des KL-Optimality design has support near right boundary", {
  # For Normal family discriminating x^2 vs b*x, the sensitivity function
  # sensKL(x) = x^2*(b*-x)^2/2 is maximised near x=5 for most values of b*.
  # By the Equivalence Theorem the optimal design must include a support point
  # near the upper boundary. The HEAVIEST weight may be at a smaller x because
  # the weighting balances sensKL across all support points.
  des <- kl_res$optdes
  expect_gte(nrow(des), 2L)             # at least 2 points (1 point => KL=0)
  expect_gte(max(des$Point), 4.0)       # upper-boundary point is in the support
})


# ---------------------------------------------------------------------------
# Same model with bounded rival (prevents collapse to KL = 0)
# ---------------------------------------------------------------------------

test_that("opt_des KL-Optimality same model with bounded rival", {
  result <- evaluate_promise(opt_des(
    "KL-Optimality",
    model        = y ~ a * exp(-b / x),
    parameters   = c("a", "b"),
    par_values   = c(1, 1500),
    design_space = c(212, 422),
    family       = "Normal",
    phi          = 1,
    rival_pars   = c(1, 2000),
    rival_lower  = c(0.5, 1800),
    rival_upper  = c(2.0, 3000),
    max_iter     = 8L
  ))$result

  expect_s3_class(result, "optdes")
  expect_gt(result$crit_value, 0)
  expect_gte(result$atwood, 0)
  expect_equal(sum(result$optdes$Weight), 1, tolerance = 1e-6)
})


# ---------------------------------------------------------------------------
# print / summary / plot
# ---------------------------------------------------------------------------

test_that("print.optdes works for KL-Optimality", {
  expect_output(print(kl_res), "KL-Optimality")
})

test_that("summary.optdes works for KL-Optimality", {
  expect_output(summary(kl_res), "KL-Optimality")
  expect_output(summary(kl_res), "GLM family: Normal")
  expect_output(summary(kl_res), "Optimal rival parameters")
})

test_that("plot.optdes returns ggplot for KL-Optimality (1D)", {
  p <- plot(kl_res)
  expect_s3_class(p, "gg")
})


# ---------------------------------------------------------------------------
# design_efficiency
# ---------------------------------------------------------------------------

test_that("design_efficiency for KL returns numeric in (0, 2)", {
  eff <- evaluate_promise(design_efficiency(kl_res$optdes, kl_res))$result
  expect_true(is.numeric(eff))
  expect_gt(eff, 0)
  expect_lt(eff, 2)   # allows slight overshooting due to inner-opt noise
})

test_that("design_efficiency KL with 3-point uniform design is positive", {
  alt <- data.frame(Point = c(1, 3, 5), Weight = c(1/3, 1/3, 1/3))
  eff <- evaluate_promise(design_efficiency(alt, kl_res))$result
  expect_gt(eff, 0)
})


# ---------------------------------------------------------------------------
# Input validation
# ---------------------------------------------------------------------------

test_that("opt_des KL-Optimality rejects invalid GLM family", {
  expect_error(
    opt_des("KL-Optimality",
            y ~ a * x^2, "a", 1, c(1, 5),
            rival_model = y ~ b * x, rival_params = "b", rival_pars = 3,
            family = "Cauchy"),
    "Unknown GLM family"
  )
})

test_that("opt_des KL-Optimality rejects non-positive phi", {
  expect_error(
    opt_des("KL-Optimality",
            y ~ a * x^2, "a", 1, c(1, 5),
            rival_model = y ~ b * x, rival_params = "b", rival_pars = 3,
            phi = 0),
    "phi"
  )
})

test_that("check_inputs rejects KL-Optimality spelled wrong", {
  expect_error(
    opt_des("KL-optimality",    # lowercase o
            y ~ a * x^2, "a", 1, c(1, 5)),
    "valid criterion"
  )
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  fig.width  = 6,
  fig.height = 4,
  out.width  = "90%"
)

## ----setup--------------------------------------------------------------------
library(optedr)

## ----region-1d----------------------------------------------------------------
init_des <- data.frame(
  Point  = c(30, 60, 90),
  Weight = c(1/3, 1/3, 1/3)
)

region <- get_augment_region(
  criterion           = "D-Optimality",
  init_design         = init_des,
  alpha               = 0.25,
  model               = y ~ 10^(a - b / (c + x)),
  parameters          = c("a", "b", "c"),
  par_values          = c(8.07131, 1730.63, 233.426),
  design_space        = c(1, 100),
  calc_optimal_design = FALSE,
  delta_val           = 0.85
)
print(region)

## ----augment-1d---------------------------------------------------------------
new_pt <- mean(region$region[1:2])

augmented <- augment_design(
  criterion           = "D-Optimality",
  init_design         = init_des,
  alpha               = 0.25,
  model               = y ~ 10^(a - b / (c + x)),
  parameters          = c("a", "b", "c"),
  par_values          = c(8.07131, 1730.63, 233.426),
  design_space        = c(1, 100),
  calc_optimal_design = FALSE,
  delta_val           = 0.85,
  new_points          = data.frame(Point = new_pt, Weight = 1)
)
print(augmented)
cat("Sum of weights:", sum(augmented$Weight), "\n")

## ----efficiency-1d------------------------------------------------------------
result_opt <- opt_des(
  "D-Optimality",
  y ~ 10^(a - b / (c + x)), c("a", "b", "c"),
  c(8.07131, 1730.63, 233.426), c(1, 100)
)

eff_before <- design_efficiency(init_des, result_opt)
eff_after  <- design_efficiency(augmented, result_opt)

cat("Efficiency before augmenting:", round(eff_before * 100, 2), "%\n")
cat("Efficiency after augmenting: ", round(eff_after  * 100, 2), "%\n")
cat("Gain:                        ", round((eff_after - eff_before) * 100, 2),
    "percentage points\n")

## ----augment-with-opt, eval=FALSE---------------------------------------------
# region_opt <- get_augment_region(
#   criterion           = "D-Optimality",
#   init_design         = init_des,
#   alpha               = 0.25,
#   model               = y ~ 10^(a - b / (c + x)),
#   parameters          = c("a", "b", "c"),
#   par_values          = c(8.07131, 1730.63, 233.426),
#   design_space        = c(1, 100),
#   calc_optimal_design = TRUE,
#   delta_val           = 0.85
# )

## ----region-2d----------------------------------------------------------------
init_2d <- data.frame(
  x1     = c(0.8, 10, 5),
  x2     = c(10, 0.8, 5),
  Weight = c(1/3, 1/3, 1/3)
)

result_2D <- opt_des(
  criterion    = "D-Optimality",
  model        = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters   = c("Vmax", "K1", "K2"),
  par_values   = c(1, 1, 1),
  design_space = list(x1 = c(0.1, 10), x2 = c(0.1, 10))
)

region_2d <- get_augment_region(
  criterion           = "D-Optimality",
  init_design         = init_2d,
  alpha               = 0.25,
  model               = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters          = c("Vmax", "K1", "K2"),
  par_values          = c(1, 1, 1),
  design_space        = list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
  calc_optimal_design = FALSE,
  delta_val           = 0.85
)

## ----augment-2d---------------------------------------------------------------
best_2d <- region_2d$region[which.max(region_2d$region$efficiency), ]

eff_antes <- suppressMessages(design_efficiency(init_2d, result_2D))

aug_2d <- augment_design(
  criterion           = "D-Optimality",
  init_design         = init_2d,
  alpha               = 0.25,
  model               = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters          = c("Vmax", "K1", "K2"),
  par_values          = c(1, 1, 1),
  design_space        = list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
  calc_optimal_design = FALSE,
  delta_val           = 0.85,
  new_points          = data.frame(x1 = best_2d$x1, x2 = best_2d$x2, Weight = 1)
)

eff_despues <- suppressMessages(design_efficiency(aug_2d, result_2D))

cat("Efficiency before:", round(eff_antes  * 100, 2), "%\n")
cat("Efficiency after: ", round(eff_despues * 100, 2), "%\n")
print(aug_2d)

## ----region-3d----------------------------------------------------------------
init_3d <- data.frame(
  x1     = c(0.8, 10,  10,  0.8, 10),
  x2     = c(10,  0.8, 10,  10,  0.8),
  x3     = c(10,  10,  0.8, 0.8, 10),
  Weight = rep(0.2, 5)
)

region_3d <- get_augment_region(
  criterion           = "D-Optimality",
  init_design         = init_3d,
  alpha               = 0.45,
  model               = y ~ Vmax * x1 * x2 * x3 / ((K1+x1) * (K2+x2) * (K3+x3)),
  parameters          = c("Vmax", "K1", "K2", "K3"),
  par_values          = c(1, 1, 1, 1),
  design_space        = list(x1 = c(0.1, 10), x2 = c(0.1, 10), x3 = c(0.1, 10)),
  calc_optimal_design = FALSE,
  delta_val           = 0.93
)
cat("Number of candidate points:", nrow(region_3d$region), "\n")
plot(region_3d$plot)

## ----augment-ds---------------------------------------------------------------
region_ds <- get_augment_region(
  criterion           = "Ds-Optimality",
  init_design         = init_2d,
  alpha               = 0.25,
  model               = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters          = c("Vmax", "K1", "K2"),
  par_values          = c(1, 1, 1),
  design_space        = list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
  calc_optimal_design = FALSE,
  par_int             = c(1),
  delta_val           = 0.85,
  n_lhs               = 5000
)

best_ds <- region_ds$region[which.max(region_ds$region$efficiency), ]

aug_ds <- augment_design(
  criterion           = "Ds-Optimality",
  init_design         = init_2d,
  alpha               = 0.25,
  model               = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters          = c("Vmax", "K1", "K2"),
  par_values          = c(1, 1, 1),
  design_space        = list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
  calc_optimal_design = FALSE,
  par_int             = c(1),
  delta_val           = 0.85,
  new_points          = data.frame(x1 = best_ds$x1, x2 = best_ds$x2, Weight = 1),
  n_lhs               = 5000
)
print(aug_ds)

#' Master function for the cocktail algorithm, that calls the appropriate one given the criterion.
#'
#' @description
#' Depending on the \code{criterion} the cocktail algorithm for the chosen criterion is called,
#' and the necessary parameters for the functions are given from the user input.
#'
#' @param init_design optional dataframe with the initial design for the algorithm.
#' @param grad function of partial derivatives of the model.
#' @param criterion character variable with the chosen optimality criterion.
#' @param par_int numeric vector with the index of the parameters of interest (Ds-Optimality).
#' @param matB optional matrix for L-optimality.
#' @param design_space named list with bounds for each design variable.
#' @param grid.length numeric value for the sensitivity search grid / LHS size.
#' @param join_thresh numeric value for the merge heuristic.
#' @param delete_thresh numeric value for the weight deletion threshold.
#' @param k number of unknown parameters of the model.
#' @param delta_weights numeric value in (0, 1), parameter of the algorithm.
#' @param tol numeric value for convergence of the weight loop.
#' @param tol2 numeric value for the outer stop condition.
#' @param max_iter maximum number of outer iterations.
#' @param compound preprocessed list of compound criterion specifications (internal use).
#' @param kl_spec preprocessed list of KL-Optimality specifications (internal use).
#'
#' @return An object of class \code{optdes}.
#'
#' @family cocktail algorithms
WFMult <- function(init_design, grad, criterion, par_int = NA, matB = NA,
                   design_space, grid.length, join_thresh, delete_thresh,
                   k, delta_weights, tol, tol2, max_iter, compound = NULL,
                   kl_spec = NULL) {
  if (identical(criterion, "D-Optimality"))
    return(DWFMult(init_design, grad, design_space, grid.length,
                   join_thresh, delete_thresh, k, delta_weights, tol, tol2, max_iter))
  else if (identical(criterion, "Ds-Optimality"))
    return(DsWFMult(init_design, grad, par_int, design_space, grid.length,
                    join_thresh, delete_thresh, delta_weights, tol, tol2, max_iter))
  else if (identical(criterion, "A-Optimality"))
    return(IWFMult(init_design, grad, diag(k), design_space, grid.length,
                   join_thresh, delete_thresh, delta_weights, tol, tol2, "A-Optimality", max_iter))
  else if (identical(criterion, "I-Optimality"))
    return(IWFMult(init_design, grad, matB, design_space, grid.length,
                   join_thresh, delete_thresh, delta_weights, tol, tol2, "I-Optimality", max_iter))
  else if (identical(criterion, "L-Optimality"))
    return(IWFMult(init_design, grad, matB, design_space, grid.length,
                   join_thresh, delete_thresh, delta_weights, tol, tol2, "L-Optimality", max_iter))
  else if (identical(criterion, "Compound"))
    return(CWFMult(init_design, grad, compound, design_space, grid.length,
                   join_thresh, delete_thresh, delta_weights, tol, tol2, max_iter))
  else if (identical(criterion, "KL-Optimality"))
    return(KLWFMult(init_design,
                    kl_spec$kl_fun, kl_spec$beta2_init,
                    kl_spec$lower, kl_spec$upper,
                    design_space, grid.length,
                    join_thresh, delete_thresh,
                    delta_weights, tol, tol2, max_iter,
                    kl_meta = kl_spec$kl_meta))
}


#' Cocktail Algorithm implementation for D-Optimality
#'
#' @inherit WFMult return params
#' @family cocktail algorithms
DWFMult <- function(init_design, grad, design_space, grid.length,
                    join_thresh, delete_thresh, k, delta_weights, tol, tol2, max_iter) {
  multi   <- is_multifactor(attr(grad, "design_vars"))
  dc      <- coord_cols(init_design)
  maxiter_weights <- 100L
  crit_val <- numeric(max_iter * (maxiter_weights + 1L) + 1L)
  index <- 1L
  cli::cli_progress_bar("Calculating optimal design")

  for (i in seq_len(max_iter)) {
    cli::cli_progress_update()
    M     <- inf_mat(grad, init_design)
    crit_val[index] <- dcrit(M, k)
    index <- index + 1L
    sensM <- dsens(grad, M)
    xmax  <- findmax(sensM, design_space, grid.length)
    if ((as.numeric(sensM(xmax)) - k) / k < tol2) {
      message("\n", crayon::blue(cli::symbol$info),
              " Stop condition reached: difference between sensitivity and criterion < ", tol2)
      break
    }
    init_design <- update_design(init_design, xmax, join_thresh, 1/(index + 2))
    iter <- 1L; stopw <- FALSE
    while (!stopw) {
      weightsInit <- init_design$Weight
      M     <- inf_mat(grad, init_design)
      crit_val[index] <- dcrit(M, k)
      index <- index + 1L
      sensM <- dsens(grad, M)
      init_design$Weight <- update_weights(init_design, sensM, k, delta_weights)
      stopw <- max(abs(weightsInit - init_design$Weight)) < tol || iter >= maxiter_weights
      iter  <- iter + 1L
    }
    init_design <- delete_points(init_design, delete_thresh)
    if (i %% 5 == 0) init_design <- update_design_total(init_design, join_thresh)
    if (i == max_iter)
      message("\n", crayon::blue(cli::symbol$info),
              " Stop condition not reached, max iterations performed")
  }

  cli::cli_progress_update(force = TRUE)
  base::cat("")
  crit_val[index] <- dcrit(M, k)
  crit_val <- crit_val[seq_len(index)]
  if (!all(is.finite(crit_val)))
    stop("Criterion function returned non-finite values (NA, NaN or Inf). ",
         "The model is likely not identifiable in all specified parameters - ",
         "check for parameter redundancy (see warnings above).",
         call. = FALSE)
  conv_plot <- plot_convergence(data.frame("criteria" = crit_val,
                                           "step"     = seq_along(crit_val)))

  init_design <- init_design[order(init_design[[dc[1L]]]), ]
  rownames(init_design) <- NULL

  M     <- inf_mat(grad, init_design)
  sensM <- dsens(grad, M)
  xmax  <- findmax(sensM, design_space, grid.length * 10L)
  atwood <- k / as.numeric(sensM(xmax)) * 100
  check_atwood(atwood)
  message(crayon::blue(cli::symbol$info), " The lower bound for efficiency is ", atwood, "%")

  plot_opt <- .make_sens_plot(multi, design_space, sensM, dc, init_design, k)

  l_return <- list("optdes" = init_design, "convergence" = conv_plot, "sens" = plot_opt,
                   "criterion" = "D-Optimality", "crit_value" = crit_val[length(crit_val)],
                   "atwood" = atwood)
  attr(l_return, "hidden_value")  <- k
  attr(l_return, "gradient")      <- grad
  attr(l_return, "design_space")  <- design_space
  attr(l_return, "crit_function") <- function(design) { M <- inf_mat(grad, design); dcrit(M, k) }
  class(l_return) <- "optdes"
  l_return
}


#' Cocktail Algorithm implementation for Ds-Optimality
#'
#' @inherit WFMult return params
#' @family cocktail algorithms
DsWFMult <- function(init_design, grad, par_int, design_space, grid.length,
                     join_thresh, delete_thresh, delta_weights, tol, tol2, max_iter) {
  multi   <- is_multifactor(attr(grad, "design_vars"))
  dc      <- coord_cols(init_design)
  maxiter_weights <- 100L
  crit_val <- numeric(max_iter * (maxiter_weights + 1L) + 1L)
  index <- 1L
  cli::cli_progress_bar("Calculating optimal design")

  for (i in seq_len(max_iter)) {
    cli::cli_progress_update()
    M      <- inf_mat(grad, init_design)
    crit_val[index] <- dscrit(M, par_int)
    index <- index + 1L
    sensDs <- dssens(grad, M, par_int)
    xmax   <- findmax(sensDs, design_space, grid.length)
    if ((as.numeric(sensDs(xmax)) - length(par_int)) / length(par_int) < tol2) {
      message("\n", crayon::blue(cli::symbol$info),
              " Stop condition reached: difference between sensitivity and criterion < ", tol2)
      break
    }
    init_design <- update_design(init_design, xmax, join_thresh, 1/(index + 2))
    iter <- 1L; stopw <- FALSE
    while (!stopw) {
      weightsInit <- init_design$Weight
      crit_val[index] <- dscrit(M, par_int)
      index <- index + 1L
      M      <- inf_mat(grad, init_design)
      sensDs <- dssens(grad, M, par_int)
      init_design$Weight <- update_weightsDS(init_design, sensDs, length(par_int), delta_weights)
      stopw <- max(abs(weightsInit - init_design$Weight)) < tol || iter >= maxiter_weights
      iter  <- iter + 1L
    }
    init_design <- delete_points(init_design, delete_thresh)
    if (i %% 5 == 0) init_design <- update_design_total(init_design, join_thresh)
    if (i == max_iter)
      message("\n", crayon::blue(cli::symbol$info),
              " Stop condition not reached, max iterations performed")
  }

  cli::cli_progress_update(force = TRUE)
  crit_val[index] <- dscrit(M, par_int)
  crit_val <- crit_val[seq_len(index)]
  if (!all(is.finite(crit_val)))
    stop("Criterion function returned non-finite values (NA, NaN or Inf). ",
         "The model is likely not identifiable in all specified parameters - ",
         "check for parameter redundancy (see warnings above).",
         call. = FALSE)
  conv_plot <- plot_convergence(data.frame("criteria" = crit_val,
                                           "step"     = seq_along(crit_val)))

  init_design <- init_design[order(init_design[[dc[1L]]]), ]
  rownames(init_design) <- NULL

  M      <- inf_mat(grad, init_design)
  sensM  <- dssens(grad, M, par_int)
  xmax   <- findmax(sensM, design_space, grid.length * 10L)
  atwood <- (2 - as.numeric(sensM(xmax)) / length(par_int)) * 100
  check_atwood(atwood)
  message(crayon::blue(cli::symbol$info), " The lower bound for efficiency is ", atwood, "%")

  plot_opt <- .make_sens_plot(multi, design_space, sensM, dc, init_design, length(par_int))

  l_return <- list("optdes" = init_design, "convergence" = conv_plot, "sens" = plot_opt,
                   "criterion" = "Ds-Optimality", "crit_value" = crit_val[length(crit_val)],
                   "atwood" = atwood)
  attr(l_return, "hidden_value")  <- par_int
  attr(l_return, "gradient")      <- grad
  attr(l_return, "design_space")  <- design_space
  attr(l_return, "crit_function") <- function(design) { M <- inf_mat(grad, design); dscrit(M, par_int) }
  class(l_return) <- "optdes"
  l_return
}


#' Cocktail Algorithm implementation for L-, I- and A-Optimality
#'
#' @inherit WFMult return params
#' @family cocktail algorithms
IWFMult <- function(init_design, grad, matB, design_space, grid.length,
                    join_thresh, delete_thresh, delta_weights, tol, tol2, criterion, max_iter) {
  multi   <- is_multifactor(attr(grad, "design_vars"))
  dc      <- coord_cols(init_design)
  maxiter_weights <- 100L
  crit_val <- numeric(max_iter * (maxiter_weights + 1L) + 1L)
  index <- 1L
  cli::cli_progress_bar("Calculating optimal design")

  for (i in seq_len(max_iter)) {
    cli::cli_progress_update()
    M     <- inf_mat(grad, init_design)
    crit_val[index] <- icrit(M, matB)
    index <- index + 1L
    sensI <- isens(grad, M, matB)
    xmax  <- findmax(sensI, design_space, grid.length)
    if ((as.numeric(sensI(xmax)) - crit_val[index - 1L]) < tol2) {
      message("\n", crayon::blue(cli::symbol$info),
              " Stop condition reached: difference between sensitivity and criterion < ", tol2)
      break
    }
    init_design <- update_design(init_design, xmax, join_thresh, 1/(index + 2))
    iter <- 1L; stopw <- FALSE
    while (!stopw) {
      weightsInit <- init_design$Weight
      M     <- inf_mat(grad, init_design)
      crit  <- icrit(M, matB)
      crit_val[index] <- crit
      index <- index + 1L
      sensI <- isens(grad, M, matB)
      init_design$Weight <- update_weightsI(init_design, sensI, crit, delta_weights)
      stopw <- max(abs(weightsInit - init_design$Weight)) < tol || iter >= maxiter_weights
      iter  <- iter + 1L
    }
    init_design <- delete_points(init_design, delete_thresh)
    if (i %% 5 == 0) init_design <- update_design_total(init_design, join_thresh)
    if (i == max_iter)
      message("\n", crayon::blue(cli::symbol$info),
              " Stop condition not reached, max iterations performed")
  }

  cli::cli_progress_update(force = TRUE)
  M     <- inf_mat(grad, init_design)
  crit_val[index] <- icrit(M, matB)
  crit_val <- crit_val[seq_len(index)]
  if (!all(is.finite(crit_val)))
    stop("Criterion function returned non-finite values (NA, NaN or Inf). ",
         "The model is likely not identifiable in all specified parameters - ",
         "check for parameter redundancy (see warnings above).",
         call. = FALSE)
  conv_plot <- plot_convergence(data.frame("criteria" = crit_val,
                                           "step"     = seq_along(crit_val)))

  init_design <- init_design[order(init_design[[dc[1L]]]), ]
  rownames(init_design) <- NULL

  M     <- inf_mat(grad, init_design)
  sensM <- isens(grad, M, matB)
  xmax  <- findmax(sensM, design_space, grid.length * 10L)
  atwood <- (2 - as.numeric(sensM(xmax)) / icrit(M, matB)) * 100
  check_atwood(atwood)
  message(crayon::blue(cli::symbol$info), " The lower bound for efficiency is ", atwood, "%")

  plot_opt <- .make_sens_plot(multi, design_space, sensM, dc, init_design, icrit(M, matB))

  l_return <- list("optdes" = init_design, "convergence" = conv_plot, "sens" = plot_opt,
                   "criterion" = criterion, "crit_value" = crit_val[length(crit_val)],
                   "atwood" = atwood)
  attr(l_return, "hidden_value")  <- matB
  attr(l_return, "gradient")      <- grad
  attr(l_return, "design_space")  <- design_space
  attr(l_return, "crit_function") <- function(design) { M <- inf_mat(grad, design); icrit(M, matB) }
  class(l_return) <- "optdes"
  l_return
}


#' Cocktail Algorithm implementation for Compound Optimality
#'
#' @inherit WFMult return params
#' @param compound_specs preprocessed list of per-criterion specifications.
#' @family cocktail algorithms
CWFMult <- function(init_design, grad, compound_specs, design_space, grid.length,
                    join_thresh, delete_thresh, delta_weights, tol, tol2, max_iter) {
  multi   <- is_multifactor(attr(grad, "design_vars"))
  dc      <- coord_cols(init_design)
  maxiter_weights <- 100L
  crit_val <- numeric(max_iter * (maxiter_weights + 1L) + 1L)
  index <- 1L
  cli::cli_progress_bar("Calculating optimal design")

  for (i in seq_len(max_iter)) {
    cli::cli_progress_update()
    M     <- inf_mat(grad, init_design)
    crit_val[index] <- ccrit(compound_specs, M)
    index <- index + 1L
    sensC <- csens(compound_specs, grad, M)
    thrC  <- compound_threshold(compound_specs, M)
    xmax  <- findmax(sensC, design_space, grid.length)
    if ((as.numeric(sensC(xmax)) - thrC) < tol2) {
      message("\n", crayon::blue(cli::symbol$info),
              " Stop condition reached: difference between sensitivity and criterion < ", tol2)
      break
    }
    init_design <- update_design(init_design, xmax, join_thresh, 1/(index + 2))
    iter <- 1L; stopw <- FALSE
    while (!stopw) {
      weightsInit <- init_design$Weight
      M     <- inf_mat(grad, init_design)
      crit_val[index] <- ccrit(compound_specs, M)
      index <- index + 1L
      sensC <- csens(compound_specs, grad, M)
      thrC  <- compound_threshold(compound_specs, M)
      init_design$Weight <- update_weightsI(init_design, sensC, thrC, delta_weights)
      stopw <- max(abs(weightsInit - init_design$Weight)) < tol || iter >= maxiter_weights
      iter  <- iter + 1L
    }
    init_design <- delete_points(init_design, delete_thresh)
    if (i %% 5 == 0) init_design <- update_design_total(init_design, join_thresh)
    if (i == max_iter)
      message("\n", crayon::blue(cli::symbol$info),
              " Stop condition not reached, max iterations performed")
  }

  cli::cli_progress_update(force = TRUE)
  M     <- inf_mat(grad, init_design)
  crit_val[index] <- ccrit(compound_specs, M)
  crit_val <- crit_val[seq_len(index)]
  if (!all(is.finite(crit_val)))
    stop("Criterion function returned non-finite values (NA, NaN or Inf). ",
         "The model is likely not identifiable in all specified parameters - ",
         "check for parameter redundancy (see warnings above).",
         call. = FALSE)
  conv_plot <- plot_convergence(data.frame("criteria" = crit_val,
                                           "step"     = seq_along(crit_val)))

  init_design <- init_design[order(init_design[[dc[1L]]]), ]
  rownames(init_design) <- NULL

  M     <- inf_mat(grad, init_design)
  sensM <- csens(compound_specs, grad, M)
  xmax  <- findmax(sensM, design_space, grid.length * 10L)
  thrC  <- compound_threshold(compound_specs, M)
  atwood <- thrC / as.numeric(sensM(xmax)) * 100
  check_atwood(atwood)
  message(crayon::blue(cli::symbol$info), " The lower bound for efficiency is ", atwood, "%")

  plot_opt <- .make_sens_plot(multi, design_space, sensM, dc, init_design, thrC)

  l_return <- list("optdes"      = init_design,
                   "convergence" = conv_plot,
                   "sens"        = plot_opt,
                   "criterion"   = "Compound",
                   "crit_value"  = crit_val[length(crit_val)],
                   "atwood"      = atwood)
  attr(l_return, "hidden_value")  <- compound_specs
  attr(l_return, "gradient")      <- grad
  attr(l_return, "design_space")  <- design_space
  attr(l_return, "crit_function") <- function(design) { M <- inf_mat(grad, design); ccrit(compound_specs, M) }
  class(l_return) <- "optdes"
  l_return
}


# Dispatch plot creation based on design dimensionality.
# 1D: sensitivity curve; d=2: heatmap; d>2: NULL.
.make_sens_plot <- function(multi, design_space, sens_fn, dc, design, criterion_value) {
  if (!multi) {
    plot_sens(design_space[[1L]][1L], design_space[[1L]][2L], sens_fn, criterion_value)
  } else if (length(design_space) == 2L) {
    plot_sens_2d(design_space, sens_fn, design[, c(dc, "Weight"), drop = FALSE], criterion_value)
  } else {
    NULL
  }
}


#' Calculates the optimal design for a specified criterion
#'
#' @description
#' The opt_des function calculates the optimal design for an optimality criterion and a model input from the user.
#' Supports single-factor models (design variable named \code{x}) and multi-factor models
#' (design variables named \code{x1}, \code{x2}, \ldots, detected automatically from the formula).
#'
#' @param criterion character variable with the chosen optimality criterion.
#' @param model formula describing the model. For single-factor models use \code{x} as the
#'   design variable; for multi-factor models use \code{x1}, \code{x2}, etc.
#' @param parameters character vector with the parameter names.
#' @param par_values numeric vector with the nominal parameter values.
#' @param design_space For single-factor models: numeric vector \code{c(min, max)}.
#'   For multi-factor models: named list \code{list(x1 = c(min, max), x2 = c(min, max), ...)}.
#' @param init_design optional dataframe with the initial design. For single-factor models
#'   use columns \code{Point} and \code{Weight}; for multi-factor models use one column per
#'   design variable plus \code{Weight}.
#' @param join_thresh optional numeric threshold for merging nearby design points.
#' @param delete_thresh optional numeric minimum weight to keep a support point.
#' @param delta optional numeric in (0, 1), damping parameter of the algorithm.
#' @param tol optional numeric for convergence of the weight loop.
#' @param tol2 optional numeric for the outer stop condition.
#' @param par_int optional numeric vector of parameter indices for Ds-optimality.
#' @param matB optional k x k matrix for L-optimality.
#' @param reg_int optional bounds for the I-optimality integration region.
#'   Single-factor: \code{c(min, max)}.  Multi-factor: named list matching \code{design_space}.
#' @param max_iter optional integer maximum number of outer cocktail iterations.
#' @param distribution character variable specifying the response distribution.
#' @param weight_fun optional variance-structure weight function.
#' @param compound optional list of criterion specifications for \code{criterion = "Compound"}.
#'   Each element must be a named list with at least \code{criterion} (character) and
#'   \code{weight} (positive numeric). Additional fields per sub-criterion:
#'   \code{par_int} (Ds-Optimality), \code{reg_int} (I-Optimality), \code{matB} (L-Optimality).
#'   Weights are normalised to sum to 1. Example:
#'   \code{list(list(criterion="D-Optimality", weight=0.7),
#'              list(criterion="I-Optimality", weight=0.3, reg_int=c(380,422)))}.
#' @param rival_model optional formula for the rival model used with
#'   \code{criterion = "KL-Optimality"}. Defaults to \code{model} (same structure, rival
#'   parameters explored by inner optimisation).
#' @param rival_params optional character vector of rival model parameter names. Defaults to
#'   \code{parameters}.
#' @param rival_pars optional numeric vector of initial rival parameter values for the inner
#'   optimisation. Defaults to \code{par_values}.
#' @param family character; GLM family for \code{criterion = "KL-Optimality"}. One of
#'   \code{"Normal"}, \code{"Poisson"}, \code{"Binomial"}, \code{"Gamma"}. Default \code{"Normal"}.
#' @param phi positive numeric dispersion parameter for KL-Optimality. Default \code{1}.
#' @param rival_lower optional numeric vector of lower bounds for rival parameters in the
#'   inner optimisation. Defaults to \code{-Inf} for each parameter.
#' @param rival_upper optional numeric vector of upper bounds for rival parameters in the
#'   inner optimisation. Defaults to \code{Inf} for each parameter.
#' @param kl_fun optional user-supplied \code{function(x, beta2)} giving the point
#'   KL divergence at design point \code{x} for rival parameters \code{beta2}, as built by
#'   \code{\link{make_kl_fun}}. When provided, bypasses the \code{family}/\code{rival_model}
#'   path entirely; \code{rival_pars} must still be supplied for the inner optimisation.
#'
#' @return a list of class \code{optdes} with components \code{optdes}, \code{convergence},
#'   \code{sens}, \code{criterion}, \code{crit_value}, and \code{atwood}.
#' @export
#'
#' @examples
#' # Single-factor (backward compatible)
#' opt_des("D-Optimality", y ~ a * exp(-b / x), c("a", "b"), c(1, 1500), c(212, 422))
#'
#' \donttest{
#' # Two-factor Michaelis-Menten bisubstrate model
#' opt_des("D-Optimality",
#'         y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
#'         c("Vmax", "K1", "K2"), c(1, 1, 1),
#'         list(x1 = c(0.1, 10), x2 = c(0.1, 10)))
#'
#' # Compound D+I (70% D, 30% I) for Antoine equation
#' opt_des("Compound",
#'         y ~ 10^(a - b/(c + x)), c("a","b","c"),
#'         c(8.07131, 1730.63, 233.426), c(1, 100),
#'         compound = list(
#'           list(criterion = "D-Optimality", weight = 0.7),
#'           list(criterion = "I-Optimality", weight = 0.3, reg_int = c(60, 100))
#'         ))
#'
#' # KL-Optimality: discriminate quadratic from linear mean model (Normal)
#' opt_des("KL-Optimality",
#'         model        = y ~ a * x^2,
#'         parameters   = c("a"),
#'         par_values   = c(1),
#'         design_space = c(1, 5),
#'         rival_model  = y ~ b * x,
#'         rival_params = c("b"),
#'         rival_pars   = c(3),
#'         family       = "Normal",
#'         phi          = 1)
#' }
opt_des <- function(criterion, model, parameters,
                    par_values   = c(1),
                    design_space,
                    init_design  = NULL,
                    join_thresh  = -1,
                    delete_thresh = 0.02,
                    delta        = 1 / 2,
                    tol          = 0.00001,
                    tol2         = 0.00001,
                    par_int      = NULL,
                    matB         = NULL,
                    reg_int      = NULL,
                    max_iter     = 21L,
                    distribution = NA,
                    weight_fun   = function(x) 1,
                    compound     = NULL,
                    rival_model  = NULL,
                    rival_params = NULL,
                    rival_pars   = NULL,
                    family       = "Normal",
                    phi          = 1,
                    rival_lower  = NULL,
                    rival_upper  = NULL,
                    kl_fun       = NULL) {

  k <- length(parameters)
  if (identical(par_values, c(1))) par_values <- rep(1, k)

  # -- Auto-detect design variables and normalise design_space --------------
  design_vars  <- detect_design_vars(model, parameters)
  multi        <- is_multifactor(design_vars)
  design_space <- canonicalise_design_space(design_space, design_vars)

  # For single-factor: also accept reversed bounds
  if (!multi) {
    bnds <- design_space[[1L]]
    if (bnds[1L] > bnds[2L]) design_space[[1L]] <- rev(bnds)
  }

  # -- Initial design --------------------------------------------------------
  n0 <- k * (k + 1L) / 2L + 1L
  if (is.null(init_design)) {
    if (!multi) {
      bnds <- design_space[[1L]]
      init_design <- data.frame(
        "Point"  = seq(bnds[1L], bnds[2L], length.out = n0),
        "Weight" = rep(1 / n0, n0)
      )
    } else {
      pts <- lhs_sample(n0, design_space)
      init_design <- as.data.frame(pts)
      init_design$Weight <- 1 / n0
    }
  } else {
    # Normalise user-supplied design column names (accepts "Point" for 1D)
    init_design <- normalize_design_cols(init_design, design_vars)
  }

  # -- Input validation ------------------------------------------------------
  check_inputs(
    criterion, model, parameters, par_values, design_space, init_design,
    join_thresh, delete_thresh, delta, tol, tol2, par_int, matB, reg_int, weight_fun
  )

  # -- Weight function and gradient ------------------------------------------
  if (!is.na(distribution))
    weight_fun <- weight_function(model, parameters, par_values, distribution = distribution)
  grad <- gradient(model, parameters, par_values, weight_fun)

  # -- join_thresh default ---------------------------------------------------
  if (join_thresh == -1)
    join_thresh <- min(sapply(design_space, diff)) / 10

  # -- I-Optimality: build matB ----------------------------------------------
  if (identical(criterion, "I-Optimality") && !is.null(reg_int))
    matB <- integrate_reg_int(grad, k, reg_int)

  # -- Compound: preprocess specs (normalise weights, build matB per component) -
  compound_specs <- NULL
  if (identical(criterion, "Compound")) {
    if (is.null(compound) || length(compound) < 2L)
      stop("criterion = 'Compound' requires compound list with at least 2 specifications.",
           call. = FALSE)
    ws <- sapply(compound, function(s) {
      if (is.null(s$weight) || !is.numeric(s$weight) || s$weight <= 0)
        stop("Each compound element must have a positive numeric 'weight'.", call. = FALSE)
      s$weight
    })
    # Warn if weights don't already sum to 1 (before normalising)
    if (abs(sum(ws) - 1) > 1e-8)
      message(crayon::yellow(cli::symbol$warning),
              " compound weights sum to ", round(sum(ws), 6),
              " - normalising to 1.")
    ws <- ws / sum(ws)

    # Warn about duplicate components (same criterion + same auxiliary params)
    .compound_key <- function(s) {
      key <- s$criterion
      if (identical(s$criterion, "Ds-Optimality") && !is.null(s$par_int))
        key <- paste0(key, ":par_int=", paste(sort(s$par_int), collapse=","))
      if (identical(s$criterion, "I-Optimality")  && !is.null(s$reg_int))
        key <- paste0(key, ":reg_int=", paste(s$reg_int, collapse=","))
      if (identical(s$criterion, "L-Optimality")  && !is.null(s$matB))
        key <- paste0(key, ":matB=<custom>")
      key
    }
    keys <- sapply(compound, .compound_key)
    dups <- keys[duplicated(keys)]
    if (length(dups) > 0L)
      warning("Duplicate compound components detected (", paste(unique(dups), collapse=", "),
              "). Their weights will be summed - consider merging them.",
              call. = FALSE)

    compound_specs <- lapply(seq_along(compound), function(i) {
      spec <- compound[[i]]
      spec$weight <- ws[i]
      spec$k      <- k
      ci <- spec$criterion
      if (!ci %in% c("D-Optimality","Ds-Optimality","A-Optimality","I-Optimality","L-Optimality"))
        stop("Unknown sub-criterion '", ci, "' in compound list.", call. = FALSE)
      if (identical(ci, "A-Optimality"))
        spec$matB <- diag(k)
      if (identical(ci, "I-Optimality") && !is.null(spec$reg_int))
        spec$matB <- integrate_reg_int(grad, k, spec$reg_int)
      if (identical(ci, "L-Optimality") && is.null(spec$matB))
        stop("L-Optimality component requires 'matB'.", call. = FALSE)
      spec
    })
  }

  # -- KL-Optimality: build kl_fun and inner-opt spec -----------------------
  kl_spec <- NULL
  if (identical(criterion, "KL-Optimality")) {
    if (!is.null(kl_fun)) {
      # User-supplied kl_fun path
      if (!is.function(kl_fun))
        stop("'kl_fun' must be a function(x, beta2).", call. = FALSE)
      if (is.null(rival_pars))
        stop("'rival_pars' must provide initial rival parameters when 'kl_fun' is used.",
             call. = FALSE)
      lower_eff <- if (is.null(rival_lower)) rep(-Inf, length(rival_pars)) else rival_lower
      upper_eff <- if (is.null(rival_upper)) rep( Inf, length(rival_pars)) else rival_upper
      kl_spec <- list(
        kl_fun     = kl_fun,
        beta2_init = rival_pars,
        lower      = lower_eff,
        upper      = upper_eff,
        kl_meta    = list(type = "kl_fun")
      )
    } else {
      # Standard family-based path: build kl_fun from model evaluators
      if (!is.character(family) || length(family) != 1L)
        stop("'family' must be a single character string for KL-Optimality.", call. = FALSE)
      if (!is.numeric(phi) || phi <= 0)
        stop("'phi' must be a positive number for KL-Optimality.", call. = FALSE)
      rival_model_eff  <- if (is.null(rival_model))  model      else rival_model
      rival_params_eff <- if (is.null(rival_params)) parameters else rival_params
      rival_pars_eff   <- if (is.null(rival_pars))   par_values else rival_pars
      lower_eff <- if (is.null(rival_lower)) rep(-Inf, length(rival_params_eff)) else rival_lower
      upper_eff <- if (is.null(rival_upper)) rep( Inf, length(rival_params_eff)) else rival_upper
      fam           <- make_glm_family(family)
      mu1_fn        <- .make_mu_eval(model, parameters, par_values)
      rival_eval_fn <- .make_rival_eval(rival_model_eff, rival_params_eff)
      kl_fun_built  <- local({
        mu1_ <- mu1_fn; riv_ <- rival_eval_fn; fam_ <- fam; phi_ <- phi
        function(x, beta2) .kl_at_point(x, mu1_, riv_, beta2, fam_, phi_)
      })
      kl_spec <- list(
        kl_fun     = kl_fun_built,
        beta2_init = rival_pars_eff,
        lower      = lower_eff,
        upper      = upper_eff,
        kl_meta    = list(type   = "family",
                          family = fam,
                          phi    = phi)
      )
    }
  }

  # -- Run cocktail algorithm ------------------------------------------------
  output <- WFMult(init_design, grad, criterion,
                   par_int = par_int, matB,
                   design_space, 1000L,
                   join_thresh, delete_thresh, k, delta, tol, tol2, max_iter,
                   compound = compound_specs,
                   kl_spec  = kl_spec)

  attr(output, "model")      <- model
  attr(output, "weight_fun") <- weight_fun
  output
}

library(optedr)

# All tests are error-only: fast, no algorithm execution.
# Shared fixtures -------------------------------------------------------
model  <- y ~ a * exp(-b / x)
params <- c("a", "b")
pvals  <- c(1, 1500)
dspace <- c(212, 422)

# Criterion -------------------------------------------------------------

test_that("invalid criterion strings error", {
  expect_error(opt_des("E-Optimality",  model, params, pvals, dspace))
  expect_error(opt_des("d-optimality",  model, params, pvals, dspace))
  expect_error(opt_des("",              model, params, pvals, dspace))
})

# Model -----------------------------------------------------------------

test_that("non-formula model errors", {
  expect_error(opt_des("D-Optimality", "a * exp(-b/x)", params, pvals, dspace))
  expect_error(opt_des("D-Optimality", 42,              params, pvals, dspace))
})

test_that("model with parameters not matching formula errors", {
  expect_error(opt_des("D-Optimality", y ~ a * exp(-b / x), c("a", "c"), pvals, dspace))
})

test_that("model without x variable errors", {
  expect_error(opt_des("D-Optimality", y ~ a + b, params, pvals, dspace))
})

# par_values ------------------------------------------------------------

test_that("non-numeric par_values errors", {
  expect_error(opt_des("D-Optimality", model, params, c("1", "1500"), dspace))
})

test_that("mismatched parameters / par_values length errors", {
  # Three values for two parameters -> error
  expect_error(opt_des("D-Optimality", model, params, c(1, 1500, 3), dspace))
  # Note: c(1) is the sentinel default and is expanded internally to rep(1, k),
  # so it does NOT error — that behaviour is intentional and tested elsewhere.
})

# design_space ----------------------------------------------------------

test_that("design_space with wrong length errors", {
  expect_error(opt_des("D-Optimality", model, params, pvals, c(212)))
  expect_error(opt_des("D-Optimality", model, params, pvals, c(212, 300, 422)))
})

test_that("non-numeric design_space errors", {
  expect_error(opt_des("D-Optimality", model, params, pvals, c("a", "b")))
})

# init_design -----------------------------------------------------------

test_that("non-dataframe init_design errors", {
  expect_error(
    opt_des("D-Optimality", model, params, pvals, dspace,
            init_design = list(Point = c(212, 422), Weight = c(0.5, 0.5)))
  )
})

test_that("init_design with wrong column names errors", {
  expect_error(
    opt_des("D-Optimality", model, params, pvals, dspace,
            init_design = data.frame(X = c(212, 422), Weight = c(0.5, 0.5)))
  )
})

# Numeric thresholds ----------------------------------------------------

test_that("non-numeric delete_thresh errors", {
  expect_error(opt_des("D-Optimality", model, params, pvals, dspace,
                       delete_thresh = "big"))
})

test_that("delete_thresh at boundary values errors", {
  expect_error(opt_des("D-Optimality", model, params, pvals, dspace, delete_thresh = 0))
  expect_error(opt_des("D-Optimality", model, params, pvals, dspace, delete_thresh = 1))
})

test_that("delta at boundary values errors", {
  expect_error(opt_des("D-Optimality", model, params, pvals, dspace, delta = 0))
  expect_error(opt_des("D-Optimality", model, params, pvals, dspace, delta = 1))
})

test_that("non-positive tol or tol2 errors", {
  expect_error(opt_des("D-Optimality", model, params, pvals, dspace, tol  = 0))
  expect_error(opt_des("D-Optimality", model, params, pvals, dspace, tol  = -1))
  expect_error(opt_des("D-Optimality", model, params, pvals, dspace, tol2 = 0))
})

# Ds-specific -----------------------------------------------------------

test_that("Ds-Optimality without par_int errors", {
  expect_error(opt_des("Ds-Optimality", model, params, pvals, dspace))
})

test_that("Ds-Optimality with non-integer par_int errors", {
  expect_error(opt_des("Ds-Optimality", model, params, pvals, dspace, par_int = c(1.5)))
})

test_that("Ds-Optimality with out-of-bounds par_int errors", {
  expect_error(opt_des("Ds-Optimality", model, params, pvals, dspace, par_int = c(5L)))
})

test_that("Ds-Optimality with par_int covering all parameters errors", {
  expect_error(opt_des("Ds-Optimality", model, params, pvals, dspace, par_int = c(1L, 2L)))
})

# I-specific ------------------------------------------------------------

test_that("I-Optimality without reg_int errors", {
  expect_error(opt_des("I-Optimality", model, params, pvals, dspace))
})

test_that("I-Optimality with wrong-length reg_int errors", {
  expect_error(opt_des("I-Optimality", model, params, pvals, dspace, reg_int = c(380)))
})

test_that("I-Optimality with non-numeric reg_int errors", {
  expect_error(opt_des("I-Optimality", model, params, pvals, dspace,
                       reg_int = c("380", "422")))
})

# L-specific ------------------------------------------------------------

test_that("L-Optimality without matB errors", {
  expect_error(opt_des("L-Optimality", model, params, pvals, dspace))
})

test_that("L-Optimality with wrong-size matB errors", {
  expect_error(opt_des("L-Optimality", model, params, pvals, dspace, matB = matrix(1, 3, 3)))
  expect_error(opt_des("L-Optimality", model, params, pvals, dspace, matB = matrix(1:6, 2, 3)))
})

# weight_fun ------------------------------------------------------------

test_that("non-function weight_fun errors", {
  expect_error(opt_des("D-Optimality", model, params, pvals, dspace, weight_fun = 1))
  expect_error(opt_des("D-Optimality", model, params, pvals, dspace, weight_fun = "id"))
})

library(optedr)

# Shared fixtures -------------------------------------------------------
model  <- y ~ a * exp(-b / x)
params <- c("a", "b")
pvals  <- c(1, 1500)
dspace <- c(212, 422)

# Result structure (uses D-opt to avoid repeating the slow call) --------

test_that("opt_des result has all documented components", {
  res <- evaluate_promise(opt_des("D-Optimality", model, params, pvals, dspace))$result

  expect_named(res, c("optdes", "convergence", "sens", "criterion",
                       "crit_value", "atwood"), ignore.order = TRUE)
  expect_s3_class(res, "optdes")
  expect_equal(res$criterion, "D-Optimality")
  expect_true(is.numeric(res$crit_value))
  expect_true(is.numeric(res$atwood))
  expect_gte(res$atwood, 0)
  expect_lte(res$atwood, 100)
  expect_true(!is.null(attr(res, "gradient")))
  expect_true(!is.null(attr(res, "hidden_value")))
  expect_true(!is.null(attr(res, "crit_function")))
})

test_that("print.optdes, summary.optdes and plot.optdes work without error", {
  res <- evaluate_promise(opt_des("D-Optimality", model, params, pvals, dspace))$result
  expect_output(print(res))
  expect_output(summary(res))
  expect_s3_class(plot(res), "ggplot")
})

# max_iter parameter ----------------------------------------------------

test_that("max_iter controls outer iterations: more -> same or better atwood", {
  r5  <- evaluate_promise(opt_des("D-Optimality", model, params, pvals, dspace, max_iter = 3L))$result
  r21 <- evaluate_promise(opt_des("D-Optimality", model, params, pvals, dspace, max_iter = 21L))$result
  expect_gte(r21$atwood, r5$atwood - 1)   # within 1 percentage point
})

# A-Optimality ----------------------------------------------------------

test_that("opt_des A-Optimality returns valid design", {
  res <- evaluate_promise(opt_des("A-Optimality", model, params, pvals, dspace))$result
  expect_s3_class(res, "optdes")
  expect_equal(res$criterion, "A-Optimality")
  expect_equal(sum(res$optdes$Weight), 1, tolerance = 1e-6)
  expect_true(all(res$optdes$Point >= dspace[1] & res$optdes$Point <= dspace[2]))
  expect_gte(res$atwood, 0)
  expect_lte(res$atwood, 100)
})

# I-Optimality ----------------------------------------------------------

test_that("opt_des I-Optimality returns valid design", {
  res <- evaluate_promise(opt_des(
    "I-Optimality", model, params, pvals, dspace, reg_int = c(380, 422)
  ))$result
  expect_s3_class(res, "optdes")
  expect_equal(res$criterion, "I-Optimality")
  expect_equal(sum(res$optdes$Weight), 1, tolerance = 1e-6)
  expect_gte(res$atwood, 0)
  expect_lte(res$atwood, 100)
})

# L-Optimality ----------------------------------------------------------

test_that("opt_des L-Optimality with matB = I returns valid design", {
  res <- evaluate_promise(opt_des(
    "L-Optimality", model, params, pvals, dspace, matB = diag(2)
  ))$result
  expect_s3_class(res, "optdes")
  expect_equal(res$criterion, "L-Optimality")
  expect_equal(sum(res$optdes$Weight), 1, tolerance = 1e-6)
  expect_gte(res$atwood, 0)
  expect_lte(res$atwood, 100)
})

# distribution parameter ------------------------------------------------

test_that("opt_des with distribution = Gamma returns valid design", {
  res <- evaluate_promise(opt_des(
    "D-Optimality", model, params, pvals, dspace, distribution = "Gamma"
  ))$result
  expect_s3_class(res, "optdes")
  expect_equal(sum(res$optdes$Weight), 1, tolerance = 1e-6)
})

# Non-identifiable model ------------------------------------------------

test_that("non-identifiable model (A-opt) warns about degenerate Atwood bound", {
  expect_warning(
    suppressMessages(opt_des(
      "A-Optimality",
      y ~ (w * c * k * x) / ((1 - x) * (1 + (c - 1) * k * x)),
      c("w", "c", "k"), c(1, 10, 0.5), c(0.05, 0.8)
    )),
    "outside \\[0, 100\\]"
  )
})

test_that("non-identifiable model (D-opt) errors with informative message", {
  expect_error(
    suppressWarnings(opt_des(
      "D-Optimality",
      y ~ (w * c * k * x) / ((1 - x) * (1 + (c - 1) * k * x)),
      c("w", "c", "k"), c(1, 10, 0.5), c(0.05, 0.8)
    )),
    "identifiable"
  )
})

# crit_function attribute -----------------------------------------------

test_that("crit_function attribute evaluates correctly on the optimal design", {
  res      <- evaluate_promise(opt_des("D-Optimality", model, params, pvals, dspace))$result
  crit_fn  <- attr(res, "crit_function")
  crit_val <- crit_fn(res$optdes)
  expect_equal(crit_val, res$crit_value, tolerance = 1e-6)
})

library(optedr)

test_that("opt_des D-Optimality converges to a high-efficiency design", {
  result <- evaluate_promise(opt_des(
    "D-Optimality",
    model = y ~ a * exp(-b / x),
    parameters = c("a", "b"),
    par_values = c(1, 1500),
    design_space = c(212, 422)
  ))$result

  # D-optimal design for this model has 2 support points
  expect_equal(nrow(result$optdes), 2L)
  expect_named(result$optdes, c("Point", "Weight"))

  # Points within design space
  expect_true(all(result$optdes$Point >= 212 & result$optdes$Point <= 422))

  # Weights sum to 1
  expect_equal(sum(result$optdes$Weight), 1, tolerance = 1e-6)

  # Atwood lower bound ≥ 99 % (tight convergence)
  expect_gte(result$atwood, 99)

  # Support points roughly at the known D-optimal positions (329, 422)
  expect_equal(round(sort(result$optdes$Point)), c(329, 422))

  # Roughly equal weights
  expect_equal(result$optdes$Weight, c(0.5, 0.5), tolerance = 0.01)
})

test_that("opt_des Ds-Optimality converges to a high-efficiency design", {
  result <- evaluate_promise(opt_des(
    "Ds-Optimality",
    y ~ th0 * exp(x / th1),
    c("th0", "th1"),
    c(10.4963, -3.2940),
    c(0.94, 30),
    par_int = c(1)
  ))$result

  expect_equal(nrow(result$optdes), 2L)
  expect_named(result$optdes, c("Point", "Weight"))
  expect_true(all(result$optdes$Point >= 0.94 & result$optdes$Point <= 30))
  expect_equal(sum(result$optdes$Weight), 1, tolerance = 1e-6)
  expect_gte(result$atwood, 99)
})

test_that("opt_des wrong input throws error", {
  # Wrong criterion
  expect_error(opt_des(
    "E-Optimality",
    model = y ~ a * exp(-b / x),
    parameters = c("a", "b"),
    par_values = c(1, 1500),
    design_space = c(212, 422)
  ))

  # Missing design_space
  expect_error(opt_des(
    "D-Optimality",
    model = y ~ a * exp(-b / x),
    parameters = c("a", "b"),
    par_values = c(1, 1500)
  ))

  # Ds-Optimality without par_int
  expect_error(opt_des(
    "Ds-Optimality",
    y ~ th0 * exp(x / th1),
    c("th0", "th1"),
    c(10.4963, -3.2940),
    c(0.94, 30)
  ))
})

#' Efficient Round
#'
#' @description
#' Takes an approximate design, and a number of points and converts the design to
#' an approximate design. It uses the multiplier (n - l/2) and evens the total
#' number of observations afterwards.
#'
#' @param design a dataframe with a \code{Weight} column and one or more design-variable columns
#'   (\code{Point} for single-factor designs, \code{x1}, \code{x2}, \ldots for multi-factor designs).
#' @param n an integer that represents the desired number of observations of the exact design
#' @param tol optional parameter for the consideration of an integer in the rounding process
#' @param seed optional integer seed for reproducibility. When the rounded weights sum to less
#'   than \code{n}, a random tie-breaking step is used; supplying \code{seed} makes that step
#'   deterministic by calling \code{set.seed(seed)} immediately before it. \code{NULL} (default)
#'   leaves the global RNG state unchanged.
#'
#' @return a data.frame with the same columns as \code{design}, with \code{Weight} replaced by
#'   integer observation counts summing to \code{n}.
#' @export
#'
#' @examples
#' design_test <- data.frame("Point" = seq(1, 5, length.out = 7),
#'          "Weight" = c(0.1, 0.0001, 0.2, 0.134, 0.073, 0.2111, 0.2818))
#'
#' efficient_round(design_test, 20)
#'
#' exact_design <- efficient_round(design_test, 21)
#' aprox_design <- exact_design
#' aprox_design$Weight <- aprox_design$Weight/sum(aprox_design$Weight)
#'
#' # Reproducible tie-breaking
#' efficient_round(design_test, 20, seed = 42)
efficient_round <- function(design, n, tol = 0.00001, seed = NULL){
  if(n%%1!=0 | n <= 0){
    stop("n must be a possitive integer")
  }
  else if(!is.data.frame(design) || !"Weight" %in% names(design)){
    stop("design must be a data.frame with at least a 'Weight' column")
  }
  else{
    l <- nrow(design)
    app_weights <- design[["Weight"]] * (n - l/2)
    candidates_to_increase <- min(abs(c(app_weights %% 1, app_weights %% 1 - 1))) < tol
    app_weights[candidates_to_increase] <- round(app_weights[candidates_to_increase])
    app_weights <- ceiling(app_weights)
    if(sum(app_weights) > n){
      message(crayon::blue(cli::symbol$info), " The proposed size of rounding is greater than n: \n", paste(app_weights, collapse = " "))
      dif <- app_weights - design[["Weight"]] * n
      decrease_order <- order(dif, decreasing = TRUE)
      index <- 1
      while(sum(app_weights) > n){
        if(app_weights[decrease_order[index]] > 1)
          app_weights[decrease_order[index]] <- app_weights[decrease_order[index]] - 1
        index <- index + 1
        if(index > l){
          dif <- app_weights - design[["Weight"]] * n
          decrease_order <- order(dif, decreasing = TRUE)
          index <- 1
          while(sum(app_weights) > n){
            app_weights[decrease_order[index]] <- app_weights[decrease_order[index]] - 1
            index <- index + 1
          }
        }
      }
      message(crayon::blue(cli::symbol$info), " An alternative with size n is returned")
    }
    else if(sum(app_weights) < n){
      if (!is.null(seed)) set.seed(seed)
      candidates_to_increase <- sample(candidates_to_increase)
      index <- 1
      while(sum(app_weights) < n){
        message(crayon::blue(cli::symbol$info), " The proposed size of rounding is smaller than n: \n", paste(app_weights, collapse = " "))
        app_weights[candidates_to_increase[index]] <- app_weights[candidates_to_increase[index]] + 1
        index <- index + 1
        if(index > length(candidates_to_increase)){
          dif <- app_weights - design[["Weight"]] * n
          increase_order <- order(dif, decreasing = FALSE)
          index <- 1
          while(sum(app_weights) < n){
            app_weights[increase_order[index]] <- app_weights[increase_order[index]] + 1
            index <- index + 1
          }
        }
      }
      message(crayon::blue(cli::symbol$info), " An alternative with size n is returned")
    }
    design[["Weight"]] <- app_weights
    return(design)
  }
}



# Returns a human-readable order-of-magnitude time estimate for 2^k evaluations.
estimate_combo_time <- function(k) {
  n <- 2^k
  if      (n <   1e4) "< 1 second"
  else if (n <   1e5) "a few seconds"
  else if (n <   5e5) "tens of seconds"
  else if (n <   1e7) "minutes"
  else if (n <   1e8) "tens of minutes"
  else                "hours or more"
}


#' Combinatorial round
#'
#' @description
#' Given an approximate design and a number of points, computes all the possible combinations of
#' roundings of each point to the nearest integer, keeps the ones that amount to the requested number of points,
#' and returns the one with the best value for the criterion function.
#'
#' The search is exhaustive and requires \eqn{2^k} evaluations where \eqn{k} is the number of
#' support points.  For designs with more than \code{max_support} support points the function
#' requests confirmation (interactive sessions) or stops with an informative error
#' (non-interactive sessions), unless \code{ask = FALSE}.
#'
#' @param design either a dataframe with the design to round, or an object of class \code{"optdes"}.
#'   If a dataframe, the criterion, model and parameters must be specified. It must have a \code{Weight}
#'   column and one or more design-variable columns (\code{Point} for single-factor,
#'   \code{x1}, \code{x2}, \ldots for multi-factor).
#' @param n integer with the desired number of points of the resulting design.
#' @param criterion character variable with the chosen optimality criterion. Can be one of the following:
#'   * 'D-Optimality'
#'   * 'Ds-Optimality'
#'   * 'A-Optimality'
#'   * 'I-Optimality'
#'   * 'L-Optimality'
#' @param model formula describing the model. Must use \code{x} (single-factor) or
#'   \code{x1}, \code{x2}, \ldots (multi-factor) as design variables.
#' @param parameters character vector with the parameters of the models, as written in the \code{formula}.
#' @param par_values numeric vector with the parameters nominal values, in the same order as given in \code{parameters}.
#' @param weight_fun optional one variable function that represents the square of the structure of variance, in case of heteroscedastic variance of the response.
#' @param par_int optional numeric vector with the index of the \code{parameters} of interest for Ds-optimality.
#' @param reg_int optional numeric vector with the ranges of integration, for I-optimality.
#' @param matB optional matrix of dimensions k x k, for L-optimality.
#' @param max_support integer. Number of support points above which the function triggers the
#'   confirmation mechanism. Default is 15 (\eqn{2^{15} \approx 32\,000} combinations).
#' @param ask logical. If \code{TRUE} (default) and the design exceeds \code{max_support}:
#'   in an interactive session the user is prompted; in a non-interactive session an error is raised.
#'   Set \code{ask = FALSE} to skip confirmation in scripts or pipelines (a message is still emitted).
#'
#' @return A data.frame with the rounded design to n number of points, or \code{NULL} invisibly
#'   if the user declines the confirmation prompt.
#' @export
#'
#' @examples
#' aprox_design <- opt_des("D-Optimality", y ~ a * exp(-b / x), c("a", "b"), c(1, 1500), c(212, 422))
#' combinatorial_round(aprox_design, 27)
combinatorial_round <- function(design, n,
                                criterion  = NULL,
                                model      = NULL,
                                parameters = NULL,
                                par_values = NULL,
                                weight_fun = function(x) 1,
                                par_int    = NULL,
                                reg_int    = NULL,
                                matB       = NULL,
                                max_support = 15L,
                                ask         = TRUE) {
  # --- Step 1: extract design_df (needed to know k before the guard) --------
  if (inherits(design, "optdes")) {
    design_df  <- design$optdes
    criterion  <- design$criterion
    crit_funct <- attr(design, "crit_function")
  } else {
    design_df <- design
    design_df$Weight <- design_df$Weight / sum(design_df$Weight)
  }

  # --- Step 2: large-design guard -------------------------------------------
  k <- nrow(design_df)
  if (k > max_support) {
    n_combos <- 2^k
    time_est <- estimate_combo_time(k)
    info <- paste0(
      "combinatorial_round: the design has ", k, " support points",
      " (max_support = ", max_support, ").\n",
      "  Combinations to evaluate : ",
      format(n_combos, big.mark = ",", scientific = FALSE),
      "  (2^", k, ")\n",
      "  Estimated time           : ", time_est, " (hardware-dependent)\n",
      "  Fast alternative         : efficient_round(design, n)"
    )
    if (ask) {
      if (!interactive()) {
        stop(info, "\n",
             "Set ask = FALSE to proceed without confirmation.",
             call. = FALSE)
      }
      cat(info, "\n")
      ans <- readline("Continue? [y/N]: ")
      if (!tolower(trimws(ans)) %in% c("y", "yes")) {
        message("Aborted. Use efficient_round(design, n) for a fast approximate result.")
        return(invisible(NULL))
      }
    } else {
      message(info)
    }
  }

  # --- Step 3: build criterion function (data.frame path only) --------------
  if (!inherits(design, "optdes")) {
    if (is.null(criterion))
      stop(crayon::red(cli::symbol$cross),
           " criterion must be specified or an 'optdes' object must be provided",
           call. = FALSE)
    grad <- gradient(model, parameters, par_values, weight_fun)
    if (identical(criterion, "D-Optimality")) {
      crit_funct <- function(design_df) {
        M <- inf_mat(grad, design_df); dcrit(M, length(parameters))
      }
    } else if (identical(criterion, "Ds-Optimality")) {
      crit_funct <- function(design_df) {
        M <- inf_mat(grad, design_df); dscrit(M, par_int)
      }
    } else if (identical(criterion, "A-Optimality")) {
      crit_funct <- function(design_df) {
        M <- inf_mat(grad, design_df); icrit(M, diag(length(parameters)))
      }
    } else if (identical(criterion, "I-Optimality")) {
      if (is.null(reg_int))
        stop(crayon::red(cli::symbol$cross),
             " reg_int must be specified for I-Optimality", call. = FALSE)
      matB <- integrate_reg_int(grad, length(par_values), reg_int)
      crit_funct <- function(design_df) {
        M <- inf_mat(grad, design_df); icrit(M, matB)
      }
    } else if (identical(criterion, "L-Optimality")) {
      crit_funct <- function(design_df) {
        M <- inf_mat(grad, design_df); icrit(M, matB)
      }
    } else {
      stop(crayon::red(cli::symbol$cross),
           " Invalid criterion. Choose from: D-Optimality, Ds-Optimality, ",
           "A-Optimality, I-Optimality, L-Optimality", call. = FALSE)
    }
  }

  # --- Step 4: exhaustive search --------------------------------------------
  prod_weights    <- design_df$Weight * n
  combinations_df <- data.frame(
    matrix(c(ceiling(prod_weights), floor(prod_weights)), nrow = 2, byrow = TRUE)
  )
  combinations_df        <- expand.grid(data = combinations_df)
  combinations_df$n      <- rowSums(combinations_df)
  combinations_df        <- combinations_df[combinations_df$n == n, ]
  cc <- coord_cols(design_df)
  combinations_df$crit   <- purrr::map_dbl(
    seq_len(nrow(combinations_df)),
    function(x) {
      temp        <- design_df[, cc, drop = FALSE]
      temp$Weight <- unlist(combinations_df[x, seq_len(ncol(combinations_df) - 1L)]) / n
      crit_funct(temp)
    }
  )
  opt_comb        <- combinations_df[which.min(combinations_df$crit), ]
  output          <- design_df[, cc, drop = FALSE]
  output$Weight   <- unlist(opt_comb[1L, seq_len(ncol(combinations_df) - 2L)])
  rownames(output) <- NULL
  output
}

library(optedr)

# efficient_round -------------------------------------------------------

test_that("rounding works", {
  design_test <- data.frame("Point" = seq(1, 5, length.out = 7),
                             "Weight" = c(0.1, 0.0001, 0.2, 0.134, 0.073, 0.2111, 0.2818))

  eff1    <- evaluate_promise(efficient_round(design_test, 21))
  rounded1 <- data.frame("Point"  = c(1.000000, 1.666667, 2.333333, 3, 3.666667, 4.333333, 5),
                          "Weight" = c(2, 1, 4, 3, 2, 4, 5))
  expect_true(all(mapply(`==`, round(eff1$result, 6), round(rounded1, 6))))

  eff2    <- evaluate_promise(efficient_round(design_test, 20))
  rounded2 <- data.frame("Point"  = c(1.000000, 1.666667, 2.333333, 3, 3.666667, 4.333333, 5),
                          "Weight" = c(2, 1, 4, 3, 1, 4, 5))
  expect_true(all(mapply(`==`, round(eff2$result, 6), round(rounded2, 6))))
  expect_equal(eff2$messages, c("i The proposed size of rounding is greater than n: \n2 1 4 3 2 4 5\n",
                                "i An alternative with size n is returned\n"))
})

test_that("rounding errors", {
  design_test <- data.frame("Point" = seq(1, 5, length.out = 7),
                             "Weight" = c(0.1, 0.0001, 0.2, 0.134, 0.073, 0.2111, 0.2818))

  expect_error(efficient_round(design_test, -21), "n must be a possitive integer")
  expect_error(efficient_round(design_test,  6.3), "n must be a possitive integer")

  # design without 'Weight' column is still rejected
  design_no_weight <- data.frame("Point" = seq(1, 5, length.out = 7), "Obs" = rep(1, 7))
  expect_error(efficient_round(design_no_weight, 15), "'Weight'")

  # design with non-standard coord column name now works (only Weight is required)
  design_test2 <- data.frame("Points" = seq(1, 5, length.out = 7),
                              "Weight" = c(0.1, 0.0001, 0.2, 0.134, 0.073, 0.2111, 0.2818))
  expect_no_error(evaluate_promise(efficient_round(design_test2, 15)))
})

test_that("efficient_round seed parameter gives reproducible results", {
  design_test <- data.frame("Point" = seq(1, 5, length.out = 7),
                             "Weight" = c(0.1, 0.0001, 0.2, 0.134, 0.073, 0.2111, 0.2818))
  r1 <- evaluate_promise(efficient_round(design_test, 21, seed = 42L))$result
  r2 <- evaluate_promise(efficient_round(design_test, 21, seed = 42L))$result
  expect_equal(r1, r2)
})

test_that("efficient_round with seed gives same result as without seed when sample() not needed", {
  design_test <- data.frame("Point" = seq(1, 5, length.out = 7),
                             "Weight" = c(0.1, 0.0001, 0.2, 0.134, 0.073, 0.2111, 0.2818))
  r_seed    <- evaluate_promise(efficient_round(design_test, 21, seed = 1L))$result
  r_no_seed <- evaluate_promise(efficient_round(design_test, 21))$result
  expect_equal(r_seed, r_no_seed)
})

# combinatorial_round ---------------------------------------------------

test_that("combinatorial_round works for D-Optimality from optdes object", {
  aprox <- evaluate_promise(
    opt_des("D-Optimality", y ~ a * exp(-b / x), c("a", "b"), c(1, 1500), c(212, 422))
  )$result
  result <- combinatorial_round(aprox, 27)
  expect_s3_class(result, "data.frame")
  expect_named(result, c("Point", "Weight"))
  expect_equal(sum(result$Weight), 27L)
  expect_true(all(result$Weight >= 0))
})

test_that("combinatorial_round errors in non-interactive context when k > max_support", {
  aprox <- evaluate_promise(
    opt_des("D-Optimality", y ~ a * exp(-b / x), c("a", "b"), c(1, 1500), c(212, 422))
  )$result
  # max_support = 1 forces the guard even for a 2-point design
  expect_error(
    combinatorial_round(aprox, 10, max_support = 1L),
    "support points"
  )
})

test_that("combinatorial_round with ask = FALSE emits message and proceeds", {
  aprox <- evaluate_promise(
    opt_des("D-Optimality", y ~ a * exp(-b / x), c("a", "b"), c(1, 1500), c(212, 422))
  )$result
  # max_support = 1: would normally ask, but ask = FALSE skips to computation
  expect_message(
    combinatorial_round(aprox, 10, max_support = 1L, ask = FALSE),
    "support points"
  )
})

test_that("combinatorial_round max_support = Inf never triggers the guard", {
  aprox <- evaluate_promise(
    opt_des("D-Optimality", y ~ a * exp(-b / x), c("a", "b"), c(1, 1500), c(212, 422))
  )$result
  # With Inf threshold, no message or error regardless of design size
  expect_no_condition(combinatorial_round(aprox, 10, max_support = Inf))
})

#' Check Inputs
#'
#' @description
#' Function to check that the inputs given to the function \code{opt_des} are correct. If not, throws the
#' correspondent error message.
#'
#' @inheritParams opt_des
#'
#'
check_inputs <- function(criterion, model, parameters, par_values, design_space,
                         init_design,
                         join_thresh,
                         delete_thresh,
                         delta,
                         tol,
                         tol2,
                         par_int,
                         matB,
                         reg_int,
                         weight_fun) {
  error_msg <- ""
  # Check for valid criterion
  if (!criterion %in% c("D-Optimality", "Ds-Optimality", "A-Optimality",
                        "I-Optimality", "L-Optimality", "Compound", "KL-Optimality")) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross),
                        " Choose a valid criterion: D-Optimality, Ds-Optimality, A-Optimality, I-Optimality, L-Optimality, Compound or KL-Optimality")
  }

  # Check that the model is a formula
  if (!rlang::is_formula(model)) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " The model must be a formula")
  }

  # Check that parameters is a character vector
  if (!is.character(parameters) || !is.atomic(parameters) || !is.vector(parameters)) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " parameters must be a character vector")
  }

  # check that all the parameters are in the formula
  if (!all(purrr::map_lgl(parameters, function(x) grepl(x, paste(format(model),collapse=""))))) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " The model must contain all the parameters")
  }
  # Check that at least one design variable is present (x for 1D, x1/x2/... for multi-factor)
  if (!grepl("x", paste(format(model), collapse = ""))) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross),
                        " The model must contain a design variable: 'x' for single-factor models or 'x1', 'x2', ... for multi-factor models")
  }

  # Check that par_values is a numeric vector
  if (!is.numeric(par_values) || !is.atomic(par_values) || !is.vector(par_values)) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " par_values must be a numeric vector")
  }

  # Check that there's a nominal value for each parameter
  if (length(parameters) != length(par_values)) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " parameters must be of the same length as par_values")
  }

  # design_space has been canonicalised to a named list by opt_des before this call
  if (!is.list(design_space) ||
      !all(sapply(design_space, function(ds) is.numeric(ds) && length(ds) == 2L))) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross),
                        " design_space must be a named list of numeric c(min, max) pairs")
  }

  # Check that init_design is a dataframe
  if (!is.data.frame(init_design)) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " init_design must be a dataframe")
  }

  # Check the names of init_design: last column must be "Weight";
  # remaining columns are the design variables (already normalised by opt_des)
  if (!"Weight" %in% names(init_design) || ncol(init_design) < 2L) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross),
                        " init_design must have a 'Weight' column and at least one design-variable column")
  }

  # Check that join_thresh is numeric
  if (!is.numeric(join_thresh)) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " join_thresh must be numeric")
  }

  # Check that delete_thresh is numeric and between 0 and 1
  if (!is.numeric(delete_thresh)) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " delete_thresh must be numeric")
  }

  if (delete_thresh >= 1 || delete_thresh <= 0) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " delete_thresh must be in (0, 1)")
  }


  # Check that delta is numeric and between 0 and 1
  if (!is.numeric(delta)) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " delta must be numeric")
  }

  if (delta >= 1 || delta <= 0) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " delta must be in (0, 1)")
  }

  # Check that tol is numeric and greater than 0
  if (!is.numeric(tol)) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " tol must be numeric")
  }

  if (tol <= 0) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " tol must be in greater than 0")
  }

  # Check that tol2 is numeric and greater than 0
  if (!is.numeric(tol2)) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " tol2 must be numeric")
  }

  if (tol2 <= 0) {
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " tol2 must be in greater than 0")
  }


  # If Ds-Opt check par int
  if (criterion == "Ds-Optimality") {
    # Check that par_int is a numeric vector
    if (!is.numeric(par_int) || !is.atomic(par_int) || !is.vector(par_int)) {
      error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " par_int must be numeric vector")
    }
    # Check that all are indexes
    if (!is.null(par_int)) {
      if (any(par_int %% 1 != 0)) {
        error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " par_int must contain only integers")
      }
      if (any(par_int > length(parameters))) {
        error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " par_int index out of bonds")
      }
      if (length(par_int) >= length(parameters)) {
        error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " par_int has length equal or greater than the number of parameters. Consider using D-Optimality")
      }
    }
  }

  # If I-Optimality check reg_int
  if (criterion == "I-Optimality") {
    if (is.null(reg_int)) {
      error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " reg_int must be provided for I-Optimality")
    }
    if (!is.null(reg_int)) {
      if (is.numeric(reg_int)) {
        # 1D: must be c(min, max)
        if (length(reg_int) != 2L)
          error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross),
                              " reg_int must be a numeric vector of length 2 for single-factor models")
      } else if (is.list(reg_int)) {
        # Multi-factor: named list of c(min, max) pairs
        if (!all(sapply(reg_int, function(r) is.numeric(r) && length(r) == 2L)))
          error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross),
                              " Each element of reg_int must be a numeric c(min, max) pair")
      } else {
        error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross),
                            " reg_int must be a numeric vector c(min, max) for 1D or a named list for multi-factor models")
      }
    }
  }
  # If L-Optimality check matB
  if (criterion == "L-Optimality") {
    # Check that matB is provided
    if (is.null(matB)) {
      error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " matB must be provided for L-Optimality")
    }
    # Check that matB is a matrix with the correct length
    if (!is.null(matB) && (!is.matrix(matB) || nrow(matB) != length(parameters) || ncol(matB) != length(parameters))) {
      error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " matB must be a square matrix with as many rows as parameters")
    }
  }
  if(!is.function(weight_fun)){
    error_msg <- paste0(error_msg, "\n", crayon::red(cli::symbol$cross), " weight_fun must be a function")
  }

  # If there are errors, send them to the user
  if (!identical(error_msg, "")) {
    stop(error_msg, call. = FALSE)
  }
}

Presentation
------------

This Shiny application presents different ways of applying the
methodology to add points to a design with controlled efficiency for a
number of different models. The aim is to provide an interactive tool
that demonstrates the methodological procedures experimenters can follow
in order to modify or enhance their designs.

The working of the app is as follows

-   The user should first decide the model to be used and the associated
    parameters- at least the space of the design- in Set-Up & Optimal
    Design. The user should then calculate the optimal design for their
    choice.
-   Custom Design is one of the subsequent choices.
    -   In Design Proposal, the user chooses the weight to give to new
        points and a consequent efficiency, and the app gives the design
        resulting from adding all the points of the limits of the
        feasible region for this user choice. The feasible region can be
        visualized beforehand.
    -   In Restricted Design, the choice of weight and efficiency must
        also be made, which gives the feasible region. Then the user can
        add points within the region to the design up to the desired
        weight (if over or under that number, the weights will be scaled
        down/up) and calculate the efficiency of their design.
-   Your Design is another possibility if you want more freedom for the
    design to be evaluated:
    -   First you should Build your design, and you can then evaluate
        its the efficiency.
    -   Then, in Enhance my design, an analogous procedure to that in
        Restricted Design can be followed, where, after the choice of
        weights and efficiency, the user adds points in the feasible
        region. The difference is that rather than start with the
        optimal design, the initial design is the user input in Build
        your design.

**NOTE**: the inputs for the user to choose are on the left sidebar,
while the main body of the app is used only to reflect the outputs.

---
title: "Presentación"
author: "Carlos de la Calle"
date: "21/03/2020"
output: html_document
fontsize: 12pt
---


## Presentation

This Shiny Application is a tool for calculating optimal designs for Antoine's Equation given different initial estimations of the unknown parameters, $\theta^{(0)}$, and design space, $\mathcal{X}$.

This application uses the *Cocktail Algorithm* to calculate optimal designs for Antoine's Equation. The details of the algorithm can be found in the original work accompanying this application.

The functioning of the different tabs of the app are as follows

* <u>Presentation</u>: description of how the app works.
* <u>Optimal design</u>: calculates $D-$, $D_s-$, $A-$ and $I-$ optimal designs for Antoine's Equation. The user determines the probability distribution of the reponse (homoscedastic or heteroscedastic), the criteria and the associated parameters by either manually selecting them or choosing a substance from the sample list included. Then, after making extra choices if necessary, the user calculates the optimal design by clicking *Calculate Optimal Design*. <br />
When the optimal design is algorithmically computed, there is the chance to show the ``convergence of the algorithm''. The graphic represents the value of the criterion function for each step of the algorithm, either adding a new point or each iteration of the multiplicative weight optimization strategy.  
* <u>Custom design</u>: after a design has been calculated in *Optimal Design*, this menu allows the user to create their own design by adding points and their corresponding weights, and to compare the efficiency of their design with the optimum design calculated before. Points need to be in the design space of the optimal design, and their weights are standardized to sum to 1.
* <u>Contact</u>: information for contacting the authors of the app.


**NOTE**: The inputs for the user to choose are on the left sidebar, while the main body of the app is used only to reflect the outputs.

**NOTE**: For some set of parameters the Information Matrix might be poorly conditioned, which causes the inverse calculating function to fail, and hence an error is prompted to the user.

Contact
-------

This project has been developed by Carlos de la Calle-Arroyo, to
illustrate the results of his joint work with Mariano Amo-Salas, Jesús
López-Fidalgo and Licesio Rodríguez-Aragón. If you find any error, have
any doubts about the content, result or methodology, or just want to get
more detail or give feedback you can write the developer at:
<a href="mailto:carlos.callearroyo@uclm.es" class="email">carlos.callearroyo@uclm.es</a>.

Presentation
------------

This shinny application presents a tool to calculate optimal designs for
Antoine’s Equation given different initial estimations of the unknown
parameters, *θ*<sup>(0)</sup>, and design space, 𝒳.

This application uses the *Cocktail Algorithm* to calculate optimal
designs for Antoine’s Equation. The details of the algorithm can be
found in the original work accompaning this application.

The functioning of the different tabs of the app are as follows

-   <u>Presentation</u>: description on how the app works.
-   Optimal design: calculates *D*−, *D*<sub>*s*</sub>, *A*− and
    *I*−optimal designs for Antoine’s Equation. The user decides the
    criterion and the associated parameters by either manually selecting
    them or choosing a substance from the sample list included. Then,
    after doing extra choices if necessary, the user calculates the
    optimal design by clicking *Calculate Optimal Design*.
-   Custom design: The Custom Design menu lets the user create his own
    design, and compare the efficiency of their design with the optimum
    design calculated before.
-   Efficiency of industry design: lets you calculate the efficiency of
    some commonly used designs for different processes:
    -   Equidistant: Designs supported in both extremes of the space of
        the design, with *n* points that are equidistant between them
        and uniform weights.
    -   Arithmetic: Designs supported in the one of the extremes (top)
        of the space of the design, with *n* points that follow an
        arithmetic progression and have uniform weights, and the
        leftmost point is choose in a way that optimizes the efficiency
        of the resulting design.
    -   Geometric: Designs supported in the one of the extremes (top) of
        the space of the design, with *n* points that follow a geometric
        progression and have uniform weights, and the leftmost point is
        choose in a way that optimizes the efficiency of the resulting
        design.
-   Contact: information for contacting the authors of the app.

**NOTE**: the inputs for the user to choose are on the left sidebar,
while the main body of the app is used only to reflect the outputs.

**NOTE**: for some set of parameters the Information Matrix might be bad
conditioned, which causes the inverse calculating function to fail, and
hence an error is promped to the user.

---
title: "Presentación"
author: "Carlos de la Calle"
date: "5/10/2020"
output: md_document
---


## Contact

This project has been developed by Carlos de la Calle-Arroyo, to illustrate the results of his joint work with Jesús López-Fidalgo and Licesio Rodríguez-Aragón. If you find any error, have any doubts about the content, result or methodology, or just want to get more detail or give feedback you can write the developer at: carlos.callearroyo@uclm.es. 

---
title: "Presentación"
author: "Carlos de la Calle"
date: "5/10/2020"
output: md_document
---


## Help

This application uses the *Cocktail Algorithm* to calculate optimal designs for Antoine's Equation. The details of the algorithm can be found in the original work accompaning this application.

For some set of parameters, usually with no physical meaning, the Information Matrix might be bad conditioned, which causes the algorithm to fail. Similar phenomenons might happen when trying to optimise for *Arithmetic* and *Geometric* industry designs for certain criteria.

Help
----

The processes and examples shown here are based in the theoretical
results previous to this work. Given a choice for the weights of points
to add, *α*, and desired efficiency, *e**f**f*<sub>*D*</sub> (bounded by
the first and the initial design, *ζ*), one can calculate the points
that satisfy:

$$
eff\_{D} \\geq (1-\\alpha) \\left\\{ 1+\\frac{\\alpha d(x\_{1},\\zeta)}{1-\\alpha} \\right\\}^{1/m}.
$$

In out work, it has been proved that choosing any number of points,
distributing the weight *α* among them as desired, the efficiency is
greater or equal to the choice of *e**f**f*<sub>*D*</sub>.

The points that satisfy that condition for the choices of the user on
each section of the application are what we call the “feasible region”,
from which the users can choose their points with the choice of
efficiency as a minimum for the efficiency of the resulting design.

The optimal designs are calculated through their analytical expressions,
as are the sensitivity functions.

---
title: "Presentación"
author: "Carlos de la Calle"
date: "5/10/2020"
output: md_document
---


## Help

The processes and examples shown here are based in the theoretical results previous to this work. Given a choice for the weights of points to add, $\alpha$, and desired efficiency, $eff_{D}$ (bounded by the first and the initial design, $\zeta$), one can calculate the points that satisfy:

\[
eff_{D} \geq (1-\alpha) \left\{ 1+\frac{\alpha d(x_{1},\zeta)}{1-\alpha} \right\}^{1/m}.
\]

In out work, it has been proved that choosing any number of points, distributing the weight $\alpha$ among them as desired, the efficiency is greater or equal to the choice of $eff_D$. 

The points that satisfy that condition for the choices of the user on each section of the application are what we call the "feasible region", from which the users can choose their points with the choice of efficiency as a minimum for the efficiency of the resulting design.

The optimal designs are calculated through their analytical expressions, as are the sensitivity functions. 

Contact
-------

This project has been developed by Carlos de la Calle-Arroyo, to
illustrate the results of his joint work with Jesús López-Fidalgo and
Licesio Rodríguez-Aragón. If you find any error, have any doubts about
the content, result or methodology, or just want to get more detail or
give feedback you can write the developer at:
<a href="mailto:carlos.callearroyo@uclm.es" class="email">carlos.callearroyo@uclm.es</a>.

---
title: "Introduction to optedr: optimal designs for non-linear models"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to optedr: optimal designs for non-linear models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  fig.width  = 6,
  fig.height = 4,
  out.width  = "90%"
)
```

```{r setup}
library(optedr)
```

## The optimal design problem

In non-linear regression the information that an experiment provides about the model parameters depends on *where* observations are taken, not just how many. The Fisher information matrix at a design point $x$ is

$$M(\xi) = \int f(x)\, f(x)^{\top} \, \xi(dx),$$

where $f(x) = \nabla_\theta \mu(x, \theta)$ is the gradient of the mean response. Different optimality criteria summarise $M(\xi)$ into a single number to minimise:

| Criterion | Objective | Key argument |
|-----------|-----------|-------------|
| D-Optimality | $-\log\det M$ | — |
| Ds-Optimality | focuses on a subset of parameters | `par_int` |
| A-Optimality | $\operatorname{tr} M^{-1}$ | — |
| I-Optimality | integrated prediction variance over a region | `reg_int` |
| L-Optimality | $\operatorname{tr}(B \, M^{-1})$ for user-supplied $B$ | `matB` |

All are computed with `opt_des()`, which implements the cocktail algorithm (Yu, 2011).

## A first example: D-Optimality in one factor

The Antoine equation models vapour pressure as a function of temperature. With two parameters and one design variable the D-optimal design has exactly two support points.

```{r d-optimal}
result_D <- opt_des(
  criterion    = "D-Optimality",
  model        = y ~ a * exp(-b / x),
  parameters   = c("a", "b"),
  par_values   = c(1, 1500),
  design_space = c(212, 422)
)
result_D
```

`plot()` shows the sensitivity function $d(x, \xi)$. By the Equivalence Theorem the design is optimal if and only if $d(x, \xi) \leq k$ everywhere, with equality at every support point.

```{r d-plot, fig.cap = "Sensitivity function for the D-optimal design."}
plot(result_D)
```

`summary()` gives a fuller breakdown including the criterion value and convergence diagnostics.

```{r d-summary}
summary(result_D)
```

The `$atwood` component reports the Atwood criterion (percentage deviation from the theoretical optimum bound): values close to 100 % confirm convergence.

## Other optimality criteria

### Ds-Optimality

`par_int` selects the indices of the parameters of interest. Here we focus only on `th0`:

```{r ds-optimal}
result_Ds <- opt_des(
  criterion    = "Ds-Optimality",
  model        = y ~ th0 * exp(x / th1),
  parameters   = c("th0", "th1"),
  par_values   = c(10.4963, -3.2940),
  design_space = c(0.94, 30),
  par_int      = c(1)
)
result_Ds
```

### A-Optimality

Minimises the average variance of the parameter estimators:

```{r a-optimal}
result_A <- opt_des(
  criterion    = "A-Optimality",
  model        = y ~ a * exp(-b / x),
  parameters   = c("a", "b"),
  par_values   = c(1, 1500),
  design_space = c(212, 422)
)
result_A
```

### I-Optimality

Minimises the integrated prediction variance over a region of interest `reg_int`. Useful when prediction in a specific sub-range matters more than global estimation:

```{r i-optimal}
result_I <- opt_des(
  criterion    = "I-Optimality",
  model        = y ~ a * exp(-b / x),
  parameters   = c("a", "b"),
  par_values   = c(1, 1500),
  design_space = c(212, 422),
  reg_int      = c(380, 422)
)
result_I
```

### L-Optimality

`matB` is a symmetric positive-semidefinite $k \times k$ matrix that selects which variances to minimise. Setting `matB = diag(c(1, 0))` focuses entirely on the variance of `a`:

```{r l-optimal}
result_L <- opt_des(
  criterion    = "L-Optimality",
  model        = y ~ a * exp(-b / x),
  parameters   = c("a", "b"),
  par_values   = c(1, 1500),
  design_space = c(212, 422),
  matB         = diag(c(1, 0))
)
result_L
```

Comparing D- and L-optimal designs shows how the support points shift when precision is concentrated on one parameter:

```{r l-compare}
cat("D-optimal support:\n"); print(result_D$optdes)
cat("L-optimal support:\n"); print(result_L$optdes)
```

## Multi-dimensional design spaces

For models with two or more design variables pass `design_space` as a named list. Variable names in the list must match those used in `model`.

### Two-factor model: bisubstrate Michaelis-Menten

Enzyme kinetics with two substrates: $\mu = V_{\max} x_1 x_2 / [(K_1 + x_1)(K_2 + x_2)]$.

```{r 2d-optimal}
result_2D <- opt_des(
  criterion    = "D-Optimality",
  model        = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters   = c("Vmax", "K1", "K2"),
  par_values   = c(1, 1, 1),
  design_space = list(x1 = c(0.1, 10), x2 = c(0.1, 10))
)
result_2D
```

`plot()` for a two-factor model returns a viridis heatmap of the sensitivity function, with the support points overlaid in red:

```{r 2d-plot, fig.cap = "Sensitivity heatmap for the 2D D-optimal design."}
plot(result_2D)
```

L-Optimality extends naturally to multi-dimensional spaces. Here we minimise only the variance of $K_1$:

```{r 2d-l}
result_2D_L <- opt_des(
  criterion    = "L-Optimality",
  model        = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters   = c("Vmax", "K1", "K2"),
  par_values   = c(1, 1, 1),
  design_space = list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
  matB         = diag(c(0, 1, 0))
)
result_2D_L
```

### Three factors and beyond

For $d \geq 3$ factors `plot()` switches to a scatter-matrix with $\binom{d}{2}$ panels, one per pair of variables. Point size is proportional to weight.

```{r 3d-optimal}
result_3D <- opt_des(
  criterion    = "D-Optimality",
  model        = y ~ Vmax * x1 * x2 * x3 / ((K1+x1) * (K2+x2) * (K3+x3)),
  parameters   = c("Vmax", "K1", "K2", "K3"),
  par_values   = c(1, 1, 1, 1),
  design_space = list(x1 = c(0.1, 10), x2 = c(0.1, 10), x3 = c(0.1, 10))
)
result_3D
plot(result_3D)
```

## Compound criterion

Sometimes no single criterion captures the experimental goals. `criterion = "Compound"` combines any set of criteria as a weighted sum of their sensitivity functions. Weights are normalised automatically.

```{r compound}
result_DI <- opt_des(
  criterion    = "Compound",
  model        = y ~ 10^(a - b / (c + x)),
  parameters   = c("a", "b", "c"),
  par_values   = c(8.07131, 1730.63, 233.426),
  design_space = c(1, 100),
  compound     = list(
    list(criterion = "D-Optimality", weight = 0.7),
    list(criterion = "I-Optimality", weight = 0.3, reg_int = c(60, 100))
  )
)
result_DI
```

Comparing with the pure D-optimal design shows how the composite shifts one support point toward the prediction region:

```{r compound-compare}
result_D_ant <- opt_des(
  "D-Optimality",
  y ~ 10^(a - b / (c + x)), c("a", "b", "c"),
  c(8.07131, 1730.63, 233.426), c(1, 100)
)
cat("D-optimal:\n");         print(result_D_ant$optdes)
cat("Compound D+I (70/30):\n"); print(result_DI$optdes)
```

## Design efficiency

`design_efficiency()` measures the fraction of the optimal information that a given design achieves: an efficiency of 0.80 means the design needs $1/0.8 = 1.25\times$ more observations to match the optimal.

```{r efficiency}
design_ad_hoc <- data.frame(
  Point  = c(220, 300, 400),
  Weight = c(1/3, 1/3, 1/3)
)
eff <- design_efficiency(design_ad_hoc, result_D)
cat("Efficiency of ad-hoc design:", round(eff * 100, 2), "%\n")
```

For multi-dimensional designs pass a data frame with one column per factor plus a `Weight` column:

```{r efficiency-2d}
corners_2d <- data.frame(
  x1     = c(0.1, 10,  0.1, 10),
  x2     = c(0.1, 0.1, 10,  10),
  Weight = rep(0.25, 4)
)
eff_2d <- design_efficiency(corners_2d, result_2D)
cat("Efficiency of corner design vs 2D D-optimal:", round(eff_2d * 100, 2), "%\n")
```

## Rounding to exact designs

Optimal designs assign continuous weights. Two functions convert them to integer replication counts for a given total $n$.

### `efficient_round()`

Uses the multiplier $n - l/2$ (where $l$ is the number of support points) and adjusts to guarantee the total is exactly $n$:

```{r efficient-round}
exact_design <- efficient_round(result_D$optdes, n = 20)
print(exact_design)
cat("Total observations:", sum(exact_design$Weight), "\n")
```

### `combinatorial_round()`

Exhaustive floor/ceiling search: optimal for small $k$ (number of support points) since it checks all $2^k$ combinations. Pass the `optdes` object so the function can use the criterion value for comparison:

```{r combo-round}
combo_design <- combinatorial_round(result_D, n = 10)
print(combo_design)
```

Once rounded, compare the exact design's efficiency against the approximate optimum:

```{r round-eff}
approx <- exact_design
approx$Weight <- approx$Weight / sum(approx$Weight)
cat("Efficiency of rounded design:", round(design_efficiency(approx, result_D) * 100, 2), "%\n")
```

---
title: "KL-Optimality: designs for model discrimination"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{KL-Optimality: designs for model discrimination}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  fig.width  = 6,
  fig.height = 4,
  out.width  = "90%"
)
```

```{r setup}
library(optedr)
```

## Background

Standard optimality criteria (D, A, I, …) assume the model is correctly specified and aim to estimate its parameters as precisely as possible. KL-Optimality addresses a different question: *given two competing models, where should observations be taken to tell them apart most easily?*

The criterion is based on the Kullback-Leibler divergence between the reference model $f_1$ and the best-fitting version of the rival $f_2$:

$$\Phi_{\text{KL}}(\xi) = \int \min_{\beta_2} \text{KL}(f_1(y \mid x) \,\|\, f_2(y \mid x, \beta_2))\, \xi(dx).$$

The design $\xi^*$ that maximises $\Phi_{\text{KL}}$ places observations where the two models are most distinguishable on average.

**Reference:** López-Fidalgo, Tommasi & Trandafir (2007). *An optimal experimental design criterion for discriminating between non-normal models.* J. R. Stat. Soc. B, 69(2), 231–242.

## API overview

```r
opt_des(
  "KL-Optimality",
  model        = <reference model formula>,
  parameters   = <reference parameter names>,
  par_values   = <nominal values>,
  design_space = <scalar interval or named list>,
  # rival specification:
  rival_model  = <rival formula>,       # default: same as model
  rival_params = <rival param names>,   # default: same as parameters
  rival_pars   = <starting values for internal optimisation>,
  rival_lower  = <lower bounds for rival params>,
  rival_upper  = <upper bounds for rival params>,
  # GLM family:
  family       = "Normal",   # "Poisson", "Binomial", "Gamma"
  phi          = 1,          # dispersion parameter
  # alternative: supply a custom KL function
  kl_fun       = NULL
)
```

The internal optimisation over rival parameters is done automatically. The optimal rival values are stored as a hidden attribute and shown by `summary()`.

## Example 1: Michaelis-Menten vs linear rival (Normal, 1D)

**Context.** Enzyme kinetics with one substrate. The reference model is Michaelis-Menten $\mu(x) = V_{\max} x / (K_m + x)$ with $V_{\max} = 2$, $K_m = 1$. The rival is a linear rate $\mu(x) = a x$, valid only when $x \ll K_m$. At high concentrations MM saturates while the linear model keeps growing — that is where the models diverge most.

```{r kl-mm}
result_kl_mm <- opt_des(
  "KL-Optimality",
  model        = y ~ Vmax * x / (Km + x),
  parameters   = c("Vmax", "Km"),
  par_values   = c(2, 1),
  design_space = c(0.1, 5),
  rival_model  = y ~ a * x,
  rival_params = c("a"),
  rival_pars   = c(1),
  rival_lower  = c(0.2),
  rival_upper  = c(2.5),
  family       = "Normal",
  phi          = 1
)
result_kl_mm
```

`summary()` shows the GLM family and the optimal rival parameters found by the internal optimisation:

```{r kl-mm-summary}
summary(result_kl_mm)
```

The sensitivity plot shows $d_{\text{KL}}(x, \xi)$ with the Equivalence Theorem bound (dashed). Support points concentrate in the saturation region where MM and the linear rival differ most:

```{r kl-mm-plot, fig.cap = "KL sensitivity function: MM vs linear rival."}
plot(result_kl_mm)
```

Efficiency of a five-point uniform design relative to the KL-optimal:

```{r kl-mm-eff}
design_unif <- data.frame(
  Point  = c(0.1, 1.3, 2.5, 3.8, 5.0),
  Weight = rep(1/5, 5)
)
eff_kl <- design_efficiency(design_unif, result_kl_mm)
cat("Efficiency of uniform design:", round(eff_kl * 100, 2), "%\n")
```

## Example 2: exponential decay, Poisson family

**Context.** A drug clearance study where the event count follows a Poisson process with rate $\mu(x) = \exp(a - bx)$. The reference model has nominal decay rate $b = 0.5$. The rival represents a faster-elimination regime ($b \in [0.8, 1.5]$), which could arise from drug interactions. The KL divergence for Poisson is $\text{KL} = \mu_2 - \mu_1 - \mu_1 \log(\mu_2/\mu_1)$; since the rival decays faster, the gap is largest at long follow-up times.

```{r kl-poisson}
result_kl_pois <- opt_des(
  "KL-Optimality",
  model        = y ~ exp(a - b * x),
  parameters   = c("a", "b"),
  par_values   = c(2, 0.5),
  design_space = c(0, 4),
  rival_pars   = c(2, 1.0),
  rival_lower  = c(1.5, 0.8),
  rival_upper  = c(2.5, 1.5),
  family       = "Poisson",
  phi          = 1
)
result_kl_pois
summary(result_kl_pois)
```

```{r kl-poisson-plot, fig.cap = "KL sensitivity for Poisson decay model."}
plot(result_kl_pois)
```

The optimal rival parameters found internally confirm the closest rival within the constraint:

```{r kl-poisson-rival}
hv <- attr(result_kl_pois, "hidden_value")
cat("Optimal rival: a =", round(hv$beta2_star[1], 3),
    " b =", round(hv$beta2_star[2], 3), "\n")
```

## Using `make_kl_fun()` for custom KL functions

For some discrimination problems the KL divergence does not reduce to the standard cumulant-generating-function form that `opt_des()` uses internally. `make_kl_fun()` automates the construction of the KL function from two model-family pairs.

### Supported pairs

| Combination | Formula |
|-------------|---------|
| Same family, same $\phi$ | Standard cumulant form |
| Normal vs Normal, $\phi_1 \neq \phi_2$ | $\frac{1}{2}\left[\log\frac{\phi_2}{\phi_1} + \frac{\phi_1}{\phi_2} + \frac{(\mu_1-\mu_2)^2}{\phi_2} - 1\right]$ |
| Gamma vs Gamma, different shape | Closed form with digamma/lgamma |

For other pairs, supply `kl_fun` directly as a function `(x, beta2) -> scalar`.

### Example 3: Normal with different variances (1D)

**Context.** Same exponential decay model, but now the rival has variance $\phi_2 = 4$ (four times larger). The KL function includes a constant term $\frac{1}{2}[\log(\phi_2/\phi_1) + \phi_1/\phi_2 - 1]$ even when the means coincide, so the design must also account for the variance difference.

```{r kl-variances}
kl_fn_var <- make_kl_fun(
  "Normal",
  model1      = y ~ a * exp(-b * x),
  params1     = c("a", "b"),
  par_values1 = c(1, 0.5),
  phi1        = 1,
  family2     = "Normal",
  model2      = y ~ c * exp(-d * x),
  params2     = c("c", "d"),
  phi2        = 4
)

result_kl_var <- opt_des(
  "KL-Optimality",
  model        = y ~ a * exp(-b * x),
  parameters   = c("a", "b"),
  par_values   = c(1, 0.5),
  design_space = c(0, 4),
  kl_fun       = kl_fn_var,
  rival_pars   = c(1, 1.0),
  rival_lower  = c(0.5, 0.8),
  rival_upper  = c(2.0, 1.5)
)
result_kl_var
summary(result_kl_var)
```

```{r kl-var-plot, fig.cap = "KL sensitivity: Normal phi=1 vs Normal phi=4."}
plot(result_kl_var)
```

### Example 4: two-factor MM vs linear rival (`make_kl_fun`, 2D)

`make_kl_fun()` works with models that use different subsets of the design variables. Here the reference uses $(x_1, x_2)$ (bisubstrate MM) while the rival depends only on $x_1$ (single-substrate linear approximation):

```{r kl-2d}
kl_fn_2d <- make_kl_fun(
  "Normal",
  model1      = y ~ Vmax * x1 * x2 / ((Km1 + x1) * (Km2 + x2)),
  params1     = c("Vmax", "Km1", "Km2"),
  par_values1 = c(1, 1, 1),
  model2      = y ~ alpha * x1,
  params2     = "alpha"
)

result_kl_2d <- opt_des(
  "KL-Optimality",
  model        = y ~ Vmax * x1 * x2 / ((Km1 + x1) * (Km2 + x2)),
  parameters   = c("Vmax", "Km1", "Km2"),
  par_values   = c(1, 1, 1),
  design_space = list(x1 = c(0.1, 5), x2 = c(0.1, 5)),
  kl_fun       = kl_fn_2d,
  rival_pars   = c(0.5),
  rival_lower  = c(0.05),
  rival_upper  = c(2.0)
)
result_kl_2d
```

The heatmap concentrates support where the difference between the bisubstrate hyperbola and the linear rival is greatest (saturation regime in both $x_1$ and $x_2$):

```{r kl-2d-plot, fig.cap = "KL sensitivity heatmap: 2D MM vs linear rival."}
plot(result_kl_2d)
```

## Example 5: discriminating error structures — the Hill model

**Reference:** De la Calle-Arroyo, Leorato, Rodríguez-Aragón & Tommasi (2025). *Augmented designs to choose between constant absolute and relative errors.* Chemometrics and Intelligent Laboratory Systems, doi:10.1016/j.chemolab.2025.105374.

**Context.** Dose-response experiments with the Hill (4-parameter Emax) model:

$$\eta(x, \beta) = (E_{\text{con}} - b) \cdot \frac{(x/\text{IC}_{50})^s}{1 + (x/\text{IC}_{50})^s} + b$$

with $E_{\text{con}} = 1.70$, $b = 0.137$, $\text{IC}_{50} = 111$, $s = -1.03$ and $x \in [0.01, 1500]$ nM. Two error structures are competing:

- **Relative error** (reference): $\varepsilon \sim N(0,\, \sigma_{\text{rel}}^2 \cdot \eta^2)$
- **Absolute error** (rival): $\varepsilon \sim N(0,\, \sigma_{\text{abs}}^2)$

The KL divergence minimised over the rival variance $\sigma_{\text{abs}}^2$ is

$$\text{KL}(x,\, \sigma_{\text{abs}}^2) = \frac{1}{2}\!\left(\frac{\eta^2(x)}{\sigma_{\text{abs}}^2} - 1 + \log\frac{\sigma_{\text{abs}}^2}{\eta^2(x)}\right).$$

We supply this directly as `kl_fun` since it is not covered by the standard Normal-vs-Normal formula (the reference variance is $x$-dependent):

```{r kl-hill}
kl_fun_hill <- function(x, beta2) {
  sigma_sq <- beta2[1]
  eta <- (1.70 - 0.137) * (x / 111)^(-1.03) / (1 + (x / 111)^(-1.03)) + 0.137
  0.5 * (eta^2 / sigma_sq - 1 + log(sigma_sq / eta^2))
}

result_kl_hill <- opt_des(
  "KL-Optimality",
  model        = y ~ (Econ - b) * (x / IC50)^s / (1 + (x / IC50)^s) + b,
  parameters   = c("Econ", "b", "IC50", "s"),
  par_values   = c(1.70, 0.137, 111, -1.03),
  design_space = c(0.01, 1500),
  kl_fun       = kl_fun_hill,
  rival_pars   = c(1.0),
  rival_lower  = c(1e-4),
  rival_upper  = c(1e6)
)
result_kl_hill
summary(result_kl_hill)
```

```{r kl-hill-plot, fig.cap = "KL sensitivity for the Hill model (error structure discrimination)."}
plot(result_kl_hill)
```

The KL-optimal design places all weight at the two extremes of the design space. This matches the result reported in the paper (support at $x = 0.01$ with weight 0.23 and $x = 1500$ with weight 0.77). The Atwood criterion at 100 % confirms the design is exactly optimal.

The optimal rival variance recovered internally:

```{r kl-hill-rival}
hv_hill <- attr(result_kl_hill, "hidden_value")
cat("Optimal rival sigma_abs^2 =", round(hv_hill$beta2_star, 4), "\n")
```

Note that with only two support points this design cannot estimate all four parameters of the Hill model. The paper proposes combining it with a D-optimal design via the augmentation workflow described in `vignette("optedr-augment")`.

---
title: "Augmenting designs with controlled efficiency loss"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Augmenting designs with controlled efficiency loss}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  fig.width  = 6,
  fig.height = 4,
  out.width  = "90%"
)
```

```{r setup}
library(optedr)
```

## Motivation

In practice an experiment is rarely designed from scratch. A researcher may already have data collected at certain conditions and want to add new observations to improve estimation — without discarding what has already been measured. The key question is: *where* can new points be placed so that the efficiency of the augmented design stays above an acceptable threshold?

**optedr** answers this question with two functions used in sequence:

1. `get_augment_region()` — computes the candidate region: the set of design points whose addition keeps the D-efficiency of the augmented design above a user-specified threshold `delta_val`.
2. `augment_design()` — adds a chosen point to the initial design and rescales the weights.

Both functions support the same optimality criteria as `opt_des()` and work for any number of factors.

## Key parameters

| Parameter | Role |
|-----------|------|
| `init_design` | Current design (data frame with `Point`/`Weight` in 1D, or factor columns + `Weight` in multi-D) |
| `alpha` | Fraction of total weight assigned to the new point after augmentation |
| `delta_val` | Minimum acceptable D-efficiency of the augmented design |
| `calc_optimal_design` | If `TRUE`, also computes the optimal design and uses it as the reference for efficiency |
| `new_points` | Data frame of points to add (non-interactive mode); omit for interactive mode |
| `par_int` | Indices of parameters of interest (Ds-Optimality only) |
| `n_lhs` | Number of Latin-Hypercube candidates for the region search (multi-D) |

## One-factor augmentation

### Step 1: compute the candidate region

We start with a uniform three-point design for Antoine's equation and look for points that keep the D-efficiency of the augmented design above 85 %.

```{r region-1d}
init_des <- data.frame(
  Point  = c(30, 60, 90),
  Weight = c(1/3, 1/3, 1/3)
)

region <- get_augment_region(
  criterion           = "D-Optimality",
  init_design         = init_des,
  alpha               = 0.25,
  model               = y ~ 10^(a - b / (c + x)),
  parameters          = c("a", "b", "c"),
  par_values          = c(8.07131, 1730.63, 233.426),
  design_space        = c(1, 100),
  calc_optimal_design = FALSE,
  delta_val           = 0.85
)
print(region)
```

`region$region` is a data frame of candidate intervals. Each row gives a lower and upper bound on the design space where the new point can be placed.

### Step 2: choose a point and augment

```{r augment-1d}
new_pt <- mean(region$region[1:2])

augmented <- augment_design(
  criterion           = "D-Optimality",
  init_design         = init_des,
  alpha               = 0.25,
  model               = y ~ 10^(a - b / (c + x)),
  parameters          = c("a", "b", "c"),
  par_values          = c(8.07131, 1730.63, 233.426),
  design_space        = c(1, 100),
  calc_optimal_design = FALSE,
  delta_val           = 0.85,
  new_points          = data.frame(Point = new_pt, Weight = 1)
)
print(augmented)
cat("Sum of weights:", sum(augmented$Weight), "\n")
```

### Comparing efficiency before and after

```{r efficiency-1d}
result_opt <- opt_des(
  "D-Optimality",
  y ~ 10^(a - b / (c + x)), c("a", "b", "c"),
  c(8.07131, 1730.63, 233.426), c(1, 100)
)

eff_before <- design_efficiency(init_des, result_opt)
eff_after  <- design_efficiency(augmented, result_opt)

cat("Efficiency before augmenting:", round(eff_before * 100, 2), "%\n")
cat("Efficiency after augmenting: ", round(eff_after  * 100, 2), "%\n")
cat("Gain:                        ", round((eff_after - eff_before) * 100, 2),
    "percentage points\n")
```

### Using the optimal design as reference (`calc_optimal_design = TRUE`)

When `calc_optimal_design = TRUE`, the function internally computes the optimal design and uses it to define the efficiency threshold. This is the recommended mode when no optimal design has been computed yet:

```{r augment-with-opt, eval=FALSE}
region_opt <- get_augment_region(
  criterion           = "D-Optimality",
  init_design         = init_des,
  alpha               = 0.25,
  model               = y ~ 10^(a - b / (c + x)),
  parameters          = c("a", "b", "c"),
  par_values          = c(8.07131, 1730.63, 233.426),
  design_space        = c(1, 100),
  calc_optimal_design = TRUE,
  delta_val           = 0.85
)
```

## Two-factor augmentation

In multi-dimensional spaces `get_augment_region()` samples candidate points with a Latin Hypercube (controlled by `n_lhs`) and returns a data frame of candidates together with their estimated efficiency gain. A heatmap of the efficiency function is displayed automatically.

### Initial design and candidate region

```{r region-2d}
init_2d <- data.frame(
  x1     = c(0.8, 10, 5),
  x2     = c(10, 0.8, 5),
  Weight = c(1/3, 1/3, 1/3)
)

result_2D <- opt_des(
  criterion    = "D-Optimality",
  model        = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters   = c("Vmax", "K1", "K2"),
  par_values   = c(1, 1, 1),
  design_space = list(x1 = c(0.1, 10), x2 = c(0.1, 10))
)

region_2d <- get_augment_region(
  criterion           = "D-Optimality",
  init_design         = init_2d,
  alpha               = 0.25,
  model               = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters          = c("Vmax", "K1", "K2"),
  par_values          = c(1, 1, 1),
  design_space        = list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
  calc_optimal_design = FALSE,
  delta_val           = 0.85
)
```

`region_2d$region` is a data frame of sampled candidates, each with an `efficiency` column. Pick the candidate that maximises efficiency:

```{r augment-2d}
best_2d <- region_2d$region[which.max(region_2d$region$efficiency), ]

eff_antes <- suppressMessages(design_efficiency(init_2d, result_2D))

aug_2d <- augment_design(
  criterion           = "D-Optimality",
  init_design         = init_2d,
  alpha               = 0.25,
  model               = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters          = c("Vmax", "K1", "K2"),
  par_values          = c(1, 1, 1),
  design_space        = list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
  calc_optimal_design = FALSE,
  delta_val           = 0.85,
  new_points          = data.frame(x1 = best_2d$x1, x2 = best_2d$x2, Weight = 1)
)

eff_despues <- suppressMessages(design_efficiency(aug_2d, result_2D))

cat("Efficiency before:", round(eff_antes  * 100, 2), "%\n")
cat("Efficiency after: ", round(eff_despues * 100, 2), "%\n")
print(aug_2d)
```

## Three-factor augmentation

For three or more factors the candidate region is displayed as a scatter-matrix coloured by candidate/non-candidate status, with the current design shown as triangles.

```{r region-3d}
init_3d <- data.frame(
  x1     = c(0.8, 10,  10,  0.8, 10),
  x2     = c(10,  0.8, 10,  10,  0.8),
  x3     = c(10,  10,  0.8, 0.8, 10),
  Weight = rep(0.2, 5)
)

region_3d <- get_augment_region(
  criterion           = "D-Optimality",
  init_design         = init_3d,
  alpha               = 0.45,
  model               = y ~ Vmax * x1 * x2 * x3 / ((K1+x1) * (K2+x2) * (K3+x3)),
  parameters          = c("Vmax", "K1", "K2", "K3"),
  par_values          = c(1, 1, 1, 1),
  design_space        = list(x1 = c(0.1, 10), x2 = c(0.1, 10), x3 = c(0.1, 10)),
  calc_optimal_design = FALSE,
  delta_val           = 0.93
)
cat("Number of candidate points:", nrow(region_3d$region), "\n")
plot(region_3d$plot)
```

## Augmenting with Ds-Optimality

When the goal is to augment while preserving estimation quality for a *subset* of parameters, use `criterion = "Ds-Optimality"` and pass `par_int`:

```{r augment-ds}
region_ds <- get_augment_region(
  criterion           = "Ds-Optimality",
  init_design         = init_2d,
  alpha               = 0.25,
  model               = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters          = c("Vmax", "K1", "K2"),
  par_values          = c(1, 1, 1),
  design_space        = list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
  calc_optimal_design = FALSE,
  par_int             = c(1),
  delta_val           = 0.85,
  n_lhs               = 5000
)

best_ds <- region_ds$region[which.max(region_ds$region$efficiency), ]

aug_ds <- augment_design(
  criterion           = "Ds-Optimality",
  init_design         = init_2d,
  alpha               = 0.25,
  model               = y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
  parameters          = c("Vmax", "K1", "K2"),
  par_values          = c(1, 1, 1),
  design_space        = list(x1 = c(0.1, 10), x2 = c(0.1, 10)),
  calc_optimal_design = FALSE,
  par_int             = c(1),
  delta_val           = 0.85,
  new_points          = data.frame(x1 = best_ds$x1, x2 = best_ds$x2, Weight = 1),
  n_lhs               = 5000
)
print(aug_ds)
```

## Interactive mode

Omitting `new_points` (and `delta_val`) from both functions triggers an interactive session where the package plots the candidate region and asks the user to type a point. This mode is documented in `?augment_design`.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{findminval}
\alias{findminval}
\title{Find Minimum Value}
\usage{
findminval(sens, design_space, grid.length)
}
\arguments{
\item{sens}{A numeric function to evaluate (scalar for 1D, named numeric vector for multi-factor).}

\item{design_space}{Numeric vector \code{c(min, max)} for single-factor models, or a
named list \code{list(x1 = c(min, max), ...)} for multi-factor models.}

\item{grid.length}{Number of grid points (1D) or LHS samples (multi-factor).}
}
\value{
The value of the minimum
}
\description{
Searches the minimum of a function over a grid on the design space.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inf_mat_sens.R
\name{sens}
\alias{sens}
\title{Master function to calculate the sensitivity function}
\usage{
sens(Criterion, grad, M, par_int = c(1), matB = NA)
}
\arguments{
\item{Criterion}{character variable with the chosen optimality criterion. Can be one of the following:
\itemize{
\item 'D-Optimality'
\item 'Ds-Optimality'
\item 'A-Optimality'
\item 'I-Optimality'
\item 'L-Optimality'
}}

\item{grad}{A function in a single variable that returns the partial derivatives vector of the model.}

\item{M}{Information Matrix for the sensitivity function.}

\item{par_int}{Numeric vector of the indexes of the parameters of interest for Ds-Optimality.}

\item{matB}{Matrix resulting from the integration of the one-point Information Matrix along the interest
region or lineal matrix for L-Optimality.}
}
\value{
The sensitivity function as a matrix of single variable.
}
\description{
Calculates the sensitivity function given the desired \code{Criterion}, an information matrix and other
necessary values depending on the chosen criterion.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wf_mult.R
\name{opt_des}
\alias{opt_des}
\title{Calculates the optimal design for a specified criterion}
\usage{
opt_des(
  criterion,
  model,
  parameters,
  par_values = c(1),
  design_space,
  init_design = NULL,
  join_thresh = -1,
  delete_thresh = 0.02,
  delta = 1/2,
  tol = 1e-05,
  tol2 = 1e-05,
  par_int = NULL,
  matB = NULL,
  reg_int = NULL,
  max_iter = 21L,
  distribution = NA,
  weight_fun = function(x) 1,
  compound = NULL,
  rival_model = NULL,
  rival_params = NULL,
  rival_pars = NULL,
  family = "Normal",
  phi = 1,
  rival_lower = NULL,
  rival_upper = NULL,
  kl_fun = NULL
)
}
\arguments{
\item{criterion}{character variable with the chosen optimality criterion.}

\item{model}{formula describing the model. For single-factor models use \code{x} as the
design variable; for multi-factor models use \code{x1}, \code{x2}, etc.}

\item{parameters}{character vector with the parameter names.}

\item{par_values}{numeric vector with the nominal parameter values.}

\item{design_space}{For single-factor models: numeric vector \code{c(min, max)}.
For multi-factor models: named list \code{list(x1 = c(min, max), x2 = c(min, max), ...)}.}

\item{init_design}{optional dataframe with the initial design. For single-factor models
use columns \code{Point} and \code{Weight}; for multi-factor models use one column per
design variable plus \code{Weight}.}

\item{join_thresh}{optional numeric threshold for merging nearby design points.}

\item{delete_thresh}{optional numeric minimum weight to keep a support point.}

\item{delta}{optional numeric in (0, 1), damping parameter of the algorithm.}

\item{tol}{optional numeric for convergence of the weight loop.}

\item{tol2}{optional numeric for the outer stop condition.}

\item{par_int}{optional numeric vector of parameter indices for Ds-optimality.}

\item{matB}{optional k x k matrix for L-optimality.}

\item{reg_int}{optional bounds for the I-optimality integration region.
Single-factor: \code{c(min, max)}.  Multi-factor: named list matching \code{design_space}.}

\item{max_iter}{optional integer maximum number of outer cocktail iterations.}

\item{distribution}{character variable specifying the response distribution.}

\item{weight_fun}{optional variance-structure weight function.}

\item{compound}{optional list of criterion specifications for \code{criterion = "Compound"}.
Each element must be a named list with at least \code{criterion} (character) and
\code{weight} (positive numeric). Additional fields per sub-criterion:
\code{par_int} (Ds-Optimality), \code{reg_int} (I-Optimality), \code{matB} (L-Optimality).
Weights are normalised to sum to 1. Example:
\code{list(list(criterion="D-Optimality", weight=0.7),
             list(criterion="I-Optimality", weight=0.3, reg_int=c(380,422)))}.}

\item{rival_model}{optional formula for the rival model used with
\code{criterion = "KL-Optimality"}. Defaults to \code{model} (same structure, rival
parameters explored by inner optimisation).}

\item{rival_params}{optional character vector of rival model parameter names. Defaults to
\code{parameters}.}

\item{rival_pars}{optional numeric vector of initial rival parameter values for the inner
optimisation. Defaults to \code{par_values}.}

\item{family}{character; GLM family for \code{criterion = "KL-Optimality"}. One of
\code{"Normal"}, \code{"Poisson"}, \code{"Binomial"}, \code{"Gamma"}. Default \code{"Normal"}.}

\item{phi}{positive numeric dispersion parameter for KL-Optimality. Default \code{1}.}

\item{rival_lower}{optional numeric vector of lower bounds for rival parameters in the
inner optimisation. Defaults to \code{-Inf} for each parameter.}

\item{rival_upper}{optional numeric vector of upper bounds for rival parameters in the
inner optimisation. Defaults to \code{Inf} for each parameter.}

\item{kl_fun}{optional user-supplied \code{function(x, beta2)} giving the point
KL divergence at design point \code{x} for rival parameters \code{beta2}, as built by
\code{\link{make_kl_fun}}. When provided, bypasses the \code{family}/\code{rival_model}
path entirely; \code{rival_pars} must still be supplied for the inner optimisation.}
}
\value{
a list of class \code{optdes} with components \code{optdes}, \code{convergence},
\code{sens}, \code{criterion}, \code{crit_value}, and \code{atwood}.
}
\description{
The opt_des function calculates the optimal design for an optimality criterion and a model input from the user.
Supports single-factor models (design variable named \code{x}) and multi-factor models
(design variables named \code{x1}, \code{x2}, \ldots, detected automatically from the formula).
}
\examples{
# Single-factor (backward compatible)
opt_des("D-Optimality", y ~ a * exp(-b / x), c("a", "b"), c(1, 1500), c(212, 422))

\donttest{
# Two-factor Michaelis-Menten bisubstrate model
opt_des("D-Optimality",
        y ~ Vmax * x1 * x2 / ((K1 + x1) * (K2 + x2)),
        c("Vmax", "K1", "K2"), c(1, 1, 1),
        list(x1 = c(0.1, 10), x2 = c(0.1, 10)))

# Compound D+I (70\% D, 30\% I) for Antoine equation
opt_des("Compound",
        y ~ 10^(a - b/(c + x)), c("a","b","c"),
        c(8.07131, 1730.63, 233.426), c(1, 100),
        compound = list(
          list(criterion = "D-Optimality", weight = 0.7),
          list(criterion = "I-Optimality", weight = 0.3, reg_int = c(60, 100))
        ))

# KL-Optimality: discriminate quadratic from linear mean model (Normal)
opt_des("KL-Optimality",
        model        = y ~ a * x^2,
        parameters   = c("a"),
        par_values   = c(1),
        design_space = c(1, 5),
        rival_model  = y ~ b * x,
        rival_params = c("b"),
        rival_pars   = c(3),
        family       = "Normal",
        phi          = 1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crit_eff.R
\name{dcrit}
\alias{dcrit}
\title{Criterion function for D-Optimality}
\usage{
dcrit(M, k)
}
\arguments{
\item{M}{information matrix for which the criterion value wants to be calculated.}

\item{k}{numeric variable with the number of parameters of the model. Taken from the number of rows of the matrix if omitted.}
}
\value{
numeric value of the D-optimality criterion for the information matrix.
}
\description{
Calculates the value of the D-Optimality criterion function, which follows the expression:
\deqn{\phi_D = \left(\frac{1}{|M|}\right)^{1/k}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{getCross2}
\alias{getCross2}
\title{Give effective limits to candidate points region}
\usage{
getCross2(cross, min, max, start, par)
}
\arguments{
\item{cross}{Vector of crosspoints in the sensitivity function given an efficiency and weight}

\item{min}{Minimum of the space of the design}

\item{max}{Maximum of the space of the design}

\item{start}{Boolean that gives the effective start of the candidate points region}

\item{par}{Boolean with the parity of the region}
}
\value{
Vector of effective limits of the candidate points region. Taken
in pairs from the beginning delimit the region.
}
\description{
Given the start of the candidates points region, the parity of the crosspoints
and the boundaries of the space of the design returns the effective limits of
the candidate points region. Those points, taken in pairs from the first to
the last delimit the region.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny_examples.R
\name{shiny_optimal}
\alias{shiny_optimal}
\title{Shiny Optimal}
\usage{
shiny_optimal()
}
\description{
Launches the demo Shiny application to calculate optimal designs for
Antoine's Equation. Requires an interactive R session; if called
non-interactively a message with the web URL is emitted instead.
}
\examples{
shiny_optimal()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{findmax}
\alias{findmax}
\title{Find Maximum}
\usage{
findmax(sens, design_space, grid.length, n_starts = 5L)
}
\arguments{
\item{sens}{Sensitivity function (scalar for 1D, named numeric vector for multi-factor).}

\item{design_space}{Numeric vector \code{c(min, max)} or named list.}

\item{grid.length}{Number of grid / LHS points for the initial sweep.}

\item{n_starts}{Number of local-optimisation restarts (multi-factor only).}
}
\value{
The design point (scalar or named numeric vector) at which \code{sens} is maximised.
}
\description{
Searches the location of the maximum of a sensitivity function over the design space.
For single-factor models the search uses a regular grid followed by direct selection.
For multi-factor models a Latin Hypercube Sample is evaluated and then refined with
L-BFGS-B local optimisation from the \code{n_starts} best candidate points.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{summary.optdes}
\alias{summary.optdes}
\title{Summary function for optdes}
\usage{
\method{summary}{optdes}(object, ...)
}
\arguments{
\item{object}{An object of class \code{optdes}.}

\item{...}{Possible extra arguments for the summary}
}
\description{
Summary function for optdes
}
\examples{
rri <- opt_des(criterion = "I-Optimality", model = y ~ a * exp(-b / x),
  parameters = c("a", "b"), par_values = c(1, 1500), design_space = c(212, 422),
  reg_int = c(380, 422))
summary(rri)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{update_design}
\alias{update_design}
\title{Update Design with new point}
\usage{
update_design(design, xmax, delta, new_weight)
}
\arguments{
\item{design}{Design to update. It's a dataframe with two columns:
\itemize{
\item \code{Point} contains the support points of the design.
\item \code{Weight} contains the corresponding weights of the \code{Point}s.
}}

\item{xmax}{The point to add as a numeric value.}

\item{delta}{Threshold which defines how close the new point has to be to any of the existing ones in order to
merge with them.}

\item{new_weight}{Number with the weight for the new point.}
}
\value{
The updated design.
}
\description{
Updates a design adding a new point to it. If the added point is closer than \code{delta} to an existing
point of the design, the two points are merged together as their arithmetic average. Then updates the weights
to be equal to all points of the design.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crit_eff.R
\name{design_efficiency}
\alias{design_efficiency}
\title{Efficiency between optimal design and a user given design}
\usage{
design_efficiency(design, opt_des_obj)
}
\arguments{
\item{design}{dataframe that represents the design. Must have two columns:
\itemize{
\item \code{Point} contains the support points of the design.
\item \code{Weight} contains the corresponding weights of the \code{Point}s.
}}

\item{opt_des_obj}{an object given by the function \code{opt_des}.}
}
\value{
The efficiency as a value between 0 and 1
}
\description{
Takes an optimal design provided from the function \code{opt_des} and a user given design and compares their
efficiency
}
\examples{
result <- opt_des("D-Optimality", y ~ a * exp(-b / x), c("a", "b"), c(1, 1500), c(212, 422))
design <- data.frame("Point" = c(220, 240, 400), "Weight" = c(1 / 3, 1 / 3, 1 / 3))
design_efficiency(design, result)
}
\seealso{
opt_des
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/augment.R
\name{get_dsaugment_region}
\alias{get_dsaugment_region}
\title{Get Ds-augment region}
\usage{
get_dsaugment_region(
  init_design,
  alpha,
  model,
  parameters,
  par_values,
  par_int,
  design_space,
  calc_optimal_design,
  weight_fun = function(x) 1,
  delta_val = NULL
)
}
\arguments{
\item{init_design}{dataframe with "Point" and "Weight" columns that represents the initial design to augment}

\item{alpha}{combined weight of the new points}

\item{model}{formula that represent the model with x as the independent variable}

\item{parameters}{character vector with the unknown parameters of the model to estimate}

\item{par_values}{numeric vector with the initial values of the unknown parameters}

\item{par_int}{optional numeric vector with the index of the \code{parameters} of interest for Ds-optimality.}

\item{design_space}{numeric vector with the limits of the space of the design}

\item{calc_optimal_design}{boolean parameter, if TRUE, the optimal design is calculated and efficiencies of the initial and augmented design are given}

\item{weight_fun}{optional one variable function that represents the square of the structure of variance, in case of heteroscedastic variance of the response}

\item{delta_val}{optional numeric value for the minimum relative efficiency. If \code{NULL} (default),
the user is prompted interactively. Providing this value enables non-interactive use.}
}
\value{
A vector of the points limiting the candidate points region
}
\description{
Given a model, calculates the candidate points region for Ds-Optimality. The user gives an initial
design for which he would like to add points and specifies the weight of the new points. Then he
is prompted to choose a minimum efficiency. After that, the candidate points region is calculated.
}
\examples{
\donttest{
init_des <- data.frame("Point" = c(30, 60, 90), "Weight" = c(1/3, 1/3, 1/3))
get_augment_region("D-Optimality", init_des, 0.25, y ~ 10^(a - b/(c + x)),
  c("a", "b", "c"), c(8.07131, 1730.63, 233.426), c(1, 100), FALSE,
  delta_val = 0.85)
}
}
\seealso{
Other augment region: 
\code{\link{get_laugment_region}()}
}
\concept{augment region}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/efficient_round.R
\name{efficient_round}
\alias{efficient_round}
\title{Efficient Round}
\usage{
efficient_round(design, n, tol = 1e-05, seed = NULL)
}
\arguments{
\item{design}{a dataframe with a \code{Weight} column and one or more design-variable columns
(\code{Point} for single-factor designs, \code{x1}, \code{x2}, \ldots for multi-factor designs).}

\item{n}{an integer that represents the desired number of observations of the exact design}

\item{tol}{optional parameter for the consideration of an integer in the rounding process}

\item{seed}{optional integer seed for reproducibility. When the rounded weights sum to less
than \code{n}, a random tie-breaking step is used; supplying \code{seed} makes that step
deterministic by calling \code{set.seed(seed)} immediately before it. \code{NULL} (default)
leaves the global RNG state unchanged.}
}
\value{
a data.frame with the same columns as \code{design}, with \code{Weight} replaced by
integer observation counts summing to \code{n}.
}
\description{
Takes an approximate design, and a number of points and converts the design to
an approximate design. It uses the multiplier (n - l/2) and evens the total
number of observations afterwards.
}
\examples{
design_test <- data.frame("Point" = seq(1, 5, length.out = 7),
         "Weight" = c(0.1, 0.0001, 0.2, 0.134, 0.073, 0.2111, 0.2818))

efficient_round(design_test, 20)

exact_design <- efficient_round(design_test, 21)
aprox_design <- exact_design
aprox_design$Weight <- aprox_design$Weight/sum(aprox_design$Weight)

# Reproducible tie-breaking
efficient_round(design_test, 20, seed = 42)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crit_eff.R
\name{icrit}
\alias{icrit}
\title{Criterion function for I-Optimality and L-Optimality}
\usage{
icrit(M, matB)
}
\arguments{
\item{M}{information matrix for which the criterion value wants to be calculated.}

\item{matB}{matrix of the integral of the information matrix over the interest region. Identity matrix for
A-Optimality.}
}
\value{
Numeric value of the I-optimality criterion for the information matrix.
}
\description{
Calculates the value of the I-Optimality criterion function, which follows the expression:
\deqn{\phi_I = Tr(M^{-1} \cdot B)}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detect_design.R
\name{detect_design_vars}
\alias{detect_design_vars}
\title{Detect design variables from a model formula}
\usage{
detect_design_vars(model, parameters)
}
\arguments{
\item{model}{A formula describing the model.}

\item{parameters}{Character vector of parameter names (excluded from design variables).}
}
\value{
Character vector of design variable names, sorted in numerical order for
multi-factor models (e.g. \code{c("x1", "x2", "x10")}).
}
\description{
Inspects the formula to identify which symbols are design variables (as opposed to
parameters or the response).  Enforces the naming convention: \code{"x"} for
single-factor models and \code{"x1"}, \code{"x2"}, \ldots for multi-factor models.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crit_eff.R
\name{eff}
\alias{eff}
\title{Efficiency between two Information Matrices}
\usage{
eff(criterion, mat1, mat2, k = 0, intPars = c(1), matB = NA)
}
\arguments{
\item{criterion}{character variable with the chosen optimality criterion. Can be one of the following:
\itemize{
\item 'D-Optimality'
\item 'Ds-Optimality'
\item 'A-Optimality'
\item 'I-Optimality'
\item 'L-Optimality'
}}

\item{mat1}{first information matrix, for the numerator.}

\item{mat2}{second information matrix, for the denominator.}

\item{k}{number of parameters of the model. Taken from the number of rows of the matrix if omitted.}

\item{intPars}{numeric vector with the index of the parameters of interest of the model. Only for "Ds-Optimality".}

\item{matB}{matrix of the integral of the information matrix over the interest region. Only for "I-Optimality".}
}
\value{
Efficiency of first design with respect to the second design, as a decimal number.
}
\description{
Efficiency between two Information Matrices
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{update_weightsDS}
\alias{update_weightsDS}
\title{Update weight Ds-Optimality}
\usage{
update_weightsDS(design, sens, s, delta)
}
\arguments{
\item{design}{Design to optimize the weights from. It's a dataframe with two columns:
\itemize{
\item \code{Point} contains the support points of the design.
\item \code{Weight} contains the corresponding weights of the \code{Point}s.
}}

\item{sens}{Sensibility function for the design and model.}

\item{s}{number of parameters of interest of the model}

\item{delta}{A parameter of the algorithm that can be tuned. Must be \eqn{0< delta < 1}.}
}
\value{
returns the new weights of the design after one iteration.
}
\description{
Implementation of the weight update formula for Ds-Optimality used to optimize the weights of a design,
which is to be applied iteratively until no sizable changes happen.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{update_design_total}
\alias{update_design_total}
\title{Merge close points of a design}
\usage{
update_design_total(design, delta)
}
\arguments{
\item{design}{The design to update. It's a dataframe with two columns:
\itemize{
\item \code{Point} contains the support points of the design.
\item \code{Weight} contains the corresponding weights of the \code{Point}s.
}}

\item{delta}{Threshold which defines how close two points have to be to any of the existing ones in order to
merge with them.}
}
\value{
The updated design.
}
\description{
Takes a design and merge together all points that are closer between them than a certain threshold \code{delta}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wf_mult.R
\name{IWFMult}
\alias{IWFMult}
\title{Cocktail Algorithm implementation for L-, I- and A-Optimality}
\usage{
IWFMult(
  init_design,
  grad,
  matB,
  design_space,
  grid.length,
  join_thresh,
  delete_thresh,
  delta_weights,
  tol,
  tol2,
  criterion,
  max_iter
)
}
\arguments{
\item{init_design}{optional dataframe with the initial design for the algorithm.}

\item{grad}{function of partial derivatives of the model.}

\item{matB}{optional matrix for L-optimality.}

\item{design_space}{named list with bounds for each design variable.}

\item{grid.length}{numeric value for the sensitivity search grid / LHS size.}

\item{join_thresh}{numeric value for the merge heuristic.}

\item{delete_thresh}{numeric value for the weight deletion threshold.}

\item{delta_weights}{numeric value in (0, 1), parameter of the algorithm.}

\item{tol}{numeric value for convergence of the weight loop.}

\item{tol2}{numeric value for the outer stop condition.}

\item{criterion}{character variable with the chosen optimality criterion.}

\item{max_iter}{maximum number of outer iterations.}
}
\value{
An object of class \code{optdes}.
}
\description{
Cocktail Algorithm implementation for L-, I- and A-Optimality
}
\seealso{
Other cocktail algorithms: 
\code{\link{CWFMult}()},
\code{\link{DWFMult}()},
\code{\link{DsWFMult}()},
\code{\link{KLWFMult}()},
\code{\link{WFMult}()}
}
\concept{cocktail algorithms}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kl_optim.R
\name{make_kl_fun}
\alias{make_kl_fun}
\title{Build a KL-divergence point function for use with opt_des()}
\usage{
make_kl_fun(
  family1,
  model1,
  params1,
  par_values1,
  family2 = family1,
  model2 = model1,
  params2 = params1,
  phi1 = 1,
  phi2 = phi1
)
}
\arguments{
\item{family1}{character; reference distribution (\code{"Normal"},
\code{"Poisson"}, \code{"Binomial"} or \code{"Gamma"}).}

\item{model1}{formula; reference model mean function.}

\item{params1}{character vector; parameter names in \code{model1}.}

\item{par_values1}{numeric vector; nominal values for the reference parameters.}

\item{family2}{character; rival distribution (default: same as \code{family1}).}

\item{model2}{formula; rival model mean function (default: same as \code{model1}).}

\item{params2}{character vector; rival parameter names (optimised internally).
Default: same as \code{params1}.}

\item{phi1}{positive numeric; dispersion of the reference
(\code{Normal}: variance; \code{Gamma}: \eqn{1/\text{shape}}).}

\item{phi2}{positive numeric; dispersion of the rival (default: same as \code{phi1}).}
}
\value{
A function \code{function(x, beta2)} giving the point KL divergence.
Works for both 1-D (\code{x} scalar) and multi-factor designs (\code{x}
named numeric vector).
}
\description{
Returns a \code{function(x, beta2)} computing the point KL divergence at
design point \code{x} given rival parameters \code{beta2}.  The result is
passed to \code{\link{opt_des}} via the \code{kl_fun} argument, allowing
discrimination between models with different family, dispersion, or mean
structure without having to derive the formula manually.

Supported family pairs:
\itemize{
\item Same family and same \code{phi}: any of
\code{"Normal"}, \code{"Poisson"}, \code{"Binomial"}, \code{"Gamma"}.
Uses the standard exponential-family cumulant formula.
\item \code{"Normal"} vs \code{"Normal"} with \code{phi1 != phi2}:
\eqn{KL = \frac{1}{2}\bigl[\log(\phi_2/\phi_1) + \phi_1/\phi_2 + (\mu_1-\mu_2)^2/\phi_2 - 1\bigr]}.
\item \code{"Gamma"} vs \code{"Gamma"} with different shape
(\eqn{k_i = 1/\phi_i}): closed form involving \code{digamma} and
\code{lgamma}.
}
For other cross-family pairs provide \code{kl_fun} directly.
}
\examples{
# Same family (Normal), different model structures
kl_fn <- make_kl_fun(
  "Normal",
  model1 = y ~ Vmax * x / (Km + x), params1 = c("Vmax", "Km"),
  par_values1 = c(2, 1),
  model2 = y ~ a * x, params2 = "a"
)
kl_fn(x = 1, beta2 = 0.5)

\donttest{
# Normal vs Normal with different variance (phi2 = 4)
kl_fn2 <- make_kl_fun(
  "Normal",
  model1 = y ~ a * exp(-b * x), params1 = c("a", "b"),
  par_values1 = c(1, 0.5), phi1 = 1,
  family2 = "Normal",
  model2 = y ~ c * exp(-d * x), params2 = c("c", "d"), phi2 = 4
)
opt_des("KL-Optimality",
        model = y ~ a * exp(-b * x), parameters = c("a", "b"),
        par_values = c(1, 0.5), design_space = c(0, 4),
        kl_fun = kl_fn2, rival_pars = c(1, 1),
        rival_lower = c(0.5, 0.8), rival_upper = c(2, 1.5))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{delete_points}
\alias{delete_points}
\title{Remove low weight points}
\usage{
delete_points(design, delta)
}
\arguments{
\item{design}{The design from which to remove points as a dataframe with two columns:
\itemize{
\item \code{Point} contains the support points of the design.
\item \code{Weight} contains the corresponding weights of the \code{Point}s.
}}

\item{delta}{The threshold from which the points with such a weight or lower will be removed.}
}
\value{
The design without the removed points.
}
\description{
Removes the points of a design with a weight lower than a threshold, \code{delta}, and distributes that weights
proportionally to the rest of the points.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/augment.R
\name{add_points}
\alias{add_points}
\title{Update design given crosspoints and alpha}
\usage{
add_points(points, alpha, design)
}
\arguments{
\item{points}{Points to be added to the design}

\item{alpha}{Combined weight of the new points to be added to the design}

\item{design}{A design as a dataframe with "Point" and "Weight" columns}
}
\value{
A design as a dataframe with "Point" and "Weight" columns that is the
addition of the design and the new points
}
\description{
Given a set of points, a weight and the design, the function adds these points
to the new design with uniform weight, and combined weight alpha
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{update_sequence}
\alias{update_sequence}
\title{Deletes duplicates points}
\usage{
update_sequence(points, tol)
}
\arguments{
\item{points}{Points to be updated}

\item{tol}{Tolerance for which two points are considered the same}
}
\value{
The points without duplicates
}
\description{
Within a vector of points, deletes points that are close enough (less than the tol parameter).
Returns the points without the "duplicates"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{getPar}
\alias{getPar}
\title{Parity of the crosspoints}
\usage{
getPar(cross)
}
\arguments{
\item{cross}{Vector of crosspoints in the sensitivity function given an efficiency and weight}
}
\value{
True if the number of crosspoints is even, false otherwise
}
\description{
Determines if the number of crosspoints is even or odd given the vector
of crosspoints
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/augment.R
\name{crosspoints}
\alias{crosspoints}
\title{Calculate crosspoints}
\usage{
crosspoints(val, sens, gridlength, tol, xmin, xmax)
}
\arguments{
\item{val}{Efficiency value to solve in the curve relationing the space of the design and efficiency of new design}

\item{sens}{Sensitivity function of the design for the model}

\item{gridlength}{Number of points in the initial grid used to bracket roots}

\item{tol}{Tolerance for root refinement passed to \code{uniroot}}

\item{xmin}{Minimum of the space of the design}

\item{xmax}{Maximum of the space of the design}
}
\value{
A numeric vector of crosspoints that define the candidate points region
}
\description{
Given the parameters for augmenting a design, this function calculates the
crosspoints in the efficiency function that delimit the candidate points
region
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inf_mat_sens.R
\name{isens}
\alias{isens}
\title{Sensitivity function for I-Optimality}
\usage{
isens(grad, M, matB)
}
\arguments{
\item{grad}{A function in a single variable that returns the partial derivatives vector of the model.}

\item{M}{Information Matrix for the sensitivity function.}

\item{matB}{Matrix resulting from the integration of the one-point Information Matrix along the interest
region or lineal matrix for L-Optimality.}
}
\value{
The sensitivity function as a matrix of single variable.
}
\description{
Calculates the sensitivity function from the gradient vector, the Information Matrix and the integral of the
one-point Identity Matrix over the interest region. If instead the identity matrix is used, it can be used
for A-Optimality.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kl_optim.R
\name{KLWFMult}
\alias{KLWFMult}
\title{Cocktail Algorithm for KL-Optimality}
\usage{
KLWFMult(
  init_design,
  kl_fun,
  beta2_init,
  lower,
  upper,
  design_space,
  grid.length,
  join_thresh,
  delete_thresh,
  delta_weights,
  tol,
  tol2,
  max_iter,
  kl_meta = list(type = "kl_fun")
)
}
\arguments{
\item{init_design}{optional dataframe with the initial design for the algorithm.}

\item{kl_fun}{function(x, beta2); returns the KL divergence at design point
\code{x} given rival parameters \code{beta2}.}

\item{beta2_init}{numeric vector of initial rival parameter values.}

\item{lower}{lower bounds for rival parameters in the inner optimisation.}

\item{upper}{upper bounds for rival parameters in the inner optimisation.}

\item{design_space}{named list with bounds for each design variable.}

\item{grid.length}{numeric value for the sensitivity search grid / LHS size.}

\item{join_thresh}{numeric value for the merge heuristic.}

\item{delete_thresh}{numeric value for the weight deletion threshold.}

\item{delta_weights}{numeric value in (0, 1), parameter of the algorithm.}

\item{tol}{numeric value for convergence of the weight loop.}

\item{tol2}{numeric value for the outer stop condition.}

\item{max_iter}{maximum number of outer iterations.}

\item{kl_meta}{list with summary metadata (\code{type}, and optionally
\code{family} and \code{phi} for the standard path).}
}
\value{
An object of class \code{optdes}.
}
\description{
Cocktail Algorithm for KL-Optimality
}
\seealso{
Other cocktail algorithms: 
\code{\link{CWFMult}()},
\code{\link{DWFMult}()},
\code{\link{DsWFMult}()},
\code{\link{IWFMult}()},
\code{\link{WFMult}()}
}
\concept{cocktail algorithms}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inf_mat_sens.R
\name{dsens}
\alias{dsens}
\title{Sensitivity function for D-Optimality}
\usage{
dsens(grad, M)
}
\arguments{
\item{grad}{A function in a single variable that returns the partial derivatives vector of the model.}

\item{M}{Information Matrix for the sensitivity function.}
}
\value{
The sensitivity function as a matrix of single variable.
}
\description{
Calculates the sensitivity function from the gradient vector and the Identity Matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inf_mat_sens.R
\name{gradient}
\alias{gradient}
\title{Gradient function}
\usage{
gradient(model, char_vars, values, weight_fun = function(x) 1)
}
\arguments{
\item{model}{formula describing the model, which must contain only \code{x}, the parameters defined in
\code{char_vars} and the numerical operators.}

\item{char_vars}{character vector of the parameters of the model.}

\item{values}{numeric vector with the nominal values of the parameters in \code{char_vars}.}

\item{weight_fun}{optional function variable that represents the square of the structure of variance, in case of heteroscedastic variance of the response}
}
\value{
A function depending on \code{x} that's the gradient of the \code{model} with respect to \code{char_vars}
}
\description{
Calculates the gradient function of a \code{model} with respect to the parameters, \code{char_vars}, evaluates
it at the provided \code{values} and returns the result as a function of the variable \code{x}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{plot_convergence}
\alias{plot_convergence}
\title{Plot Convergence of the algorithm}
\usage{
plot_convergence(convergence)
}
\arguments{
\item{convergence}{A dataframe with two columns:
\itemize{
\item \code{criteria} contains value of the criterion on each step.
\item \code{step} contains number of the step.
}}
}
\value{
A ggplot object with the \code{criteria} in the \code{y} axis and \code{step} in the \code{x} axis.
}
\description{
Plots the criterion value on each of the steps of the algorithm, both for optimizing weights and points,
against the total step number.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inf_mat_sens.R
\name{dssens}
\alias{dssens}
\title{Sensitivity function for Ds-Optimality}
\usage{
dssens(grad, M, par_int)
}
\arguments{
\item{grad}{A function in a single variable that returns the partial derivatives vector of the model.}

\item{M}{Information Matrix for the sensitivity function.}

\item{par_int}{Numeric vector of the indexes of the parameters of interest for Ds-Optimality.}
}
\value{
The sensitivity function as a matrix of single variable.
}
\description{
Calculates the sensitivity function from the gradient vector, the Identity Matrix and the parameters of
interest.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny_examples.R
\name{shiny_augment}
\alias{shiny_augment}
\title{Shiny D-augment}
\usage{
shiny_augment()
}
\description{
Launches the demo Shiny application to D-augment several pre-specified
models. Requires an interactive R session; if called non-interactively
a message with the web URL is emitted instead.
}
\examples{
shiny_augment()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crit_eff.R
\name{dscrit}
\alias{dscrit}
\title{Criterion function for Ds-Optimality}
\usage{
dscrit(M, par_int)
}
\arguments{
\item{M}{information matrix for which the criterion value wants to be calculated.}

\item{par_int}{numeric vector with the index of the parameters of interest of the model.}
}
\value{
Numeric value of the Ds-optimality criterion for the information matrix.
}
\description{
Calculates the value of the Ds-Optimality criterion function, which follows the expression:
\deqn{\phi_{Ds} = \left(\frac{|M_{22}|}{|M|}\right)^{1/s}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{update_weightsI}
\alias{update_weightsI}
\title{Update weight I-Optimality}
\usage{
update_weightsI(design, sens, crit, delta)
}
\arguments{
\item{design}{Design to optimize the weights from. It's a dataframe with two columns:
\itemize{
\item \code{Point} contains the support points of the design.
\item \code{Weight} contains the corresponding weights of the \code{Point}s.
}}

\item{sens}{Sensibility function for the design and model.}

\item{crit}{Value of the criterion function for I-Optimality.}

\item{delta}{A parameter of the algorithm that can be tuned. Must be \eqn{0< delta < 1}.}
}
\value{
returns the new weights of the design after one iteration.
}
\description{
Implementation of the weight update formula for I-Optimality used to optimize the weights of a design,
which is to be applied iteratively until no sizable changes happen. A-Optimality if instead of the
integral matrix the identity function is used.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{plot_sens}
\alias{plot_sens}
\title{Plot sensitivity function}
\usage{
plot_sens(min, max, sens_function, criterion_value)
}
\arguments{
\item{min}{Minimum of the space of the design, used in the limits of the representation.}

\item{max}{Maximum of the space of the design, used in the limits of the representation.}

\item{sens_function}{A single variable function, the sensitivity function.}

\item{criterion_value}{A numeric value representing the other side of the inequality of the Equivalence Theorem.}
}
\value{
A \code{ggplot} object that represents the sensitivity function
}
\description{
Plots the sensitivity function and the value of the Equivalence Theorem as an horizontal line, which helps
assess the optimality of the design of the given sensitivity function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input_checks.R
\name{check_inputs}
\alias{check_inputs}
\title{Check Inputs}
\usage{
check_inputs(
  criterion,
  model,
  parameters,
  par_values,
  design_space,
  init_design,
  join_thresh,
  delete_thresh,
  delta,
  tol,
  tol2,
  par_int,
  matB,
  reg_int,
  weight_fun
)
}
\arguments{
\item{criterion}{character variable with the chosen optimality criterion.}

\item{model}{formula describing the model. For single-factor models use \code{x} as the
design variable; for multi-factor models use \code{x1}, \code{x2}, etc.}

\item{parameters}{character vector with the parameter names.}

\item{par_values}{numeric vector with the nominal parameter values.}

\item{design_space}{For single-factor models: numeric vector \code{c(min, max)}.
For multi-factor models: named list \code{list(x1 = c(min, max), x2 = c(min, max), ...)}.}

\item{init_design}{optional dataframe with the initial design. For single-factor models
use columns \code{Point} and \code{Weight}; for multi-factor models use one column per
design variable plus \code{Weight}.}

\item{join_thresh}{optional numeric threshold for merging nearby design points.}

\item{delete_thresh}{optional numeric minimum weight to keep a support point.}

\item{delta}{optional numeric in (0, 1), damping parameter of the algorithm.}

\item{tol}{optional numeric for convergence of the weight loop.}

\item{tol2}{optional numeric for the outer stop condition.}

\item{par_int}{optional numeric vector of parameter indices for Ds-optimality.}

\item{matB}{optional k x k matrix for L-optimality.}

\item{reg_int}{optional bounds for the I-optimality integration region.
Single-factor: \code{c(min, max)}.  Multi-factor: named list matching \code{design_space}.}

\item{weight_fun}{optional variance-structure weight function.}
}
\description{
Function to check that the inputs given to the function \code{opt_des} are correct. If not, throws the
correspondent error message.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/augment.R
\name{add_design}
\alias{add_design}
\title{Add two designs}
\usage{
add_design(design_1, design_2, alpha)
}
\arguments{
\item{design_1}{A dataframe with 'Point' and 'Weight' as columns that represent the first design to add}

\item{design_2}{A dataframe with 'Point' and 'Weight' as columns that represent the second design to add}

\item{alpha}{Weight of the first design}
}
\value{
A design as a dataframe with the weighted addition of the two designs
}
\description{
Add two designs
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/efficient_round.R
\name{combinatorial_round}
\alias{combinatorial_round}
\title{Combinatorial round}
\usage{
combinatorial_round(
  design,
  n,
  criterion = NULL,
  model = NULL,
  parameters = NULL,
  par_values = NULL,
  weight_fun = function(x) 1,
  par_int = NULL,
  reg_int = NULL,
  matB = NULL,
  max_support = 15L,
  ask = TRUE
)
}
\arguments{
\item{design}{either a dataframe with the design to round, or an object of class \code{"optdes"}.
If a dataframe, the criterion, model and parameters must be specified. It must have a \code{Weight}
column and one or more design-variable columns (\code{Point} for single-factor,
\code{x1}, \code{x2}, \ldots for multi-factor).}

\item{n}{integer with the desired number of points of the resulting design.}

\item{criterion}{character variable with the chosen optimality criterion. Can be one of the following:
\itemize{
\item 'D-Optimality'
\item 'Ds-Optimality'
\item 'A-Optimality'
\item 'I-Optimality'
\item 'L-Optimality'
}}

\item{model}{formula describing the model. Must use \code{x} (single-factor) or
\code{x1}, \code{x2}, \ldots (multi-factor) as design variables.}

\item{parameters}{character vector with the parameters of the models, as written in the \code{formula}.}

\item{par_values}{numeric vector with the parameters nominal values, in the same order as given in \code{parameters}.}

\item{weight_fun}{optional one variable function that represents the square of the structure of variance, in case of heteroscedastic variance of the response.}

\item{par_int}{optional numeric vector with the index of the \code{parameters} of interest for Ds-optimality.}

\item{reg_int}{optional numeric vector with the ranges of integration, for I-optimality.}

\item{matB}{optional matrix of dimensions k x k, for L-optimality.}

\item{max_support}{integer. Number of support points above which the function triggers the
confirmation mechanism. Default is 15 (\eqn{2^{15} \approx 32\,000} combinations).}

\item{ask}{logical. If \code{TRUE} (default) and the design exceeds \code{max_support}:
in an interactive session the user is prompted; in a non-interactive session an error is raised.
Set \code{ask = FALSE} to skip confirmation in scripts or pipelines (a message is still emitted).}
}
\value{
A data.frame with the rounded design to n number of points, or \code{NULL} invisibly
if the user declines the confirmation prompt.
}
\description{
Given an approximate design and a number of points, computes all the possible combinations of
roundings of each point to the nearest integer, keeps the ones that amount to the requested number of points,
and returns the one with the best value for the criterion function.

The search is exhaustive and requires \eqn{2^k} evaluations where \eqn{k} is the number of
support points.  For designs with more than \code{max_support} support points the function
requests confirmation (interactive sessions) or stops with an informative error
(non-interactive sessions), unless \code{ask = FALSE}.
}
\examples{
aprox_design <- opt_des("D-Optimality", y ~ a * exp(-b / x), c("a", "b"), c(1, 1500), c(212, 422))
combinatorial_round(aprox_design, 27)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inf_mat_sens.R
\name{inf_mat}
\alias{inf_mat}
\title{Information Matrix}
\usage{
inf_mat(grad, design)
}
\arguments{
\item{grad}{A function in a single variable that returns the partial derivatives vector of the model.}

\item{design}{A dataframe that represents the design. Must have two columns:
\itemize{
\item \code{Point} contains the support points of the design.
\item \code{Weight} contains the corresponding weights of the \code{Point}s.
}}
}
\value{
The information matrix of the design, a \eqn{k\times k} matrix where k is the length of the gradient.
}
\description{
Given the gradient vector of a model in a single variable model and a design, calculates the information matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wf_mult.R
\name{WFMult}
\alias{WFMult}
\title{Master function for the cocktail algorithm, that calls the appropriate one given the criterion.}
\usage{
WFMult(
  init_design,
  grad,
  criterion,
  par_int = NA,
  matB = NA,
  design_space,
  grid.length,
  join_thresh,
  delete_thresh,
  k,
  delta_weights,
  tol,
  tol2,
  max_iter,
  compound = NULL,
  kl_spec = NULL
)
}
\arguments{
\item{init_design}{optional dataframe with the initial design for the algorithm.}

\item{grad}{function of partial derivatives of the model.}

\item{criterion}{character variable with the chosen optimality criterion.}

\item{par_int}{numeric vector with the index of the parameters of interest (Ds-Optimality).}

\item{matB}{optional matrix for L-optimality.}

\item{design_space}{named list with bounds for each design variable.}

\item{grid.length}{numeric value for the sensitivity search grid / LHS size.}

\item{join_thresh}{numeric value for the merge heuristic.}

\item{delete_thresh}{numeric value for the weight deletion threshold.}

\item{k}{number of unknown parameters of the model.}

\item{delta_weights}{numeric value in (0, 1), parameter of the algorithm.}

\item{tol}{numeric value for convergence of the weight loop.}

\item{tol2}{numeric value for the outer stop condition.}

\item{max_iter}{maximum number of outer iterations.}

\item{compound}{preprocessed list of compound criterion specifications (internal use).}

\item{kl_spec}{preprocessed list of KL-Optimality specifications (internal use).}
}
\value{
An object of class \code{optdes}.
}
\description{
Depending on the \code{criterion} the cocktail algorithm for the chosen criterion is called,
and the necessary parameters for the functions are given from the user input.
}
\seealso{
Other cocktail algorithms: 
\code{\link{CWFMult}()},
\code{\link{DWFMult}()},
\code{\link{DsWFMult}()},
\code{\link{IWFMult}()},
\code{\link{KLWFMult}()}
}
\concept{cocktail algorithms}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{plot.optdes}
\alias{plot.optdes}
\title{Plot function for optdes}
\usage{
\method{plot}{optdes}(x, ...)
}
\arguments{
\item{x}{An object of class \code{optdes}.}

\item{...}{Possible extra arguments (currently unused).}
}
\description{
For single-factor models, overlays the support points on the sensitivity function curve.
For two-factor models, shows a heatmap of the sensitivity function with the support
points overlaid and the Equivalence Theorem contour highlighted.
For models with more than two factors, shows a pairwise scatter matrix with one
panel per pair of design variables and point size proportional to weight.
}
\examples{
rri <- opt_des(criterion = "I-Optimality", model = y ~ a * exp(-b / x),
  parameters = c("a", "b"), par_values = c(1, 1500), design_space = c(212, 422),
  reg_int = c(380, 422))
plot(rri)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/augment.R
\name{print.augment_region}
\alias{print.augment_region}
\title{Print method for augment_region objects}
\usage{
\method{print}{augment_region}(x, ...)
}
\arguments{
\item{x}{An object of class \code{augment_region} returned by \code{get_augment_region}.}

\item{...}{Unused.}
}
\description{
Print method for augment_region objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{getStart}
\alias{getStart}
\title{Find where the candidate points region starts}
\usage{
getStart(cross, min, max, val, sens_opt)
}
\arguments{
\item{cross}{Vector of crosspoints in the sensitivity function given an efficiency and weight}

\item{min}{Minimum of the space of the design}

\item{max}{Maximum of the space of the design}

\item{val}{Value of the sensitivity function at the crosspoints}

\item{sens_opt}{Sensitivity function}
}
\value{
True if the candidate points region starts on the minimum, False otherwise
}
\description{
Given the crosspoints and the sensitivity function, this function
finds where the candidate points region starts, either on the extreme of
the space of the design or the first crosspoints
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{tr}
\alias{tr}
\title{Trace}
\usage{
tr(M)
}
\arguments{
\item{M}{The matrix from which to calculate the trace.}
}
\value{
The trace of the matrix.
}
\description{
Return the mathematical trace of a matrix, the sum of its diagonal elements.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glm_families.R
\name{make_glm_family}
\alias{make_glm_family}
\title{GLM family specification for KL-Optimality}
\usage{
make_glm_family(name)
}
\arguments{
\item{name}{character; one of \code{"Normal"}, \code{"Poisson"},
\code{"Binomial"} or \code{"Gamma"}.}
}
\value{
A named list with elements \code{b}, \code{b_prime}, \code{b_dbl},
\code{link}, \code{link_inv} and \code{name}.
}
\description{
Returns a GLM family object for use with \code{criterion = "KL-Optimality"}.
The family encodes the cumulant generating function and canonical link needed
to compute the Kullback-Leibler divergence between two distributions.
}
\examples{
make_glm_family("Poisson")
make_glm_family("Normal")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crit_eff.R
\name{crit}
\alias{crit}
\title{Master function for the criterion function}
\usage{
crit(criterion, M, k = 0, par_int = c(1), matB = NA)
}
\arguments{
\item{criterion}{character variable with the chosen optimality criterion. Can be one of the following:
\itemize{
\item 'D-Optimality'
\item 'Ds-Optimality'
\item 'A-Optimality'
\item 'I-Optimality'
\item 'L-Optimality'
}}

\item{M}{information matrix for which the criterion value wants to be calculated.}

\item{k}{numeric variable with the number of parameters of the model. Taken from the number of rows of the matrix if omitted.}

\item{par_int}{numeric vector with the index of the parameters of interest of the model. Only for "Ds-Optimality".}

\item{matB}{optional matrix of dimensions k x k, for I- and L-optimality.}
}
\value{
Numeric value of the optimality criterion for the information matrix.
}
\description{
Depending on the criterion input, the function returns the output of the corresponding criterion function given
the information matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{weight_function}
\alias{weight_function}
\title{Weight function per distribution}
\usage{
weight_function(model, char_vars, values, distribution = "Normal")
}
\arguments{
\item{model}{formula describing the model to use. Must use \code{x} as the design variable.}

\item{char_vars}{character vector with the parameters of the models, as written in the \code{formula}}

\item{values}{numeric vector with the parameters nominal values, in the same order as given in \code{parameters}.}

\item{distribution}{character variable specifying the probability distribution of the response. Can be one of the following:
\itemize{
\item 'Normal', for normal homoscedastic (default)
\item 'Gamma', which can be used for exponential or normal heteroscedastic with constant relative error
\item 'Poisson'
\item 'Logistic'
}}
}
\value{
one variable function that represents the square of the structure of variance, in case of heteroscedastic variance of the response.
}
\description{
Weight function per distribution
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{integrate_reg_int}
\alias{integrate_reg_int}
\title{Integrate IM}
\usage{
integrate_reg_int(grad, k, reg_int)
}
\arguments{
\item{grad}{function of partial derivatives of the model.}

\item{k}{number of unknown parameters of the model.}

\item{reg_int}{optional numeric vector of two components with the bounds of the interest region for I-Optimality.}
}
\value{
The integrated information matrix.
}
\description{
Integrates the information matrix over the region of interest to calculate matrix B to be used in I-Optimality
calculation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/augment.R
\name{get_augment_region}
\alias{get_augment_region}
\title{Get Augment Regions}
\usage{
get_augment_region(
  criterion,
  init_design,
  alpha,
  model,
  parameters,
  par_values,
  design_space,
  calc_optimal_design,
  par_int = NA,
  matB = NA,
  distribution = NA,
  weight_fun = function(x) 1,
  delta_val = NULL,
  n_lhs = 2000L
)
}
\arguments{
\item{criterion}{character with the chosen optimality criterion. Can be one of the following:
\itemize{
\item 'D-Optimality'
\item 'Ds-Optimality'
\item 'A-Optimality'
\item 'I-Optimality'
\item 'L-Optimality'
}}

\item{init_design}{dataframe with "Point" and "Weight" columns that represents the initial design to augment}

\item{alpha}{combined weight of the new points}

\item{model}{formula that represent the model with x as the independent variable}

\item{parameters}{character vector with the unknown parameters of the model to estimate}

\item{par_values}{numeric vector with the initial values of the unknown parameters}

\item{design_space}{numeric vector with the limits of the space of the design}

\item{calc_optimal_design}{boolean parameter, if TRUE, the optimal design is calculated and efficiencies of the initial and augmented design are given}

\item{par_int}{optional numeric vector with the index of the \code{parameters} of interest for Ds-optimality.}

\item{matB}{optional matrix of dimensions k x k, for L-optimality.}

\item{distribution}{character specifying the probability distribution of the response. Can be one of the following:
\itemize{
\item 'Homoscedasticity'
\item 'Gamma', which can be used for exponential or normal heteroscedastic with constant relative error
\item 'Poisson'
\item 'Logistic'
\item 'Log-Normal' (work in progress)
}}

\item{weight_fun}{optional one variable function that represents the square of the structure of variance, in case of heteroscedastic variance of the response}

\item{delta_val}{optional numeric value for the minimum relative efficiency. If \code{NULL} (default),
the user is prompted interactively. Providing this value enables non-interactive use.}

\item{n_lhs}{integer number of LHS points used to visualise and sample the candidate region for
multi-factor models with \code{d > 2} (default 2000). Increase for smoother scatter plots.}
}
\value{
A vector of the points limiting the candidate points region
}
\description{
Given a model and criterion, calculates the candidate points region. The user gives an initial design for which
he would like to add points and specifies the weight of the new points. Then he is prompted
to choose a minimum efficiency. After that, the candidate points region is calculated.
}
\examples{
\donttest{
init_des <- data.frame("Point" = c(30, 60, 90), "Weight" = c(1/3, 1/3, 1/3))
get_augment_region("D-Optimality", init_des, 0.25, y ~ 10^(a - b/(c + x)),
  c("a", "b", "c"), c(8.07131, 1730.63, 233.426), c(1, 100), FALSE,
  delta_val = 0.85)
}
}
